Package: pkgmaker
Type: Package
Title: Development Utilities for R Packages
Version: 0.32.10
Date: 2023-05-03
Authors@R: person("Renaud", "Gaujoux", email = "renozao@protonmail.com", role = c("aut", "cre"))
Description: Provides some low-level utilities to use for package
    development. It currently provides managers for multiple package specific
    options and registries, vignette, unit test and bibtex related utilities.
    It serves as a base package for packages like 'NMF', 'RcppOctave', 'doRNG', and
    as an incubator package for other general purposes utilities, that will
    eventually be packaged separately.
    It is still under heavy development and changes in the interface(s) are
    more than likely to happen.
License: GPL (>= 2)
URL: https://renozao.github.io/pkgmaker/
BugReports: https://github.com/renozao/pkgmaker/issues
LazyLoad: yes
Encoding: UTF-8
Depends: R (>= 3.0.0), methods, stats, registry
Imports: tools, grDevices, utils, graphics, codetools, digest, stringr,
        xtable, withr, assertthat
Suggests: devtools (>= 0.8), roxygen2, RUnit, testthat, knitr,
        rmarkdown, markdown (>= 1.3), yaml, Biobase, datasets,
        rbibutils (>= 1.3)
Collate: 'bibtex.R' 'colors.R' 'data.R' 'namespace.R' 'utils.R'
        'devutils.R' 'files.R' 'graphics.R' 'options.R' 'logging.R'
        'unitTests.R' 'is.R' 'knitr.R' 'registry.R' 'package-extra.R'
        'package.R' 'packages.R' 'project.R' 'rd.R' 'user.R'
        'vignette.R' 'with.r'
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-05-03 08:01:53 UTC; renaud
Author: Renaud Gaujoux [aut, cre]
Maintainer: Renaud Gaujoux <renozao@protonmail.com>
Repository: CRAN
Date/Publication: 2023-05-03 11:30:08 UTC
Built: R 4.4.0; ; 2026-07-18 10:24:03 UTC; unix

Package: pkgmaker
Type: Package
Title: Development Utilities for R Packages
Version: 0.32.10
Date: 2023-05-03
Authors@R: person("Renaud", "Gaujoux", email = "renozao@protonmail.com", role = c("aut", "cre"))
Description: Provides some low-level utilities to use for package
    development. It currently provides managers for multiple package specific
    options and registries, vignette, unit test and bibtex related utilities.
    It serves as a base package for packages like 'NMF', 'RcppOctave', 'doRNG', and
    as an incubator package for other general purposes utilities, that will
    eventually be packaged separately.
    It is still under heavy development and changes in the interface(s) are
    more than likely to happen.
License: GPL (>= 2)
URL: https://renozao.github.io/pkgmaker/
BugReports: https://github.com/renozao/pkgmaker/issues
LazyLoad: yes
Encoding: UTF-8
Depends: R (>= 3.0.0), methods, stats, registry
Imports: tools, grDevices, utils, graphics, codetools, digest, stringr,
        xtable, withr, assertthat
Suggests: devtools (>= 0.8), roxygen2, RUnit, testthat, knitr,
        rmarkdown, markdown (>= 1.3), yaml, Biobase, datasets,
        rbibutils (>= 1.3)
Collate: 'bibtex.R' 'colors.R' 'data.R' 'namespace.R' 'utils.R'
        'devutils.R' 'files.R' 'graphics.R' 'options.R' 'logging.R'
        'unitTests.R' 'is.R' 'knitr.R' 'registry.R' 'package-extra.R'
        'package.R' 'packages.R' 'project.R' 'rd.R' 'user.R'
        'vignette.R' 'with.r'
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2023-05-03 08:01:53 UTC; renaud
Author: Renaud Gaujoux [aut, cre]
Maintainer: Renaud Gaujoux <renozao@protonmail.com>
Repository: CRAN
Date/Publication: 2023-05-03 11:30:08 UTC

# Generated by roxygen2: do not edit by hand

S3method("$",ExposeAttribute)
S3method("$<-",ExposeAttribute)
S3method("[[",package_options)
S3method("[[<-",package_options)
S3method(.DollarNames,ExposeAttribute)
S3method(addnames,array)
S3method(addnames,default)
S3method(addnames,matrix)
S3method(addnames,vector)
S3method(format,package_metaregistry)
S3method(format,package_subregistry)
S3method(print,ExposeAttribute)
S3method(print,package_metaregistry)
S3method(print,package_options)
S3method(print,str_diff)
S3method(runVignette,default)
S3method(runVignette,rnw_knitr)
S3method(runVignette,rnw_sweave)
S3method(xtable,package_metaregistry)
export("attr_mode<-")
export(.silenceF)
export(CRAN)
export(ExposeAttribute)
export(R.CMD)
export(R.SHLIB)
export(R.exec)
export(RdSection2latex)
export(Rversion)
export(Sys.getenv_value)
export(addNamespaceExport)
export(add_lib)
export(addnames)
export(allFormals)
export(alphacol)
export(as.package_options)
export(as.rnw)
export(as_package)
export(askUser)
export(attr_mode)
export(cgetAnywhere)
export(charmap)
export(checkPlot)
export(checkWarning)
export(citecmd)
export(compactVignettes)
export(compile_src)
export(digest_function)
export(exitCheck)
export(expand_dots)
export(expand_list)
export(extractLocalFun)
export(factor2character)
export(file_extension)
export(find_devpackage)
export(getBiocMirror)
export(getBiocRepos)
export(getLoadingNamespace)
export(gfile)
export(hasArg2)
export(hasEnvar)
export(hasNames)
export(hasPackageRegistry)
export(hook_backspace)
export(hook_toggle)
export(hook_try)
export(inSweave)
export(install.extrapackages)
export(install.extras)
export(irequire)
export(is.dir)
export(is.file)
export(is.verbose)
export(isCHECK)
export(isCRAN_timing)
export(isCRANcheck)
export(isDevNamespace)
export(isInteger)
export(isLoadingNamespace)
export(isManualVignette)
export(isNamespaceLoaded2)
export(isNumber)
export(isPackageInstalled)
export(isReal)
export(isString)
export(is_NA)
export(is_option_symlink)
export(iterCount)
export(knit_ex)
export(latex_bibliography)
export(latex_preamble)
export(ldata)
export(libname)
export(library_project)
export(list.data)
export(list.libs)
export(listPackageOptions)
export(lmessage)
export(load_all_file)
export(load_project)
export(log_append)
export(lsilent)
export(lverbose)
export(makeFakeVignette)
export(makeUnitVignette)
export(make_vignette_auxfiles)
export(messagef)
export(mfrow)
export(mkoptions)
export(mrequire)
export(new2)
export(ns_get)
export(onLoad)
export(onUnload)
export(oneoffVariable)
export(option_symlink)
export(option_symlink_target)
export(orderVersion)
export(packageData)
export(packageEnv)
export(packageExtra)
export(packageExtraHandler)
export(packageExtraRunner)
export(packageOptions)
export(packagePath)
export(packageReference)
export(packageReferenceFile)
export(packageRegistries)
export(packageRegistry)
export(packageTestEnv)
export(package_bibliography)
export(parsePackageCitation)
export(pkgreg_fetch)
export(pkgreg_remove)
export(postponeAction)
export(qlibrary)
export(qrequire)
export(quickinstall)
export(read.yaml_section)
export(regfetch)
export(render_notes)
export(reorder_columns)
export(require.quiet)
export(requirePackage)
export(requireRUnit)
export(rnw)
export(rnwChildren)
export(rnwCompiler)
export(rnwDriver)
export(rnwIncludes)
export(rnwWrapper)
export(runPostponedAction)
export(runVignette)
export(sVariable)
export(setBiocMirror)
export(setCRANMirror)
export(setPackageExtra)
export(setPackageExtraHandler)
export(setPackageRegistry)
export(setPackageRegistryEntry)
export(setupPackageOptions)
export(simpleRegistry)
export(sortVersion)
export(source_files)
export(str_bs)
export(str_class)
export(str_desc)
export(str_diff)
export(str_dim)
export(str_fun)
export(str_hash)
export(str_md5sum)
export(str_ns)
export(str_out)
export(str_pkg)
export(sys.function_digest)
export(sys.function_frame)
export(sys.function_nframe)
export(sys.source_file)
export(testRversion)
export(topns)
export(topns_name)
export(unit.test)
export(unlist_)
export(unlist_with_sep)
export(userData)
export(userIs)
export(using_collate)
export(using_dir)
export(using_env)
export(using_envvar)
export(using_lib)
export(using_libpaths)
export(using_locale)
export(using_options)
export(using_par)
export(using_path)
export(utest)
export(utestFramework)
export(utestPath)
export(vignetteMakefile)
export(vmessage)
export(winbuild)
export(wnote)
export(write.pkgbib)
export(writeUnitVignette)
if (getRversion() < "3.5.0") export(isFALSE)
import(codetools)
import(digest)
import(grDevices)
import(graphics)
import(methods)
import(registry)
import(stats)
import(stringr)
import(utils)
importFrom(assertthat,assert_that)
importFrom(assertthat,is.number)
importFrom(assertthat,is.scalar)
importFrom(tools,Rd_db)
importFrom(tools,md5sum)
importFrom(withr,with_options)
importFrom(xtable,xtable)

# Run all unit tests in installed directory unitTests
# 
# Author: Renaud Gaujoux
###############################################################################

pkgmaker::utest('package:pkgmaker', quiet=FALSE)

# Package specific option system
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

#' @include unitTests.R
#' @include devutils.R
NULL

#' Package Specific Options
#' 
#' The following functions to access/set the options from the set are assigned 
#' in \code{envir}:
#' \describe{
#' \item{<subset>Options}{}
#' \item{<subset>GetOption}{}
#' }
#' 
#' @param ... a single named list or named arguments that provide the default 
#' options and their values.
#' @param NAME name of the set of options.
#' This is used as a prefix for the name of the associated global 
#' option: \code{package:<name>}.
#' @param ENVIR environment where the option wrapper functions will be defined.
#' No function is defined if \code{ENVIR=NULL} 
#' @param RESET a logical that indicates whether the option set should overwrite
#' one that already exists if necessary. 
#' The default is \code{FALSE} (i.e. no reset), except when loading a namespace, 
#' either from an installed package or a development package -- with devtools. 
#' If \code{FALSE}, an error is thrown if trying to setup options with the same name.
#'
#' @export
setupPackageOptions <- function(..., NAME=NULL, ENVIR=topenv(parent.frame()), RESET = isLoadingNamespace()){
	
	defaults <- .list_or_named_dots(...)
	
	# do not write into the Global environment
	e <- parent.frame()
	if( missing(ENVIR) && identical(e, .GlobalEnv) ){
		ENVIR <- NULL
	}
	
	# get calling package
	pkg <- packageName(.Global=TRUE)
	
	# prefix for the wrapper functions
	fprefix <- if( is.null(NAME) ) tolower(pkg) else NAME
	
	# define name for the option set
	optname <- pkg
	if( !is.null(NAME) )
		optname <- paste(optname, NAME, sep=':')
	
	# create package_options object
	optobj <- as.package_options(optname, defaults=defaults)
	
	# check if options with the same key are not already registered
	OLD <- getOption(optobj$name)
	if( !is.null(OLD) && !RESET )
		stop("Package specific options '", OLD$name, "' already exist: " 
				, " (", length(OLD$options())," default option(s))")
	
	# register the package_options object in global options
	message(if( is.null(OLD) ) "Setting" else "Resetting"
			, " package specific options: ", optobj$name
			, " (", length(optobj$options())," default option(s))")
	options(setNames(list(optobj), optobj$name))
	
	# (re)load registered package_options object from global options
	optobj <- getOption(optobj$name)
	stopifnot( !is.null(optobj) )
	
	# define wrapper functions in the supplied environment
	if( !is.null(ENVIR) ){
		isfun <- unlist(eapply(optobj, is.function))
		isfun <- isfun[names(isfun) != 'newOptions']
		ifun <- which(isfun)
		lapply(names(isfun)[ifun], function(x){
			f <- get(x, envir=optobj)
			assign(paste(fprefix, x, sep='.'), f, envir=ENVIR)
		})
	}
	
	# return package_options object
	optobj
}

is.package_options <- function(x){
	is(x, 'package_options')
}

#' @S3method print package_options
print.package_options <- function(x, ...){
	cat("<Package specific options: ", x$name, ">\n", sep='')
	cat("Registered: ", !is.null(getOption(x$name)), "\n", sep='')
	def <- if( identical(x$.options, x$.defaults) ) " <as default>"
	# show options
	if( length(x$.options) ){
		cat("Options",def,":\n", sep='');
		str(x$.options) 
	}else 
		cat("Options: none\n")
	# show defaults
	if( is.null(def) ){
		cat("Defaults:\n"); str(x$.defaults)
	}
}


#' \code{option_symlink} creates a symbolic link to option \code{x}.
#' 
#' @export
#' @rdname options
option_symlink <- function(x){
	if( !is.character(x) )
		stop("Symbolic link options must be character strings")
	structure(x, class='option_symlink')
}
#' \code{is_option_symlink} tests if \code{x} is a symbolic link option.
#' 
#' @param opts a list of options
#' 
#' @export
#' @rdname options
is_option_symlink <- function(x, opts){
	if( missing(opts) ) is(x, 'option_symlink')
	else is(opts[[x]], 'option_symlink')
}

#' \code{option_symlink_target} returns the end target option of a symbolic link 
#' option \code{x}.
#' 
#' @export
#' @rdname options
option_symlink_target <- function(x, opts){
	
	if( !is.list(opts) )
		stop("invalid argument `opts`: must be a list object")
	
	n <- 0
	track <- NULL
	while( is_option_symlink(x, opts) ){
		if( x %in% track )
			stop("cycling symbolic link options: ", str_out(c(track, x), Inf, sep=' -> '))
		track <- c(track, x)
		x <- opts[[x]]
		n <- n + 1
		
	}
	x
	
}

# unit test for option symbolic links
unit.test('option_symlink', {

	opt <- setupPackageOptions(a=1,b=2,c=option_symlink('a'),d=4)
	
	.test <- function(msg){
		checkIdentical(names(opt$options('a')), 'a', paste(msg, " - options: name of target is ok"))
		checkIdentical(names(opt$options('c')), 'c', paste(msg, " - options: name of link is ok"))
		checkIdentical(opt$options('c'), setNames(opt$options('a'), 'c'), paste(msg, " - options: link ok"))
		checkIdentical(opt$getOption('a'), opt$getOption('c'), paste(msg, " - getOption: link ok"))
	}
	
	.test('Default')
	opt$options(a=100)
	.test('After setting target')
	opt$options(c=50)
	.test('After setting link')
			
})

#' \code{as.package_options} creates an object such as the 
#' ones used to stores package specific options.
#' 
#' @param x a character string, a list or an object of class 
#' \code{package_options}.
#' @param defaults \code{NULL} or a list of default options 
#' with their values.   
#'
#' @export
#' @rdname options
as.package_options <- function(..., defaults=NULL){
	
	args <- .list_or_named_dots(...)
	
	x <- if( is.null(names(args)) ) args[[1]] 
	if( !is.null(names(args)) ) defaults <- args
	if( is.null(x) ) x <- basename(tempfile(''))
	
	# early exit if already a package_options object
	if( is.package_options(x) ){
		
		# new defaults?: clone into a new package_options object
		if( !missing(defaults) && is.list(defaults) ){
			optname <- basename(tempfile(str_c(x$name, '_')))
			x <- as.package_options(x$.options, defaults)
			x$name <- optname
		}
	
		return(x)
	}
	
	# create a package_options object
	.OPTOBJ <- structure(list2env(list(name=NULL, .options=NULL, .defaults=defaults))
						, class='package_options')
	
	if( is.character(x) ){
		
		# build name as 'package:*'
		x <- sub("^package:", '', x)
		.OPTOBJ$name <- paste('package:', x[1L], sep='')
		
	}else if( is.list(x) ){
		.OPTOBJ$name <- tempfile('package:')
		.OPTOBJ$.options <- x
	}else
		stop("Invalid argument `x`: must be a character string or a list.")
	
	# define options() 
	.OPTOBJ$options <- function(...){
		# call .options on package_options object
		.options(..., .DATA=.OPTOBJ)
	}
	# define getOption
	.OPTOBJ$getOption <- function (x, default = NULL) 
	{
		# use local specific function options()
		options <- .OPTOBJ$options
		
		if (missing(default)) 
			return(options(x)[[1L]])
		if (x %in% names(options())) 
			options(x)[[1L]]
		else default
	}
	# define newOption
	.OPTOBJ$newOptions <- function(...){
		defs <- .list_or_named_dots(..., named.only=TRUE)
		
		lapply(seq_along(defs),
			function(i){
			name <- names(defs)[i]
			value <- defs[[i]]
			# check defaults
            in_opts <- name %in% names(.OPTOBJ$.defaults) && !identical(.OPTOBJ$.defaults[[name]], value)
			if( in_opts && !isLoadingNamespace() ){
				message("Skipping option ", .OPTOBJ$name, "::`", name, "`: already defined with another default value")
		    }else{
                if( in_opts )
                    message("Overwriting option ", .OPTOBJ$name, "::`", name, "` : already defined with another default value")
				.OPTOBJ$.defaults[[name]] <- value
				.OPTOBJ$.options[[name]] <- value
			}
		})
		invisible()
	}
	# define resetOptions
	.OPTOBJ$resetOptions <- function(..., ALL=FALSE){
		
		defaults <- .OPTOBJ$.defaults
		if( ALL ){
			.OPTOBJ$.options <- NULL
		}
		if( length(list(...)) > 0L ){
			onames <- c(...)
			if( !is.character(onames) )
				stop('character strings expected for resetting option names')
			defaults <- defaults[names(defaults) %in% onames]
			if( length(not_default <- onames[!onames %in% names(defaults)]) ){
				.OPTOBJ$.options[not_default] <- NULL
			}
		}
		if( length(defaults) ){
			.OPTOBJ$options(defaults)
		}
	}
	# define showOptions
	.OPTOBJ$printOptions <- function() print(.OPTOBJ)
	
	# initialise with default options 
	.OPTOBJ$resetOptions()
	
	# return pacakge_options object
	.OPTOBJ
}


#' The method \code{[[} is equivalent to \code{options()} or \code{getOption(...)}:
#' e.g. \code{obj[[]]} returns the list of options defined in \code{obj}, and 
#' \code{obj[['abc']]} returns the value of option \code{'abc'}.
#' 
#' @param ... arguments passed to \code{getOption} (only first one is used). 
#'  
#' @rdname options 
#' @S3method [[ package_options
"[[.package_options" <- function(x, ...){
	if( missing(..1) ) x$options()
	else x$getOption(..1)
}

#' @S3method [[<- package_options
"[[<-.package_options" <- function(x, i, value){
	x$.options[[i]] <- value 
}


##' @S3method [[ package_options
#`[[.package_options` <- function(x, ..., follow=FALSE){
#	
#	if( missing(..1) ) as.list(x$.options)
#	else if( follow ){
#		x$.options[[option_symlink_target(..1, x)]]
#	}else x$.options[[..1]]
#}
#
##' @S3method [[<- package_options
#`[[<-.package_options` <- function(x, i, ..., value){
#	
#	follow <- if( missing(..1) ) FALSE else ..1 
#	if( follow ){
#		old <- x[[i]]
#		if( is_option_symlink(old) && !is_option_symlink(value) )
#			x$.options[[option_symlink_target(i, x)]] <- value
#	}else x$.options[[i]] <- value
#}

.list_or_named_dots <- function(..., named.only=FALSE){
	
	dots <- list(...)
	if( length(dots) == 0L ) return()
	
	params <- dots
	if( is.null(names(dots)) && length(dots)==1L ){
		if ( is.list(dots[[1L]]) ){ 
			params <- dots[[1L]]
			if( is.null(names(params)) || any(names(params)=='') )
				stop("single list argument must only have named elements")
		}
	}
	if( named.only ){
		if( is.null(names(params)) || any(names(params)=='') )
			stop("all arguments be named")
	}
	
	params
}

#' Quick Option-like Feature
#' 
#' \code{mkoptions} is a function that returns a function that 
#' behaves like \code{\link[base]{options}}, with an attached 
#' internal/local list of key-value pairs.
#' 
#' @rdname local-options
#' @seealso \code{\link{setupPackageOptions}}
#' @export
#' 
#' @examples
#' f <- mkoptions(a=3, b=list(1,2,3))
#' str(f())
#' f('a')
#' f('b')
#' str(old <- f(a = 10))
#' str(f())
#' f(old)
#' str(f())
#' 
mkoptions <- function(...){
	
	.DATA <- new.env(parent=emptyenv())
	.defaults <- list(...)
	.DATA$.options <- list(...)
	function(...){		
		.options(..., .DATA=.DATA)
	}
}

#' \code{.options} is a low-level function that mimics the behaviour 
#' of the base function \code{\link[base]{options}}, given a set 
#' of key-value pairs.
#' It is the workhorse function used in \code{mkoptions} and package-specific
#' option sets (see \code{\link{setupPackageOptions}})
#' 
#' @param ... list of keys or key-value pairs.
#' For \code{mkoptions} these define inital/default key-value pairs. 
#' @param .DATA a list or an environment with an element \code{.options}.
#' 
#' @rdname local-options
.options <- function(..., .DATA){
	
	opts <- if( is.package_options(.DATA) || is.environment(.DATA) ) .DATA$.options else .DATA
	
	params <- .list_or_named_dots(...)
	# return complete option list if no other argument was passed
	if( is.null(params) ) return(opts)
	
	# initialise opts to an empty list if necessary 
	if( is.null(opts) ) opts <- list()
	stopifnot( is.list(opts) )
	
	# READ ACCESS
	if ( is.null(names(params)) ){
		if( !is.character(c(...)) )
			stop('character strings expected for option names')
		
		cparams <- c(...)
		# retrieve options as a list (use sapply to also get non-existing options)
		res <- sapply(cparams, function(n){
					# follow link if necessary
					opts[[option_symlink_target(n, opts)]]
				}, simplify=FALSE)
		return(res)
	}
	
	# WRITE ACCESS
	old <- sapply(names(params), 
			function(name){
				# assign the new value into the options environment
				val <- params[[name]]
				old <- opts[[name]]
				# change value of target if symlink and the new value is not a symlink
				if( is_option_symlink(old) && !is_option_symlink(val) )
					opts[[option_symlink_target(name, opts)]] <<- val
				else
					opts[[name]] <<- val
				# return the option's old value
				old
			}
			, simplify = FALSE
	)	
	#old <- old[!sapply(old, is.null)]

	# update package_options object in place if necessary (NB: it is an environment)
	if( is.package_options(.DATA) || is.environment(.DATA) ) .DATA$.options <- opts
	
	# return old values of the modified options
	return( invisible(old) )
}

#' \code{packageOptions} provides access to package specific options from a 
#' given package that were defined with \code{setupPackageOptions}, and behaves as the base function \code{\link[base]{options}}.
#' 
#' @param PACKAGE a package name
#' @inheritParams base::options 
#' 
#' @export
#' @rdname options
packageOptions <- function(..., PACKAGE = packageName()){
		
	# create/retrieve a package_options object from .DATA
	optobj <- as.package_options(PACKAGE)
	optobj <- getOption(optobj$name)
	
	# call the package_options object's options() function
	optobj$options(...)
}

#' \code{listPackageOptions} returns the names of all option 
#' currently defined with \code{setupPackageOptions}.
#' 
#' @return a character vector (possibly empty).
#'
#' @export
#' @rdname options 
#' @examples
#' listPackageOptions()
#' 
listPackageOptions <- function(){
	grep('^package:', names(options()), value=TRUE)
} 

# Unit tests for options
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

context('Package specific options')

test_that('setupPackageOptions', {
	
	opt <- setupPackageOptions(a=1,b=2,c=option_symlink('a'),d=4, RESET=TRUE)
	
	.test <- function(msg){
		expect_identical(names(opt$options('a')), 'a', paste(msg, " - options: name of target is ok"))
    expect_identical(names(opt$options('c')), 'c', paste(msg, " - options: name of link is ok"))
    expect_identical(opt$options('c'), setNames(opt$options('a'), 'c'), paste(msg, " - options: link ok"))
    expect_identical(opt$getOption('a'), opt$getOption('c'), paste(msg, " - getOption: link ok"))
	}
	
	.test('Default')
	opt$options(a=100)
	.test('After setting target')
	opt$options(c=50)
	.test('After setting link')

})

test_that('resetOptions', {
	opt <- setupPackageOptions(a=1,b=2,c=option_symlink('a'),d=4, RESET=TRUE)
	
	.checkOptions <- function(y, msg) expect_identical(opt$options(), y, msg)
	ref <- opt$options()
	# simple set
	opt$options(a=10)
	x <- ref
	x$a <- 10
	.checkOptions(x, 'change default option works')
	opt$resetOptions()
	.checkOptions(ref, 'default options are reset after resetOptions()')
	
	# new option
	opt$options(aaa=10)
	x <- ref
	x$aaa <- 10
	.checkOptions(x, 'add new option works')
	opt$resetOptions()
	.checkOptions(c(ref, aaa=10), 'new option kept after resetOptions()')
	opt$resetOptions(ALL=TRUE)
	.checkOptions(ref, 'all options are reset to default and new options are removed when ALL=TRUE')
	opt$options(a=20, b='c', d='toto', aaa=25)
	x <- ref
	x$a <- 20
	x$b <- 'c'
	x$d <- 'toto'
	x$aaa <- 25
	.checkOptions(x, '2 default options + 1 new are set correctly')
	opt$resetOptions('a', 'b')
	x$a <- 1
	x$b <- 2
	.checkOptions(x, 'only the specified options are reset to default when passed in argument')
	opt$resetOptions('bbb')
	.checkOptions(x, 'if some options are not present it does not affect the result')
	opt$resetOptions('aaa')
	x$aaa <- NULL
	.checkOptions(x, 'new options get removed')
	
})

# Filesystem related functions
# 
# Author: Renaud Gaujoux
###############################################################################

#' Library Files Utilities
#' 
#' Lists binary library files in a directory
#' 
#' @param dir directory
#' @param all.platforms a logical that indicates whether to list library files for 
#' the current platform only (default) or all platforms (Unix, Windows, Mac).
#' @param ... extra arguments passed to \code{\link{list.files}}.
#' 
#' @return a character vector
#' @export
#' @rdname libutils
list.libs <- function(dir, ..., all.platforms=FALSE){
	
	p <- if( !all.platforms ){
		str_c("\\", .Platform$dynlib.ext, "$")
	}else{
		p <- str_c("(\\.", c('so', 'dll'), , ')', collapse='|')
		str_c(p, '$')
	}
	list.files(dir, pattern=p, ...)
}

#' \code{libname} extracts library names from a path, removing the 
#' directory part of the path, as well as the platform 
#' specific library extension.
#' 
#' @param x a filename
#' 
#' @export
#' @rdname libutils
#' 
#' @examples
#' 
#' libname('mylib.so')
#' libname('/some/path/somewhere/mylib.dll') 
#' 
libname <- function(x){
	sub(str_c("\\", .Platform$dynlib.ext, "$"), "", basename(x))
}


#' Source Multiple Files
#' 
#' Vectorised version of \code{source}.
#' 
#' @param x character vector containing filenames
#' @inheritParams base::list.files
#' @param ... extra arguments passed to \code{\link{source}}.
#' 
#' @export
source_files <- function(x, pattern=NULL, ...){
	if( length(x) == 1L && is.dir(x) )
		x <- list.files(x, pattern=pattern, full.names=TRUE)
	invisible(sapply(x, source, ...))
}

#' Extract File Extension
#' 
#' @param x path as a character vector.
#' @param ext extension to append instead of the original extension.
#' 
#' @export
#' 
#' @examples
#' 
#' file_extension('alpha.txt')
#' file_extension(paste('aa.tt', 1:5, sep=''))
#' # change extension
#' file_extension(paste('aa.tt', 1:5, sep=''), 'pdf')
#' file_extension(paste('aatt', 1:5, sep=''), 'pdf')
#' 
file_extension <- function(x, ext=NULL){
	
	if( is.null(ext) ) sub(".*\\.([^.]{3})$","\\1",x)
	else str_c(sub("(.*)(\\.([^.]{3}))$","\\1", x), '.', sub("^.", '', ext))
}

## ----setup, include=FALSE------------------------------------------------
pkg <- 'pkgmaker'
require( pkg, character.only=TRUE )
prettyVersion <- packageDescription(pkg)$Version
prettyDate <- format(Sys.Date(), '%B %e, %Y')
authors <- packageDescription(pkg)$Author

# General test utility functions to check the type of objects
# 
# Author: Renaud Gaujoux
# Creation: 30 Apr 2012
###############################################################################
#' @include unitTests.R
NULL

#' Testing Object Type 
#' 
#' @name is_something
#' @rdname is_something
#' 
#' @return \code{TRUE} or \code{FALSE}
NULL

#' \code{is_NA} tests if a variable is exactly NA (logical, character, numeric or integer)
#' 
#' @param x an R object
#' @rdname is_something
#' @export
is_NA <- function(x){ 
	identical(x, NA) || identical(x, as.character(NA)) || identical(x, as.numeric(NA)) || identical(x, as.integer(NA))
}

#' \code{isFALSE} Tests if a variable is exactly FALSE.
#' 
#' @rdname is_something
#' @seealso \code{\link{isTRUE}}
#' @export
isFALSE <- function(x) identical(x, FALSE)

#' \code{isNumber} tests if a variable is a single number
#' 
#' @rdname is_something
#' @export
isNumber <- function(x){ 
	is.numeric(x) && length(x) == 1
}

#' \code{isReal} tests if a variable is a single real number
#' 
#' @rdname is_something
#' @export
isReal <- function(x){ 
	isNumber(x) && !is.integer(x)
}

#' \code{isInteger} tests if an object is a single integer
#' @rdname is_something
#' @export
isInteger <- function(x){ 
	is.integer(x) && length(x) == 1
}


#' \code{isString} tests if an object is a character string. 
#' 
#' @param y character string to compare with.
#' @param ignore.case logical that indicates if the comparison 
#' should be case sensistive.
#' 
#' @rdname is_something
#' @export
isString <- function(x, y, ignore.case=FALSE){
	if( res <- is.character(x) && length(x) == 1L ){
		if( !missing(y) ){
			if( !isString(y) ) stop("Invalid argument 'y': must be a string itself.")
			if( ignore.case ) {
				x <- toupper(x)
				y <- toupper(y)
			}
			res <-  x == y
		}
	}
	res
}

#' \code{is.dir} tests if a filename is a directory.
#' 
#' @rdname is_something
#' @export
is.dir <- function(x) file_test('-d', x)


#' \code{is.file} tests if a filename is a file.
#' 
#' @rdname is_something
#' @export
is.file <- function(x) file_test('-f', x)

#' \code{hasNames} tests if an object has names.
#' 
#' @param all logical that indicates if the object needs all names non empty
#' @rdname is_something
#' @export
hasNames <- function(x, all=FALSE){
	nm <- names(x)
	if( length(x) == 0L ) TRUE
	else !is.null(nm) && (!all || !is.element('', nm) )
}

unit.test(hasNames, {
			
			add_names <- function(x) setNames(x, letters[1:length(x)])
			# vector
			checkTrue(hasNames( add_names(1:10) ))
			checkTrue(hasNames( add_names(1:10) , all=TRUE))
			checkTrue(hasNames( c(add_names(1:10),11) ))
			checkTrue(!hasNames( c(add_names(1:10),11) , all=TRUE))
			# list
			checkTrue(hasNames( add_names(list(1,2,3)) ))
			checkTrue(hasNames( add_names(list(1,2,3)) , all=TRUE))
			checkTrue(hasNames( c(add_names(list(1,2,3)),11) ))
			checkTrue(!hasNames( c(add_names(list(1,2,3)),11) , all=TRUE))
			
		})

# Unit tests utilities
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

#' @include utils.R
#' @include logging.R 
NULL

#' Load RUnit Compatible Package
#' 
#' Loads the package responsible for the implementation of the RUnit framework,
#' choosing amongst \sQuote{RUnitX}, \sQuote{svUnit} and \sQuote{RUnit}.
#' 
#' @param ... arguments passed to \code{\link{requirePackage}}.
#' 
#' @return nothing
#' @export
#' 
requireRUnit <- local({
			
	.cache <- NULL
	function(...){
		
		if( !is.null(.cache) ) return(.cache)
		
		has_pkg <- function(x) length(find.package(x, quiet=TRUE)) > 0L
		
		ruf <- c('RUnit', 'svUnit')
		runit <- NULL
		for( pkg in ruf){
			if( require.quiet(pkg, character.only=TRUE) ){
				runit <- pkg
				break
			}
		}
		
		if( is.null(runit) )
			stop("Cannot find any package providing RUnit framework.")
		message("Using RUnit framework provider: ", runit)
		
		.cache <<- runit
		# return name of the loaded framework 
		invisible(runit)
	}
	
})


# Borrowed from RUnit::.existsTestLogger
.existsTestLogger <- function(envir = .GlobalEnv){
    exists(".testLogger", envir = envir) && inherits(.testLogger, "TestLogger")
}

#' Enhancing RUnit Logger
#' 
#' Adds a function or a local variable to RUnit global logger.
#' 
#' @param name name of the function or variable to add 
#' @param value object to append to the logger.
#' If \code{value} is a function it is added to the list and is accessible via 
#' \code{.testLogger$name}.
#' If \code{value} is a variable it is added to the local environment and is 
#' therefore accessible in all logging functions.
#' @param logger an optional RUnit logger object. 
#' If missing or \code{NULL}, the object \code{.testLogger} is searched in  
#' \code{.GlobalEnv} -- and an error is thrown if it does not exist. 
#' 
#' @return the modified logger object. Note that the global object is also 
#' modified if \code{logger} is \code{NULL}.
#' 
addToLogger <- function(name, value, logger=NULL){
	
	
	logobj <- 
		if( !is.null(logger) ) logger
		else{
			if( !.existsTestLogger() )
				stop("No global logger exists")
			
			get('.testLogger', envir=.GlobalEnv)
		}
	
	# get local logger environment
	logenv <- environment(logobj$incrementCheckNum)

	if( is.function(value) ){# add function to logger
		if( is.null(logobj[[name]]) ){
			environment(value) <- logenv 
			logobj[[name]] <- value
			
			# update global logger if necessary
			if( is.null(logger) ){
				ge <- .GlobalEnv
				assign('.testLogger', logobj, envir=ge)
			}
		}
	}else{ # assign object in logger's local environment if not already there
		if( !exists(name, envir=logenv) )
			assign(name, value, envir=logenv)
	}
	
	# return modified logger object
	logobj
}

#' Plot in Unit Tests
#' 
#' Saves a plot in a PNG file that will be included in unit test HTML reports.
#' 
#' @param expr expression that generate th eplot
#' @param width plot width
#' @param height plot height (not used)  
#' @param msg plot msg explaining the plot . It will be used as the caption
#' 
#' @export
#' @keywords internal
checkPlot <- function(expr, msg=NULL, width=1000, height=NULL){
	
	# get stuff from RUnit
	uf <- requireRUnit()
	if( is.null(uf) || uf != 'RUnit' ) return(TRUE)
	#.existsTestLogger <- RUnit:::.existsTestLogger	
	.testLogger <- if( .existsTestLogger() ) .GlobalEnv$.testLogger
	
	if (missing(expr)) {
		stop("'expr' is missing.")
	}
	
	
	plotfile <- 
	if (.existsTestLogger()) {
		
		.testLogger$incrementCheckNum()
		
		if( is.null(.testLogger$setPlot) ){
			# add .plot list to logger environment
			addToLogger('.plots', NULL)
			
			
			# add function setPlot to logger
			.plots <- NULL # to trick R CMD check 
			addToLogger('setPlot', 
				function(name, msg=''){
					##@bdescr
					## add a plot to a test function.
					##@edescr
					##@in testFuncName : [character] name of test function
					##@in name : [character] filename
					##@in msg : [character] message string
					##@edescr	
					
					if( is.null(.plots) ) .plots <<- list()
					.plots[[name]] <<- msg
				}
			)
			
			# add function setPlot to logger
			.getTestData <- 
				.currentTestSuiteName <- 
				.currentSourceFileName <- 
				.getCheckNum <- NULL # not to get NOTES is R CMD check
			addToLogger('getPlotfile', 
				function(name, msg=''){
					
					td <- .getTestData()
					# TODO from test function name
					#fname <- tail(names(td[[.currentTestSuiteName]]$sourceFileResults[[.currentSourceFileName]]), 1L)
					fname <- basename(tempfile(paste(.currentTestSuiteName, '_', .currentSourceFileName, '_', sep='')))
					paste(fname, .getCheckNum(), sep='_')
					
				}
			)
			
			# update local object with modified global logger
			.testLogger <- .GlobalEnv$.testLogger
		}
		
		.testLogger$getPlotfile()
	}
	else tempfile(tmpdir='.')
	
	# add extension to plot file
	plotfile <- paste(plotfile, 'png', sep='.')
	
	# reset the msg if none was provided
	if( is.null(msg) ) msg <- plotfile

	#plot in the PNG file
	png(filename=plotfile, width=width)
	
	# evaluate the expression that generates the plot
	res <- try( eval(expr, envir = parent.frame()) )
	# close the graphic device
	dev.off()
	
	# test if everything went alright
	fileinfo <- file.info(plotfile)	
	if( inherits(res, "try-error") || is.na(fileinfo$size[1]) || fileinfo$size[1] == 0 ){
		#make sure the plot file is removed
		unlink(plotfile)
		
		if (.existsTestLogger()) {
			.testLogger$setFailure()
		}
		stop("Problem when generating plot:", res, msg)
	}
	
	if (.existsTestLogger()) {
		.testLogger$setPlot(plotfile, msg)
	}
	return(TRUE)
	
}

if( FALSE ){
	
	library(NMF, lib='build/lib')
	utest('pkg/inst/tests/runit.algorithms.r', fun='test.brunet', framework='RUnit')
	
}

#' Extra Check Functions for RUnit
#' 
#' \code{checkWarning} checks if a warning is generated by an expression, and 
#' optionally follows an expected regular expression pattern.
#' 
#' @param expr an R expression
#' @param expected expected value as regular expression pattern.
#' If a logical, then it specifies if a warning is expected or not.
#' 
#' For backward compatibility, a \code{NULL} value is equivalent to \code{TRUE}.
#' @param msg informative message to add to the error in case of failure
#' 
#' @export
#' @rdname uchecks
#' 
#' @examples 
#' 
#' # check warnings
#' checkWarning({ warning('ah ah'); 3})
#' checkWarning({ warning('ah oh ah'); 3}, 'oh')
#' try( checkWarning(3) )
#' try( checkWarning({ warning('ah ah'); 3}, 'warn you') )
#' 
checkWarning <- function(expr, expected=TRUE, msg=NULL){
	
	# get stuff from RUnit
	uf <- requireRUnit()
	#.existsTestLogger <- RUnit:::.existsTestLogger	
	.testLogger <- if( .existsTestLogger() ) .GlobalEnv$.testLogger
	
	if (missing(expr)) {
		stop("'expr' is missing")
	}
#	if (is.null(silent)) {
#		silent <- FALSE
#		warning("'silent' has to be of type 'logical'. Was NULL. Set to FALSE.")
#	}
#	
	if (.existsTestLogger()) {
		.testLogger$incrementCheckNum()
	}
	
	pf <- parent.frame()
	warns <- NULL
	withCallingHandlers(eval(expr, envir = pf)
		, warning = function(w){
			warns <<- c(warns, w$message)
		}
	)
	
	# check that some warning was thrown
	if( length(warns) == 0L ){
        if( isFALSE(expected) ) return( TRUE )
		if (.existsTestLogger()) {
			.testLogger$setFailure()
		}
		stop("Warning not generated as expected\n", msg)
	}
	if( isFALSE(expected) ){
        if (.existsTestLogger()) {
			.testLogger$setFailure()
		}
		stop("Warning generated while none was expected:\n"
            , "  - Warning(s): ", if(length(warns)>1)"\n    * ",  str_out(warns, Inf, sep="\n    * ") ,"\n"
            , msg)
    }
	# check warnings
	if( is.null(expected) || isTRUE(expected) ) return(TRUE)
	if( any(grepl(expected, warns)) ) return(TRUE)
	
	# throw error
	if (.existsTestLogger()) {
		.testLogger$setFailure()
	}
	stop("Warning does not match expected pattern:\n"
		, "  - Warning(s): ", if(length(warns)>1)"\n    * ",  str_out(warns, Inf, sep="\n    * ") ,"\n"
		, "  - Pattern: '", expected,"'\n"
		, msg)
	
	TRUE
}

#' Make Vignette for Unit Tests
#' 
#' Builds a vignette for unit tests in a package using the \code{\link{utest}} 
#' and a template vignette file. 
#' 
#' @param pkg Package name
#' @param file Output file (.Rnw, .tex, or .pdf)
#' @param ... extra arguments passed to \code{\link{utest}}.
#' @param check logical that indactes the cal was made from R CMD check, in which case the vignette
#' is updated only if results of unit tests can be found in the unit test output directory, 
#' where they would have been generated by \code{\link{utest}}.
#' 
#' @return Result of running unit test suite
#'  
#' @export
#' 
makeUnitVignette <- function(pkg, file=paste(pkg, '-unitTests.pdf', sep=''), ..., check=FALSE){
	
	package <- pkg
	pkg <- sub("^package:", "", pkg)
	# generate the vignette for unit test on exit
	if( !is.null(file) )
		on.exit( writeUnitVignette(pkg, file, check=check) )
	# load this package
	if( !require(pkg, character.only = TRUE ) ){
		stop("Could not load package '", pkg, "' for testing [libPath= ", str_out(.libPaths(), Inf), "]")
	}
	
	# run unit tests if not check or if the test results are not there (e.g., R CMD build)
#	if( userIs('renaud') ){
#		env <- str_trim(capture.output(system('env', intern=TRUE)))
#		if( check )	write(env, file="~/check_env.txt")
#		else write(env, file="~/make_env.txt")
#	}
	
#	if( !check || !is.dir(utestPath(package=package)) ){
	if( !check ){
		
		# force running all tests 
		utestCheckMode(FALSE)
		
		# run unit tests
		tests <- utest(package, ...)
		
		# check for errors
		err <- getErrors(tests)
		errMsg <- NULL
		if( err$nFail > 0) {
			errMsg <- c(errMsg, sprintf( "unit test problems: %d failures\n", err$nFail))
		}
		if( err$nErr > 0) {
			errMsg <- c(errMsg, sprintf( "unit test problems: %d errors\n", err$nErr))
		}
		# stop if any failure or error occured
		if( length(errMsg) > 0L )
			stop(errMsg)
		
		# return result of unit test suite
		err
	}else{
		# do nothing: tests should have been already run by R CMD check
	}
	
}

#' Writes Unit Tests Vignette 
#' 
#' Writes a vignette that contains the results from running unit test suites.
#' 
#' @param pkg Package name
#' @param file Output Sweave (.Rnw) file
#' @param results result file or output character vector
#' @param check logical that indactes the cal was made from R CMD check, 
#' in which case the vignette is updated only if results of unit tests can 
#' be found in the unit test output directory, where they would have been 
#' generated by \code{\link{utest}}.
#' 
#' @export
#' 
writeUnitVignette <- function(pkg, file, results=NULL, check=FALSE){
	Rnw.template <- 
"
\\documentclass[10pt]{article}
%\\VignetteDepends{knitr}
%\\VignetteIndexEntry{@pkg@-unitTests}
%\\VignetteCompiler{knitr}
%\\VignetteEngine{knitr::knitr}
\\usepackage{vmargin}
\\setmargrb{0.75in}{0.75in}{0.75in}{0.75in}

<<setup, include=FALSE>>=
pkg <- '@pkg@'
require( pkg, character.only=TRUE )
prettyVersion <- packageDescription(pkg)$Version
prettyDate <- format(Sys.Date(), '%B %e, %Y')
authors <- packageDescription(pkg)$Author
@

\\usepackage[colorlinks]{hyperref}
\\author{\\Sexpr{authors}}
\\title{\\texttt{\\Sexpr{pkg}}: Unit testing results@resNote@}
\\date{\\texttt{\\Sexpr{pkg}} version \\Sexpr{prettyVersion} as of \\Sexpr{prettyDate}}
\\begin{document}
\\maketitle

@results@

\\section*{Session Information}
@sessionInfo@

\\end{document}
"
	verbatim_wrap <- function(...){
		c("\\\\begin{verbatim}\n", ..., "\n\\\\end{verbatim}")
	}
	# default is to load the unit test results from the global output directory
	if( is.null(results) ){
		upath <- utestPath(package=pkg)
		results <- list.files(upath, pattern="\\.txt$", full.names=TRUE)
		if( !length(results) ){
			results <- verbatim_wrap('Could not find any unit test result in "', upath, '"')
		}
	}
	
	if( is.file(results[1L]) ){
		resFile <- results[1L]
		name <- str_match(resFile, "([^.]+)\\.[^.]+$")[,2L]
		results <- c(str_c("\\\\section{", name, "}"), verbatim_wrap(readLines(resFile)))
	}else{
		resFile <- NULL
	}
	results <- paste(results, collapse="\n")
	
	# substitute template variables
	contents <- Rnw.template
	# package name
	contents <-	gsub("@pkg@", pkg, contents)
	# unit test results
	contents <-	gsub("@results@", results, contents)
	# session info (as when calling this function)
	contents <-	gsub("@sessionInfo@", gsub("\\", "\\\\", paste(toLatex(sessionInfo()), collapse="\n"), fixed=TRUE), contents)
	# note on how tests were performed
	resnote <- str_c("\\footnote{Vignette computed ", if( check ) ' via R CMD check/build ', ' on ', date(),"}")
	if( check ){ 
		# add path to included file if compiled from R CMD check (for debug purposes)
		lfile <- gsub("([_$])", "\\\\\\1", paste(resFile, collapse="\\\\"))
		resnote <- str_c(resnote, " \\footnote{File: '", lfile, "'}")
	}
	contents <-	gsub("@resNote@", gsub("\\", "\\\\", resnote, fixed=TRUE), contents)
	
	fileext <- toupper(file_extension(file))
	fileext <- charmatch(fileext, c('RNW', 'TEX', 'PDF'))
	if( is_NA(fileext) )
		stop("Invalid output file extension [",fileext,"] from file '", file, "'")
	
	fileRNW <- if( fileext == 1L ) file else str_c(pkg, '-unitTests.Rnw')
	fileTEX <- if( fileext == 2L ) file else str_c(pkg, '-unitTests.tex')
	filePDF <- if( fileext == 3L ) file else str_c(pkg, '-unitTests.pdf')
	
	# write into Rnw file
	writeLines(contents, fileRNW)	
	if( fileext == 1L ) return()
	
	# compile vignette
	rnw(fileRNW, fileTEX)
	if( fileext == 2L ) return()
	
	# Run texi2dvi tex file
	res <- tools::texi2dvi(fileTEX, pdf = TRUE, clean = TRUE )
	
	# copy file in main check directory
	if( check )	file.copy(filePDF, '../../..')
	res
}

# Unit test frameworks data
.UFdata <- list(
	RUnit = list(
		file_pattern="^runit.*\\.[rR]$"
		, fun_pattern="^test\\."
		, check_pattern = "^check.+"
		, check_functions = c(
				'checkTrue'
				, 'checkIdentical'
				, 'checkEquals'
				, 'checkEqualsNumeric'
				, 'checkException'
		)
	)
	, testthat = list(
		file_pattern="^test.*\\.[rR]$"
		, check_pattern = "^(expect_.+)|(test_that$)" 
		, check_functions = c(
				"test_that"
				, "expect_equal"
				, "expect_equivalent"
				, "expect_error"
				, "expect_false"
				, "expect_identical"
				, "expect_is"
				, "expect_match"
				, "expect_message"
				, "expect_output"
				, "expect_that"
				, "expect_true"
				, "expect_warning"   
		)
	)
)

#' Inferring Unit Test Framework
#' 
#' @param x an filename, a function or the body of a function
#' @param eval a logical that indicates if the value of \code{x} should be used.
#' 
#' @return the name of the framework as a character string or NULL if
#' it could not be detected.
#' 
#' @import codetools
#' @export
utestFramework <- function(x, eval=FALSE){
	
	# check if one should detect within an expression
	expr <- if( missing(eval) || !eval ) substitute(x) 
			else if( is.function(x) ) body(x)
	
	# walk code using codetools looking up for known test functions
	if( !is.null(expr) ){
		cw <- makeCodeWalker(leaf= function(e, w) if( is.symbol(e) ) cat(e, "\n"))
		s <- str_trim(capture.output(walkCode(expr, cw)))
		if( length(s) > 1L ){
			for( f in names(.UFdata) ){
				if( any(s %in% .UFdata[[f]]$check_functions) ){
					return(f)
				}
			}
		}
		# not found without evaluating
		if( !missing(eval) && !eval ) return()
		if( missing(eval) ){ # try evaluating
			return(utestFramework(x, eval=TRUE))
		}
	}
	
	if( !is.character(x) )
		stop("Invalid argument `x`: expecting a character string")
	path <- x
	framework <- NULL
	tf <- if( is.dir(path) ) list.files(path, "\\.[rR]$") else path
	for( f in names(.UFdata) ){
		if( any(grepl(.UFdata[[f]]$file_pattern, tf)) ){
			return(f)
		}
	}
	
	if( is.null(framework) )
		stop("Could not determine unit test framework used in directory: '", path, "'")
	framework
}

#' Embedded Unit Tests
#' 
#' The function \code{unit.test} provides a way to write unit tests embedded within
#' package source files.
#' These tests are stored and organised in the package namespace, and can be run using 
#' the unified interface provided by the function \code{link{utest}}.
#' Both Runit and testthat tests are supported -- and automatically detected.
#' 
#' 
#' @param x single character string used as test identifier/label
#' @param expr expression containing the actual test commands.
#' It is not evaluated, but only stored in the package namespace.
#' @param framework Unit test framework
#' @param envir the definition environment of object \code{x}.
#' 
#' @return a test function with no arguments that wrapping around \code{expr} 
#' 
#' @import digest
#' @export
#' 
unit.test <- function(x, expr, framework=NULL, envir=parent.frame()){
	
	sid <- as.character(deparse(substitute(x)))	
	hash <- suppressWarnings(digest(x))
	# get test environment
	eTest <- packageTestEnv()
	# wrap test into a function
	f <- function(){}
	environment(f) <- eTest
	body(f) <- substitute({expr})
	
	if( !grepl('"', sid) )
	{
		lmessage('Creating unit test for object: `', sid, '`')
		eval(substitute(attr(x, 'test') <- f, list(x=substitute(x), f=f)), envir)
	}else
		lmessage('Creating unit test: ', sid)
	
	# add the test to the package test environment
	eTest[[str_c(sid, ':', hash)]] <- list(test=f, name=sid, object=is.name(x))
	# return the test function
	f
}

#' Returns the package internal environment where unit tests are stored.
#' 
#' @param pkg package name.
#' If missing the caller's package is assumed. 
#' 
#' @export
packageTestEnv <- function(pkg){
	
	if( !missing(pkg) && !is.null(pkg) ){
		e <- packageEnv(pkg)
		return( e$.packageTest )
	}
	
	e <- packageEnv()
	# create test environment if necessary
	if( is.null(e$.packageTest) )
		e$.packageTest <- new.env(parent=e)
	e$.packageTest
}


list.tests <- function(x, pattern=NULL){
	
}

#unit.test(packageEnv, {print('test for packageEnv')})
#unit.test('lmlm', {print('test for something else')})

#utest <- function(x, ..., framework="RUnit", PACKAGE=NULL){
#		
#	if( missing(x) )
#		x <- packagePath('unitTests', PACKAGE=PACKAGE)
#	else if( class(x)[1] != 'character')
#		return( UseMethod('utest', x) )
#	
#	if( is.null(framework) ){
#		stop("Not implemented")
#	}else{
#		# change directory to run tests
#		owd <- setwd(x)
#		on.exit(setwd(owd))
#		# run tests under selected framework
#		class(x) <- framework
#		utest(x, ..., PACKAGE=PACKAGE)
#		# output test result
#	}
#}

#' Running Unit Tests
#' 
#' Run unit tests in a variety of settings.
#' This is still \strong{very} experimental.
#' 
#' @param x object to which a unit test is attached
#' @param ... extra arguments to allow extensions and are passed to 
#' the unit framework running funcitons. 
#'
#' @inline
#' @export
setGeneric('utest', function(x, ...) standardGeneric('utest'))
#' Run the unit test assoicated to a function. 
#' 
#' @param run a logical that indicates if the unit test should be run
setMethod('utest', 'function',
	function(x, run = TRUE){
		# get actual name of the function
		sid <- as.character(deparse(substitute(x, parent.frame())))
		# remove leading namespace specifications
		sid <- sub("^[^:]+:::?", "", sid)
		# get the package's  
		pkg <- attr(x, 'package')
		eTest <- packageTestEnv(pkg)
		if( is.null(eTest) ) return()
		tfun <- ls(eTest, pattern=str_c("^", sid, ":"))		
	}
)
#' Run a package test suite
#' 
#' @param filter pattern to match files that contain the definition of 
#' the unit tests functions to run.
#' @param fun patter to match the test functions to run.
#' @param testdir directory where to look for the test files
#' @param framework unit test framework
#' @param quiet a logical that indicates if the tests should be run silently
#' @param lib.loc path to a library where installed packages are searched for.
#' Used is of the form \code{x='package:*'}.
#'  
setMethod('utest', 'character', 
		function(x, filter="^runit.+\\.[rR]$", fun="^test\\.", ...
				, testdir='tests', framework=c('RUnit', 'testthat')
				, quiet = Sys.getenv("RCMDCHECK") != "FALSE"
				, lib.loc = NULL){
			
			cat("#########################\n")
			
			# define environment variable that identifies a test run (if not already defined) 
			if( is.na(utestCheckMode(raw = TRUE)) ){
				oldUM <- utestCheckMode(TRUE)
				on.exit( utestCheckMode(oldUM), add=TRUE)
			}
			
			#print(system('env'))
			# detect type of input string
			path <- 
					if( grepl("^package:", x) ){# installed package
						pkg <- sub("^package:", "", x)
						if( is.null(path <- path.package(pkg, quiet=TRUE)) ){
							library(pkg, character.only=TRUE, lib.loc=lib.loc)
							path <- path.package(pkg)
						}
						file.path(path, testdir)
					}else{
						# try to find a corresponding development package
						if( require.quiet(devtools) 
								&& is.package(pkg <- as.package(x, quiet=TRUE)) ){
							load_all(pkg, TRUE)
							file.path(pkg$path, 'inst', testdir)
						}else{ # assume x is a path  
							x
						}
					}
			
			# check that the path exists
			if( !file.exists(path) ){
				if( !hasArg(testdir) ){ # try another default
					opath <- path
					path <- file.path(dirname(path), 'unitTests')
					if( !file.exists(path) )
						stop("Could not find any default unit test directory ['", opath, "' nor '", path, "'].")
				} else {
					stop("Unit test directory '", path, "' does not exist")
				}
			}
			
			message("Running unit tests in: '", path, "'")
			# detect unit test framework: RUnit or testthat?
			framework <- 
					if( missing(framework) ) utestFramework(path)
					else match.arg(framework)
			message("Using unit test framework: ", framework)
			
			# load default patterns
			up <- .UFdata[[framework]]
			if( missing(filter) ) filter <- up$file_pattern
			if( missing(fun) ) fun <- up$fun_pattern
			
			# run tests
			path <- normalizePath(path)
			# remove/create output directory
			opath <- utestPath(package=x)
			if( file.exists( opath ) ){
				unlink(opath, recursive=TRUE) 
			}
			dir.create(opath, recursive=TRUE)
			# copy results in working directory on exit
			on.exit(
				{ if( file.exists(opath) )
					file.copy(opath, '.', recursive=TRUE)
				}
			, add=TRUE)
			#
			
			if( is.dir(path) ){ # all tests in a directory
				if( framework == 'RUnit' ){ # RUnit
					
					requireRUnit("Running RUnit test suites")
					s <- defineTestSuite(x, path
							, testFileRegexp=filter
							, testFuncRegexp=fun, ...)
					str(s)
					utest(s, quiet=quiet, outdir=opath)
					
				}else if( framework == 'testthat' ){ # testthat
					
					requirePackage('testthat', "Running testthat unit test suites")
					test_dir(path, filter=filter, ...)
					
				}
			}else{ # single test file
				if( framework == 'RUnit' ){ # RUnit
					
					requireRUnit("Running RUnit unit test file")
					runTestFile(path, testFuncRegexp=fun, ...)
					
				}else if( framework == 'testthat' ){ # testthat
					
					requirePackage('testthat', "Running testthat unit test file")
					test_file(path, ...)
					
				}
			}
			
		}
)

setOldClass('RUnitTestSuite')
#' Runs a RUnit test suite
#' 
#' @param outdir output directory
setMethod('utest', 'RUnitTestSuite',
	function(x, ..., quiet=FALSE, outdir=NULL){
		requireRUnit("Running RUnit test suites")
		
		pathReport <- file.path(outdir, str_c("utest.", sub("[:]", "_", x$name)))
		
		t <- system.time({
			if( quiet ){
				suppressWarnings(suppressMessages(out <- capture.output(
					tests <- runTestSuite(x, ...)
				)))
			}else 
				tests <- runTestSuite(x, ...)
		})
		
		## Report to stdout and text files
		cat("------------------- UNIT TEST SUMMARY ---------------------\n\n")
		summary_file <- paste(pathReport, ".Summary.txt", sep="")
		printTextProtocol(tests, showDetails=FALSE,	fileName=summary_file)
		# append timing
		st <- c("\nTotal execution time\n***************************"
				, paste(capture.output(print(t)), collapse="\n"))
		write(st, file=summary_file, append=TRUE)
		# detailed report
		details_file <- paste(pathReport, ".Details.txt", sep="")
		printTextProtocol(tests, showDetails=TRUE, fileName=details_file)
		write(st, file=details_file, append=TRUE)
		#
		
		## Report to HTML file
		printHTMLProtocol(tests, fileName=paste(pathReport, ".html", sep=""))
		
		## Return stop() to cause R CMD check stop in case of
		##  - failures i.e. FALSE to unit tests or
		##  - errors i.e. R errors
        tmp <- getErrors(tests)
        if(tmp$nFail > 0 | tmp$nErr > 0) {
            # filter failures and errors
            test_summary <- capture.output(.summaryRUnit(tests, c('error', 'failure'), print = TRUE))
            stop(paste0(test_summary, collapse = "\n"))
#			stop(paste("\n\nunit testing failed (#test failures: ", tmp$nFail,
#							", #R errors: ",  tmp$nErr, ")\n\n", sep=""))
		}
		
		tests
	}
)

.summaryRUnit <- function(testData, type = c('error', 'failure', 'deactivated'), print = FALSE){
    
    # taken from printTextProtocol
    res <- testData
    for (tsName in names(testData)) {
        
        if( print ){
            cat("Functions:", testData[[tsName]]$nTestFunc, "|"
                , "Errors:", testData[[tsName]]$nErr, "|"
                , "Failures:", testData[[tsName]]$nFail, "\n")
        }
        srcFileRes <- testData[[tsName]][["sourceFileResults"]]
        
        for (i in seq_along(srcFileRes)) {
            fname <- basename(names(srcFileRes)[i])
            testFuncNames <- names(srcFileRes[[i]])
            keep <- integer()
            for (j in seq_along(testFuncNames)) {
                  funcList <- srcFileRes[[i]][[testFuncNames[j]]]
                  if (funcList$kind %in% type){
                      keep <- c(keep, j)
                      if( print ){
                          if (funcList$kind == "error") {
                                cat("ERROR in ", fname, "::", testFuncNames[j], ": ", funcList$msg, sep = "")
                          }
                          else if (funcList$kind == "failure") {
                                cat("FAILURE in ", fname, "::", testFuncNames[j], ": ", funcList$msg, sep = "")
                          }
                          else if (funcList$kind == "deactivated") {
                                cat("DEACTIVATED ", fname, "::", testFuncNames[j], ": ", funcList$msg, sep = "")
                          }
                      }
                }
            }
            if( length(keep) ) res[[tsName]][["sourceFileResults"]][[i]] <- srcFileRes[[i]][keep]
            else res[[tsName]][["sourceFileResults"]] <- res[[tsName]][["sourceFileResults"]][-i]
        }
    }
    invisible(res)
}

#' Unit Tests Result Directory
#' 
#' Returns the path to the directory where the results of unit tests are stored.
#' This path is used by \code{\link{utest}} to save unit test results, which are 
#' read by \code{\link{makeUnitVignette}} to update the unit test vignette when 
#' runnning R CMD check.  
#' 
#' @param ... extra arguments passed to \code{\link{packagePath}}, e.g., \code{package}.
#' 
#' @export
utestPath <- function(...){
	packagePath('tests-results', ...)
}

#uTest <- function(file, fun, ...){
#	
#	library(RUnit)
#	tdir <- packagePath('unitTests')
#	ufiles <- list.files(tdir)
#	
#	get.tfile <- function(file){
#		i <- grep(paste(file,"(\\.[rR])?",sep=''), ufiles)
#		if( length(i) > 0L ) ufiles[i[1L]]
#		else NULL
#	}
#	
#	tfile <- file
#	if( is.null(tfile <- get.tfile(tfile)) ){
#		tfile <- paste('runit.', file, sep='')
#		if( is.null(tfile <- get.tfile(tfile)) ){
#			tfile <- paste('testthat.', file, sep='')
#			if( is.null(tfile <- get.tfile(tfile)) )
#				stop("Could not find test file '", file, "' (nor runit.% or testthat.% versions) in '", tdir, "'")
#		}
#	}
#	tfile <- file.path(tdir, tfile)
#	
#	if( !missing(fun) ){
#		e <- new.env()
#		source(tfile, local=e)
#		tfun <- fun
#		if( !exists(tfun, e, inherits=FALSE) ){
#			tfun <- paste('test.', fun, sep='')
#			if( !exists(tfun, e, , inherits=FALSE) )
#				stop("Could not find test function '", fun, "' (not test.% version) in '", tfile, "'")
#		}
#		tfun <- gsub(".", "\\.", tfun, fixed=TRUE)
#		runTestFile(tfile, testFuncRegexp=str_c("^", tfun, "$"), ...)
#	}else 
#		runTestFile(tfile, ...)
#}

# Namespace related functions
# 
# Author: Renaud Gaujoux
# Creation: 30 Apr 2012
###############################################################################


is_funcall <- function(fun){

	n <- sys.nframe()
	i <- 1
	dg <- digest(fun)
	while( i <= n ){
		f <- sys.function(i)
		ca <- sys.call(i)
#		cat(digest(f), dg, getPackageName(environment(f), FALSE), "\n")
		if( digest(f) == dg ) return(i)
		i <- i + 1
	}
	FALSE
}

is_pkgcall <- function(pkg){
	
	pkg %in% pkg_calls()
	
}

pkg_calls <- function(){
	n <- sys.nframe() - 1
	i <- 1
	res <- character()
	while( i <= n ){
		f <- sys.function(i)
		e <- environment(f)
		if( !is.null(e) ){
			pkg <- methods::getPackageName(e, create=FALSE)
			if( pkg != '' ) res <- c(res, pkg)
		}
		i <- i + 1
	}
	res
}

#' Namespace Development Functions
#' 
#' \code{getLoadingNamespace} returns information about the loading namespace.
#' It is a wrapper to \code{\link{loadingNamespaceInfo}}, that does not throw 
#' an error.
#' 
#' @param env logical that indicates that the namespace's environment (i.e. the 
#' namespace itself) should be returned.
#' @param info logical that indicates that the complete information list should 
#' be returned
#' 
#' @return the name of the loading namespace if \code{env} and \code{info} are 
#' \code{FALSE}, an environment if \code{env=TRUE}, a list with elements 
#' \code{pkgname} and \code{libname} if \code{info=TRUE}. 
#' 
#' @rdname namespace
#' @export
#' 
getLoadingNamespace <- function(env=FALSE, info=FALSE, nodev=FALSE){
	is.loading <- try(nsInfo <- loadingNamespaceInfo(), silent=TRUE)
	if( !is(is.loading, 'try-error') ){
		if( env ) asNamespace(as.name(nsInfo$pkgname))
		else if( info ){
			nsInfo$path <- file.path(nsInfo$libname, nsInfo$pkgname)
			nsInfo 
		}else nsInfo$pkgname
		
	}else if( !nodev ){ # devtools namespaces are allowed
		if( is_pkgcall('devtools') && (i <- is_funcall(devtools::load_all)) ){
			# find out the package that is currently being loaded by load_all
			e <- sys.frame(i)
			pkg <- e$pkg
			# extract namespace
			if( env ) asNamespace(pkg$package)
			else if( info ){
				list(
						pkgname = pkg$package
						, path = pkg$path
						, libname = dirname(pkg$path)
				)
			}else pkg$package
		}
	}
	else NULL
}

#' Tests if a namespace is being loaded.
#' 
#' @param ns the name of a namespace or a namespace whose loading state is tested.
#' If missing \code{isLoadingNamespace} test if any namespace is being loaded.
#' @param nodev logical that indicates if loading devtools namespace should 
#' be discarded.
#' 
#' @rdname namespace
#' @export
isLoadingNamespace <- function(ns, nodev=FALSE){
	
	if( missing(ns) ) !is.null(getLoadingNamespace(nodev=nodev))
	else{
		nspkg <- getLoadingNamespace(nodev=nodev, env=is.environment(ns))
		if( is.null(nspkg) ) FALSE
		else identical(nspkg, ns)
	}
}

#' \code{isNamespaceLoaded} tests if a given namespace is loaded, without loading it, 
#' contrary to \code{\link{isNamespace}}.
#' 
#' @rdname namespace
#' @export
isNamespaceLoaded <- function(ns){
	if( is.environment(ns) ){
		if( !isNamespace(ns) ) return(FALSE)
		else ns <- getPackageName(ns)
	}
	if( isString(ns) ) ns %in% loadedNamespaces()
	else stop("Invalid argument `ns`: only support strings and environments.")
}

#' \code{isDevNamespace} tests the -- current -- namespace is a devtools namespace.
#' 
#' @rdname namespace
#' @export
isDevNamespace <- function(ns){
	if( missing(ns) ){
		e <- parent.frame()
		ns <- methods::getPackageName(topenv(e))
	}
	
	# cannot be true if the namespace is not loaded
	if( !isNamespaceLoaded(ns) ) return( FALSE )
	# get the namespace environment
	if( isString(ns) ) ns <- asNamespace(ns)
	# check for the presence of a .__DEVTOOLS__ object
	exists('.__DEVTOOLS__', where=ns)
	
}

#' Dynamically adds exported objects into the loading namespace.   
#' 
#' @param x character vector containing the names of R objects to export in the 
#' loading namespace.
#' 
#' @rdname namespace
#' @export
addNamespaceExport <- function(x){
	ns <- pkgmaker::getLoadingNamespace(env=TRUE)
	if( !is.null(ns) ){
		namespaceExport(ns, x)
	}
}

#' \code{ns_get} gets an object from a given namespace.
#' @rdname namespace
#' @export
ns_get <- function(x, ns){
    if( !isNamespace(ns) ) ns <- asNamespace(ns)
    get(x, ns)
}

# Bibtex related functions
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################


#' Generate a Bibtex File from Package Citations
#' 
#' Generates a Bibtex file from a list of packages or all the installed packages.
#' It is useful for adding relevant citations in Sweave documents.
#' 
#' Multiple citations are handled by adding a numeric suffix to the Bibtex key 
#' (other than the first/main citation) as \code{"<pkgname>\%i"} (e.g. pkg, pkg2, pkg3).
#' 
#' This function has now been integrated by Romain François in the bibtex package.
#'
#' @encoding utf8
#'
#' @param entry a \code{\link{bibentry}} object or a character vector of package 
#' names. If \code{NULL}, then the list of all installed packages is used.
#' @param file output Bibtex file. It can be specified as a filename (as a single 
#' character string), NULL for \code{stdout}, or a \code{link{connection}} object. 
#' If \code{file} is a character string, an extension '.bib' is appended if not 
#' already present.
#' @param prefix character string to prepend to the generated packages' Bibtex key.
#' @param append a logical that indicates that the Bibtex entries should be added
#' to the file. If \code{FALSE} (default), the file is overwritten.  
#' @param verbose a logical to toggle verbosity. If \code{file=NULL}, verbosity 
#' is forced off. 
#'
#' @return the list of Bibtex objects -- invisibly.
#' @author
#' Renaud Gaujoux, based on the function \code{Rpackages.bib} 
#' from Achim Zeileis (see \emph{References}).
#' 
#' @references 
#' \emph{[R] Creating bibtex file of all installed packages?}
#' Achim Zeileis. R-help mailing list. 
#' \url{https://stat.ethz.ch/pipermail/r-help/2009-December/222201.html}
#' 
#' @seealso \code{link{connection}}, \code{link{bibentry}}
#'  
#' @export
#' @examples
#' 
#' write.pkgbib(c('bibtex', 'utils', 'tools'), file='references')
#' bibs <- bibtex::read.bib('references.bib')
#' write.pkgbib(bibs, 'references2.bib')
#' md5 <- tools::md5sum(c('references.bib', 'references2.bib'))
#' md5[1] == md5[2]
#' \dontshow{ stopifnot(md5[1] == md5[2]) }
#' 
#' # write to stdout()
#' write.pkgbib(c('bibtex', 'utils', 'tools'), file=NULL)
#' 
#' # clean up 
#' unlink(c('references.bib', 'references2.bib'))
#' 
write.pkgbib <- function(entry=NULL, file="Rpackages.bib", prefix='', append = FALSE, verbose = TRUE)
{
	# special handling of file=NULL: use stdout()
	if( is.null(file) ){
		file <- stdout()
		verbose <- FALSE
	}	
	## use all installed packages if nothing is specified
	if( is.null(entry) ){ 
		if( verbose ) message("Generating Bibtex entries for all installed packages ", appendLF=FALSE)
		entry <- unique(installed.packages()[,1])
		if( verbose ) message("[", length(entry), "]")
	}
	
	bibs <- 
			if( is(entry, 'bibentry') )	entry
			else if( is.character(entry) ){
				if( length(entry) == 0 ){
					if( verbose ) message("Empty package list: nothing to be done.")
					return(invisible())
				}
				
				pkgs <- entry
				bibs <- sapply(pkgs, function(x) try(citation(x)), simplify=FALSE)
				#bibs <- lapply(pkgs, function(x) try(toBibtex(citation(x))))
				n.installed <- length(bibs)
				
				## omit failed citation calls
				ok <- sapply(bibs, is, 'bibentry')
				pkgs <- pkgs[ok]
				bibs <- bibs[ok]
				n.converted <- sum(ok)
				
				## add bibtex keys to each entry
				pkgs <- lapply(seq_along(pkgs), function(i){
							if(length(bibs[[i]]) > 1)
								paste(prefix, pkgs[i], c('', 2:length(bibs[[i]])), sep = "") 
							else paste(prefix, pkgs[i], sep='')
				})
				pkgs <- do.call("c", pkgs)
				bibs <- do.call("c", bibs)		
				# formatting function for bibtex keys:
				# names with special characters must be enclosed in {}, others not.
				as.bibkey <- function(x){
					i <- grep("[.]", x)
					if( length(i) > 0 )
						x[i] <- paste("{", x[i], "}", sep='')
					x
				}		
				#bibs <- mapply(function(b,k){ if( is.null(b$key) ) b$key <- as.bibkey(k); b}, bibs, pkgs, SIMPLIFY=FALSE)
				bibs <- mapply(function(b,k){ if( is.null(b$key) ) b$key <- k; b}, bibs, pkgs, SIMPLIFY=FALSE)
				bibs <- do.call("c", bibs)
				
				if(verbose) message("Converted ", n.converted, " of ", n.installed, " package citations to BibTeX")					
				bibs
			} else
				stop("Invalid argument `entry`: expected a bibentry object or a character vector of package names.")
	
	if( length(bibs) == 0 ){
		if( verbose ) message("Empty bibentry list: nothing to be done.")
		return(invisible())
	}
	
	## write everything to the .bib file
	not_anonymous <- !identical(file,'')
	fh <- if( is.character(file) ){
				if( not_anonymous && !grepl("\\.bib$", file) ) # add .bib extension if necessary 
					file <- paste(file, '.bib', sep='')
				fh <- file(file, open = if(append && not_anonymous) "a+" else "w+" )
				if( not_anonymous )
					on.exit( if( isOpen(fh) ) close(fh) )
				fh
			} else if( is(file, 'connection') )
				file
			else
				stop("Invalid argument `file`: expected a filename, NULL, or a connection [", class(file), "]")
	
	if( !is(fh, 'connection') )
		stop("Invalid connection: ", fh)		
	file.desc <- summary(fh)['description']
	
	if( verbose ) message(if( append ) "Adding " else "Writing ", length(bibs) , " Bibtex entries ... ", appendLF=FALSE)
	writeLines(toBibtex(bibs), fh)
	if(verbose) message("OK\nResults written to file '", file.desc, "'")
	
	## return Bibtex items invisibly
	if( !not_anonymous ) attr(bibs, 'connection') <- fh 
	invisible(bibs)
}

#' @rdname pkgmaker-defunct
write.bib <- function(...){
	.Defunct('write.pkgbib', package = 'pkgmaker')
}

#' Package References
#' 
#' Create a citation string from package specific BibTex entries, suitable to 
#' be used in Rd files.
#' The entries are looked in a file named REFERNCES.bib in the package's root 
#' directory (i.e. inst/ in development mode).
#'  
#' @param key character vector of BibTex keys
#' @param short logical that indicates if the reference should be shorten as 
#' First Author et al. if it has more than one author.
#' @return a character string containing the text formated BibTex entries
#'  
#' @export
packageReference <- function(key, short=FALSE){
	bibs <- bibtex::read.bib(file=packageReferenceFile())
	k <- sapply(bibs, function(x) x$key)
    mk <- match(key, k)
	sel <- mk[!is.na(mk)]
	if( !length(sel) ) return("")
	if( !short ){
		paste(format(bibs[sel]), collapse="\n\n")
	}else{
		sapply(bibs[sel], function(x){
					if( length(x$author$family) <= 1L ) 
						paste(x$author$family, '(', x$year, ')', sep='')				
					else{
						paste(x$author$family[[1]], ' et al. (', x$year, ')', sep='')
					}
				})
	} 
}

#' Citing Package References
#' 
#' Create a citation command from package specific BibTex entries, suitable to 
#' be used in Rd files or Latex documents.
#' The entries are looked in a file named REFERNCES.bib in the package's root 
#' directory (i.e. inst/ in development mode).
#'  
#' @param key character vector of BibTex keys
#' @param ... extra arguments passed to \code{format.bibentry}.
#' @param REFERENCES package or bibentry specification
#' @return a character string containing the text formated BibTex entries
#'  
#' @export
#' 
citecmd <- local({
	
	.init <- list(REFERENCES=NULL, KEYS=NULL) 
	.cache <- .init
	function(key, ..., REFERENCES=NULL){
		 
		# detect package name if necessary
		if( is.null(REFERENCES) ){
			# reset if explicitly passed NULL
			if( hasArg(REFERENCES) ) .cache <<- .init
			
			if( is.null(.cache$REFERENCES) ){
				pkg <- Sys.getenv('R_PACKAGE_NAME')
				if( !nchar(pkg) )
					pkg <- Sys.getenv('R_INSTALL_PKG')
				if( !nchar(pkg) )
					pkg <- Sys.getenv('MAKE_R_PACKAGE')
				if( !nchar(pkg) )
					stop("Could not identify package")
				# load REFERENCES from detected package
				.cache$REFERENCES <<- bibtex::read.bib(package=pkg)
			}
			REFERENCES <- .cache$REFERENCES
		}
		
		# load relevant Bibtex file
		REFERENCES <- if( is(REFERENCES, 'bibentry') ) REFERENCES
				else if( is.character(REFERENCES) ){
					p <- str_match(REFERENCES, "^package:(.*)")[,2]
					if( is.na(p) ) bibtex::read.bib(file=REFERENCES)
					else bibtex::read.bib(package=p)
				}else
					stop("Invalid argument `REFERENCES`: expected bibentry object or character string [", class(REFERENCES), "]")
		
		# update the cache if no keys are provided
		if( missing(key) ){
			.cache$REFERENCES <<- REFERENCES
			if( hasArg(REFERENCES) ) return(invisible(.cache$KEYS))
			else return(.cache$KEYS)
		}
		
		# check key type
		if( !is.character(key) )
			stop("Invalid argument `key`: must be a character vector.")
		
		# extract the Bibtex keys
		refkey <- sapply(REFERENCES, function(x) x$key)
		pkgs <- str_match(key, "^package:(.*)")[,2]
		nokey <- !key %in% refkey
		i_pkgs <- which(nokey && !is.na(pkgs))
		if( length(i_pkgs) > 0L ){
			# only include \cite{key} if running Sweave
			.cache$KEYS <<- unique(c(.cache$KEYS, key[i_pkgs]))
			key[i_pkgs] <- pkgs[i_pkgs] 
		}
		paste("\\cite{", key, "}", sep='')
#		if( inSweave() ) paste("\\cite{", k, "}", sep='')
#		else paste(format(REFERENCES[k %in% key], ...), collapse="\n\n")
	}
})

citecmd_pkg <- function(key, ...){
	citecmd(str_c('package:', key), ...)
}

#' Bibtex Utilities
#' 
#' \code{packageReferenceFile} returns the path to a package REFERENCES.bib file.
#' 
#' @param PACKAGE package name
#' 
#' @rdname bibtex
packageReferenceFile <- function(PACKAGE=NULL) packagePath('REFERENCES.bib', package=PACKAGE)

# Package extra action registry
# 
# Author: renaud
###############################################################################

#' @include devutils.R 
NULL

.getExtraEnv <- function(package){
	if( missing(package) || is.null(package) ) where <- topns(FALSE)
	else if( isString(package) ) {
		package <- sub("^package:", "", package)
		if( package == 'R_GlobalEnv') where <- .GlobalEnv
		else where <- asNamespace(package)
	}
	else stop("Invalid argument `package`: must be missing or a package name.")
	where
}

# extra handler registry
extra_handlers <- setPackageRegistry('extra_handler', 'function' 
									, description = 'Handler functions for package-specific extra tasks'
									, entrydesc = 'extra handler')
							
# extra action registry
extra_actions <- registry()
extra_actions$set_field("key", type="character", is_key = TRUE, index_FUN = match_exact)
extra_actions$set_field("package", type="character", is_key = TRUE, index_FUN = match_exact)
extra_actions$set_field("handler", type='character', is_mandatory=TRUE, is_key=TRUE)
extra_actions$set_field("args", type='list', default=list())
extra_actions <- setPackageRegistry('extra_action', extra_actions
									, description = 'Handler functions for package-specific extra actions'
									, entrydesc = 'extra action')

#' Install/Run Extra Things After Standard Package Installation
#' 
#' @description
#' These functions define a framework to register actions for which default sets of arguments
#' can be defined when (lazy-)loading a package, and run later on, e.g., after the package 
#' is installed using dedicated commands.
#' 
#' \code{setPackageExtraHandler} defines main action handler functions, for which 
#' actions are defined as a set of arguments and registered using \code{setPackageExtra}. 
#'  
#' @param handler name of a handler, e.g, \code{'install'}.
#' It must be unique across all handlers registered by any other packages.  
#' @param fun handler function that will be called with the arguments registered
#' with \code{packageExtra(name, ...)}
#' @param package package name where to store/look for the internal registries.
#' End users should not need to use this argument.
#' 
#' @return the runner function associated with the newly registered handler,
#' as built by \code{packageExtraRunner}.  
#'  
#' @rdname packageExtra
#' @export
setPackageExtraHandler <- function(handler, fun, ...){
	
	# add entry to the registry
	setPackageRegistryEntry('extra_handler', handler, fun, ...)
	# build associated runner
	runner <- packageExtraRunner(handler)
}

#' \code{packageExtraHandler} retrieves a given handler from the registry. 
#' 
#' @param ... extra arguments passed to internal function calls.
#' In \code{packageExtraHandler}, these are passed to \code{\link{pkgreg_fetch}}.
#' 
#' In \code{setPackageExtra}, these define default arguments for the handler function. 
#' These are overwritten by arguments in the call to runner function if any.
#'  
#' @rdname packageExtra
#' @export 
packageExtraHandler <- function(handler=NULL, ...){
	# load handler from registry
	pkgreg_fetch('extra_handler', key=handler, ...)
}
#' \code{setPackageExtra} registers extra actions for a given handler.
#' 
#' For example, calling \code{setPackageExtra('install', pkgs='non_CRAN_pkg', repos='http://non-standard-repo')}
#' in a source file of package 'myPkg' registers the call 
#' \code{install.packages('non_CRAN_pkg', repos='http://non-standard-repo', ...)}
#' in a registry internal to the package. 
#' All calls to \code{setPackageExtra('install', ...)} can then be run by the user, as
#' a post installation step via \code{install.extrapackages('myPkg', ..)}.
#' 
#' @param extra name of the extra action.
#' @param .wrap logical that indicates if a function that runs the extra action should
#' be returned or only the default arguments
#' 
#' @rdname packageExtra
#' @export
setPackageExtra <- function(handler, extra, ...){
	
	# check that a handler is defined in the registry
	fhandler <- packageExtraHandler(handler, exact=TRUE, error=FALSE)
	if( is.null(fhandler) ){
		handlers <- packageExtraHandler()
		stop("Could not register action '", extra, "': handler '", handler, "' is not defined"
				, if( length(handlers) ){
					str_c(".\n  Available handlers are: ", str_out(handlers, Inf))
				} else " [handler registry is empty]." )
	}
	args <- list(...)
	pkg <- packageName(topenv(parent.frame()), .Global=TRUE)
	setPackageRegistryEntry('extra_action', key=extra, handler=handler, args=args
							, package = pkg 
							, msg=str_c(" for handler '", handler, "'"))
}


.wrapExtra <- function(fhandler, args=list()){
	
	# define wrapper function
	f <- function(...){
		cl <- match.call()
		cl[[1L]] <- as.name('fhandler')
		# add default arguments
		lapply(names(args), function(a){
			if( !a %in% names(cl) )
				cl[[a]] <<- as.name(substitute(a, list(a=a)))
		})
		eval(cl)
	}
	# set registered arguments as default arguments
	formals(f) <- c(args, formals(f))
	f
}
#' \code{packageExtra} retrieve a given extra action, either as its registry entry,
#' or as a function that would perform the given action.
#' @rdname packageExtra
#' @export
packageExtra <- function(handler=NULL, extra=NULL, package=NULL, .wrap=FALSE){
	
	# load extra registry
	extras <- pkgreg_fetch('extra_action', key=extra, handler=handler, package=package
						, exact=TRUE, all=!.wrap)
	
	# return whole registry if no other argument is provided
	if( missing(handler) || is.null(extra) || !.wrap ) return( extras )
		
	args <- extras$args
	fhandler <- packageExtraHandler(handler, package='pkgmaker')
	if( is.null(fhandler) ){
		handlers <- packageExtraHandler(package='pkgmaker')
		stop("Could not find action handler '", handler, "' in pkgmaker global handler registry.\n"
				, "  Available handlers are: ", str_out(handlers, Inf))
	}
	# define wrapper function
	.wrapExtra(fhandler, args)		
}
#' \code{packageExtraRunner} defines a function to run all or some of the actions registered 
#' for a given handler in a given package.
#' For example, the function \code{install.extrapackages} is the runner defined for the extra handler \code{'install'} 
#' via \code{packageExtraRunner('install')}.
#' 
#' @param .verbose logical that indicates if verbose messages about the extra actions being
#' run should be displayed.
#' 
#' @rdname packageExtra
#' @export
packageExtraRunner <- function(handler){

	.handler <- handler
	function(package, extra=NULL, handler=NULL, ..., .verbose=getOption('verbose')){
		
		if( missing(handler) ) handler <- .handler
		.local <- function(p, ...){
			# load list of extras
			extras <- packageExtra(handler=handler, extra=extra, package=p)
			# execute extras
			sapply(extras, 
				function(def, ...){
					e <- def$key
					h <- def$handler
					f <- packageExtra(handler=h, extra=e, package=p, .wrap=TRUE)
					if( .verbose ){
						message("# Running extra action '", h, ':', e, "' ...")
						message("# Action: ", str_fun(f))
						on.exit( message("# ERROR [", e, "]\n") )
					}
					res <- f(...)
					if( .verbose ){
						on.exit()
						message("# OK [", e, "]\n")
					}
					res
				}
			, ...)
		}
		invisible(sapply(package, .local, ...))
	}
}

#' \code{install.extrapackages} runs all extra actions registered for a given package.
#' 
#' @rdname packageExtra
#' @export
install.extras <- packageExtraRunner(NULL)
#' \code{install.extrapackages} install sets of packages that can enhance a 
#' package, but may not be available from CRAN.
#' 
#' \code{install.extrapackages} is defined as the extra handler for 
#' the extra action handler \code{'install.packages'}.
#' All arguments in \code{...} are passed to \code{\link{install.packages}}.
#' By default, packages that are already installed are not re-installed.
#' An extra argument \code{force} allows to force their installation.
#' The packages are loaded if their installation is successful. 
#' 
#' @rdname packageExtra
#' @export
install.extrapackages <- setPackageExtraHandler('install.packages', 
	function(pkgs, ..., force=FALSE){
		res <- sapply(pkgs, function(pkg, ...){
			if( force || !require.quiet(pkg, character.only=TRUE) ){
				install.packages(pkg, ...)
				require(pkg, character.only=TRUE)
			}else message("Loaded extra package: ", pkg)
		}, ...)
	}
)

# Color utilities
# 
# Author: Renaud Gaujoux
# Created: 30 Nov 2012
###############################################################################

#' Colour utilities
#' 
#' \code{alphacol} adds an alpha value to a colour specification and convert to 
#' a hexadecimal colour string.
#'  
#' @inheritParams grDevices::col2rgb
#' 
#' @importFrom grDevices col2rgb
#' @export
#' @examples
#' 
#' # Alphas
#' alphacol('red') # do nothing
#' alphacol('red', 10)
#' alphacol('#aabbcc', 5)
#' alphacol(4, 5)
#' 
alphacol <- function(col, alpha=FALSE){
	apply(as.character(as.hexmode(col2rgb(col, alpha))), 2, function(x) paste("#", paste(x, collapse=''), sep=''))
}

# Unit test for utils
# 
# Author: Renaud Gaujoux
###############################################################################

library(stringr)

test.errorCheck <- function(){
	
	f <- function(err=''){
		success <- exitCheck()
		on.exit( if(success()) cat("no error\n") else cat("with error\n") )
		
		if( err=='error' ) stop('There is an error')
		if( err=='try' ) try(stop('Catched error'), silent=TRUE)
		if( err=='tryCatch' ) tryCatch(stop('Catched error'), error = function(e){})
		
		success(1+1)
	}
	
	# without error
	out <- capture.output(res <- f())
	checkIdentical(res, 2, 'If no error: return result')
	checkIdentical(out, 'no error', 'If no error: correctly detected no error')
	
	# with error
	out <- capture.output(res <- try(f('error'), silent=TRUE))
	checkTrue( is(res, 'try-error'), 'If error: effectively throws an error')
	checkIdentical(out, 'with error', 'If error: correctly detected the error')
	
	# with try-caught error 
	out <- capture.output(res <- f('try'))
	checkIdentical( res, 2, 'If try-catched error: return result')
	checkIdentical(out, 'no error', 'If try-catched error: correctly detected no error')
	
	# with tryCatch-caught error 
	out <- capture.output(res <- f('tryCatch'))
	checkIdentical( res, 2, 'If tryCatch-catched error: return result')
	checkIdentical(out, 'no error', 'If tryCatch-catched error: correctly detected no error')
}


test.ExposeAttribute <- function(){
	
	
	x <- 1:10
	checkIdentical(ExposeAttribute(x), {attr_mode(x) <- 'rw'; x}
		, "Using ExposeAttribute() and attr_mode <- 'rw' is equivalent")
	x <- 1:10
	checkIdentical(capture.output(print(ExposeAttribute(x, a='r', b='rw'))), capture.output(print(x))
		, "Printing object with exposed attribute is identical to plain print")

	checkSet <- function(x, name, msg, ...){
		attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		eval(parse(text=str_c('y$', name, ' <- 1')))
		attr_mode(y) <- NULL 
		checkIdentical(x, y, msg)
	}
	checkSetException <- function(x, name, msg, ...){
		y <- ExposeAttribute(x, ...)
		checkException(eval(parse(text=str_c('y$', name, ' <- 1'))), msg)
	}
	
	checkSet(x, 'a', "Set works if default")
	checkSet(x, 'a', .MODE='rw', "Set works if all args are 'rw'")
	checkSet(x, 'a', a='rw', "Set works if specified arg is 'rw'")
	checkSet(x, 'a', a='w', "Set works if specified arg is 'w'")
	checkSet(x, 'a', a='rw', b='r', "Set works if specified arg is 'rw', even if others are not")
	checkSet(x, 'ab', ab='rw', `a.*`='r', "Set works if specified arg is 'rw', even if another match is not")
	checkSetException(x, 'a', .MODE='r', "Set throws an error if access right is 'r'")
	checkSetException(x, 'a', a='r', "Set throws an error if specific access right is 'r'")
	checkSetException(x, 'a', a='', "Set throws an error if specific access right is ''")
	
	checkGet <- function(x, name, msg, ...){
		attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		a <- eval(parse(text=str_c('y$', name)))
		checkIdentical(attr(x, name), a, msg)
	}
	checkGetException <- function(x, name, msg, ...){
		y <- ExposeAttribute(x, ...)
		checkException(eval(parse(text=str_c('y$', name))), msg)
	}
	
	checkGet(x, 'a', "Get works if default")
	checkGet(x, 'a', .MODE='rw', "Get works if all args are 'rw'")
	checkGet(x, 'a', a='rw', "Get works if specified arg is 'rw'")
	checkGet(x, 'a', a='r', "Get works if specified arg is 'r'")
	checkGet(x, 'a', a='rw', b='w', "Get works if specified arg is 'rw', even if others are not")
	checkGet(x, 'ab', ab='r', `a.*`='w', "Get works if specified arg is 'rw', even if another match is not")
	checkGetException(x, 'a', .MODE='w', "Get throws an error if access right is 'r'")
	checkGetException(x, 'a', a='w', "Get throws an error if specific access right is 'r'")
	checkGetException(x, 'a', a='', "Get throws an error if specific access right is ''")
	
	
}


test.Sys.getenv_value <- function(){
    
    on.exit( Sys.unsetenv('TOTO') )
    
    # undefined returns FALSE
    checkIdentical(Sys.getenv_value('TOTO'), FALSE, 'undefined returns FALSE')
    # raw undefined returns NA
    checkIdentical(Sys.getenv_value('TOTO', raw = TRUE), as.character(NA), 'raw undefined returns NA')
    
    Sys.setenv(TOTO='bla')
    checkIdentical(Sys.getenv_value('TOTO'), 'bla', 'defined returns value')
    
    # anything false-like returns FALSE
    Sys.setenv(TOTO='false');
    checkIdentical(Sys.getenv_value('TOTO'), FALSE, '"false" returns FALSE')
    Sys.setenv(TOTO='FALSE');
    checkIdentical(Sys.getenv_value('TOTO'), FALSE, '"FALSE" returns FALSE')
    Sys.setenv(TOTO='0');
    checkIdentical(Sys.getenv_value('TOTO'), FALSE, '"0" returns FALSE')
    
}


test.str_bs <- function(){
    
    checkIdentical(str_bs("abcd"), "abcd", "No backspace returns string unchanged")
    checkIdentical(str_bs("abcd\b"), "abc", "One backspace at the end is OK")
    checkIdentical(str_bs("\babcd"), "abcd", "One leading backspace is OK")
    checkIdentical(str_bs("abcd\b\b"), "ab", "Two backspaces at the end is OK")
    checkIdentical(str_bs("abcd\b\b\b"), "a", "Three backspaces at the end is OK")
    checkIdentical(str_bs("abcd\b\b\b\b"), "", "As many backspaces as characters at the end is OK")
    checkIdentical(str_bs("abcd\b\be"), "abe", "Backspace in the middle is OK")
}

# Unit test for utils
# 
# Author: Renaud Gaujoux
###############################################################################

context('Utilities')

library(stringr)

test_that('errorCheck', {
	
	f <- function(err=''){
		success <- exitCheck()
		on.exit( if(success()) cat("no error\n") else cat("with error\n") )
		
		if( err=='error' ) stop('There is an error')
		if( err=='try' ) try(stop('Catched error'), silent=TRUE)
		if( err=='tryCatch' ) tryCatch(stop('Catched error'), error = function(e){})
		
		success(1+1)
	}
	
	# without error
	out <- capture.output(res <- f())
	expect_identical(res, 2, 'If no error: return result')
	expect_identical(out, 'no error', 'If no error: correctly detected no error')
	
	# with error
	out <- capture.output(res <- try(f('error'), silent=TRUE))
	expect_true( is(res, 'try-error'), 'If error: effectively throws an error')
	expect_identical(out, 'with error', 'If error: correctly detected the error')
	
	# with try-caught error 
	out <- capture.output(res <- f('try'))
	expect_identical( res, 2, 'If try-catched error: return result')
	expect_identical(out, 'no error', 'If try-catched error: correctly detected no error')
	
	# with tryCatch-caught error 
	out <- capture.output(res <- f('tryCatch'))
	expect_identical( res, 2, 'If tryCatch-catched error: return result')
	expect_identical(out, 'no error', 'If tryCatch-catched error: correctly detected no error')
})


test_that('ExposeAttribute', {
	
	
	x <- 1:10
	expect_identical(ExposeAttribute(x), {attr_mode(x) <- 'rw'; x}
		, "Using ExposeAttribute() and attr_mode <- 'rw' is equivalent")
	x <- 1:10
	expect_identical(capture.output(print(ExposeAttribute(x, a='r', b='rw'))), capture.output(print(x))
		, "Printing object with exposed attribute is identical to plain print")

	checkSet <- function(x, name, msg, ...){
		attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		eval(parse(text=str_c('y$', name, ' <- 1')))
		attr_mode(y) <- NULL 
		expect_identical(x, y, msg)
	}
	checkSetException <- function(x, name, msg, regexp, ...){
		y <- ExposeAttribute(x, ...)
		expect_error(eval(parse(text=str_c('y$', name, ' <- 1'))), regexp, info = msg)
	}
	
	checkSet(x, 'a', "Set works if default")
	checkSet(x, 'a', .MODE='rw', "Set works if all args are 'rw'")
	checkSet(x, 'a', a='rw', "Set works if specified arg is 'rw'")
	checkSet(x, 'a', a='w', "Set works if specified arg is 'w'")
	checkSet(x, 'a', a='rw', b='r', "Set works if specified arg is 'rw', even if others are not")
	checkSet(x, 'ab', ab='rw', `a.*`='r', "Set works if specified arg is 'rw', even if another match is not")
	checkSetException(x, 'a', .MODE='r', "Set throws an error if access right is 'r'", "Could not write attribute 'a'.*permission denied.*mode='r'")
	checkSetException(x, 'a', a='r', "Set throws an error if specific access right is 'r'", "Could not write attribute 'a'.*permission denied.*mode='r'")
	checkSetException(x, 'a', a='', "Set throws an error if specific access right is ''", "Could not write attribute 'a'.*permission denied.*mode=''")
	
	checkGet <- function(x, name, msg, ...){
		attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		a <- eval(parse(text=str_c('y$', name)))
		expect_identical(attr(x, name), a, msg)
	}
	checkGetException <- function(x, name, msg, regexp, ...){
    attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		expect_error(eval(parse(text=str_c('y$', name))), regexp, info = msg)
	}
	
	checkGet(x, 'a', "Get works if default")
	checkGet(x, 'a', .MODE='rw', "Get works if all args are 'rw'")
	checkGet(x, 'a', a='rw', "Get works if specified arg is 'rw'")
	checkGet(x, 'a', a='r', "Get works if specified arg is 'r'")
	checkGet(x, 'a', a='rw', b='w', "Get works if specified arg is 'rw', even if others are not")
	checkGet(x, 'ab', ab='r', `a.*`='w', "Get works if specified arg is 'rw', even if another match is not")
	checkGetException(x, 'a', .MODE='w', "Get throws an error if access right is only 'w'", "Could not access exposed attribute 'a'.*permission denied.*mode='w'")
	checkGetException(x, 'a', a='w', "Get throws an error if specific access right is only 'w'", "Could not access exposed attribute 'a'.*permission denied.*mode='w'")
	checkGetException(x, 'a', a='', "Get throws an error if specific access right is ''", "Could not access exposed attribute 'a'.*permission denied.*mode=''")
	
	
})


test_that('Sys.getenv_value', {
    
    on.exit( Sys.unsetenv('TOTO') )
    
    # undefined returns FALSE
    expect_identical(Sys.getenv_value('TOTO'), FALSE, 'undefined returns FALSE')
    # raw undefined returns NA
    expect_identical(Sys.getenv_value('TOTO', raw = TRUE), as.character(NA), 'raw undefined returns NA')
    
    Sys.setenv(TOTO='bla')
    expect_identical(Sys.getenv_value('TOTO'), 'bla', 'defined returns value')
    
    # anything false-like returns FALSE
    Sys.setenv(TOTO='false');
    expect_identical(Sys.getenv_value('TOTO'), FALSE, '"false" returns FALSE')
    Sys.setenv(TOTO='FALSE');
    expect_identical(Sys.getenv_value('TOTO'), FALSE, '"FALSE" returns FALSE')
    Sys.setenv(TOTO='0');
    expect_identical(Sys.getenv_value('TOTO'), FALSE, '"0" returns FALSE')
    
})


test_that('.str_bs', {
    
    expect_identical(str_bs("abcd"), "abcd", "No backspace returns string unchanged")
    expect_identical(str_bs("abcd\b"), "abc", "One backspace at the end is OK")
    expect_identical(str_bs("\babcd"), "abcd", "One leading backspace is OK")
    expect_identical(str_bs("abcd\b\b"), "ab", "Two backspaces at the end is OK")
    expect_identical(str_bs("abcd\b\b\b"), "a", "Three backspaces at the end is OK")
    expect_identical(str_bs("abcd\b\b\b\b"), "", "As many backspaces as characters at the end is OK")
    expect_identical(str_bs("abcd\b\be"), "abe", "Backspace in the middle is OK")
})

test_that("ldata", {
      
  # parameter check error
  expect_error(ldata(NA), "Invalid argument 'list':.* NULL .* character vector")
  expect_error(ldata(1L), "Invalid argument 'list':.* NULL .* character vector")
  for(v in list(1L, "", character()))
    expect_error(ldata(package = v), "Invalid argument 'package':.* NULL .* non-empty string")
  for(v in list(1L, c(TRUE, FALSE), logical()))
    expect_error(ldata(error = v), "Invalid argument 'error':.* single logical")
  for(v in list(1L, c(TRUE, FALSE), logical()))
    expect_error(ldata(simplify = v), "Invalid argument 'simplify':.* single logical")
  
  # load single data
  a <- ldata("iris", package = "datasets")
  expect_true(is.data.frame(a))
  e <- environment()
  expect_true(exists("iris", envir = e, inherit = FALSE), "Dataset is loaded in caller environment")
  expect_identical(e[["iris"]], a)
  expect_true(!exists("iris", envir = .GlobalEnv, inherit = FALSE), "Dataset is not loaded in .GlobalEnv")
  expect_error(ldata("blabla", package = "datasets"), "object 'blabla' not found")
  expect_identical(ldata("iris", package = "datasets", error = FALSE), a, "Data found returns correct data if error = FALSE")
  expect_identical(ldata("blabla", package = "datasets", error = FALSE), NULL, "Data not found returns NULL if error = FALSE")
  expect_identical(ldata("blabla", package = "datasets", error = FALSE, simplify = FALSE)
                      , list(blabla = NULL), "Data not found returns named list with NULL element if error = FALSE and not simplifying")
  expect_identical(ldata(c("blabla", "iris"), package = "datasets", error = FALSE)
                    , list(blabla = NULL, iris = a), "Some data not found with error = FALSE returns partially filled list")
  expect_identical(ldata(c("iris", "blabla"), package = "datasets", error = FALSE)
                    , list(iris = a, blabla = NULL), "Some data not found with error = FALSE returns partially filled list (order is honored)")
  
  # check that argument stringsAsFactors is honoured
  expect_true(is.factor(ldata("iris", package = "datasets", stringsAsFactors = TRUE)[["Species"]]) && 
                is.factor(e[["iris"]][["Species"]]))
  expect_true(is.character(ldata("iris", package = "datasets", stringsAsFactors = FALSE)[["Species"]]) && 
                is.character(e[["iris"]][["Species"]]))
})

# Unit test for utils
# 
# Author: Renaud Gaujoux
###############################################################################

context('Utilities')

library(stringr)

test_that('errorCheck', {
	
	f <- function(err=''){
		success <- exitCheck()
		on.exit( if(success()) cat("no error\n") else cat("with error\n") )
		
		if( err=='error' ) stop('There is an error')
		if( err=='try' ) try(stop('Catched error'), silent=TRUE)
		if( err=='tryCatch' ) tryCatch(stop('Catched error'), error = function(e){})
		
		success(1+1)
	}
	
	# without error
	out <- capture.output(res <- f())
	expect_identical(res, 2, 'If no error: return result')
	expect_identical(out, 'no error', 'If no error: correctly detected no error')
	
	# with error
	out <- capture.output(res <- try(f('error'), silent=TRUE))
	expect_true( is(res, 'try-error'), 'If error: effectively throws an error')
	expect_identical(out, 'with error', 'If error: correctly detected the error')
	
	# with try-caught error 
	out <- capture.output(res <- f('try'))
	expect_identical( res, 2, 'If try-catched error: return result')
	expect_identical(out, 'no error', 'If try-catched error: correctly detected no error')
	
	# with tryCatch-caught error 
	out <- capture.output(res <- f('tryCatch'))
	expect_identical( res, 2, 'If tryCatch-catched error: return result')
	expect_identical(out, 'no error', 'If tryCatch-catched error: correctly detected no error')
})


test_that('ExposeAttribute', {
	
	
	x <- 1:10
	expect_identical(ExposeAttribute(x), {attr_mode(x) <- 'rw'; x}
		, "Using ExposeAttribute() and attr_mode <- 'rw' is equivalent")
	x <- 1:10
	expect_identical(capture.output(print(ExposeAttribute(x, a='r', b='rw'))), capture.output(print(x))
		, "Printing object with exposed attribute is identical to plain print")

	checkSet <- function(x, name, msg, ...){
		attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		eval(parse(text=str_c('y$', name, ' <- 1')))
		attr_mode(y) <- NULL 
		expect_identical(x, y, msg)
	}
	checkSetException <- function(x, name, msg, regexp, ...){
		y <- ExposeAttribute(x, ...)
		expect_error(eval(parse(text=str_c('y$', name, ' <- 1'))), regexp, info = msg)
	}
	
	checkSet(x, 'a', "Set works if default")
	checkSet(x, 'a', .MODE='rw', "Set works if all args are 'rw'")
	checkSet(x, 'a', a='rw', "Set works if specified arg is 'rw'")
	checkSet(x, 'a', a='w', "Set works if specified arg is 'w'")
	checkSet(x, 'a', a='rw', b='r', "Set works if specified arg is 'rw', even if others are not")
	checkSet(x, 'ab', ab='rw', `a.*`='r', "Set works if specified arg is 'rw', even if another match is not")
	checkSetException(x, 'a', .MODE='r', "Set throws an error if access right is 'r'", "Could not write attribute 'a'.*permission denied.*mode='r'")
	checkSetException(x, 'a', a='r', "Set throws an error if specific access right is 'r'", "Could not write attribute 'a'.*permission denied.*mode='r'")
	checkSetException(x, 'a', a='', "Set throws an error if specific access right is ''", "Could not write attribute 'a'.*permission denied.*mode=''")
	
	checkGet <- function(x, name, msg, ...){
		attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		a <- eval(parse(text=str_c('y$', name)))
		expect_identical(attr(x, name), a, msg)
	}
	checkGetException <- function(x, name, msg, regexp, ...){
    attr(x, name) <- 1
		y <- ExposeAttribute(x, ...)
		expect_error(eval(parse(text=str_c('y$', name))), regexp, info = msg)
	}
	
	checkGet(x, 'a', "Get works if default")
	checkGet(x, 'a', .MODE='rw', "Get works if all args are 'rw'")
	checkGet(x, 'a', a='rw', "Get works if specified arg is 'rw'")
	checkGet(x, 'a', a='r', "Get works if specified arg is 'r'")
	checkGet(x, 'a', a='rw', b='w', "Get works if specified arg is 'rw', even if others are not")
	checkGet(x, 'ab', ab='r', `a.*`='w', "Get works if specified arg is 'rw', even if another match is not")
	checkGetException(x, 'a', .MODE='w', "Get throws an error if access right is only 'w'", "Could not access exposed attribute 'a'.*permission denied.*mode='w'")
	checkGetException(x, 'a', a='w', "Get throws an error if specific access right is only 'w'", "Could not access exposed attribute 'a'.*permission denied.*mode='w'")
	checkGetException(x, 'a', a='', "Get throws an error if specific access right is ''", "Could not access exposed attribute 'a'.*permission denied.*mode=''")
	
	
})


test_that('Sys.getenv_value', {
    
    on.exit( Sys.unsetenv('TOTO') )
    
    # undefined returns FALSE
    expect_identical(Sys.getenv_value('TOTO'), FALSE, 'undefined returns FALSE')
    # raw undefined returns NA
    expect_identical(Sys.getenv_value('TOTO', raw = TRUE), as.character(NA), 'raw undefined returns NA')
    
    Sys.setenv(TOTO='bla')
    expect_identical(Sys.getenv_value('TOTO'), 'bla', 'defined returns value')
    
    # anything false-like returns FALSE
    Sys.setenv(TOTO='false');
    expect_identical(Sys.getenv_value('TOTO'), FALSE, '"false" returns FALSE')
    Sys.setenv(TOTO='FALSE');
    expect_identical(Sys.getenv_value('TOTO'), FALSE, '"FALSE" returns FALSE')
    Sys.setenv(TOTO='0');
    expect_identical(Sys.getenv_value('TOTO'), FALSE, '"0" returns FALSE')
    
})


test_that('.str_bs', {
    
    expect_identical(str_bs("abcd"), "abcd", "No backspace returns string unchanged")
    expect_identical(str_bs("abcd\b"), "abc", "One backspace at the end is OK")
    expect_identical(str_bs("\babcd"), "abcd", "One leading backspace is OK")
    expect_identical(str_bs("abcd\b\b"), "ab", "Two backspaces at the end is OK")
    expect_identical(str_bs("abcd\b\b\b"), "a", "Three backspaces at the end is OK")
    expect_identical(str_bs("abcd\b\b\b\b"), "", "As many backspaces as characters at the end is OK")
    expect_identical(str_bs("abcd\b\be"), "abe", "Backspace in the middle is OK")
})

test_that("ldata", {
      
  # parameter check error
  expect_error(ldata(NA), "Invalid argument 'list':.* NULL .* character vector")
  expect_error(ldata(1L), "Invalid argument 'list':.* NULL .* character vector")
  for(v in list(1L, "", character()))
    expect_error(ldata(package = v), "Invalid argument 'package':.* NULL .* non-empty string")
  for(v in list(1L, c(TRUE, FALSE), logical()))
    expect_error(ldata(error = v), "Invalid argument 'error':.* single logical")
  for(v in list(1L, c(TRUE, FALSE), logical()))
    expect_error(ldata(simplify = v), "Invalid argument 'simplify':.* single logical")
  
  # load single data
  a <- ldata("iris", package = "datasets")
  expect_true(is.data.frame(a))
  e <- environment()
  expect_true(exists("iris", envir = e, inherit = FALSE), "Dataset is loaded in caller environment")
  expect_identical(e[["iris"]], a)
  expect_true(!exists("iris", envir = .GlobalEnv, inherit = FALSE), "Dataset is not loaded in .GlobalEnv")
  expect_error(ldata("blabla", package = "datasets"), "object 'blabla' not found")
  expect_identical(ldata("iris", package = "datasets", error = FALSE), a, "Data found returns correct data if error = FALSE")
  expect_identical(ldata("blabla", package = "datasets", error = FALSE), NULL, "Data not found returns NULL if error = FALSE")
  expect_identical(ldata("blabla", package = "datasets", error = FALSE, simplify = FALSE)
                      , list(blabla = NULL), "Data not found returns named list with NULL element if error = FALSE and not simplifying")
  expect_identical(ldata(c("blabla", "iris"), package = "datasets", error = FALSE)
                    , list(blabla = NULL, iris = a), "Some data not found with error = FALSE returns partially filled list")
  expect_identical(ldata(c("iris", "blabla"), package = "datasets", error = FALSE)
                    , list(iris = a, blabla = NULL), "Some data not found with error = FALSE returns partially filled list (order is honored)")
  
})

# Unit tests for options
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

test.option_link <- function(){
	
	opt <- setupPackageOptions(a=1,b=2,c=option_symlink('a'),d=4, RESET=TRUE)
	
	.test <- function(msg){
		checkIdentical(names(opt$options('a')), 'a', paste(msg, " - options: name of target is ok"))
		checkIdentical(names(opt$options('c')), 'c', paste(msg, " - options: name of link is ok"))
		checkIdentical(opt$options('c'), setNames(opt$options('a'), 'c'), paste(msg, " - options: link ok"))
		checkIdentical(opt$getOption('a'), opt$getOption('c'), paste(msg, " - getOption: link ok"))
	}
	
	.test('Default')
	opt$options(a=100)
	.test('After setting target')
	opt$options(c=50)
	.test('After setting link')

}

test.resetOptions <- function(){
	opt <- setupPackageOptions(a=1,b=2,c=option_symlink('a'),d=4, RESET=TRUE)
	
	.checkOptions <- function(y, msg) checkIdentical(opt$options(), y, msg)
	ref <- opt$options()
	# simple set
	opt$options(a=10)
	x <- ref
	x$a <- 10
	.checkOptions(x, 'change default option works')
	opt$resetOptions()
	.checkOptions(ref, 'default options are reset after resetOptions()')
	
	# new option
	opt$options(aaa=10)
	x <- ref
	x$aaa <- 10
	.checkOptions(x, 'add new option works')
	opt$resetOptions()
	.checkOptions(c(ref, aaa=10), 'new option kept after resetOptions()')
	opt$resetOptions(ALL=TRUE)
	.checkOptions(ref, 'all options are reset to default and new options are removed when ALL=TRUE')
	opt$options(a=20, b='c', d='toto', aaa=25)
	x <- ref
	x$a <- 20
	x$b <- 'c'
	x$d <- 'toto'
	x$aaa <- 25
	.checkOptions(x, '2 default options + 1 new are set correctly')
	opt$resetOptions('a', 'b')
	x$a <- 1
	x$b <- 2
	.checkOptions(x, 'only the specified options are reset to default when passed in argument')
	opt$resetOptions('bbb')
	.checkOptions(x, 'if some options are not present it does not affect the result')
	opt$resetOptions('aaa')
	x$aaa <- NULL
	.checkOptions(x, 'new options get removed')
	
}

# Registry utility functions
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

#' @include devutils.R
#' @include is.R
NULL

#' Fix Registry Access Functions 
#' 
#' Fixes some of the member functions for registry objects, as defined in the 
#' \pkg{registry} package.
#' The main fixed issue is due to the fact that key filtering does not return 
#' the correct entry, when an entry key is a prefix of another entry key,
#' even when passed the exact key. 
#' 
#' @param regobj registry object
#' 
#' @family registry-internals
#' @keywords internal
fix_registry <- function(regobj){
	
	# get private environment
	.REGENV <- environment(environment(regobj$n_of_entries)$f)
	
	# do not fix twice
	if( isFixed <- exists('.isFixed', .REGENV, inherits=FALSE) ){
		return(regobj)
	}
	
#	message("REGENV:\n", capture.output(print(ls(.REGENV, all=TRUE))))
#	message("env(delete_entry)\n", capture.output(print(ls(environment(environment(regobj$delete_entry)$f), all=TRUE))))
	# dummy variables for R CMD check
	PERMISSIONS <- 
	.get_entry_indices <-  
	.get_entry_names <- 
	SEALED_ENTRIES <-
	DATA <- 
	.delete_entry <- 
	NULL

	# .get_entries
	.get_entries <- get('.get_entries', .REGENV, inherits=FALSE)
	.get_fields <- get('.get_fields', .REGENV, inherits=FALSE)
	hook <- function(...){
		# remove fields that are not keys
		fld <- .get_fields()
		if( length(fld) ){
			keyfield <- names(fld[sapply(fld, function(x) isTRUE(x$is_key) )])
			index_fields <- list(...)
			if( !is.null(names(index_fields)) ){
				i <- match(keyfield, names(index_fields))
				index_fields <- index_fields[i[!is.na(i)]]
			}
		}
		do.call(.get_entries, index_fields)
	}
	assign('.get_entries', hook, .REGENV)
	#
	
	# fix bug in delete_entry
	hook <- function(...){
		key <- list(...)
		isString <- function(x) is.character(x) && length(x) == 1L
		if( length(key) == 1L && isString(key[[1L]]) ){
			
			errmsg <- paste0("Could not delete entry '", key[[1L]],"': ")
			if (!PERMISSIONS["delete_entries"]) 
				stop(errmsg, "deletion of entries denied due to restricted permissions.", call. = FALSE)
			entry_index <- .get_entry_indices(key)
			
			# fix: check for exact match (on full key)
			if( key[[1L]] %in% .get_entry_names() ){
				entry_index <- match(key[[1L]], .get_entry_names())
			}
			#end_fix
			if( !length(entry_index) ){
#				if( !quiet ) warning(errmsg, "not in registry.", immediate.=TRUE, call. = FALSE)
				return()
			}
			if (length(entry_index) != 1)
				stop(errmsg, "multiple matches.", call. = FALSE)
			if (entry_index %in% SEALED_ENTRIES) 
				stop(errmsg, "deletion of entry not allowed in sealed registry.", call. = FALSE)
			DATA[entry_index] <<- NULL
		} else .delete_entry(...)
	}
	environment(hook) <- .REGENV
	regobj$delete_entry <- hook
	#

	# fix bug in get_entry
	hook <- function(...){
		key <- list(...)
		isString <- function(x) is.character(x) && length(x) == 1L
		if( length(key) == 1L && isString(key[[1L]]) ){
			res <- .get_entries(...)
			if( key[[1L]] %in% names(res) )
				res[[key[[1L]]]]
			else res[[1L]] 
		}else .get_entries(...)[[1]]
	}
	environment(hook) <- .REGENV
	regobj$get_entry <- hook
	#

	# flag the registry as fixed
	assign('.isFixed', TRUE, .REGENV)
	# return fixed registry
	regobj
}


testreg <- function(){
	regobj <- registry()
	regobj$set_field("X", type = TRUE)
	regobj$set_field("Y", type = "character")
	regobj$set_field("index", type = "character", is_key = TRUE,
			index_FUN = match_partial_ignorecase)
	# fix
	regobj <- fix_registry(regobj)
	
	regobj$set_entry(X = TRUE, Y = "bla", index = "test")
	regobj$set_entry(X = TRUE, Y = "bloblo", index = "test2")
	regobj$set_entry(X = FALSE, Y = "foo", index = c("test", "bar"))
	
	regobj
}

#' Creates or Retrieves a Package Meta Registry
#' 
#' This function is used in \code{\link{setPackageRegistry}} and 
#' \code{\link{packageRegistry}} to create or query meta registries.
#' 
#' @keywords internal
.packageMetaRegistry <- function(package, quiet=FALSE, create=FALSE){
	
#	library(registry)
	metaregname <- '.packageRegistry'
	# get package environment
	e <- packageEnv(package)	
	# get namespace name
	nm <- packageName(e)
	
	# create registry environment if necessary
	if( !exists(metaregname, e, inherits=FALSE) ){
		if( !create ){
			if( quiet ) return(NULL)
			# throw error
			stop("Meta registry in package `", nm, "` does not exist.")
		}
#		if( !isLoadingNamespace(e) ){
#			stop("Can only create a package meta-registry when loading it"
#				," [loading namespace: ", if(!is.null(ns <- getLoadingNamespace()) ) ns else 'none', "].")
#		}
		message("Creating meta registry in package '", nm, "' ... ", appendLF=FALSE)
		# create registry object with special classes for the registry and entries
		meta <- registry(c(paste('package', nm, 'subregistry', sep='_'), 'package_subregistry')
				, c(paste('package', nm, 'metaregistry', sep='_'), 'package_metaregistry'))
		## set fields
		# access key
		meta$set_field("key", type="character", is_key = TRUE, index_FUN = match_exact)
		# sub-registry object
		meta$set_field("regobj", type="registry", is_mandatory = TRUE)
		# human readable description
		meta$set_field("description", type="character", is_mandatory = TRUE)
		# short object description
		meta$set_field("entrydesc", type="character", is_mandatory = TRUE)
		# parent package = owner of the primary registry
		meta$set_field("parent", type="character", default = '')
		# owner package (its value is forced)
		meta$set_field("package", type="character", default = nm, alternatives=nm)
		#
		
		# fix registry
		meta <- fix_registry(meta)
		# add package attribute
		attr(meta, 'package') <- nm
		# store within the calling package environment
		assign(metaregname,  meta, envir = e)
		message('OK')
	}
	# get package meta registry
	get(metaregname, envir=e, inherits = FALSE)
}

#' Package Registry
#' 
#' \code{packageRegistry} provides ways to create query package specific 
#' registries.
#' 
#' Package registries are organised in a meta-registry (a registry of registries) within a package's namespace. 
#' Each registry can be used to store sets of built-in or user-defined objects 
#' in an organised way, e.g. algorithms or datasets.
#' 
#' A package meta-registry is a \code{\link[registry:regobj]{registry}} object,
#' whose entries are \code{\link[registry:regobj]{registry}} objects themselves.
#' A sub-registry entry is defined by the following fields:
#' \describe{
#' \item{key}{The sub-registry's accession key/identifier (a character string).}
#' \item{regobj}{The sub-registry itself (a \code{registry} object)}
#' \item{description}{Human readable description of the purpose of the registry (a character string)}
#' \item{description}{Short human readable description of the type of entries (a character string)}
#' \item{package}{owner package, which is forced to be the package in which the meta registry
#' is defined.}
#' \item{parent}{The name of the package that holds the parent registry, which we 
#' call the primary package.
#' This field is non empty for cross-package registries, i.e. registries that 
#' derive from primary package's own registry.
#' Their entries are defined when (lazy-)loading the dependent package's namespace.}
#' }
#' 
#' Note that this function cannot be called from the global environment, but from 
#' a package namespace, e.g., when a package is lazy-loaded on installation or loaded
#' via the function \code{\link[devtools]{load_all}} from the \pkg{devtools} package.  
#'   
#' @param regname Name of a sub-registry, used as its identifier.
#' @param quiet a logical that indicates that one should return the (meta-)registry if it exists, 
#' or \code{NULL} otherwise, without throwing any error.
#' @param entry logical that indicates if the corresponding meta registry entry should 
#' be directly returned, without any other processing.
#' @param update logical that indicates if the package registry should be updated, by adding/removing 
#' entries from other loaded/unloaded packages. 
#' @param package package where to store or look for the registry.
#' @return a \code{\link[registry:regobj]{registry}} object or \code{NULL} (see argument 
#' \code{quiet}).
#' 
#' @import registry
#' @rdname registry
#' @export
packageRegistry <- function(regname=NULL, quiet=FALSE, entry=FALSE, update=!entry, package=topenv(parent.frame())){
	
#	library(registry)
	metaregname <- '.packageRegistry'
	name <- regname

	# get package environment
	e <- packageEnv(package)	
	# get namespace name
	nm <- packageName(e)
	
	# get package meta-registry
	pkgreg <- .packageMetaRegistry(package, quiet)
	
	# return meta registry if no name is specified
	if( is.null(name) )	return(pkgreg)
	else{
		if( is.null(pkgreg) ){
			if( quiet ) return(NULL)
			# throw error
			stop("Could not find registry '", name, "' in package `", nm, "`: meta registry does not exist.")	
		} 
		# retrieve sub-registry entry
		nm <- packageSlot(pkgreg) 
		reg <- regfetch(pkgreg, key=name, exact=TRUE, error=FALSE)
		
		if( is.null(reg) ){# not found
			if( quiet ) return(NULL)
			# throw error
			stop("Could not find registry `", name, "` in package `", nm, "`.")
		}else{
			# synchronise and substitute by primary sub-registry (if necessary)
			if( update ) reg <- .update_pkgreg(reg)
			# return plain registry entry if requested
			if( entry )	return(reg)
			# return sub-registry object
			reg$regobj
		}
	}
}

.update_pkgreg <- local({
	.cacheNS <- list()
	.cacheMD5 <- list()
	function(regentry){
		
		verbose <- getOption('verbose')
		
		# directly return entry if:
		# - one is loading the namespace of the package (primary or not)
		if( isLoadingNamespace(regentry$package) ) return(regentry) 		
		# - not a primary registry
		if( nchar(regentry$parent) > 0L ) return(regentry)
		
		primary <- regentry$package
		primaryreg <- regentry$regobj
		key <- regentry$key
		fullkey <- str_c(primary, '::', key)
		# sync if loaded packages changed
		hash <- digest(c(.cacheNS[[fullkey]], ns <- loadedNamespaces()))
#		print(ns)
#		print(.cacheNS)
		if( !identical(hash, .cacheMD5[[fullkey]]) ){
			if( verbose ) message('Updating registry ', fullkey, " ... ", appendLF=FALSE)
			# remove entries from unloaded packages
			if( length(.cacheNS[[fullkey]]) && length(notloaded <- setdiff(.cacheNS[[fullkey]], ns)) ){
				ndel <- sapply(notloaded, function(p){
					if( verbose > 1L ) message("\n Removing entries from package ", p, " ... ", appendLF=FALSE)
					e <- primaryreg$get_entry_names()
					n <- sapply(e, function(x){
						rec <- primaryreg$get_entry(x)
						if( rec$REGISTERINGpackage == p ){
							primaryreg$delete_entry(x)
							1L
						}else 0L
					})
					if( verbose > 1L ) message('OK [', sum(n), ']')
					sum(n)
				})
			}
			
			# list packages that have local versions of this registry
			reglist <- packageRegistries(fullkey)
#			print(reglist)
			pkgs <- names(reglist)
			# add entries from new packages into the primary registry
			if( length(miss <- setdiff(pkgs, .cacheNS[[fullkey]])) ){
				nadd <- sapply(miss, function(p){
					if( verbose > 1L ) message("\n Adding entries from package ", p, " ... ", appendLF=FALSE)
					reg <- packageRegistry(fullkey, package=p)
					e <- reg$get_entries()
					n <- sapply(e, function(x){
						# add entry if it does not exists already
						oldentry <- regfetch(primaryreg, KEYS=x, exact=TRUE, error=FALSE)
						if( is.null(oldentry) ){
							do.call(primaryreg$set_entry, x)
							1L
						}else 0L				
					})
					if( verbose > 1L ) message('OK [', sum(n), ']')
					sum(n)
				})
			}
			# store contributing packages and MD5 hash
			.cacheNS[[fullkey]] <<- pkgs
			.cacheMD5[[fullkey]] <<- digest(c(.cacheNS[[fullkey]], ns)) 
			if( verbose ) message('OK')
		}
		
		regentry
	}
})


#' \code{packageRegistries} lists registries from loaded packages.
#' 
#' @param primary logical that indicates if only primary registries 
#' should be listed.
#'  
#' @rdname registry
#' @export
packageRegistries <- function(regname=NULL, package=NULL, primary=FALSE){
	lns <- loadedNamespaces()
	if( !is.null(package) ) lns <- lns[lns %in% package]
	
	# early exit if no namespace
	if( !length(lns) ) return( character() )
	
	res <- lapply(lns, function(ns){
		reg <- packageRegistry(package=ns, quiet=TRUE)
		if( is.null(reg) ) return( character() )
		regnames <- reg$get_entry_names()
		res <- setNames(regnames, rep(ns, length(regnames)))
		if( primary ){
			pr <- sapply(res, function(n) reg$get_entry(n)$parent)
			res <- res[ nchar(pr) == 0L ]
		}
		res
	})

	res <- unlist(res)			
	if( !is.null(regname) ){
		res <- res[res == regname]
		if( primary && length(res) > 1L ){
			warning("Package registry - Found multiple primary registries '", regname, "' in packages "
					, str_out(res, Inf), " [using first one only]")
			res <- res[1L]
		}
	}
	res
}

#' \code{hasPackageRegistry} tells if a given package has a meta-registry or 
#' a given registry. 
#' 
#' @rdname registry
#' @export
hasPackageRegistry <- function(regname=NULL, package){
	isNamespaceLoaded(package) && !is.null( packageRegistry(regname, package=package, quiet=TRUE, entry=TRUE) )
}

#' @S3method format package_subregistry
format.package_subregistry <- function(x, ...){
	c(Key = x$key
	, Description = x$description
	, Entries = x$regobj$n_of_entries()
	, Parent = x$parent)	
}

#' @S3method format package_metaregistry
format.package_metaregistry <- function(x, ...){
	rec <- x$get_entries()
	data.frame(t(sapply(rec, base::format, ...))[, -1L, drop=FALSE])	 
}

#' @S3method print package_metaregistry
print.package_metaregistry <- function(x, ...){	
	NextMethod('print') #registry:::print.registry(x)
	print(format(x, ...))
}

#' @S3method xtable package_metaregistry 
#' @importFrom xtable xtable
xtable.package_metaregistry <- function(x, ...){
	d <- format(x)
	xtable::xtable(d, ...)
}

#' \code{setPackageRegistry} creates a package-specific registry within a package.
#'  
#' Each package sub-registry has its own set of fields.
#' Sub-registries defined by passing a character string in argument \code{regobj} of 
#' \code{setPackageRegistry} have the following fields: \code{'key'} and \code{'object'}
#' 
#' @param regobj a \code{\link[registry:regobj]{registry}} object or a single character 
#' string that indicates the class of the objects that are stored in the 
#' sub-registry.
#' See details for the list of the sub-registry's fields in this latter case.
#' @param description short description line about the registry.
#' It is recommended to provide such description as it makes clearer the purpose of the 
#' registry.
#' This description is shown when the registry object is printed/formated/listed.
#' @param entrydesc human readable description that is used in log messages 
#' when registering/removing entries.
#' @param ... named values used to set extra information about the new registry, that 
#' are stored in the corresponding fields of the meta-registry.
#' Currently not used, as no extra field other than \code{'description'} is defined.
#' @param overwrite a logical that indicate if an existing registry with the same 
#' should be overwritten if it exists.
#' 
#' @inheritParams packageRegistry
#' @rdname registry
#' @export
setPackageRegistry <- function(regname, regobj
								, description='', entrydesc=NA
								, ...
								, package=topenv(parent.frame())
								, overwrite=FALSE){
	
#	library(registry)
	
	# force overwrite in dev mode
	if( missing(overwrite) && isDevNamespace(package) ){
		overwrite <- TRUE
	}
	# check if sub-registry already exists
	oldreg <- packageRegistry(regname, quiet=TRUE, package=package)
	if( !is.null(oldreg) && !overwrite ){
		return( oldreg )
	}
	
	# get meta-registry (force creation)
	regenv <- .packageMetaRegistry(package, create=TRUE)
	nm <- packageSlot(regenv)
	ns_str <- str_c("package '", nm, "'")
	
	if( !is.null(oldreg) ){
		if( !overwrite ){
			if( isLoadingNamespace() ){ # exit if loading a namespace
				message("NOTE: Did not create registry '", regname,"' in ", ns_str, ": registry already exists.")
				return(oldreg)
			}
			stop("Could not create registry '", regname,"' in ", ns_str, ": registry already exists")
		}else{
			message("Remove registry '", regname,"' from ", ns_str)
			regenv$delete_entry(regname)
		}
	}
	message("Creating registry '", regname,"' in ", ns_str, ' ... ', appendLF=FALSE)
	
	.add_regclass <- function(x, newcl, before){
		cl <- class(x)
		ir <- which(cl == before)
		class(x) <- c(if( ir > 1 ) cl[1:(ir-1)] 
				, newcl, cl[ir:length(cl)])
		x
	}
	
	pkgregclass <- c(paste(regname, 'package_registry', sep='_'), 'package_registry')
	if( is.character(regobj) ){# regobj specifies the S4 class of the registry entries
		objtype <- regobj[1]
		regobj <- registry(entry_class = paste(regname, 'entry', sep='_')
						, registry_class = c(pkgregclass, 'object_subregistry'))
		# access key
		regobj$set_field("key", type="character", is_key = TRUE
				, index_FUN = match_partial_ignorecase)
		# object
		regobj$set_field("object", type=objtype, is_mandatory=TRUE, validity_FUN = validObject)
	}else if( is(regobj, 'registry') ){
		if( !is(regobj, 'package_registry') ){
			regobj <- .add_regclass(regobj, pkgregclass, 'registry')
		}
	}else{
		message('ERROR')
		stop("Invalid argument 'regobj': must be a class name or a registry object.")
	}
	# add field for REGISTERING package
	if( !"REGISTERINGpackage" %in% regobj$get_field_names() )
		regobj$set_field("REGISTERINGpackage", type='character', is_mandatory=TRUE, index_FUN=match_exact)
	# fix registry object
	regobj <- fix_registry(regobj)
	# add package
	attr(regobj, 'package') <- nm
	
	# create new meta entry
	regenv$set_entry(key=regname, regobj=regobj
					, description=description, entrydesc=entrydesc
					, ...)
	message('OK')
	# return newly created registry
	regenv$get_entry(regname)$regobj
}

regkeys <- function(regobj, ...){
	
	# get keys
	fld <- regobj$get_fields()
	keyfield <- names(fld[sapply(fld, function(x) isTRUE(x$is_key) )])
	if( nargs() == 1L ) return(keyfield)
	
	index_fields <- list(...)
	if( is.null(names(index_fields)) && length(index_fields)==1L )
		index_fields <- index_fields[[1L]]
	index_fields <- index_fields[!sapply(index_fields, is.null)]
	if( !length(index_fields) ) return(list())
	
	# remove fields that are not keys
	i <- match(keyfield, names(index_fields))
	index_fields[i[!is.na(i)]]
}

#' Finds an entry in a registry.
#' 
#' This function provides extra control on how entries are queried 
#' from a \code{\link[registry:regobj]{registry}} object.
#' 
#' @param regobj a registry object
#' @param ... key value(s) to look up.
#' If multiple indexes are used, then the primary key should come first.
#' @param all logical to indicate if hidden keys (starting with a '.') should be 
#' returned and output in message.
#' @param error a logical that indicates if an error should be thrown if the key has no match 
#' or multiple matches
#' @param exact a logical that indicates if matching should be exact or partial.
#' Note that if exact matches exist then they are returned, independently of the
#' value of \code{exact}.
#' @param KEYS alternative way of passing the key value(s).
#' If not missing, then arguments in \code{...} are discarded.
#' @param verbose a logical that indicates if verbosity should be toggle on
#' @param entry a logical that indicates if the 
#' @param msg a header to use in case of error.
#' 
#' @export
regfetch <- function(regobj, ..., all=FALSE, error=TRUE, exact=FALSE
						, KEYS = NULL
						, verbose=FALSE, entry=FALSE, msg=NULL){
	
	# load the registry package
#	library(registry)
	# list -- all -- keys if no key is specified
	keylist <- allkeys <- regobj$get_entry_names()
	if( !all ) keylist <- grep("^[^.]", keylist, value=TRUE)
	
	index_fields <- if( !is.null(KEYS) ){
		if( !is.list(KEYS) ) stop("Invalid argument <KEYS>: must be a list of field values.")
		KEYS
	}else list(...)
	# extract primary key
	key <- if( length(index_fields) ){
		# remove fields that are not keys if named list
		if( !is.null(names(index_fields)) )
			index_fields <- regkeys(regobj, index_fields)
		if( length(index_fields) ){
			paste(unlist(index_fields), collapse='_')
			str_out(index_fields, Inf, use.names=TRUE)
		}
	}
	if( is.null(key) ){
		return(keylist)
	}
	
	# set verbosity level
	if( !missing(verbose) ){
		ol <- lverbose(verbose)
		on.exit( lverbose(ol) )
	}
	
	if( !is.null(msg) ) msg <- str_c(msg, ' - ')
	
	if( regobj$n_of_entries() == 0L ){
		if( error )	stop(msg, "Registry is empty: no matching entry for key ", dQuote(key), ".")
		else return(NULL)
	}
	
	# get entry
	d <- do.call(regobj$get_entries, index_fields)
	# no entry found
	if( is.null(d) ){
		if( error ){
			stop(msg, "No matching entry for key ", dQuote(key), " in the registry."
							, "\n  Use one of: ", str_wrap(str_out(sort(allkeys), Inf), exdent=2), '.')
		}else return(NULL)
	}
	
	# look for exact matches
	if( is.list(index_fields) ){
		ex <- sapply(d, function(x) all(mapply(identical, index_fields, x[names(index_fields)])))
	}else{
		ex <- names(d) == index_fields
	} 
		
	# limit to exact mathes
	if( length(i <- which(ex)) ){
		d <- d[i]
	}else if( exact ){
		if( error ){
			stop(msg, "No exact match for key '", key, "' in the registry."
					, "\n  Use one of: ", str_wrap(str_out(allkeys, Inf), exdent=2), '.')
		}else return(NULL) 
	}
	
	if( all ) return(d)
	
	# multiple match
#	str(d)
	if( length(d) > 1L ){
		if( error ){
			stop(msg, "Multiple entries found for key ", dQuote(key), ": ", str_out(sort(names(d)), Inf), '.')
		}else return(NA)
	}
	
	# check single match
	if( length(d) != 1L )
		stop("Unexpected error: more than one entry was selected.")
	
	# return single match
	d <- d[[1L]]
	
	# return registry object if the entry is an automatic sub-registry
	if( !entry && is(regobj, 'object_subregistry') ) d$object
	else d
}
#' \code{pkgreg_fetch} fetches entries in a package registry, as set up by 
#' \code{\link{setPackageRegistry}}.
#' 
#' \code{pkgreg_fetch} loads the requested package registry and uses \code{regfetch} 
#' to retrieve data from it.
#' 
#' @inheritParams setPackageRegistry
#'  
#' @rdname regfetch
#' @export
pkgreg_fetch <- function(regname, ..., msg=NULL, where=topenv(parent.frame())){
	# get package registry
	regentry <- packageRegistry(regname, package=where, entry=TRUE, update=TRUE)
	# define addon error message
	if( missing(msg) && !is_NA(regentry$entrydesc) ) msg <- regentry$entrydesc
	# fetch from registry
	regfetch(regentry$regobj, ..., msg=msg)
}

#' \code{pkgreg_remove} removes an entry from a package registry.
#' 
#' @param quiet a logical that indicates if the operation should be performed quietly, 
#' without throwing errors or warnings.
#' 
#' @rdname regfetch
#' @export
pkgreg_remove <- function(regname, ..., msg=NULL, where=topenv(parent.frame()), quiet=FALSE){
	# get package registry
	regentry <- packageRegistry(regname, package=where, entry=TRUE, update=TRUE)
	# define addon error message
	if( missing(msg) && !is_NA(regentry$entrydesc) ) msg <- regentry$entrydesc
	# fetch from registry
	entry <- regfetch(regentry$regobj, ..., exact=TRUE, error=FALSE, all=TRUE, msg=msg)
	
	res <- if( !is.null(entry) ){
		# get the method registry and the method's fullname
		name <- names(entry)
		
		if( !quiet ){
			msg <- paste0("Removing ", msg, " '", name, "' from registry '", regname, "'")
			message(msg, ' ... ', appendLF=FALSE)
		}
		# delete from registry
		regentry$regobj$delete_entry(name)
		if( !quiet ) message('OK')
		TRUE
	}else{
		if( !quiet ){
			name <- str_out(list(...), Inf, use.names=TRUE)
			warning("Could not remove ", msg, " '", name, "': no matching registry entry.", call.=FALSE)
		}
		FALSE
	}

	if( quiet ) invisible(res)
	else res
}

extract_pkg <- function(x){
	sub("^(([^:]+)::)?(.*)", "\\2", x)
}

#' Automatic S4 Class for Registry Entries
#' 
#' @param registry a registry object
#' @param Class name of the class to generate
#' @param ... extra arguments passed to \code{\link{setClass}}.
#' 
setClassRegistry <- function(registry, Class, ...){
	
#	setClass(Class, representation, prototype, contains=character(),
#			validity, access, where, version, sealed, package,
#			S3methods = FALSE)
	
	f <- registry$get_fields()
	slots <- sapply(f, '[[', 'type', simplify=FALSE)
	
	args <- list(Class, representation=do.call('representation', slots))
#	if( !hasArg(validity) ){
#		.validity <-
#		sapply(f, function(x){
#			if(x$is_mandatory)
#				function(object){
#					if()
#				}
#		})
#		args$validity <- function(object){
#			
#		}
#	}
	do.call('setClass', c(args, ...))
}

#' \code{setPackageRegistryEntry} adds an entry in a package registry.
#' 
#' @param key entry identifier.
#' @param where package name or namespace that owns the registry. 
#' @param verbose a logical that indicates if verbosity should be toggle on.
#' @param msg addon message to print at the end of the output log line, 
#' when \code{verbose=TRUE}.
#' 
#' @rdname registry
#' @export
setPackageRegistryEntry <- function(regname, key, ..., overwrite=FALSE, verbose=FALSE
									, where=topenv(parent.frame()), msg=NULL){
	
	if( isLoadingNamespace() ){
		verbose <- TRUE
		if( missing(overwrite) ) overwrite <- TRUE
	}
	registry <- regname
	package <- where
	
	# check if the name provided is not empty
	if( nchar(key) == 0 ) stop('Invalid argument <key>: cannot be an empty string.')
	
	# build full key, that includes the name of the top calling namespace
	fullkey <- key
	top_ns <- topns(strict=FALSE)
	#
	
	# retrieve package registry (it must exist or this will throw an error)
	package <- packageEnv(package)
	subregentry <- packageRegistry(registry, package=package, entry=TRUE, update=TRUE)
	# get regobj (do that to ensure it is updated with entries from other packages)
	regobj <- subregentry$regobj
	
	# setup complete list of fields
	fields <- list(...)
	objdesc <- if( !is_NA(subregentry$entrydesc) ) subregentry$entrydesc else paste(registry, 'object')
	objdesc <- paste(objdesc, " '", key, "'", sep='')
	if( length(fields)==1L ){
		objdesc <- paste(objdesc, ' [', class(fields[[1L]]), ']', sep='')
		if( is.null(names(fields)) && is(regobj, 'object_subregistry') )
			names(fields) <- 'object'
	}
	fields$key <- key
	regpkg <- packageName(top_ns, .Global=TRUE)
	fields$REGISTERINGpackage <- regpkg
#	str(fields)
	#
	
	# check if the object is already registered
	oldentry <- regfetch(regobj, KEYS=fields, exact=TRUE, error=FALSE, all=TRUE)
	# error if already exists and not overwriting		
	if( !is.null(oldentry) && !overwrite ){ 
		if( verbose ) message("ERROR")
		stop("Cannot register ", objdesc, ": key already exists.")	
	}
	
	# add entry
	if( verbose ){
		action <- if( is.null(oldentry) ) 'Registering' else 'Replacing'
		message(action, " ", objdesc, msg, " ... ", appendLF=FALSE)
	}
	# delete old entry
	if( !is.null(oldentry) ){
		regobj$delete_entry(names(oldentry)[1L])
	}
	# do add entry
	do.call(regobj$set_entry, fields)
	if( verbose ) message("OK")
	
	# if the registration happens during loading another package: 
	# create local registry and add entry to it.
	# It will be merged to the main registry on the next call to 
	# packageRegistry after the package is loaded.
	lns <- getLoadingNamespace(env=TRUE)
	if( !is.null(lns <- getLoadingNamespace(env=TRUE)) && !identical(lns, package) ){
		# clone registry
		if( nchar(subregentry$parent) ){
			warning("Deriving package registry '", registry, "' in package ", lns
					, " from ", subregentry$parent, " instead of ", subregentry$package, immediate.=TRUE)
			parent <- subregentry$parent
		}else parent <- subregentry$package
		fullregistry <- str_c(parent, '::', registry)
		
		if( is.null(locregobj <- packageRegistry(fullregistry, package=lns, quiet=TRUE)) ){
			# clone registry
			locregobj <- clone_regobj(regobj, empty=TRUE)
			# attach to loading namespace
			locregobj <- setPackageRegistry(fullregistry, locregobj
											, description = subregentry$description
											, entrydesc = subregentry$entrydesc
											, parent = parent
											, package = lns)
		}
		
		action <- 'Adding'
		if( !is.null(locentry <- regfetch(locregobj, KEYS=fields, exact=TRUE, error=FALSE, all=TRUE)) ){
			action <- 'Overwriting'
			locregobj$delete_entry(names(locentry)[1L])
		}
		# add entry into local registry
		if( verbose ) message(action, " entry '", key, "' in registry '", fullregistry, "' ... ", appendLF=FALSE)
		do.call(locregobj$set_entry, fields)
		if( verbose ) message("OK")
	}
	#
	
	# return registered object
	regfetch(regobj, KEYS=fields, exact=TRUE)

}

# clone a registry object
clone_regobj <- function(regobj, empty=FALSE){
	tmp <- tempfile('registry')
	on.exit(unlink(tmp))
	saveRDS(regobj, file=tmp)
	newreg <- readRDS(tmp)
	# empty entries if necessary
	if( empty ){
		sapply(newreg$get_entry_names(), newreg$delete_entry)
	}
	newreg
}

# Graphics utilities
# 
# Author: Renaud Gaujoux
# Created: 05 Dec 2012
###############################################################################

#' Utility Functions for Graphics
#' 
#' @name graphics-utils
#' @rdname graphics-utils
NULL

#' \code{mfrow} returns a 2-long numeric vector suitable to use in \code{\link{par}(mfrow=x)}, 
#' that will arrange \code{n} panels in a single plot. 
#' 
#' @param n number of plots to be arranged.
#' 
#' @rdname graphics-utils
#' @export
#' @examples
#'  
#' mfrow(1)
#' mfrow(2)
#' mfrow(3)
#' mfrow(4)
#' mfrow(10)
mfrow <- function(n){
	if( n == 1 ) c(1, 1)
	else if( n == 2 ) c(1, 2)
	else if( n <= 4 ) c(2, 2)
	else if( n <= 6 ) c(3, 2)
	else if( n <= 9 ) c(3, 3)
	else{
		sn <- floor(n / 3)
		c(sn + if( sn %% 3 ) 1 else 0, 3)
	}
}

round.pretty <- function(x, min=2){
	
	if( is.null(x) ) return(NULL)		
	n <- 0
	y <- round(sort(x), n)
	while( any(diff(y)==0) ){
		n <- n+1
		y <- round(sort(x), n)
	}	
	round(x, max(min,n))
}

# Command Line Interface utils
# 
# Author: Renaud Gaujoux
# Created: May 9, 2013
###############################################################################

.silenceF <- function(f, verbose=FALSE){
    
    if( verbose ) f
    else{
        function(...){
            capture.output(suppressPackageStartupMessages(suppressMessages(res <- f(...)))); 
            res
        }
    }
}

qlibrary <- .silenceF(library, verbose = FALSE)

smessage <- function(..., indent = 0L, item = NULL, appendLF = FALSE){
    if( is.null(item) ){ # choose item from indent
        .item <- c('*', '*', '-', '-', '>', '>') 
        item <- .item[indent+1]
    }
    indent <- if( indent ) paste0(rep(' ', indent), collapse='') else ''
    if( nzchar(item) ) item <- paste0(item, ' ')
    message(indent, item, ..., appendLF = appendLF)
}

CLIfile <- function(full = FALSE){
    pattern <- "--file=(.*)"
    if( !length(f <- grep(pattern, commandArgs(), value = TRUE)) ) ''
    else{
        pf <- gsub(pattern, "\\1", f)
        if( full ) pf
        else basename(pf)
    }
}

#' Enhanced Command Line Argument Parser
#' 
#' Extends the capabilities of package \pkg{argparse}, e.g., in defining sub commands.
#' 
#' @param prog program name
#' @param description program description
#' @param ... extra arguments passed to \code{\link[argparse]{ArgumentParser}}.
#' @param epilog epilog messages to display at the end of the man pages
#' @param show.defaults logical that indicates if default arugment values should 
#' be displayed. 
#' 
#' @export
CLIArgumentParser <- function(prog = CLIfile(), description = '', ..., epilog = '', show.defaults = TRUE){
    
    # load argparse
    suppressMessages( library(argparse, quietly = TRUE) )
    
    .flag_newlines <- function(x){
        gsub("\n", "", x)
    }
    
    .special <- '__@@##@@__'
    epilog <- paste0(.special, epilog)
    p <- ArgumentParser(prog = prog, description = .flag_newlines(description), ..., epilog = .flag_newlines(epilog))
    
    # change argument formatter if required
    if( show.defaults ){
        i <- grep("argparse\\.ArgumentParser", p$python_code)
        inst <- p$python_code[i]
        p$python_code[i] <- paste0(substr(inst, 1, nchar(inst)-1), ', formatter_class=argparse.ArgumentDefaultsHelpFormatter)')
    }
    
    p <- proto(p)
    
    # add add_command function
    p$command_loc <- .special
    p$prog <- prog
    p$exec <- if( nchar(exec_path <- CLIfile(full = TRUE)) ) normalizePath(CLIfile(full = TRUE)) else '' 
    p$command <- list()
    p$command_idefault <- 0L
    p$command_default <- function(.){
        if( .$command_idefault ) names(.$command)[.$command_idefault]
        else ''
    }
    
    # add a (sub-)command
    p$add_command <- function(., command, help='', ..., default = FALSE){
        # add command argument if necessary
        if( !length(.$command) ){
            .$.super$add_argument('command', help = paste0(.$prog, ' command to run'))
        }
        # store command
        .$command[command] <- help	
        # store command as default
        if( default ) .$command_idefault <- length(.$command)
    }
    #
    
    p$add_argument <- function(., ..., help = ''){
        .flag_newlines <- function(x){
            gsub("\n", "", x)
        }
        help <- .flag_newlines(help)
        .$.super$add_argument(..., help = help)
    }
    
    # overload print_usage
    p$print_usage <- function(.){
        .$.super$print_usage()
        if( length(.$command) ){
            cat("\n  Use --help for listing all available commands\n")
        }
    }
    #
    
    # overload print_help to add command descriptions
    p$print_help <- function(.){
        
        # get formatted help
        h <- paste(capture.output(.$.super$print_help()), collapse="\n")
#		# fix new lines if necessary
#		nl <- strsplit(h, "##NL##")[[1]]
#		if( length(nl) > 1L ){
#			indent <- nchar(gsub("^([ ]+).*", "\\1", tail(strsplit(nl[1], "\n")[[1L]], 1L)))
#			i <- 2:length(nl)
#			print(sprintf(paste0("%", indent, 's'), ''))
#			nl[i] <- paste0(sprintf(paste0("%", indent, 's'), ''), nl[i])
#			h <- paste0(nl, collapse="\n")
#		}
        
        cmds <- ''
        if( length(.$command) ){
            # format command help
            lm <- max(nchar(names(.$command)))
            fmt <- paste0("  %-", lm, "s")
            cmds <- strwrap(.$command, indent = 4, exdent = 2 + lm + 4, width = 80, simplify = FALSE)
            cmds <- sapply(cmds, paste, collapse = "\n")
            cmds <- paste0(sprintf(fmt, names(.$command)), cmds)
            cmds <- paste0('Commands:\n', paste(cmds, collapse = "\n"))
        }
        h <- gsub(.$command_loc, cmds, h, fixed = TRUE)
        cat(h, sep="\n")
    }
    #
    
    # add function call_string
    p$call_string <- function(., args = commandArgs(TRUE)){
        paste(.$prog, paste0(args, collapse = ' '))
    }
    
    e <- parent.frame()
    p$locenvir <- parent.env(e)
            
    # commmand parer
    p$parse_cmd <- function(., ...){
#        print(ls(.$locenvir))
        pkgmaker::parseCMD(., ..., envir = .$locenvir)
    }
    
    p
}

# combine argument parsers
.combineParser <- function(p1, p2){
    
    if( length(i <- grep("^parser\\.add_argument", p2$python_code)) ){
        p1$.that$python_code <- c(p1$python_code, p2$python_code[i])
    }
    p1
}

.hasArgument <- function(ARGS){
    function(x) length(ARGS[[x]]) && nzchar(ARGS[[x]])
}

logMessage <- function(..., appendLF = TRUE, extfile = NULL){
    
    # output to external file as well
    if( !is.null(extfile) ){
        cat(..., if( appendLF ) "\n", sep ='', file = extfile, append = TRUE)
    }
    message(..., appendLF = appendLF)
    
}


#' \code{parseCMD} parse command line arguments for sub-commands, 
#' and dispatch to the associated function.
#' 
#' @param parser parser object as returned by \code{CLIArgumentParser}.
#' @param ARGS command line argument to parse, as a named list or a character string.
#' @param debug logical that indicate if debugging information should be printed.
#' @param envir environment that contains where the sub-command functions are looked for. 
#' 
#' @export
#' @rdname CLIArgumentParser
parseCMD <- function(parser, ARGS = commandArgs(TRUE), debug = FALSE, envir = parent.frame()){
    
    if( isString(ARGS) == 1L ){ # used in dev/debugging
        ARGS <- strsplit(ARGS, ' ')[[1]]
    }
    # fix quotes to avoid python JSON parsing error
    ARGS <- gsub("'", "\"", ARGS)
    
    library(pkgmaker, quietly = TRUE)
    # define command line arguments
    prog <- parser$prog
    
    # check validity of command
    # shows usage/help in trivial calls
    if( !length(ARGS) ){
        parser$print_usage()
        return( invisible(parser) )
    }else if( !grepl("^-", ARGS[1L]) ){ # first argument is the command
        command <- ARGS[1L]
        if( !command %in% names(parser$command) ){
            stop("unknown ", prog," command '", command, "'\n"
                    , "  Available commands: ", paste0(names(parser$command), collapse = ', ') 
            #, paste(capture.output(parser$print_usage()), collapse = "\n")
            )
        }
    }else if( any(ARGS %in% c('-h', '--help')) ){
        parser$print_help()
        return( invisible(parser) )
    }else{
        
        # default command if any
        if( nzchar(parser$command_default()) )
            ARGS <- c(parser$command_default(), ARGS)
        else{
            stop("Missing command:\n  "
                    , paste(capture.output(parser$print_usage()), collapse = "\n")
                    , "\n  Available command(s): ", str_out(names(parser$command), Inf, quote=FALSE)
                    , call. = FALSE)
        }
    }
    
    # get command-specific parser
    command <- ARGS[1L]
    cmd_funame <- paste0('CLI_', command)
    if( !exists(cmd_funame, envir, inherits = TRUE) ){
#    if( is.null(cmd_fun <- getFunction(cmd_funame, mustFind = FALSE)) ){
        stop("Could not execute ", prog , " command ", command, ": did not find CLI entry point '", cmd_funame, "'")
    }
    cmd_fun <- get(cmd_funame, envir, inherits = TRUE)
    cmd_parser <- cmd_fun(ARGS=NULL)
    ARGS <- ARGS[-1L]
    
    if( !length(ARGS) ){
        # show command line
        cmd_parser$print_usage()
        invisible(cmd_parser)
    }else if( any(ARGS %in% c('-h', '--help')) ){
        cmd_parser$print_help()
        return( invisible(cmd_parser) )
    }else{
        
        # parse command arguments
        args <- cmd_parser$parse_args(ARGS)
        
        # log call and parsed arguments		
        if( debug ){
            message('Call: ', parser$call_string(ARGS))
            message('Parsed arguments:')
            str(args)
        }
        #
        
        # call command handler
        cmd_fun(ARGS = args)
    }
}

#' Package Specific Command Line Interface
#' 
#' @param package package name
#' @param altfile alternative file that defines the main CLI entry point. 
#' That is a function named \code{CLI}, which takes the list of parsed command line 
#' arguments as its first argument.
#' @param local logical that indicates if the main CLI function should be 
#' defined and evaluated in a local environment, or in the user's Global 
#' environment.
#' @param ARGS list of parsed arguments passed to the main CLI function.
#' @param ... extra arugments passed to the package's CLI function. 
#'   
#' @export
packageCLI <- function(package, altfile = NULL, local = TRUE, ARGS = commandArgs(TRUE), ...){
    
    master_cli <- if( !is.null(package) ) system.file('scripts', 'CLI.R', package = package)
    else if( is.null(altfile) ){
        stop('Could not load CLI definition: argument `package` or `altfile` is required')
    }
    if( !length(master_cli) || !nzchar(master_cli) ){
        master_cli <- altfile
    }
    
    # load CLI
    source(master_cli, keep.source = TRUE, chdir = TRUE, local = local)
    if( !exists('CLI', inherits = FALSE) ){
        stop("Could not start command line interface for package '", package, "': main entry point function CLI() not found.")
    }
    CLI <- get('CLI', inherits = !local)
    
    # run CLI
    CLI(ARGS, ...)
}

# Package hooks
# 
# Author: renaud
# Creation: 26 Jun 2012
###############################################################################

#' @include utils.R
#' @include devutils.R
#' @import stats
#' @import methods
NULL

#' Default Load/Unload Functions
#' 
#' @inheritParams base::.onLoad
#' @inheritParams base::library.dynam
#' 
#' @export
#' @rdname load
#' 
#' @examples
#' 
#' # in a package namespace:
#' .onLoad <- function(libname=NULL, pkgname){
#' 
#' 	pkgmaker::onLoad(libname, pkgname)
#' 
#' }
onLoad <- function(libname=NULL, pkgname, chname=packageName()){
	
	# load compiled library normally or in devmode
	if( !is.null(libname) ){
		if( file.exists(packagePath('libs')) ){
			sapply(chname, library.dynam, package=pkgname, lib.loc=libname)
		}
	}else{
		compile_src() # compile source files and load
	}
		
}

#' @inheritParams base::.onUnload
#' @export
#' @rdname load
#' 
#' @examples
#' 
#' # in a package namespace:
#' .onUnload <- function(libpath){
#' 
#' 	pkgmaker::onUnload(libpath)
#' 
#' }
onUnload <- function(libpath) {
	
	# unload compiled library normally or in devmode
	dlls <- base::getLoadedDLLs()
	pname <- packageName()
	if ( pname %in%  names(dlls) ){
		if( !missing(libpath) )	library.dynam.unload(pname, libpath)
		else dyn.unload(dlls[[pname]][['path']])
	}
	
}


#' Postponing Actions
#' 
#' This function implement a mechanism to postpone actions, which can be executed
#' at a later stage.
#' This is useful when developing packages, where actions that need to be run in the 
#' \code{link{.onLoad}} function but can be defined close to their context.
#' 
#' @param expr expression that define the action to postpone.
#' Currently only functions are supported.
#' @param key identifier for this specific action.
#' It should be unique across the postponed actions from the same group. 
#' @param group optional parent action group.
#' This enables to define meaningful sets of actions that can be run all at once.
#' @param envir environment in which the action should be executed.
#' Currently not used.
#' @param verbose logical that toggles verbose messages.
#'
#' @import digest
#' @export
#' 
#' @examples
#' opt <- options(verbose=2)
#' 
#' # define actions
#' postponeAction(function(){print(10)}, "print")
#' postponeAction(function(){print(1:10)}, "more")
#' postponeAction()
#' # execute actions
#' runPostponedAction()
#' runPostponedAction()
#' 
#' # restore options
#' options(opt)
#' 
postponeAction <- function(expr, key=digest(tempfile()), group=NULL, envir=topns(strict=FALSE), verbose=getOption('verbose')){
	
	# do not do anything if already running delayed actions
	if( isRunningPostponedAction() ) return()
	
	ns <- topns(strict=FALSE)
	taskObj <- simpleRegistry('.__delayedTasks__', envir=ns)
	if( !missing(expr) ){
		if( missing(key) ){
			stop("Missing required argument `key` for registering/cancelling delayed action.")
		}
		# add group prefix
		if( !is.null(group) )
			key <- str_c(group, '::', key)
		#qe <- if( !is.language(expr) ) substitute(expr) else expr
		qe <- expr
		if( verbose ){
			if( !is.null(qe) ) message("# Postponing action '", key, "'")
			else{
				message("# Cancelling postponed action '", key, "'")
			}
		}
		taskObj$set(key, list(action=qe, envir=envir))
	}else{
		taskObj$names()
	}
}

#' @rdname postponeAction
#' @export
runPostponedAction <- function(group=NULL, verbose=getOption('verbose')){
	
	ns <- topns(strict=FALSE)
	taskObj <- simpleRegistry('.__delayedTasks__', envir=ns)
	
	if( verbose ){
		message("# Executing postponed "
				, if( !is.null(group) ) paste("'", group, "' ", sep='')
				, "action(s) in package '"
				, packageName(ns, .Global=TRUE), "' ... "
				, appendLF = FALSE)
	}
	# set up running flag
	isRunningPostponedAction(TRUE)
	on.exit(isRunningPostponedAction(FALSE))
	#
	# execute actions
	t <- taskObj$names()
	if( !is.null(group) ) t <- grep(str_c("^", group), t, value=TRUE)
	if( verbose > 1 && length(t) ) message()
	sapply(t, function(x){
				act <- taskObj$get(x)
				if( verbose > 1 ){
					message("** Action '", x, "' [", packageName(act$envir, .Global=TRUE), ']')
				}
				act$action()
				taskObj$set(x, NULL)
				#eval(x$expr, x$envir)
			})
	if( verbose ) message('OK [', length(t), ']')
	invisible(length(t))
}

# Tells if one is executing deferred tasks via \code{onLoad}
isRunningPostponedAction <- sVariable(FALSE)

#' Simple Package Registry
#' 
#' @param name name of the registry object, with which it will
#' be assigned in \code{envir}.
#' @param envir environment where to store the registry object.
#' Defaults to the caller's top environment.
#' @param verbose logical that toggle a verbose message when 
#' the object is first created.
#' 
#' @export
simpleRegistry <- function(name, envir=topenv(parent.frame()), verbose=FALSE){
	
	# return stored instance if it exists
	if( exists(name, envir=envir) ){
		return( invisible(get(name, envir=envir)) )
	}
	
	if( verbose ) message("# Setup simple registry '", name, "' in ", packageName(envir, .Global=TRUE))
	.name <- name
	.envir <- envir
	.data <- list()
	
	.get <- function(x){
		if( .has(x) ){
			.data[[x]]
		}
	}
	.set <- function(x, value){
		if( is.null(value) ){
			if( .has(x) ){
				.data[[x]] <<- NULL
			}
		}else{
			.data[[x]] <<- value
		}
	}
	.has <- function(x){
		x %in% names(.data)
	}
	.cleanup <- function(){
		rm(.name, envir=.envir)
	}
	.names <- function(){
		names(.data)
	}
	.length <- function(){
		length(.data)
	}
	
	.obj <- list(get=.get, set=.set, has=.has
			, cleanup=.cleanup, names=.names
			, length = .length)
	
	# assign container object
	assign(.name, .obj, envir=.envir)
	#
	invisible(.obj)
}


#' Defunct Functions in pkgmaker
#' 
#' These functions have been defunct or superseded by other 
#' functions. 
#' 
#' @param ... extra arguments
#' 
#' @rdname pkgmaker-defunct
#' @name pkgmaker-defunct
NULL

# Project: pkgmaker
# 
# Author: Renaud Gaujoux
# Created: Feb 13, 2014
###############################################################################

.PACKAGES_fields <- c('Package', 'Version')

#' Generate CRAN-like Repository Index
#' 
#' @param path path to the repository's root directory
#' @param output output filename -- relative to the repository root \code{path}.
#' @param pattern regular expression used to filter the names of the packages that will appear in  
#' the index.
#' @param title title of the index page
#' @param robots.file logical that indicates if a file \code{robots.txt} that hides the repository from 
#' search engine robots should be created. 
#' @export
write_PACKAGES_index <- function(path = '.', output = 'index.html', pattern = NULL, title = 'Packages', robots.file = TRUE){
    
    # parameters
    dir <- path
    sel <- .PACKAGES_fields
    
    # load package list from contrib
    repo_dir <- normalizePath(dir)
    contrib_dir <- contrib.url(repo_dir)
    repo <- paste0('file://', repo_dir)
    contrib <- contrib.url(repo)
    contrib_path <- contrib.url('.')
    
    # change to repo base directory
    od <- setwd(repo_dir)
    on.exit( setwd(od) )
    
    # write PACKAGES files
    makePACKAGES <- function(dir = '.'){
        od <- setwd(dir)
        on.exit( setwd(od) )
        
        smessage('Generating PACKAGES file for ', dir, ' ... ')
        n <- tools::write_PACKAGES('.', fields = sel)
        message('OK [', n, ']')
        n
    }
    makePACKAGES(contrib_path)
    
    smessage('Generating HTML page in ', repo_dir, appendLF = TRUE)
    if( robots.file ){
        write("User-agent: *\nDisallow: /\n\n", file = file.path(repo_dir, 'robots.txt'))
    }
    smessage('Reading PACKAGES file in ', contrib_path, ' ... ')
    p <- available.packages(contrib, fields = sel)
    message('OK [', nrow(p), ']')
    if( !is.null(pattern) ){
        smessage('Selecting packages matching pattern "', pattern, '" only ... ')
        i <- grep(pattern, p[, 'Package'])
        message('OK [', length(i), '/', nrow(p), ']')
        p <- p[i, , drop = FALSE]
    }
    df <- as.data.frame(p[, sel, drop = FALSE], stringsAsFactors = FALSE)
    
    # write index page
    smessage('Loading required packages ... ')
    qlibrary('ReportingTools')
    qlibrary('hwriter')
    message('OK')
    smessage('Generating ', output, ' ... ')
    index <- HTMLReport(shortName = tools::file_path_sans_ext(output), title = title)
    
    # link to source package
    linkPackage <- function(df, ...){
	    pkg_src <- file.path(sub(file.path(repo, ''), '', contrib, fixed = TRUE), as.character(df$Package))
	    df$Package <- hwrite(as.character(df$Package), link = sprintf("%s_%s.tar.gz", pkg_src, df$Version), table=FALSE)
	    df
    }
    # maintainer email
    emailMaintainer <- function(df, ...){
	    if( !is.null(df$Maintainer) ){
		    df$Maintainer <- gsub(".*<([^>]+)> *$", "\\1", df$Maintainer)
	    }
	    df
    }
    
    # publish
    publish(knit2html(quiet = TRUE, text = "Install packages from this repository as follows (in an R console):
                            
```{r, eval = FALSE}
# install BiocInstaller (only once)
source('http://bioiconductor.org/biocLite.R')
                            
# install package
BiocInstaller::biocLite('<pkgname>', siteRepos = '<URL>')
```", fragment.only = TRUE), index)
    publish(df, index, name=title, .modifyDF = list(emailMaintainer, linkPackage))
    finish(index)
    message('OK')
    message()
    invisible(normalizePath(output))
}

# Utils for R package projects
# 
# Author: Renaud Gaujoux
# Created: May 1, 2013
###############################################################################

is_pattern <- function(x){
  grepl('[*^?)$([]', x)
}
# match with exact or regular expression lookup
match_mix <- function(x, table, nomatch = NA_integer_, ignore.case = FALSE){
  
  # find regular expression patterns
  is_reg <- is_pattern(table)
  case <- function(x) if( ignore.case ) tolower(x) else x
  hit <- match(case(x), case(table[!is_reg]), nomatch = nomatch)
  
  if( any(is_reg) ){
    reg_match <- unlist_(sapply(table[is_reg], grep, x, ignore.case = ignore.case, simplify = FALSE))
    reg_match <- reg_match[!duplicated(reg_match)]
    hit[reg_match] <- pmin(hit[reg_match], match(names(reg_match), table), na.rm = TRUE)
    
  }
  
  hit[!is.na(hit)]
}

#' Test for Package Root Directory
#' 
#' Tells if a directory is a package directory, i.e. that it contains
#' a \code{DESCRIPTION} file.
#' 
#' @param x path to the directory to test
#' @param error logical that indicates if an error should be raised
#' if the directory is not a package directory.
#' 
is_package_path <- function(x, error = FALSE) {
	if (is.null(x)) return(FALSE)
	x <- normalizePath(x, mustWork = FALSE)
	x <- gsub("\\\\$", "", x)
	desc_path <- file.path(x, "DESCRIPTION")
	if( !error ){
		file.exists(x) && file.exists(desc_path)
	}else{
		if ( !file.exists(x) ) stop("Can't find directory ", x, call. = FALSE)
		if ( !file.info(x)$isdir ) stop(x, " is not a directory", call. = FALSE)
		if (!file.exists(desc_path)) stop("No DESCRIPTION file found in ", x, call. = FALSE)
		TRUE
	}
}


#' Find Path to Development Package Root Directory
#' 
#' Development packages are looked-up according to rules
#' defined in a file \code{.Rpackages} in the user's home directory. 
#' 
#' @section Specification of package path:
#' Package paths are specified in a list with:
#'   * unnamed elements: character strings give path to directories to lookup for sub-directories that match
#' exactly the package's name;
#'   * named element containing character strings: these are paths that are looked up only for packages that 
#' match the element name. If the element name contains any of the characters `*?()$^\\][`, then it is matched using 
#' regular expression.
#' 
#' @param x name of the development package to lookup.
#' @param error logical that indicates if an error is thrown when the project root directory 
#' could not be found.
#' @export
find_devpackage <- function(x, error = TRUE) 
{
	
	
	if (is_package_path(x)) {
		return(x)
	}
	
	config_path <- "~/.Rpackages"
	if (!file.exists(config_path)) {
		return(NULL)
	}
	config_path <- path.expand(config_path)
	lookup <- source(config_path)$value
  
  default_lookup <- lookup$default
  lookup <- lookup[!names(lookup) %in% 'default']
  
  # check for a match
  i <- c(match_mix(x, names(lookup), ignore.case = TRUE), which(names(lookup) %in% ''))
  if( length(i) ){
      reg_spec <- is_pattern(names(lookup))
      for(k in i){
        val <- lookup[[k]]
        n <- names(lookup)[k]
        if( grepl('*', val, fixed = TRUE) ){ # check path like a/b/*/c
          p <- gsub('*', x, val, fixed = TRUE)
          if( is_package_path(p) ){
            message("Loading path resolved by ", n, ':', val)
            return(p)
            
          }
          
        } else if( !reg_spec[k] && is_package_path(val) ){ # exact match
          message("Loading path resolved by ", n, ':', val)
          return(val) 
          
        }else if( reg_spec[k] ){
          paths <- list.dirs(val, full.names = TRUE, recursive = FALSE)
          x2 <- sub(n, "", x)
          x2[!nzchar(x2)] <- x[!nzchar(x2)]
          hit <- match(x2, basename(paths))
          p <- paths[hit[1L]]
          if( length(hit) && is_package_path(p) ){
            message("Loading path resolved by ", n, ':', val)
            return(p)
            
          }
          
        }
      }
      
  }
  
  if( !is.null(default_lookup) ) {
		default_loc <- default_lookup(x)
		if ( is_package_path(default_loc, error = error) ) {
      message("Loading path resolved by default lookup")
			return(default_loc)
		}
	}
  
  if( error ) message("Could not find package directory for project ", x) 
	NULL
}

#' Load Development Package
#' 
#' @param pkg name of the package/project to load.
#' @param reset logical that indicates if the package should be reloaded (passed to \code{\link[devtools]{load_all}}.
#' @param ... other arguments passed to \code{\link[devtools]{load_all}}.
#' @param utests logical that indicates if an environment containing the unit test functions should be created.
#' If \code{TRUE} this environment is accessible at \code{pkgname::UnitTests$test.filename.r$function.name}.
#' @param verbose logical that indicates if log messages should be printed.
#' @param addlib logical that indicates if the \code{lib/} sub-directory, if it exists, should be prepended 
#' to the library path. 
#' This enables to control the version of the loaded dependencies.
#' @param character.only logical that indicates if argument \var{pkg} should be evaluated or taken litteral. 
#' @param try.library logicatl that indicates if projects that could not be found should be looked up in 
#' the installed packages.
#' 
#' @export 
load_project <- function(pkg, reset = FALSE, ..., utests = TRUE, verbose=FALSE, addlib=TRUE, character.only = FALSE, try.library = FALSE) {
	
	if( !character.only ){
		pkg <- deparse(substitute(pkg))
		pkg <- sub("^\"(.*)\"$", "\\1", pkg)
	}
	
  # lookup dev package root directory
	devpkg_path <- find_devpackage(pkg, error = !try.library)
  # load from installed pacakges if not found and requested
  if( is.null(devpkg_path) ){
    if( !try.library ) return(invisible())
    message(sprintf("Trying to load installed package %s ... ", pkg), appendLF = FALSE)
    library(pkg, character.only = TRUE, quietly = TRUE)
    message('OK')
    return(invisible())
    
  }
  
	pkg <- devpkg_path 

  if( !requireNamespace('devtools') ){
    stop("Could not load package: required package 'devtools' is not installed.")
  }
  
	# add ../lib to the path if necessary
	if( addlib && is.character(tp <- pkg) ){
		tp <- as.package(tp)
		pdir <- normalizePath(file.path(dirname(tp$path), "lib"), mustWork=FALSE)
		if( file_test('-d', pdir) && !is.element(pdir, .libPaths()) ){
			message("Adding to .libPaths: '", pdir, "'")
			olibs <- .libPaths()
			.libPaths(c(pdir, .libPaths()))
			on.exit( .libPaths(olibs), add=TRUE )
		}
	}
	devpkg <- as.package(pkg)
	
	# load package
	op <- options(verbose=verbose)
	on.exit(options(op), add=TRUE)  
	devtools::load_all(pkg, reset = reset, ...)
	#
	
	# source unit test files if required
	udir <- file.path(devpkg$path, 'inst', c('tests', 'unitTests'))
	if( utests && length(w <- which(file.exists(udir))) ){
		message("# Sourcing unit test directory ... ", appendLF = FALSE)
		f <- list.files(udir[w[1L]], pattern = "\\.[Rr]$", full.names=TRUE)
		if( length(f) ){
#			if( !requireNamespace('RUnit') ) stop("Missing required dependency 'RUnit' to load unit tests")
			# create unit test environment
      ns_env <- getDevtoolsFunction('ns_env')
			utest_env <- new.env(parent = ns_env(devpkg))
			assign('UnitTests', utest_env, ns_env(devpkg))
			# source test files in separate sub-environments
			sapply(f, function(f){
						e <- new.env(parent = utest_env)
						assign(basename(f), e, utest_env)
						sys.source(f, envir = e)
					})
		}
		message('OK [', length(f), ']')
		# reload to export the unit test environment
		devtools::load_all(pkg, reset = FALSE, ...)
	}
	#
	
	invisible(devpkg)
}

#' @describeIn load_project shortcut for `load_project(..., try.library = TRUE)`, to load project
#' code from installed library if not found as a development project.
#' All its arguments are passed to `load_project`.
#' @export
library_project <- function(...) load_project(..., try.library = TRUE)

getDevtoolsFunction <- function(name){
  
  if( qrequire('devtools') && !is.null(fun <- ns_get(name, 'devtools')) ) return(fun)
  if( qrequire('pkgload') && !is.null(fun <- ns_get(name, 'pkgload')) ) return(fun)
  if( qrequire('pkgbuild') && !is.null(fun <- ns_get(name, envir = 'pkgbuild')) ) return(fun)
  
}

is_Mac <- function(check.gui=FALSE){
	is.mac <- (length(grep("darwin", R.version$platform)) > 0)
	# return TRUE is running on Mac (adn optionally through GUI)
	is.mac && (!check.gui || .Platform$GUI == 'AQUA')
}

R_OS <- function(){
	if( is_Mac() ) 'MacOS'
	else .Platform$OS.type
}

packageMakefile <- function(package=NULL, template=NULL, temp = FALSE, print = TRUE){
	
	capture.output(suppressMessages({
		library(pkgmaker)
        if( !requireNamespace('devtools', quietly = TRUE) ) 
                stop("Package 'devtools' is required to generate a package Makefile")
						
	}))
#	defMakeVar <- pkgmaker::defMakeVar
#	subMakeVar <- pkgmaker::subMakeVar
	
	project_path <- getwd()
	project_name <- basename(project_path)
	subproject_path_part <- ''
	if( is.null(package) || isString(package) ){
		if( isString(package) && !nzchar(package) ) package <- NULL
		lookup_dir <- c('pkg', '.')
		if( !is.null(package) ){
			lookup_dir <- c(package, lookup_dir, file.path('pkg', package))
			subproject_path_part <- file.path(package, '')
		}
		pdir <- file.path(lookup_dir, 'DESCRIPTION')
		if( !length(sd <- which(is.file(pdir))) ){
			stop("Could not detect package source directory")
		}
		package <- pdir[sd[1L]]
	}
	package <- normalizePath(package)
	p <- pkg <- as.package(dirname(package));
	pdir <- package_dir <- p[['path']];
	
	## create makefile from template
	# load template makefile
	if( is.null(template) ){
		template <- packagePath('package.mk', package='pkgmaker')
	}
	l <- paste(readLines(template), collapse="\n")
	
	# user
	cuser <- Sys.info()["user"]
	l <- defMakeVar('AUTHOR_USER', cuser, l)
	l <- defMakeVar('R_PACKAGE', pkg$package, l)
	# R_PACKAGE_PATH
	l <- defMakeVar('R_PACKAGE_PATH', package_dir, l)
	# R_PACKAGE_PROJECT
	l <- defMakeVar('R_PACKAGE_PROJECT', project_name, l)
	# R_PACKAGE_PROJECT_PATH
	l <- defMakeVar('R_PACKAGE_PROJECT_PATH', project_path, l)
	l <- defMakeVar('R_PACKAGE_SUBPROJECT_PATH_PART', subproject_path_part, l)
	# R_BIN
	l <- subMakeVar('R_BIN', R.home('bin'), l)
    # REPO_DIRS
    repo_dirs <- gsub("^\\./", "", sapply(c('source', 'win.binary', 'mac.binary'), contrib.url, repos = '.'))
    l <- defMakeVar('REPO_DIRS', paste0(repo_dirs, collapse = ' '), l)
    # BUILD_DIR
    l <- defMakeVar('BUILD_DIR', file.path(repo_dirs['source'], ''), l)
	# R_PACKAGE_TAR_GZ
	pkg_targz <- file.path(repo_dirs['source'], package_buildname(p, 'source'))
	l <- defMakeVar('R_PACKAGE_TAR_GZ', pkg_targz, l)
    # R_PACKAGE_ZIP
	pkg_zip <- file.path(repo_dirs['win.binary'], package_buildname(p, 'win.binary'))
	l <- defMakeVar('R_PACKAGE_ZIP', pkg_zip, l)
    # R_PACKAGE_TGZ
	pkg_mac <- file.path(repo_dirs['mac.binary'], package_buildname(p, 'mac.binary'))
	l <- defMakeVar('R_PACKAGE_TGZ', pkg_mac, l)
	# R_PACKAGE_TYPE	
	l <- defMakeVar('R_PACKAGE_OS', R_OS(), l)
	#

    # auto-conf variables
    init_var <- list(version = pkg$version)
    if( is.dir(file.path(package_dir, 'vignettes')) ) 
        init_var <- c(init_var, has_vignettes=TRUE)
    # dump variables
    if( length(init_var) ){
        init_var <- setNames(init_var, paste0('R_PACKAGE_', toupper(names(init_var))))
        init_var_str <- str_out(init_var, Inf, use.names = TRUE, sep = "\n")
        l <- subMakeVar('INIT_CHECKS', init_var_str, l)
    }
	
	# R_CMD_CHECK
	rlibs <- ''
    if( is.dir(devlib <- file.path(dirname(pdir), 'lib')) ){
		rlibs <- paste0("R_LIBS=", devlib, ' ')
	}
    l <- subMakeVar('R_LIBS', rlibs, l)
	#

	# create makefile
	mk <- if( temp ) tempfile('package_', tmpdir='.', fileext='.mk') else 'package.mk'
	cat(l, file=mk)
	if ( print ){
		cat(mk)
	}
	invisible(l)
}

# Vignette generation related functions
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

#' @include packages.R
NULL

rnw_message <- function(...) message("# ", ...)

#' Identifying Sweave Run
#' 
#' Tells if the current code is being executed within a Sweave document.
#' 
#' @return \code{TRUE} or \code{FALSE}
#' @export
#' 
#' @examples
#' 
#' # Not in a Sweave document 
#' inSweave()
#' 
#' # Within a Sweave document
#' 
inSweave <- function(){
#	in.sweave <- FALSE
	if ((n.parents <- length(sys.parents())) >= 3) {
		for (i in seq_len(n.parents) - 1) {
			if ("chunkopts" %in% ls(envir = sys.frame(i))) {
				chunkopts = get("chunkopts", envir = sys.frame(i))
				if (all(c("prefix.string", "label") %in% names(chunkopts))) {
#					in.sweave <- TRUE
					return(TRUE)
					break
				}
			}
		}
	}
	FALSE
}

#' Generate a Fake Vignette
#' 
#' @param src original Sweave file
#' @param out output file
#' @param PACKAGE package name where to look the source vignette
#' 
#' @export
makeFakeVignette <- function(src, out, PACKAGE=NULL){
	
	# interpret template within the package directory
	if( !is.null(PACKAGE) ){
		src <- str_c(, src)
	}
    if( identical(normalizePath(dirname(src)), normalizePath(dirname(out))) ){
        cat("# NOTE: skipped fake vignette [source in root directory]\n")
        return(invisible())
    }
	# read in template file
	l <- readLines(src)
	# extract %\Vignette commands
	vign <- l[grep("^%\\s*\\\\Vignette", l)]
	# write output file
	cat(c("\\documentclass[10pt]{article}"
		, vign
		, "\\usepackage{url}\n\\usepackage[colorlinks]{hyperref}\n\n\\begin{document}\n\\end{document}")
		, file=out, sep="\n");

}

#' LaTeX Utilities for Vignettes
#' 
#' \code{latex_preamble} outputs/returns command definition LaTeX commands to 
#' be put in the preamble of vignettes.
#' 
#' Argument \code{PACKAGE} is not required for \code{latex_preamble}, but must 
#' be correctly specified to ensure \code{biblatex=TRUE} generates the correct
#' bibliography command.
#'  
#' @param R logical that indicate if general R commands should be added 
#' (e.g. package names, inline R code format commands) 
#' @param CRAN logical that indicate if general CRAN commands should be added
#' (e.g. CRAN package citations) 
#' @param Bioconductor logical that indicate if general Bioconductor commands 
#' should be added (e.g. Bioc package citations) 
#' @param GEO logical that indicate if general GEOmnibus commands should be added
#' (e.g. urls to GEO datasets) 
#' @param ArrayExpress logical that indicate if general ArrayExpress commands 
#' should be added (e.g. urls to ArrayExpress datasets)
#' @param biblatex logical that indicates if a \code{\\bibliography} command 
#' should be added to include references from the package's REFERENCES.bib file. 
#' 
#' @param only a logical that indicates if the only the commands whose 
#' dedicated argument is not missing should be considered.
#' @param file connection where to print. If \code{NULL} the result is returned
#' silently.
#' 
#' @import stringr
#' @export
#' @rdname latex
#' @examples
#' 
#' latex_preamble()
#' latex_preamble(R=TRUE, only=TRUE)
#' latex_preamble(R=FALSE, CRAN=FALSE, GEO=FALSE)
#' latex_preamble(GEO=TRUE, only=TRUE)
#' 
latex_preamble <- function(PACKAGE, R=TRUE, CRAN=TRUE, Bioconductor=TRUE
							, GEO=TRUE, ArrayExpress=TRUE, biblatex=FALSE, only=FALSE, file=''){
	
	cmd <- "%%%% PKGMAKER COMMANDS %%%%%%
\\usepackage{xspace}
"
	
	inc <- function(arg){
		e <- parent.frame()
		(!only || eval(substitute(hasArg(arg), list(arg=substitute(arg))), e)) && arg
	}
	
	if( inc(R) ){
		cmd <- c(cmd, 
"% R
\\let\\proglang=\\textit
\\let\\code=\\texttt 
\\newcommand{\\Rcode}{\\code}
\\newcommand{\\pkgname}[1]{\\textit{#1}\\xspace}
\\newcommand{\\Rpkg}[1]{\\pkgname{#1} package\\xspace}
\\newcommand{\\citepkg}[1]{\\cite{#1}}
")
}

	if( inc(CRAN) ){
		cmd <- c(cmd,
"% CRAN
\\newcommand{\\CRANurl}[1]{\\url{http://cran.r-project.org/package=#1}}
%% CRANpkg
\\makeatletter
\\def\\CRANpkg{\\@ifstar\\@CRANpkg\\@@CRANpkg}
\\def\\@CRANpkg#1{\\href{http://cran.r-project.org/package=#1}{\\pkgname{#1}}\\footnote{\\CRANurl{#1}}}
\\def\\@@CRANpkg#1{\\href{http://cran.r-project.org/package=#1}{\\pkgname{#1}} package\\footnote{\\CRANurl{#1}}}
\\makeatother
%% citeCRANpkg
\\makeatletter
\\def\\citeCRANpkg{\\@ifstar\\@citeCRANpkg\\@@citeCRANpkg}
\\def\\@citeCRANpkg#1{\\CRANpkg{#1}\\cite*{Rpackage:#1}}
\\def\\@@citeCRANpkg#1{\\CRANpkg{#1}~\\cite{Rpackage:#1}}
\\makeatother
\\newcommand{\\CRANnmf}{\\href{http://cran.r-project.org/package=NMF}{CRAN}}
\\newcommand{\\CRANnmfURL}{\\url{http://cran.r-project.org/package=NMF}}
")
}

	if( inc(Bioconductor) ){
		cmd <- c(cmd,
"% Bioconductor
\\newcommand{\\BioCurl}[1]{\\url{http://www.bioconductor.org/packages/release/bioc/html/#1.html}}
\\newcommand{\\BioCpkg}[1]{\\href{http://www.bioconductor.org/packages/release/bioc/html/#1.html}{\\pkgname{#1}} package\\footnote{\\BioCurl{#1}}}
\\newcommand{\\citeBioCpkg}[1]{\\BioCpkg{#1}~\\cite{Rpackage:#1}}
% Bioconductor annotation
\\newcommand{\\BioCAnnurl}[1]{\\url{http://www.bioconductor.org/packages/release/data/annotation/html/#1.html}}
\\newcommand{\\BioCAnnpkg}[1]{\\href{http://www.bioconductor.org/packages/release/data/annotation/html/#1.html}{\\Rcode{#1}} annotation package\\footnote{\\BioCAnnurl{#1}}}
\\newcommand{\\citeBioCAnnpkg}[1]{\\BioCAnnpkg{#1}~\\cite{Rpackage:#1}}
")
}

	if( inc(GEO) ){
		cmd <- c(cmd, 
"% GEO
\\newcommand{\\GEOurl}[1]{\\href{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=#1}{#1}\\xspace}
\\newcommand{\\GEOhref}[1]{\\GEOurl{#1}\\footnote{\\url{http://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=#1}}}
")
	}

	if( inc(ArrayExpress) ) cmd <- c(cmd,
"% ArrayExpress
\\newcommand{\\ArrayExpressurl}[1]{\\href{http://www.ebi.ac.uk/arrayexpress/experiments/#1}{#1}\\xspace}
\\newcommand{\\ArrayExpresshref}[1]{\\ArrayExpressurl{#1}\\footnote{\\url{http://www.ebi.ac.uk/arrayexpress/experiments/#1}}}
")

	if( biblatex ){
		if( missing(PACKAGE) )
			stop("Argument `PACKAGE` is required when specifying `biblatex=TRUE`.")
		cmd <- c(cmd, latex_bibliography(PACKAGE, file=NULL))
	} 

	# output or return commands
	cmd <- c(cmd, "%%%% END: PKGMAKER COMMANDS %%%%%%\n")
	cmd <- str_c(cmd, collapse="\n")
	if( !is.null(file) ) cat(cmd, file, sep='')
	else cmd
	
}

#' \code{latex_bibliography} prints or return a LaTeX command that includes a 
#' package bibliography file if it exists.
#' 
#' @param PACKAGE package name
#' 
#' @export
#' @rdname latex
#' 
latex_bibliography <- function(PACKAGE, file=''){
	
    rpkg.bib <- "%\\bibliography{Rpackages}\n"
    cmd <- rpkg.bib
	# get REFERENCES.bib file
	reffile <- packageReferenceFile(PACKAGE=PACKAGE)
    if( is.file(reffile) ){
        cmd <- paste0(cmd, "\\bibliography{", gsub("\\.bib$", "", reffile), "}\n")
    }
	
    # add post-processing knit hook
    library(knitr)
    knit_hooks$set(document = function(x){
        # write bibfile if necessary
        if( length(pkgs <- parsePackageCitation(x)) ){
            # write bibfile
            write.pkgbib(gsub("^Rpackage:", '', pkgs), file='Rpackages.bib', prefix='Rpackage:')
            # uncomment inclusion line
            x <- gsub("%\\bibliography{Rpackages}", "\\bibliography{Rpackages}", x, fixed = TRUE)
        }
        x
    })

    if( !is.null(file) ) cat(cmd, file=file)
    else cmd
}

is.rnw <- function(x){
	is(x, 'rnw')
}


runVignette <- function(x, ...){
    # flag the vignette as being locally made
    opts <- options(R_RUNNING_MANUAL_VIGNETTE=TRUE)
    on.exit( options(opts) )
    # run
	UseMethod('runVignette')
}

# tells if a vignette is locally made
#' Identifies Manually Run Vignettes
#' 
#' \code{isManualVignette} tells if a vignette is being run through the function \code{runVignette} 
#' of \pkg{pkgmker}, allowing disabling behaviours not allowed in package vignettes that are
#' checked vi \code{R CMD check}. 
#' 
#' @rdname vignette
#' @export
isManualVignette <- function(){
    isTRUE(getOption('R_RUNNING_MANUAL_VIGNETTE'))
}

runVignette.default <- function(x, file=NULL, ...){
	stop("Vignette compiler '", class(x), "' is not supported")
} 

## #' @param fig.path specification for the figure path (used in knitr vignettes only). 
## #' If \code{TRUE} then the figure path is set to \code{'./figure/<basename>/'}.
## #' @param cache.path specification for the cache path.
## #' If \code{TRUE} then the figure path is set to \code{'./cache/<basename>/'}.
#' @S3method runVignette rnw_knitr 
runVignette.rnw_knitr <- function(x, file=NULL, ..., fig.path=TRUE, cache.path=TRUE){
	library(knitr)
	# expand path to cache to fix issue in knitr
	bname <- sub("\\..{3}$", '', basename(x$file))	
	# add suffix for windows
	if( .Platform$OS.type == 'windows' ){ 
		bname <- paste(bname, '-win', sep='')
	}
	# cache.path
	if( !isFALSE(cache.path) ){
		if( isTRUE(cache.path) ){
			cache.path <- file.path(getwd(), 'cache', bname, '/')
		}
		opts_chunk$set(cache.path=cache.path)	
	}
	# fig.path
	if( !isFALSE(fig.path) ){
		if( isTRUE(fig.path) ){
			fig.path <- file.path(getwd(), 'figure', str_c(bname,'-'))
		}
		opts_chunk$set(fig.path=fig.path)	
	}
	
	# set other options
	opts_chunk$set(...)
	
	# run knitr
	e <- new.env(parent = .GlobalEnv)
	if( FALSE && (is.null(file) || file_extension(file) %in% c('tex', 'pdf')) ){
		ofile <- if( file_extension(file) == 'pdf' ) file else NULL 
		knit2pdf(x$file, ofile, envir=e)
		if( is.null(file) ){
			# remove pdf file
			unlink(file.path(getwd(), basename(file_extension(x$file, 'pdf'))))
		} else if( file_extension(file) == 'tex' ){
			# move tex file
			file.rename(file_extension(file, 'tex'), file)
		}
	}else knit(x$file, file, envir=e)
}

#' @S3method runVignette rnw_sweave 
runVignette.rnw_sweave <- function(x, file=NULL, ...){
	res <- Sweave(x$file, driver=x$driver, ...)
	# move output file
	if( !is.null(file) ){
		file.rename(res, file)
	}
	res
}

#' Utilities for Vignettes
#' 
#' \code{rnw} provides a unified interface to run vignettes that detects
#' the type of vignette (Sweave or \code{\link[knitr]{knitr}}), and which Sweave driver 
#' to use (either automatically or from an embedded command \code{\\VignetteDriver} 
#' command).
#' 
#' @param x vignette source file specification as a path or a \code{rnw} object.
#' @param file output file
#' @param ... extra arguments passed to \code{as.rnw} that can be used to force 
#' certain building parameters.
#' @param raw a logical that indicates if the raw result for the compilation 
#' should be returned, instead of the result file path.
#'   
#' @rdname vignette
#' @export
rnw <- function(x, file=NULL, ..., raw=FALSE){
	
#	library(methods)
	# load rnw file
	x <- as.rnw(x, ...)	
	
	# setup restoration of options and graphical parameters
	opts <- options()
	gpar <- par(no.readonly=TRUE)
	on.exit( {options(opts); par(gpar)})
	
	# copy package cleveref from pkgmaker installation
	if( 'cleveref' %in% x$latexPackages ){
		clv <- packagePath('cleveref.sty', package='pkgmaker')
		message("# Copying package 'cleveref.sty' from ", dirname(clv)," ... ", appendLF=FALSE)
		wd <- if( !is.null(file) ) dirname(file) else getwd()
		file.copy(clv, wd)
		if( file.exists(file.path(wd, basename(clv))) )	message('OK') else message('ERROR')
	}
	
	# run vignette
	res <- runVignette(x, file=file, ...)
	
	# Package citations
	if( !is.null(keys <- x$cite) ){
		message("# Writing package bibtex file [", length(keys)," key(s)] ... ", appendLF=FALSE)
		write.pkgbib(keys, file='Rpackages.bib', prefix='Rpackage:', verbose=FALSE)
		message('OK')
	}
	#

	# return raw result if required
	if( raw ) return(res)

	# check for a wrapper main file
	if( !is.null(x$wrapper) ){
		res <- x$wrapper
	}
	
	invisible(res)
}

checkFile <- function(x, msg="file '%s' does not exist."){
	if( !is.file(x) ) stop(sprintf(msg, x))
	TRUE
}

checkRnwFile <- function(x){
	if( is.rnw(x) ) x <- x$file
	checkFile(x, msg="Vignette file '%s' does not exist.")
}

#' \code{as.rnw} creates a S3 \code{rnw} object that contains information
#' about a vignette, e.g., source filename, driver, fixed included files, etc..
#' 
#' @param load logical to indicate if all the object's properties should loaded, 
#' which is done by parsing the file and look up for specific tags. 
#' 
#' @rdname vignette
#' @export
as.rnw <- function(x, ..., load = TRUE){
	
	if( is.rnw(x) ) return(x)
	
	checkRnwFile(x)
	# initialise 'rnw' object
	obj <- list()
	class(obj) <- 'rnw'
	
	# store source full path
	obj$file <- normalizePath(x)
	obj$line <- NA
	if( !load ) return(obj)
	
	# detect compiler
	obj$compiler <- rnwCompiler(obj) %||% 'Sweave'
	cl <- if( obj$compiler == 'knitr' ) 'knitr' else 'sweave'
	class(obj) <- c(paste('rnw_', cl, sep=''), class(obj))
	
	# detect driver
	obj$driver <- rnwDriver(obj) %||% RweaveLatex()
	# detect wrapper
	obj$wrapper <- rnwWrapper(obj)
	# detect fixed included images
	obj$includes <- rnwIncludes(obj)
	# detect latex packages
	obj$latexPackages <- rnwLatexPackages(obj)
	# detect children vignettes
	obj$children <- rnwChildren(obj)
	# detect package citations
	obj$cite <- rnwCite(obj)
	
	# override with passed extra arguments
	if( nargs() > 1L ){
		dots <- list(...)
		obj[names(dots)] <- dots
	}
	
	# return object
	obj
} 

rnwObject <- function(...) as.rnw(..., load=FALSE)

rnwParser <- function(tag, name=tolower(tag), trim=TRUE, commented=FALSE, options=FALSE, first=FALSE){
	
	function(x, verbose=TRUE){
		x <- rnwObject(x)
		# read all lines in
		l <- readLines(x$file)
		
		# identify driver
		dr <- str_match(l, str_c("^\\s*"
								, if( commented ) '%'
								,"\\s*\\\\", tag
								, if( options ) "(\\[[^]]*\\])?"
								, "\\{([^}]*)\\}"))
		w <- which(!is.na(dr[,1L]))
		if( length(w) > 0L ){
			if( first ) w <- w[1L]
			s <- dr[w, if( options ) 3L else 2L]
			# trim if necessary
			if( trim ) s <- str_trim(s)
			if( verbose ) rnw_message("Detected ", name, ": "
								,paste("'", s, "'", sep='', collapse=', '))
			s
		}
	}
}

rnwVignetteParser <- function(tag, ...){
	rnwParser(str_c('Vignette',tag), name=tolower(tag), ..., commented=TRUE, first=TRUE)
}

rnwLatexPackages <- rnwParser('usepackage', name='LaTeX package(s)', options=TRUE)

#' \code{rnwCompiler} tries to detect the vignette compiler to use on a vignette
#' source file, e.g., \code{\link{Sweave}} or \code{\link[knitr]{knitr}}.
#' 
#' @param verbose logical that toggles verbosity
#' 
#' @rdname vignette
#' @export
rnwCompiler <- rnwVignetteParser('Compiler')

#' \code{rnwWrapper} tries to detect the type of vignette and if it is meant 
#' to be wrapped into another main file.
#' 
#' @rdname vignette
#' @export
rnwWrapper <- rnwVignetteParser('Wrapper')

#' \code{rnwDriver} tries to detect Sweave driver to use on a vignette source 
#' file, e.g., \code{SweaveCache}, \code{highlight}, etc..
#' 
#' @rdname vignette
#' @export
rnwDriver <- function(x){
	
	parse_driver <- rnwVignetteParser('Driver', trim=FALSE)
	if( !is.null(s <- parse_driver(x)) ){
		# eval text
		eval(parse(text=s))
	}
	
}  

#' \code{rnwIncludes} detects fixed includes, e.g., image or pdf files, that are 
#' required to build the final document.  
#' 
#' @rdname vignette
#' @export
rnwIncludes <- function(x){
	
	x <- rnwObject(x)
	
	# read all lines in
	l <- readLines(x$file)
	
	# identify driver
	dr <- suppressWarnings(str_match(l, "^\\s*\\\\((include)|(includegraphics)|(input))\\{([^}]*)\\}"))
	w <- which(!is.na(dr[,1L]))
	rnw_message("Detected includes: ", appendLF=FALSE)
	if( length(w) > 0L ){
		inc <- str_trim(dr[w,6L])
		message(str_out(inc))
		inc
	}else
		message("NONE")
	
}

#' \code{rnwChildren} detects included vignette documents and return them as a 
#' list of vignette objects.  
#'  
#' @rdname vignette
#' @export
rnwChildren <- function(x){
	
	x <- rnwObject(x)
	
	# read all lines in
	l <- readLines(x$file)
	
	# identify driver
	dr <- str_match(l, "^\\s*\\\\SweaveInput\\{([^}]*)\\}")
	w <- which(!is.na(dr[,1L]))
	if( length(w) > 0L ){
		inc <- dr[w,2L]
		rnw_message("Detected children: ", str_out(inc, Inf))
		owd <- setwd(dirname(x$file))
		on.exit( setwd(owd) )
		mapply(as.rnw, inc, line=w, SIMPLIFY=FALSE)
	}
	
}  

#' Formatting Package Citations in Sweave/knitr Documents
#' 
#' @param x output document, as a single string.
#' @export
parsePackageCitation <- function(x){
    
    if( length(x) > 1L ) x <- paste(x, collapse = "\n")
    
    .parse <- function(x, pattern, idx){
		dr <- str_match_all(x, pattern)
		dr <- dr[sapply(dr, length)>0L]
		unlist(lapply(dr, '[', , idx))
	}
	
	# extract package citations: \citeCRANpkg - like
    x <- gsub(".*[^%]* *\\\\begin\\{document\\}(.*)", "\\1", x)
    cite <- .parse(x, "\\\\cite((CRAN)|(BioC)|(BioCAnn))?pkg[*]?\\{([^}]*)\\}", 6L)
    # \cite{Rpackage:pkgname, ...} - like
	cite2 <- .parse(x, "\\\\cite[^{ ]*\\{([^}]*)\\}", 2L)
    if( length(cite2) ){
 		cite2 <- .parse(cite2, '.*Rpackage:([^,}]+).*', 2L)
		cite <- c(cite, cite2)
	}
	# remove Rpackage prefix
	if( length(cite) ){
        cite <- unlist(strsplit(cite, ","))
        cite <- gsub('^Rpackage:', '', cite)
	}
	
    inc <- character()
	if( length(cite) > 0L ){
		inc <- unique(str_trim(unlist(strsplit(cite, ","))))
    }
	inc  
}

#' \code{bibcite} provides an inline package citation functionnality. 
#' Technically it adds a given Bibtex key to a cache that is used at the end of the 
#' document processing to generate a .bib file with all citation keys.  
#' 
#' @param key citation Bibtex key(s) as a character vector
#' @param cache specifies what to do with the previsouly chached keys.
#' If \code{TRUE}, then \code{key} is added to the cache. 
#' If \code{NULL}, then all previously cached keys are deleted, before .
#' If a character string, then it specifies the path to a Bibtex file that is loaded 
#' to initialise the cache.
#' @param ... extra arguments passed to \code{\link[bibtex]{read.bib}}.
#' @keywords internal
cite_pkg <- local({
    .keys <- character()
    function(key, cache = NA, ...){
        # return whole cache
        if( !nargs() ) return(.keys)
        # reset cache
        if( is.null(cache) ) .keys <- character()
        else if( isString(cache) ) .keys <- read.bib(file = cache, ...)
        if( !missing(key) ){
            cat(key)
            .keys <<- c(.keys, key)
        }
    }
})

rnwCite <- function(x){
	
	x <- rnwObject(x)
	
	# read all lines in
	l <- readLines(x$file)

	.parse <- function(x, pattern, idx){
		dr <- str_match_all(x, pattern)
		dr <- dr[sapply(dr, length)>0L]
		unlist(lapply(dr, '[', , idx))
	}
	
	# extract package citations: \citeCRANpkg - like
	cite <- .parse(l, "\\\\cite((CRAN)|(BioC)|(BioCAnn))?pkg[*]?\\{([^}]*)\\}", 6L)
	# \cite{Rpackage:pkgname, ...} - like
	cite2 <- .parse(l, "\\\\cite[^{ ]*\\{([^}]*)\\}", 2L)
	if( length(cite2) ){
 		cite2 <- .parse(cite2, '.*Rpackage:([^,}]+).*', 2L)
		cite <- c(cite, cite2)
	}
	# remove Rpackage prefix
	if( length(cite) ){
    cite <- unlist(strsplit(cite, ","))
	  cite <- gsub('^Rpackage:', '', cite)
	}
	
	rnw_message("Detected package citation(s): ", appendLF=FALSE)
	if( length(cite) > 0L ){
		inc <- unique(str_trim(unlist(strsplit(cite, ","))))
		message(str_out(inc), ' [', length(inc), ']')
		inc
	}else
		message("NONE")
}

# substitute a makefile template variable 
subMakeVar <- function(mvar, value, text){
	sub(str_c('#%', mvar, '%#'), value, text, fixed=TRUE)
}
# define a makefile template variable
defMakeVar <- function(var, value, ..., mvar=var){
	subMakeVar(mvar, str_c(var, '=', value), ...)
}

quick_install <- function(path, ..., lib.loc){
	
	if( !is.dir(lib.loc) ){
		stop("Installation directory '", lib.loc, "' does not exist.")
	}
	
	olib <- .libPaths()
	.libPaths(lib.loc)
	on.exit( .libPaths(olib) )
	pkg <- devtools::install(path, ...)
	
}


vignetteCheckMode <- checkMode_function('_R_CHECK_BUILDING_VIGNETTES_')

#' \code{vignetteMakefile} returns the path to a generic makefile used to make 
#' vignettes.
#' 
#' @param package package name.
#' If \code{NULL}, a DESRIPTION file is looked for one directory up: this 
#' meant to work when building a vignette directly from a package's 
#' \code{'vignettes'} sub-directory. 
#' @param skip Vignette files to skip (basename).  
#' @param print logical that specifies if the path should be printed or
#' only returned. 
#' @param template template Makefile to use.
#' The default is to use the file \dQuote{vignette.mk} shipped with the package
#' \pkg{pkgmaker} and can be found in its install root directory.
#' @param temp logical that indicates if the generated makefile should using a 
#' temporary filename (\code{TRUE}), or simply named \dQuote{vignette.mk}
#' @param checkMode logical that indicates if the vignettes should be generated as in a 
#' CRAN check (\code{TRUE}) or in development mode, in which case \code{pdflatex}, \code{bibtex}, 
#' and, optionally, \code{qpdf} are required.
#' @param user character vector containing usernames that enforce \code{checkMode=TRUE}, 
#' if the function is called from within their session.
#' @param tests logical that enables the compilation of a vignette that gathers all unit 
#' test results.
#' Note that this means that all unit tests are run before generating the vignette.
#' However, unit tests are not (re)-run at this stage when the vignettes are built 
#' when checking the package with \code{R CMD check}.
#' 
#' @rdname vignette
#' @export
vignetteMakefile <- function(package=NULL, skip=NULL, print=TRUE, template=NULL, temp=FALSE
                             , checkMode = isCHECK() || vignetteCheckMode()
                             , user = NULL, tests=TRUE){
	
#	library(methods)
	## create makefile from template
	# load template makefile
	if( is.null(template) )
		template <- packagePath('vignette.mk', package='pkgmaker')	
	l <- paste(readLines(template), collapse="\n")
  
	# R_BIN
	l <- subMakeVar('R_BIN', R.home('bin'), l)
  #
  
    if( checkMode ){
        oldCM <- vignetteCheckMode(TRUE)
        on.exit( vignetteCheckMode(oldCM) )
    }
  # Check user: LOCAL_MODE if in declared user
	localMode <- !checkMode
	cuser <- Sys.info()["user"]
	l <- subMakeVar('VIGNETTE_USER', cuser, l)
  maintainers <- '-'
  if( !is.null(user) ){
		maintainers <- str_c(user, collapse=', ')
		if( cuser %in% user ){
			localMode <- TRUE
		}
	}
	l <- subMakeVar('VIGNETTE_MAINTAINERS', maintainers, l)
	
  # define variable LOCAL_MODE
	if( localMode ){
	  l <- defMakeVar('LOCAL_MODE', cuser, l)
  }
  
	# Package name
    pkg_dir <-  dirname(getwd())
    loc_package <- if( is.file(df <- file.path(pkg_dir, 'DESCRIPTION')) ){
        d <- try(read.dcf(df), silent=TRUE)
        d <- as.list(as.data.frame(d, stringsAsFactors=FALSE))
        d$Package
    }
    if( !is.null(loc_package) && (is.null(package) || identical(loc_package, package)) ) package <- loc_package
    else if( !identical(loc_package, package) && length(pkg_dir <- find.package(package, quiet=TRUE)) ){
		d <- packageDescription(package)
	}else{
		stop("Could not load DESCRIPTION file for package '", package, "'.")		
	}
	l <- defMakeVar('MAKE_R_PACKAGE', package, l)
    l <- subMakeVar('R_PACKAGE_DESCRIPTION', pkg_dir, l)
  # R_LIBS: add package's dev lib if necessary 
  Rlibs <- NULL
  if( localMode && is.dir(devlib <- file.path(getwd(), '..', '..', 'lib')) ){
	  Rlibs <- devlib
  }
  Rlibs <- paste(c(Rlibs, "$(TMP_INSTALL_DIR)", "$(R_LIBS)"), collapse=.Platform$path.sep)
	l <- subMakeVar('R_LIBS_DEV', Rlibs, l)
	# TMP_INSTALL_DIR: temporary install directory
	l <- subMakeVar('TMP_INSTALL_DIR', file.path(dirname(tempdir()), basename(tempfile('Rpkglib_'))), l)
	
	# Vignettes files:
    # - look into src/ for real vignettes
	# - check presence of a test directory ../tests/
	# - check current directory for non fake vignettes
	rnwFiles <- NULL
	# src
	if( is.dir('src') ) rnwFiles <- list.files('src', pattern="\\.Rnw$")
	# unit tests
	if( tests && is.dir('../tests') ) rnwFiles <- c(rnwFiles, str_c(package, '-unitTests.Rnw'))
	# non-fake vignettes
    rnwFiles <- c(rnwFiles, list.files('.', pattern="\\.Rnw$"))
	# substitute in makefile
	rnwFiles <- unique(rnwFiles)
	if( !is.null(skip) )
		rnwFiles <- setdiff(rnwFiles, skip)
	l <- subMakeVar('RNW_SRCS', paste(rnwFiles, collapse=' '), l)
	# reset pdf objects in local mode to point to ../inst/doc
	noBuildVignettes <- if( !is.null(d$BuildVignettes) ) tolower(d$BuildVignettes)=='no' else FALSE
	if( localMode && noBuildVignettes ){
        l <- defMakeVar('INST_TARGET', 1, l)
    	l <- defMakeVar('PDF_OBJS'
						, paste(file.path('../inst/doc', sub("\\.Rnw$", ".pdf", rnwFiles)), collapse=' ')
						, l)
	}
    l <- defMakeVar('PDF_OBJS'
            , paste(file.path('../inst/doc', sub("\\.Rnw$", ".pdf", rnwFiles)), collapse=' ')
            , l)
    
	# create makefile
	mk <- if( temp ) tempfile('vignette_', tmpdir='.', fileext='.mk') else 'vignette.mk'
	cat(l, file=mk)
	if ( print ){
		cat(mk)
	}
	invisible(l)
}

#' Compact PDF at Best
#' 
#' Compact PDFs using either \code{gs_quality='none'} or \code{'ebook'}, 
#' depending on which compacts best (as per CRAN check criteria).
#' 
#' @inheritParams tools::compactPDF
#' 
#' @rdname vignette
#' @export
compactVignettes <- function(paths, ...){
	
	td <- tempfile(basename(paths))
	file.copy(paths, td)
	res <- tools::compactPDF(td, gs_quality = "none", ...) # use qpdf
	diff_none <- format(res, diff = 1e5)
	res <- tools::compactPDF(td, gs_quality = "ebook", ...)
	diff_ebook <- format(res, diff = 2.5e5) # 250 KB for now
	
	if( length(diff_ebook) ){
		tools::compactPDF(paths, gs_quality = "ebook", ...)
		invisible('ebook')
	}else{
		tools::compactPDF(paths, gs_quality = "none", ...)
		invisible('none')
	}
	
}

# Project: pkgmaker
# 
# Author: renaud gaujoux
# Created: Oct 23, 2013
###############################################################################


#' Knitr Extensions
#' 
#' \code{knit_ex} is a utility function for running small knitr examples, 
#' e.g., to illustrate functionalities or issues. 
#' 
#' @param x text to knit as a character vector
#' @param ... arguments passed to \code{\link[knitr]{knit2html}} or \code{\link[knitr]{knit}}
#' @param quiet logical that indicates if knitting should be quiet (no progress bars etc..).
#' @param open logical, only used when \code{x} is in .Rmd format, that indicates 
#' if the generated document result should be open in a browse, instead of 
#' being printed on screen.
#' Not that a browser will not open in non-interactive sessions, and the result will
#' be returned invisibly.
#' 
#' @return 
#' \code{knit_ex} returns the generated code, although invisibly when \code{open=TRUE}.
#' 
#' @export
#' @examples
#' 
#' library(knitr)
#' knit_ex("1 + 1")
#' 
knit_ex <- function(x, ..., quiet = TRUE, open = FALSE){
    
    library(knitr)
    # substitute special markup for Rmd markup (necessary for knit_ex examples)
    x <- gsub("^^^", "```", x, fixed = TRUE)
    
    if( !(html_chunks <- any(grepl("```{", x, fixed = TRUE))) ){
        if( all(!grepl(">>=", x, fixed = TRUE)) ){
            x <- c("```{r}", x, "```")
            html_chunks <- TRUE   
        }
    }
    x <- paste0(x, collapse = "\n")
    if( any(html_chunks) ){
        res <- knit2html(text = x, ..., fragment.only = TRUE, quiet = quiet)
        if( open ){
            tmp <- tempfile("knit_ex", fileext = '.html')
            cat(res, file = tmp, sep = "\n") 
            if( interactive() ) browseURL(tmp)
            return(invisible(res))
        }
    }else{
        res <- knit(text = x, ..., quiet = quiet)
    }
    cat(res)
}

try_message <- function(signal = FALSE){
    function(expr){
        tryCatch(expr, error = function(e){
                if( signal ) message(e)
                else message('Error: ', conditionMessage(e))
                invisible()
            })
    }
}

#' \code{hook_try} is a knitr hook to enable showing error 
#' messages thrown by \code{\link{try}}.
#' The function is not meant to be called directly, but only registered 
#' using \code{\link{knit_hooks}} (see details on this dedicated man page).
#' 
#' \code{hook_try} simply defines a function \code{try} in \code{envir} that prints 
#' the error message if any, and is called instead of base \code{\link{try}}. 
#' 
#' @param before logical that indicates when the hook is being called: 
#' before or after the chunk is processed.
#' @param options list of current knitr chunk options 
#' @param envir environment where the chunk is evaluated
#' 
#' @rdname knit_ex
#' @export
#' @examples
#' 
#' library(knitr)
#' 
#' # standard error message is caught
#' knit_ex("stop('ah ah')")
#' 
#' # with try the error is output on stderr but not caughted by knitr
#' knit_ex("try( stop('ah ah') )")
#' 
#' # no message caught
#' knit_ex("
#' ^^^{r, include = FALSE}
#' knit_hooks$set(try = pkgmaker::hook_try)
#' ^^^
#' 
#' ^^^{r, try=TRUE}
#' try( stop('ah ah') )
#' ^^^")
#' 
hook_try <- local({
    .try_defined <- FALSE
    function(before, options, envir){
    
        # remove hacked version of try
        if( !before ){
            if( .try_defined && exists('try', envir = envir, inherits = FALSE) ){
                remove(list = 'try', envir = envir)
            }
            .try_defined <<- FALSE
            return(invisible())
        }
        
        if( !is.null(options$try) ){
            
            # signal
            do.signal <- isFALSE(options$try)
            if( isManualVignette() && isTRUE(options$try) ){
                do.signal <- TRUE
            }
            # define hacked version of try()
            .try <- try_message(do.signal)
            assign('try', .try, envir)
            .try_defined <<- TRUE
        }
    }
})


chunkOutputHook <- function(name, hook, type = c('output', 'source', 'chunk')){
    type <- match.arg(type)
    function(){
        .hook_bkp <- NULL
        function(before, options, envir){
            # do nothing if the option is not ON
            if( is.null(options[[name]]) ) return()
            
            # set/unset hook
            if( before ){
                # store current hook function
                if( is.null(.hook_bkp) ) .hook_bkp <<- knit_hooks$get(type)
                
                # define hook wrapper
                hook_wrapper <- function(x, options){
                    res <- .hook_bkp(x, options)
                    hook(res, options)
                }
                        
                args <- list()
                args[[type]] <- hook_wrapper
                do.call(knit_hooks$set, args)
            }else{
                args <- list()
                args[[type]] <- .hook_bkp
                do.call(knit_hooks$set, args)
                .hook_bkp <<- NULL
            }
        }
    }
}

#' Knitr Hook for Handling Backspace Character
#' 
#' \code{hook_backspace} is a chunk hook that enables the use of backspace
#' characters in the output (e.g., as used in progress bars), and still 
#' obtain a final output as in the console.
#' 
#' @rdname knit_ex
#' @export 
#' @examples 
#' 
#' # Correctly formatting backspaces in chunk outputs
#' tmp <- tempfile(fileext = '.Rmd')
#' cat(file = tmp, "
#' ^^^{r, include = FALSE}
#' library(knitr)
#' knit_hooks$set(backspace = pkgmaker::hook_backspace())
#' ^^^
#' Default knitr does not handle backspace and adds a special character:
#' ^^^{r}
#' cat('abc\bd')
#' ^^^
#' 
#' Using the hook backspace solves the issue:
#' ^^^{r, backspace=TRUE}
#' cat('abc\bd')
#' ^^^
#' ")
#' 
#' # knit
#' out <- knitr::knit2html(tmp, fragment.only = TRUE)
#' # look at output
#' \dontrun{
#'   browseURL(out)
#'   edit( file = out)
#' }
#' # cleanup
#' unlink(c(tmp, out))
#' 
#' 
hook_backspace <- chunkOutputHook('backspace', 
        function(x, options){
            if( !isTRUE(options$backspace) ) x
            str_bs(x)
        }
)

#' \code{str_bs} substitutes backspace characters (\\b) to produce
#' a character string as it would be displayed in the console.
#'
#' @author
#' Renaud Gaujoux
#'  
#' \code{str_bs} was adapted from a proposal from Yihui Xie.
#'  
#' @rdname str_out 
#' @export
#' @examples 
#' 
#' # Backspace substitution
#' str_bs("abc")
#' str_bs("abc\b")
#' str_bs("abc\b\b")
#' str_bs("abc\bd")
#' str_bs("abc\b\bde\b")
#' 
#' # more complex example
#' x <- "\bab\nc\bd\n\babc\b\bd"
#' cat(x, "\n")
#' y <- str_bs(x)
#' y
#' cat(y, "\n")
#' 
str_bs <- function(x){
    # remove leading backspaces
    x <- gsub("^\b+", "", x)
    # remove backspaces at beginning of line
    x <- gsub("\n\b+", '\n', x)
    while( length(grep('\b', x, fixed = TRUE)) ) 
        x <- gsub('[^\n\b][\b]', '', x)
    
    x
}

md_toggleCode <- function(){
    cat(
"
<script src=\"http://ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js\"></script>
<script type=\"text/javascript\">
// toggle visibility of R source blocks in R Markdown output
function toggle_vis(o) {
    var d = o.style.display;
    o.style.display = (d == 'block' || d == '') ? 'none':'block';
  }
function toggle_R(what) {
  var x = document.getElementsByClassName('r');
  if (x.length == 0) return;

  for (i = 0; i < x.length; i++) {
    var y = x[i];
    switch (y.tagName.toLowerCase()) {
      case 'pre':
        toggle_vis(y);
        if( what == 'setup' ){
            y.id = \"Rcode_\" + i;
        }
        break;
      case 'code':
        var z = y.parentNode;
        // pandoc uses the class 'sourceCode r' on both pre and code
        if (z.tagName.toLowerCase() == 'pre' && z.className != 'sourceCode r') {
          toggle_vis(z);
          if( what == 'setup' ){
              z.id = \"Rcode_\" + i;
              var newContent = $(\"<a href=\\\"\\\" onclick=\\\"toggle_vis(document.getElementById('\" + z.id + \"')); return false;\\\">Show/Hide R code</a>\");
              newContent.insertBefore(z);
          }
        }
        break;
    }
  }
}

$( document ).ready(function(){
    toggle_R('setup');
});
</script>"
    )
}


.js_include_jquery <- "<script src=\"http://ajax.googleapis.com/ajax/libs/jquery/1.9.1/jquery.min.js\"></script>\n"
.js_toggle_fun <- "<script type=\"text/javascript\">
function toggle_vis2(cl) {
	elt = document.getElementsByClassName(cl)
	for (i = 0; i < elt.length; i++) {
		var o = elt[i].parentNode;
        var d = o.style.display;
        o.style.display = (d == 'block' || d == '') ? 'none':'block';
	}
  }

window.onload = function(){
	toggle_vis2('Chunk_none')	
}
</script>
"

.js_def_toggle_code <- "<script type=\"text/javascript\">
// toggle visibility of R source blocks in R Markdown output
function toggle_R(what) {
  var x = document.getElementsByClassName('r');
  if (x.length == 0) return;
  function toggle_vis(o) {
    var d = o.style.display;
    o.style.display = (d == 'block' || d == '') ? 'none':'block';
  }

  for (i = 0; i < x.length; i++) {
    var y = x[i];
    switch (y.tagName.toLowerCase()) {
      case 'pre':
        toggle_vis(y);
        if( what == 'setup' ){
        y.id = \"Rcode_\" + i;
        }
        break;
      case 'code':
        var z = y.parentNode;
        // pandoc uses the class 'sourceCode r' on both pre and code
        if (z.tagName.toLowerCase() == 'pre' && z.className != 'sourceCode r') {
            toggle_vis(z);
            if( what == 'setup' ){
                z.id = \"Rcode_\" + i;
                var newContent = $(\"<a href=\\\"\\\" onclick=\\\"$('#\" + z.id + \"').toggle(); return false;\\\">Show/Hide R code</a>\");
                newContent.insertBefore(z);
            }
    	}
    	break;
    }
  }
}

$( document ).ready(function(){
    toggle_R('setup');
});
</script>"

#' \code{hook_toggle} is a chunk hook that adds clickable elements to toggle \emph{indvidual}
#' code chunks in HTML documents generated from .Rmd files.
#' 
#' @rdname knit_ex
#' @export
#' @examples
#' 
#' knit_ex("
#' 
#' Declare chunk hook:
#' ^^^{r, setup}
#' library(knitr)
#' knit_hooks$set(toggle = hook_toggle())
#' ^^^
#' 
#' The R code of this chunk can be toggled on/off, and starts visible:
#' ^^^{r, toggle=TRUE}
#' print(1:10)
#' ^^^
#' The R code of this chunk can be toggled on/off, and starts hidden:
#' ^^^{r, toggle=FALSE}
#' print(1:10)
#' ^^^
#' 
#' This is a plain chunk that cannot be toggled on/off:
#' ^^^{r}
#' print(1:10)
#' ^^^
#' 
#' Now all chunks can be toggled and start visible:
#' ^^^{r, toggle_all}
#' opts_chunk$set(toggle = TRUE)
#' ^^^
#' 
#' ^^^{r}
#' sample(5)
#' ^^^
#' 
#' To diable the toggle link, one can pass anything except TRUE/FALSE:
#' ^^^{r, toggle = NA}
#' sample(5)
#' ^^^
#' 
#' ", open = TRUE)
#' 
hook_toggle <- function(){
    .init <- TRUE
    .last_label <- NULL
    
    fn <- chunkOutputHook('toggle', type = 'source', function(x, options){
        opt <- options$toggle
        label <- options$label
        if( !isTRUE(opt) && !isFALSE(opt) ) return(x)
#        print(x)
#        x <- gsub("^\n", '', x)
        # add javascript define for toggle function
        if( .init ){
            x <- paste0(.js_toggle_fun, x)
            .init <<- FALSE
        }
        
        disp <- if( opt ) 'Chunk_block'
                else 'Chunk_none'
        id <- paste0("Rcode_", label)
        subst <- paste0("```{", id, " \\1 ", disp, "}\n")
        if( !identical(label, .last_label) ){
            .last_label <<- label
            subst <- paste0("<a href=\"\" onclick=\"toggle_vis2('", id, "'); return false;\">Show/Hide R code</a>\n", subst)
        }
        sub("```([^\n]*)\n", sprintf(subst, 'block'), x)
    })
    fn()
}

# Copied from devtools to make these functions independent of devtools
# Function names were simply changed to using_* to avoid conflict
#
# Copyright Hadley Wickam 2015

#' Execute code in temporarily altered environment.
#' 
#' These functions were extracted from the \pkg{devtools} package 
#' to make them available without a dependency to \pkg{devtools}.
#'
#' \itemize{
#'   \item \code{using_dir}: working directory
#'   \item \code{using_collate}: collation order
#'   \item \code{using_envvar}: environmental variables
#'   \item \code{using_libpaths}: library paths, replacing current libpaths
#'   \item \code{using_lib}: library paths, prepending to current libpaths
#'   \item \code{using_locale}: any locale setting
#'   \item \code{using_options}: options
#'   \item \code{using_path}: PATH environment variable
#'   \item \code{using_par}: graphics parameters
#' }
#' @section Deprecation:
#' \code{using_env} will be deprecated in devtools 1.2 and removed in
#' devtools 1.3
#'
#' @param new values for setting
#' @param code code to execute in that environment
#'
#' @author Hadley Wickham
#' @name using_something
#' @examples
#' getwd()
#' using_dir(tempdir(), getwd())
#' getwd()
#'
#' Sys.getenv("HADLEY")
#' using_envvar(c("HADLEY" = 2), Sys.getenv("HADLEY"))
#' Sys.getenv("HADLEY")
#'
#' using_envvar(c("A" = 1),
#'   using_envvar(c("A" = 2), action = "suffix", Sys.getenv("A"))
#' )
NULL

using_something <- function(set) {
  function(new, code) {
    old <- set(new)
    on.exit(set(old))
    force(code)
  }
}
is.named <- function(x) {
  !is.null(names(x)) && all(names(x) != "")
}

# env ------------------------------------------------------------------------

set_envvar <- function(envs, action = "replace") {
  if (length(envs) == 0) return()

  stopifnot(is.named(envs))
  stopifnot(is.character(action), length(action) == 1)
  action <- match.arg(action, c("replace", "prefix", "suffix"))

  old <- Sys.getenv(names(envs), names = TRUE, unset = NA)
  set <- !is.na(envs)

  both_set <- set & !is.na(old)
  if (any(both_set)) {
    if (action == "prefix") {
      envs[both_set] <- paste(envs[both_set], old[both_set])
    } else if (action == "suffix") {
      envs[both_set] <- paste(old[both_set], envs[both_set])
    }
  }

  if (any(set))  do.call("Sys.setenv", as.list(envs[set]))
  if (any(!set)) Sys.unsetenv(names(envs)[!set])

  invisible(old)
}
#' @rdname using_something
#' @param action (for \code{using_envvar} only): should new values
#'    \code{"replace"}, \code{"suffix"}, \code{"prefix"} existing environmental
#'    variables with the same name.
#' @export
using_envvar <- function(new, code, action = "replace") {
  old <- set_envvar(new, action)
  on.exit(set_envvar(old, "replace"))
  force(code)
}

#' @rdname using_something
#' @export
using_env <- function(new, code) {
  message(
    "using_env() will be deprecated in devtools 1.3.\n",
    "Please use using_envvar() instead")
  using_envvar(new, code)
}

# locale ---------------------------------------------------------------------

set_locale <- function(cats) {
  stopifnot(is.named(cats), is.character(cats))

  old <- vapply(names(cats), Sys.getlocale, character(1))

  mapply(Sys.setlocale, names(cats), cats)
  invisible(old)
}

#' @rdname using_something
#' @export
using_locale <- using_something(set_locale)

# collate --------------------------------------------------------------------

set_collate <- function(locale) set_locale(c(LC_COLLATE = locale))[[1]]
#' @rdname using_something
#' @export
using_collate <- using_something(set_collate)

# working directory ----------------------------------------------------------

#' @rdname using_something
#' @export
using_dir <- using_something(setwd)

# libpaths -------------------------------------------------------------------

set_libpaths <- function(paths) {
  libpath <- normalizePath(paths, mustWork = TRUE)

  old <- .libPaths()
  .libPaths(paths)
  invisible(old)
}

#' @rdname using_something
#' @export
using_libpaths <- using_something(set_libpaths)

# lib ------------------------------------------------------------------------

set_lib <- function(paths) {
  libpath <- normalizePath(paths %||% .libPaths()[1L], mustWork = TRUE)

  old <- .libPaths()
  .libPaths(c(libpath, .libPaths()))
  invisible(old)
}

#' @rdname using_something
#' @export
using_lib <- using_something(set_lib)

# options --------------------------------------------------------------------

set_options <- function(new_options) {
  do.call(options, as.list(new_options))
}

#' @rdname using_something
#' @export
using_options <- using_something(set_options)

# par ------------------------------------------------------------------------

#' @rdname using_something
#' @export
using_par <- using_something(par)

# path -----------------------------------------------------------------------

#' @rdname using_something
#' @export
#' @param add Combine with existing values? Currently for
#'   \code{\link{using_path}} only. If \code{FALSE} all existing
#'   paths are overwritten, which you don't usually want.
#' @param prepend logical that indicates if the new paths should
#' be added in front of the current ones.
using_path <- function(new, code, add = TRUE, prepend = FALSE) {
  if (add){
      if( prepend ) new <- c(new, get_path())
      else new <- c(get_path(), new)
  }
  old <- set_path(new)
  on.exit(set_path(old))
  force(code)
}
    
# define local versions of devtools::set/get_path
get_path <- function(){
    strsplit(Sys.getenv("PATH"), .Platform$path.sep)[[1]]
}

set_path <- function (path){
    path <- normalizePath(path, mustWork = FALSE)
    old <- get_path()
    path <- paste(path, collapse = .Platform$path.sep)
    Sys.setenv(PATH = path)
    invisible(old)
}

# Package related functions
# 
# Author: Renaud Gaujoux
# Creation: 29 Jun 2012
###############################################################################

path.protect <- function(...){
  f <- file.path(...)
  if( .Platform$OS.type == 'windows' ){
    f <- gsub("\\\\", "/", f)
  }
  paste('"', f, '"', sep='')
}

#' Quick Installation of a Source Package
#' 
#' Builds and install a minimal version of a package from its 
#' source directory.
#' 
#' @param path path to the package source directory
#' @param destdir installation directory. 
#' If \code{NULL}, the package is installed in the default installation library.
#' If \code{NA}, the package is installed in a temporary directory, whose path is returned
#' as a value.
#' @param vignettes logical that indicates if the vignettes should be 
#' rebuilt and installed.
#' @param force logical that indicates if the package should be installed even if a previous
#' installation exists in the installation library.
#' @param ... extra arguments passed to \code{\link{R.CMD}}
#' @param lib.loc library specification.
#' If \code{TRUE} then the installation directory \code{destdir} is added to the default 
#' library paths.
#' This can be usefull if dependencies are installed in this directory.
#' If \code{NULL}, then the default library path is left unchanged.
#' 
#' @return The path of the library where the package was installed.
#' 
#' @export
#' 
quickinstall <- function(path, destdir=NULL, vignettes=FALSE, force=TRUE, ..., lib.loc=if(!is.null(destdir)) TRUE){
	
	npath <- normalizePath(path)
	pkg <- as.package(path)
  
  # define installation library
	nlib <- if( !is.null(destdir) ) destdir
  else if( is_NA(destdir) ) tempfile("pkglib_")
  
  # normalize path
  if( !is.null(nlib) ){
    # create direcory if needed
    if( !is.dir(nlib) ) dir.create(nlib, recursive=TRUE)
    nlib <- normalizePath(nlib)
    
    if( !is.dir(nlib) ){
      stop("Could not install package '", pkg$package, "': installation directory '", nlib, "' does not exist.")
    }
    
    # add destination directory to default libraries
    if( isTRUE(lib.loc) ) lib.loc <- unique(c(nlib, .libPaths()))
  }
  
  # setup result string
	res <- invisible(if( !is.null(destdir) ) nlib else .libPaths()[1L])
  
  # early exit if the package already exists in the library (and not forcing install)
	message("# Check for previous package installation ... ", appendLF=FALSE)
  if( !is.null(destdir) && is.dir(file.path(nlib, pkg$package)) ){
    if( !force ){
      message("YES (skip)")
      return(res)
    }
    message("YES (replace)")
  }else message("NO")
  
	# add lib path
	ol <- set_libPaths(lib.loc)
	on.exit(set_libPaths(ol), add=TRUE)
	message("Using R Libraries: ", str_out(.libPaths(), Inf))
	
	owd <- setwd(tempdir())
	on.exit( setwd(owd), add=TRUE)
  
	# build
	message("# Building package `", pkg$package, "` in '", getwd(), "'")
	opts <- '--no-manual --no-resave-data '
	if( !vignettes ){
        vflag <- if( testRversion('>= 3.0') ) '--no-build-vignettes ' else '--no-vignettes ' 
        opts <- str_c(opts, vflag)
    }
	R.CMD('build', opts, path.protect(npath), ...)
	spkg <- paste(pkg$package, '_', pkg$version, '.tar.gz', sep='')
	if( !file.exists(spkg) ) stop('Error in building package `', pkg$package,'`')
	# install
	message("# Installing package `", pkg$package, "`"
          , if( !is.null(destdir) ){
            tmp <- if( is_NA(destdir) ) 'temporary '
            str_c("in ", tmp, "'", nlib, "'")
          })
  opts_inst <- ' --no-multiarch --no-demo --with-keep.source '
	if( !vignettes ) opts_inst <- str_c(opts_inst, '--no-docs ')
	R.CMD('INSTALL', if( !is.null(destdir) ) paste('-l', path.protect(nlib)), opts_inst, path.protect(spkg), ...)
  
  # return installation library
	invisible(res)
}

#' Require a Package
#' 
#' Require a package with a custom error message
#' 
#' @param pkg package name as a character string
#' @param ... extra arguments concatenated to for the header of the 
#' error message 
#' 
#' @export
requirePackage <- function(pkg, ...){
	
	if( !require(pkg, character.only=TRUE) ){
		if( nargs() > 1L ) stop(..., " requires package(s) ", str_out(pkg))
		else stop("Could not find required package(s) ", str_out(pkg))
	}
}

parse_deps <- function (string) 
{
	if (is.null(string)) 
		return()
	string <- gsub("\\s*\\(.*?\\)", "", string)
	pieces <- strsplit(string, ",")[[1]]
	pieces <- gsub("^\\s+|\\s+$", "", pieces)
	pieces[pieces != "R"]
}

#' List Package Dependencies
#' 
#' @param x path to package source directory or file.
#' @param all logical that indicates if all dependencies should be returned,
#' or only the required ones.
#' @param as.list logical that indicates if the result should be a list with one element
#' per type of dependency.
#' @param available a matrix of available packages (as returned by \code{\link{available.packages}}), 
#' from which the dependencies are retrieved.
#' This means that there must be a row for the package \code{x}.
#'  
#' @export
#' 
packageDependencies <- function(x, all = TRUE, as.list = FALSE, available = NULL){
    
    if( is.null(available) ) x <- as.package(x, extract = TRUE)
    else{
        p <- available[, 'Package']
        if( !x %in% p ) return(NA)
        x <- available[p == x, , drop = FALSE][1L, ]
        names(x) <- tolower(names(x))
    }
    
	d <- lapply(x[c('depends', 'imports', 'linkingto', 'suggests')], parse_deps)
	d <- unlist(d)
    d <- d[!is.na(d)]
    if( !length(d) ) return()
    names(d) <- gsub("[0-9]+$", "", names(d))
    
    # remove non-required
    if( !all ) d <- d[!names(d) %in% c('suggests')]    
    if( as.list ) d <- split(unname(d), names(d))
    d
}

.biocLite <- function(...){
	# install BiocInstaller if necessary
	if( !require.quiet('BiocInstaller') ){
		message("Installing biocLite")
		source('http://www.bioconductor.org/biocLite.R')
	}
	f <- get('biocLite', 'package:BiocInstaller')
	f(...)
}

#' Installing All Package Dependencies
#' 
#' Install all dependencies from a package source directory or 
#' package source file. 
#' 
#' @param pkg package path or source file
#' @param all logical that indicates if 'Suggests' packages
#' should be installed.
#' @param ... extra arguments passed to \code{\link{install.packages}}.
#' @param dryrun logical that indicates if the packages should be 
#' effectively installed or only shown. 
#' 
#' @export
#' @examples 
#' 
#' try( install.dependencies('Matrix', dryrun=TRUE) )
#' \dontrun{
#' install.dependencies("mypackage_1.0.tar.gz", dryrun=TRUE)
#' }
#' 
install.dependencies <- function (pkg = NULL, all=FALSE, ..., dryrun=FALSE) 
{
	pkg <- as.package(pkg, extract=TRUE)
	deps <- c(parse_deps(pkg$depends)
			, parse_deps(pkg$imports) 
			, parse_deps(pkg$linkingto)
			, if( isTRUE(all) ) parse_deps(pkg$suggests) )
	not.installed <- function(x) length(find.package(x, quiet = TRUE)) == 0
	message("Package dependencies for ", pkg$package, ": ", str_out(deps, Inf))
	deps <- Filter(not.installed, deps)
	if (length(deps) == 0){
		message("Missing: none")
		return(invisible())
	}
	message("Missing: ", str_out(deps, Inf))
	message("Installing ", length(deps), " dependencies for ", pkg$package)
	if( !dryrun ){
		.biocLite(deps, ...)
	}
	invisible(deps)
}

#' Setting Mirrors and Repositories
#' 
#' \code{setBiocMirror} sets all Bioconductor repositories (software, data, 
#' annotation, etc.).
#' so that they are directly available to \code{\link{install.packages}}.
#' It differs from \code{\link{chooseBioCmirror}} in that it effectively enables 
#' the repositories.
#' 
#' @param url or Bioconductor mirror url
#' @param version version number
#' @param unique logical that indicate if duplicated urls or names should be 
#' removed.
#'
#' @rdname mirrors
#' @export 
setBiocMirror <- function(url='http://www.bioconductor.org', version=NULL, unique=TRUE){
	
    #get all bioconductor repos      
    biocRepos <- getBiocRepos(url, version)
	
	repos <- c(biocRepos, getOption('repos'))
	if( unique ){
		nam <- names(repos)
		repos <- repos[!duplicated(repos) & (!duplicated(nam) | nam=='')]
	}
    options(repos=repos)
}

#' \code{getBiocMirror} is a shortcut for \code{getOption('BioC_mirror')}, which 
#' returns the current Bioconductor mirror as used by \code{biocLite}.
#'  
#' @export
#' @rdname mirrors
getBiocMirror <- function(){
	getOption('BioC_mirror')
}
#' \code{getBiocRepos} returns urls to all Bioconductor repositories on a 
#' given mirror.
#' 
#' @export
#' @rdname mirrors
getBiocRepos <- function(url='http://www.bioconductor.org', version=NULL){
	
	if( is.null(url) ){
		url <- getBiocMirror()
		if( is.null(url) )
			stop("No Bioconductor mirror was setup. Use `setBiocMirror`.")
	}
	
	## BioConductor CRAN-style repositories.
	## The software repo (bioc) _must_ be the first element.
	biocParts <- c(
			bioc='bioc'
			, biocData='data/annotation'
			, biocExp='data/experiment'
			, biocExtra='extra'
    )
	
	# define version suffix for bioconductor repo
	if( is.null(version) ){
		assoc <- list(`2`=c(7L, 2L))
		Rv <- as.integer(sub("([0-9]+).*", "\\1", R.version$minor))
		offset <- assoc[[R.version$major]]
	    version <- paste(R.version$major, offset[2L] + Rv - offset[1L], sep='.')
	}
	
	#add version suffix for bioconductor repo
    setNames(paste(url, 'packages', version, biocParts, sep='/'), names(biocParts))
}

#' \code{setCRANMirror} sets the preferred CRAN mirror.
#' 
#' @rdname mirrors
#' @export
setCRANMirror <- function(url=CRAN, unique=TRUE){
	
	repos <- c(CRAN=url, getOption('repos'))
	if( unique ){
		nam <- names(repos)
		repos <- repos[!duplicated(repos) & (!duplicated(nam) | nam=='')]
	}
    options(repos=repos)
}

#' \code{CRAN} simply contains the url of CRAN main mirror 
#' (\url{http://cran.r-project.org}), and aims at simplifying its use, e.g., in 
#' calls to \code{\link{install.packages}}.
#' 
#' @rdname mirrors
#' @export
#' 
#' @examples
#' \dontrun{
#' install.packages('pkgmaker', repos=CRAN)
#' }
CRAN <- 'http://cran.r-project.org'


#' Adding Package Libraries
#' 
#' Prepend/append paths to the library path list, using \code{\link{.libPaths}}.
#' 
#' This function is meant to be more convenient than \code{.libPaths}, which requires 
#' more writing if one wants to:
#' \itemize{
#' \item sequentially add libraries;
#' \item append and not prepend new path(s);
#' \item keep the standard user library in the search path.
#' }
#' 
#' @param ... paths to add to .libPath
#' @param append logical that indicates that the paths should be appended
#' rather than prepended.
#' 
#' @export
#' 
#' @examples
#' ol <- .libPaths()
#' # called sequentially, .libPaths only add the last library
#' show( .libPaths('.') )
#' show( .libPaths(tempdir()) )
#' # restore
#' .libPaths(ol)
#' 
#' # .libPaths does not keep the standard user library
#' show( .libPaths() ) 
#' show( .libPaths('.') )
#' # restore
#' .libPaths(ol)
#' 
#' # with add_lib
#' show( add_lib('.') )
#' show( add_lib(tempdir()) )
#' show( add_lib('..', append=TRUE) )
#' 
#' # restore 
#' .libPaths(ol)
#' 
add_lib <- function(..., append=FALSE){
	
	p <- 
	if( append ) c(.libPaths(), ...)
	else c(..., .libPaths())
	.libPaths(p)
}


#' Package Check Utils
#' 
#' \code{isCRANcheck} \strong{tries} to identify if one is running CRAN-like checks.
#' 
#' Currently \code{isCRANcheck} returns \code{TRUE} if the check is run with 
#' either environment variable \code{_R_CHECK_TIMINGS_} (as set by flag \code{'--timings'})
#' or \code{_R_CHECK_CRAN_INCOMINGS_} (as set by flag \code{'--as-cran'}).
#' 
#' \strong{Warning:} the checks performed on CRAN check machines are on purpose not always 
#' run with such flags, so that users cannot effectively "trick" the checks.
#' As a result, there is no guarantee this function effectively identifies such checks.
#' If really needed for honest reasons, CRAN recommends users rely on custom dedicated environment 
#' variables to enable specific tests or examples.
#' 
#' @param ... each argument specifies a set of tests to do using an AND operator.
#' The final result tests if any of the test set is true.
#' Possible values are:
#' \describe{
#' \item{\code{'timing'}}{Check if the environment variable \code{_R_CHECK_TIMINGS_} is set, 
#' as with the flag \code{'--timing'} was set.}
#' \item{\code{'cran'}}{Check if the environment variable \code{_R_CHECK_CRAN_INCOMING_} is set, 
#' as with the flag \code{'--as-cran'} was set.}
#' }
#' 
#' @references Adapted from the function \code{CRAN}
#' in the \pkg{fda} package.
#' 
#' @export
isCRANcheck <- function(...){
  
  tests <- list(...)
  if( !length(tests) ){ #default tests
	  tests <- list('timing', 'cran')
  }
  test_sets <- c(timing="_R_CHECK_TIMINGS_", cran='_R_CHECK_CRAN_INCOMING_')
  tests <- sapply(tests, function(x){
			  # convert named tests
			  if( length(i <- which(x %in% names(test_sets))) ){
				  y <- test_sets[x[i]]
				  x <- x[-i]
				  x <- c(x, y)
			  }
			  # get environment variables
			  evar <- unlist(sapply(x, Sys.getenv))
			  all(nchar(as.character(evar)) > 0)
		  })
  
  any(tests)
}
#' \code{isCRAN_timing} tells if one is running CRAN check with flag \code{'--timing'}.
#' 
#' @export
#' @rdname isCRANcheck
isCRAN_timing <- function() isCRANcheck('timing')

#' \code{isCHECK} tries harder to test if running under \code{R CMD check}.
#' It will definitely identifies check runs for: 
#' \itemize{
#' \item unit tests that use the unified unit test framework defined by \pkg{pkgmaker} (see \code{\link{utest}});
#' \item examples that are run with option \code{R_CHECK_RUNNING_EXAMPLES_ = TRUE}, 
#' which is automatically set for man pages generated with a fork of \pkg{roxygen2} (see \emph{References}).
#' }
#' 
#' Currently, \code{isCHECK} checks both CRAN expected flags, the value of environment variable
#' \code{_R_CHECK_RUNNING_UTESTS_}, and the value of option \code{R_CHECK_RUNNING_EXAMPLES_}.
#' It will return \code{TRUE} if any of these environment variables is set to 
#' anything not equivalent to \code{FALSE}, or if the option is \code{TRUE}.
#' For example, the function \code{\link{utest}} sets it to the name of the package  
#' being checked (\code{_R_CHECK_RUNNING_UTESTS_=<pkgname>}), 
#' but unit tests run as part of unit tests vignettes are run with 
#' \code{_R_CHECK_RUNNING_UTESTS_=FALSE}, so that all tests are run and reported when 
#' generating them.
#' 
#' @references \url{https://github.com/renozao/roxygen2}
#' @rdname isCRANcheck
#' @export
#' 
#' @examples
#' 
#' isCHECK()
#' 
isCHECK <- function(){
	isCRANcheck() ||  # known CRAN check flags
            !isFALSE(utestCheckMode()) ||  # unit test-specific flag
            isTRUE(getOption('R_CHECK_RUNNING_EXAMPLES_')) # roxygen generated example flag
}

#' System Environment Variables
#' 
#' @param name variable name as a character string.
#' @param raw logical that indicates if one should return the raw value or
#' the convertion of any false value to \code{FALSE}.
#' 
#' @return the value of the environment variable as a character string or 
#' \code{NA} is the variable is not defined \strong{at all}.
#' 
#' @export
#' @examples
#' 
#' # undefined returns FALSE
#' Sys.getenv_value('TOTO')
#' # raw undefined returns NA
#' Sys.getenv_value('TOTO', raw = TRUE)
#' 
#' Sys.setenv(TOTO='bla')
#' Sys.getenv_value('TOTO')
#' 
#' # anything false-like returns FALSE
#' Sys.setenv(TOTO='false'); Sys.getenv_value('TOTO')
#' Sys.setenv(TOTO='0'); Sys.getenv_value('TOTO')
#' 
#' # cleanup
#' Sys.unsetenv('TOTO')
#' 
Sys.getenv_value <- function(name, raw = FALSE){
    val <- Sys.getenv(name, unset = NA, names = FALSE)
    if( raw ) return(val)
    # convert false values to FALSE if required
    if( is.na(val) || !nchar(val) || identical(tolower(val), 'false') || val == '0' ){
        val <- FALSE
    }
    val
}

checkMode_function <- function(varname){
    
    .varname <- varname
    function(value, raw = FALSE){
        if( missing(value) ) Sys.getenv_value(.varname, raw = raw)
        else{
            old <- Sys.getenv_value(.varname, raw = TRUE)
            if( is_NA(value) ) Sys.unsetenv(.varname) # unset
            else do.call(Sys.setenv, setNames(list(value), .varname)) # set value
            # return old value
            old	
        }	
    }
}


utestCheckMode <- checkMode_function('_R_CHECK_RUNNING_UTESTS_')

# Rd utility functions
# 
# Author: Renaud Gaujoux
# Created: Mar 25, 2013
###############################################################################

#getRdFile <- function(topic, package=NULL){
#	help_call <- substitute(help(topic, package = package, try.all.packages = TRUE), 
#			list(topic = topic, package = package))
#	
#	eval(help_call)
#}

# Borrowed from tools:::RdTags
RdTags <- function (Rd) 
{
    res <- sapply(Rd, attr, "Rd_tag")
    if (!length(res)) 
        res <- character()
    res
}


#' @importFrom tools Rd_db 
getRdTag <- function(topic, tag, package){
	# extract topic
	#rd <- utils:::.getHelpFile(file=getRdFile(topic, package=package))
    rd <- Rd_db(package=package)
    found <- FALSE
    i <- sapply(rd, function(x){
        if( found ) return()
        tags <- RdTags(x)
        w <- which(tags == "\\alias")
        if( length(w <- which(sapply(x[w], function(a) a[[1]] == topic))) ){
            found <<- TRUE
            rd
        }else NULL
    })
    if( !found ) stop("Could not find topic '", topic, "' in package '", package, "'")
    w <- which(!sapply(i, is.null))
    topic_rd <- rd[[w[1L]]]
    tags <- RdTags(topic_rd)
    if( !length(w <- which(tags == tag)) )
        stop("Could not find tag '", tag, "' in help topic ", package, "::", topic)
    topic_rd[w]
}

#' Format Rd Sections into LatTeX
#' 
#' This function extract sections from Rd files and convert them into 
#' LaTeX code. 
#' This can be useful to include Rd text into vignettes, hence keeping them 
#' up to date.
#' 
#' @section Example section: This is a nice section, with a bullet list: 
#' \itemize{
#' \item tata
#' \item toto
#' }
#' 
#' @param topic Rd topic
#' @param package package in which to search the topic
#' @param i index of the section to format
#' @param notitle logical that indicates if the section's title should be removed
#' 
#' @export
#' @examples
#' RdSection2latex('RdSection2latex', package = 'pkgmaker')
#' 
RdSection2latex <- function(topic, package, i=1L, notitle=TRUE){
	rdsec <- getRdTag(topic, tag="\\section", package = package)
	if( !length(rdsec) ) return()
	ltx <- capture.output(tools::Rd2latex(rdsec[i], fragment=TRUE))
	if( notitle ){
		parts <- stringr::str_match(ltx, "\\{Section\\}")
		w <- which(!is.na(parts[, 1]))
		ltx <- ltx[seq(w[1]+1, tail(w, 1)-1)]
	}
	ltx <- paste(ltx, collapse="\n")
	# remove link commands
	ltx <- gsub("\\\\LinkA\\{([^}]+)\\}\\{([^}]+)\\}", "\\2", ltx)
	
	cat(ltx)
	invisible()
}

# Development utility functions
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

#' @include namespace.R
#' @include unitTests.R
#' @include logging.R
NULL

set_libPaths <- function(lib.loc=NULL){
  ol <- Sys.getenv('R_LIBS')
  olib <- .libPaths()
  res <- list(R_LIBS=ol, .libPaths=olib)
  if( is_NA(lib.loc) ) return(res)
  
  # add lib path
  if( is.null(lib.loc) ) lib.loc <- .libPaths()
  if( is.character(lib.loc) ){
    # current .libPaths
    .libPaths(lib.loc)
    # R_LIBS env variable
    rlibs <- paste(lib.loc, collapse=.Platform$path.sep)
    Sys.setenv(R_LIBS=rlibs)
  }else if( is.list(lib.loc) ){
    Sys.setenv(R_LIBS=lib.loc$R_LIBS)
    .libPaths(lib.loc$.libPaths)
  }
  
  res
}

#' Executing R Commands
#' 
#' \code{R.exec} executes a single R command via \code{\link{system2}}.
#' 
#' @param ... extra arguments that are concatenated and appended to 
#' the command. 
#' @param lib.loc logical that indicates if the current library locations
#' should be used.
#' If a character vector, then it is used as the library path specification.
#' 
#' @export
R.exec <- function(..., lib.loc=NULL){
	cmd <- paste(file.path(R.home('bin'), 'R'), ' ', ..., sep='', collapse='')
	# add lib path
	ol <- set_libPaths(lib.loc)
	on.exit(set_libPaths(ol))

	message(cmd)
	system(cmd, intern=interactive())
}

#' \code{R.CMD} executes R CMD commands.
#' 
#' @param cmd command to run, e.g. \sQuote{check} or \sQuote{INSTALL}.
#' 
#' @export
#' @rdname R.exec
R.CMD <- function(cmd, ...){
	R.exec('CMD ', cmd, ' ', ...)
}

#' \code{R.SHLIB} executes R CMD SHLIB commands.
#' 
#' @param libname name of the output compiled library
#' 
#' @export
#' @rdname R.exec
R.SHLIB <- function(libname, ...){
	R.CMD('SHLIB', '-o ', libname, .Platform$dynlib.ext, ...)
}

#' Compile Source Files from a Development Package
#' 
#' @param pkg the name of the package to compile
#' @param load a logical indicating whether the compiled library should be loaded
#' after the compilation (default) or not.
#' 
#' @return None
#' @export
compile_src <- function(pkg=NULL, load=TRUE){
	
	if( !is.null(pkg) ){
		library(devtools)
		p <- as.package(pkg)
		path <- p$path
	}else{
		pkg <- packageName()
		path <- packagePath(lib=NA) # do not look installed packages
	}
	
	owd <- getwd()
	on.exit(setwd(owd))
	
	# Compile code in /src
	srcdir <- file.path(path, 'src')
	message("# Checking '", srcdir, "' ... ", appendLF=FALSE)
	if( !file.exists(srcdir) ){
		message("NO")
	} else {
		message("YES")
		message("## Compiling '",srcdir,"' ##")
		setwd(srcdir)
		Sys.setenv(R_PACKAGE_DIR=path)
		R.SHLIB(pkg, " *.cpp ")
		message("## DONE")
		if( load ){
			if( existsFunction('load_dll', where='package:devtools') ){ # post 0.8
				f <- getFunction('load_dll', where='package:devtools')
				f(pkg)
			}else{ # prior 0.8
				f <- getFunction('load_c', where='package:devtools')
				f(pkg)
			}
		}
	}
}

#' Package Development Utilities
#' 
#' \code{packageEnv} is a slight modification from \code{\link{topenv}}, which 
#' returns the top environment, which in the case of development 
#' packages is the environment into which the source files are loaded by 
#' \code{\link[devtools]{load_all}}.
#' 
#' @param pkg package name. If missing the environment of the runtime caller 
#' package is returned.
#' @param skip a logical that indicates if the calling namespace should be 
#' skipped.
#' @param verbose logical that toggles verbosity
#' 
#' @rdname devutils
#' @return \code{packageEnv} returns an environment
#' @export
packageEnv <- function(pkg, skip=FALSE, verbose=FALSE){
	
	# return package namespace
	if( !missing(pkg) && !is.null(pkg) ){
		# - if the package is loaded: use asNamespace because as.environment does not
		# return a correct environment (don't know why)
		# - as.environment('package:*') will return the correct environment
		# in dev mode.
		env <- if( is.environment(pkg) ) topenv(pkg)
		else if( !is.null(path.package(pkg, quiet=TRUE)) ) asNamespace(pkg)
		else if( isLoadingNamespace(pkg) ) getLoadingNamespace(env=TRUE)
		else if( isNamespaceLoaded(pkg) ) asNamespace(pkg)
		else if( pkg %in% search() ) as.environment(pkg)
		else as.environment(str_c('package:', pkg)) # dev mode
		return(env)
	}
	
	envir = parent.frame()
#	message("parent.frame: ", str_ns(envir))
	pkgmakerEnv <- topenv()
#	message("pkgmaker ns: ", str_ns(pkgmakerEnv))

	n <- 1
	skipEnv <- pkgmakerEnv
	while( identical(e <- topenv(envir), skipEnv) 
			&& !identical(e, emptyenv()) 
			&& !identical(e, .GlobalEnv) ){
		if( verbose > 1 ) print(e)
		n <- n + 1
		envir <- parent.frame(n)
	}
	
	if( !skip ){
		if( identical(e, .BaseNamespaceEnv) ){
			if( verbose ) message("packageEnv - Inferred ", str_ns(skipEnv))
			return( skipEnv )
		}
		if( verbose ) message("packageEnv - Detected ", str_ns(e))
		return(e)
	}
	if( verbose > 1 ) message("Skipping ", str_ns(skipEnv))
	# go up one extra namespace
	skipEnv <- e
	while( identical(e <- topenv(envir), skipEnv) 
			&& !identical(e, emptyenv()) 
			&& !identical(e, .GlobalEnv) ){
		if( verbose > 1 ) print(e)
		n <- n + 1
		envir <- parent.frame(n)
	}
	if( identical(e, .BaseNamespaceEnv) ){
		if( verbose ) message("packageEnv - Inferred ", str_ns(skipEnv))
		return( skipEnv )
	}
	if( verbose ) message("packageEnv - Detected ", str_ns(e))
	return(e)
}

#' \code{topns_name} returns the name of the runtime sequence of top namespace(s), 
#' i.e. the name of the top calling package(s), from top to bottom.
#' 
#' \code{topns_name}: the top namespace is is not necessarily the namespace where \code{topns_name} 
#' is effectively called.
#' This is useful for packages that define functions that need to access the 
#' calling namespace, even from calls nested into calls to another function from
#' the same package -- in which case \code{topenv} would not give the desired 
#' environment.   
#' 
#' @param n number of namespaces to return
#' @param strict a logicical that indicates if the global environment should 
#' be considered as a valid namespace.
#' @param unique logical that indicates if the result should be reduced
#' to contain only one occurence of each namespace. 
#'  
#' @rdname devutils
#' @export
topns_name <- function(n=1L, strict=TRUE, unique=TRUE){
	
	if( n==1L && !is.null(ns <- getLoadingNamespace()) ){
		return(ns)
	}
	nf <- sys.nframe()
	i <- 0
	res <- character()
	while( i <= nf && length(res) < n ){
		e <- sys.frame(i)
		if( !strict || !identical(e, .GlobalEnv) ){
			pkg <- methods::getPackageName(e, create=FALSE)
			if( pkg != '' ){
				res <- c(res, pkg)
			}
		}
		i <- i + 1
	}
	
	if( !length(res) ){# try with packageEnv
		e <- packageEnv(skip=TRUE)
		if( isNamespace(e) ){
			res <- methods::getPackageName(e)
#			print(res)
		}else{
			#warning('Could not find top namespace.', immediate.=TRUE)
			return('')
		}
	}
	
	if( unique || n==1L ) res <- match.fun('unique')(res)
	if( length(res) || n>1L ) res else ''
}

#' \code{topns} returns the runtime top namespace, i.e. the namespace of 
#' the top calling package, possibly skipping the namespace where \code{topns} 
#' is effectively called.
#' This is useful for packages that define functions that need to access the 
#' calling namespace, even from calls nested into calls to another function from
#' the same package -- in which case \code{topenv} would not give the desired 
#' environment.
#'  
#' @rdname devutils
#' @export
topns <- function(strict=TRUE){
	ns <- topns_name(n=1L, strict=strict)
	if( ns == '.GlobalEnv' ) return( .GlobalEnv )
	else if( nchar(ns) ) asNamespace(ns)
	#packageEnv(skip=TRUE, verbose=verbose)
}

#' \code{packageName} returns the current package's name.
#' It was made internal from version 0.16, since the package \pkg{utils} 
#' exported its own \code{\link[utils]{packageName}} function in R-3.0.0. 
#' 
#' @param envir environment where to start looking for a package name.
#' The default is to use the \strong{runtime} calling package environment.
#' @param .Global a logical that indicates if calls from the global 
#' environment should throw an error (\code{FALSE}: default) or the string
#' \code{'R_GlobalEnv'}.
#' @param rm.prefix logical that indicates if an eventual prefix 'package:' 
#' should be removed from the returned string.
#' 
#' @rdname devutils
#' @return a character string
packageName <- function(envir=packageEnv(), .Global=FALSE, rm.prefix=TRUE){
	
	if( is.null(envir) ) envir <- packageEnv() 
	if( is.character(envir) ){
		return( sub("^package:", "", envir) )
	}
	
	# retrieve package environment
	e <- envir
	
	# try with name from environment
	nm <- environmentName(e)
	if( identical(e, .GlobalEnv) && .Global ) return(nm)
	else if( isNamespace(e) || identical(e, baseenv()) ) return(nm)
	else if( grepl("^package:", nm) ){# should work for devtools packages
		if( rm.prefix ) 
			nm <- sub("^package:", "", nm)
		return(nm)
	}
	
	# try to find the name from the package's environment (namespace) 
	if( exists('.packageName', e) && .packageName != 'datasets' ){
		if( .packageName != '' )
			return(.packageName)
	}
	# get the info from the loadingNamespace
	info <- getLoadingNamespace(info=TRUE)
	if( !is.null(info) ) # check whether we are loading the namespace 
		info$pkgname
	else{# error
		stop("Could not reliably determine package name [", nm, "]")
	}
}

#' \code{str_ns} formats a package environment/namespace for log/info messages.
#' 
#' @rdname devutils
#' @export
str_ns <- function(envir=packageEnv()){
	if( !is.environment(envir) )
		stop("Invalid argument: must be an environment [", class(envir), ']')
	str_c(if( isNamespace(envir) ) 'namespace' else 'environment',
			" '", packageName(envir, rm.prefix=FALSE), "'")
}


#' \code{packagePath} returns the current package's root directory, which is 
#' its installation/loading directory in the case of an installed package, or
#' its source directory served by devtools. 
#' 
#' @param package optional name of an installed package 
#' @param lib.loc path to a library of R packages where to search the package
#' @param ... arguments passed to \code{\link{file.path}}.
#' 
#' @rdname devutils
#' @return a character string
#' @export
packagePath <- function(..., package=NULL, lib.loc=NULL){
	
	# try to find the path from the package's environment (namespace)
	pname <- packageName(package)
	
	# check if one is currently loading the namespace
	path <- NULL
	if( !is.null(info <- getLoadingNamespace(info=TRUE)) && info$pkgname == pname ){
		path <- info$path
	}else {
		# try loaded/installed package
		path <- find.package(package=pname, lib.loc=lib.loc, quiet=TRUE)		
	}
	# somehow this fails when loading an installed package but is works 
	# when loading a package during the post-install check
	if( !length(path) || path == '' ){
		# get the info from the loadingNamespace
		if( !is.null(info <- getLoadingNamespace(info=TRUE)) ){
			path <- info$path
		}
	}
	stopifnot( !is.null(path) && path != '' )
	
	# for development packages: add inst prefix if necessary
	if( isDevNamespace(pname) ){
		# handle special sub-directories of the package's root directory
		dots <- list(...)
		Rdirs <- c('data', 'R', 'src', 'exec', 'tests', 'demo'
				, 'exec', 'libs', 'man', 'help', 'html'
				, 'Meta')
		if( length(dots) && !sub("^/?([^/]+).*", "\\1", ..1) %in%  Rdirs)
			path <- file.path(path,'inst')
	}
	
	# add other part of the path
	file.path(path, ...)	
}

#' \code{isPackageInstalled} checks if a package is installed.
#' 
#' @rdname devutils
#' @export
isPackageInstalled <- function(..., lib.loc=NULL){
	
	inst <- utils::installed.packages(lib.loc=lib.loc)
	pattern <- '^([a-zA-Z.]+)(_([0-9.]+)?)?$';
	res <- sapply(list(...), function(p){
				vers <- gsub(pattern, '\\3', p)
				print(vers)
				pkg <- gsub(pattern, '\\1', p)
				print(pkg)
				if( !(pkg %in% rownames(inst)) ) return(FALSE);
				p.desc <- inst[pkg,]
				if( (vers != '') && compareVersion(vers, p.desc['Version']) > 0 ) return(FALSE);
				TRUE
			})
	all(res)
}

#stripLatex <- function(x){
#	gsub("\\\\.\\{(.)\\}", "\\1", x)
#}

#' \code{as.package} is enhanced version of \code{\link[devtools]{as.package}}, 
#' that is not exported not to mask the original function.
#' It could eventually be incorporated into \code{devtools} itself.
#' Extra arguments in \code{...} are passed to \code{\link{find.package}}. 
#' 
#' @param x package specified by its installation/development path or its name
#' as \code{'package:*'}.
#' @param quiet a logical that indicate if an error should be thrown if a 
#' package is not found. It is also passed to \code{\link{find.package}}.
#' @param extract logical that indicates if DESCRIPTION of package 
#' source files should be extracted.
#' In this case there will be no valid path. 
#' 
#' @rdname devutils
as.package <- function(x, ..., quiet=FALSE, extract=FALSE){
	
	
	if( is.null(x) ) return( devtools::as.package() )
	if( devtools::is.package(x) ) return(x)
	
	if( extract && grepl("\\.tar\\.gz$", x) ){ # source file
		# extract in tempdir
		tmp <- tempfile(x)
		on.exit( unlink(tmp, recursive=TRUE) )
		pkg <- basename(sub("_[0-9.]+\\.tar\\.gz$", '', x))
		desc <- file.path(pkg, 'DESCRIPTION')
		untar(x, desc, exdir=tmp)
		return(devtools::as.package(file.path(tmp, pkg)))
	} else { # check for 'package:*'
		if( grepl('^package:', x) ){
			libs <- .libPaths()
			pkg <- sub('^package:', '', x)
			p <- lapply(libs, find.package, package=pkg, quiet=TRUE, verbose=FALSE)
			p <- unlist(p[sapply(p, length)>0])
			if( !length(p) ){
				if( !quiet )
					stop("Could not find installed package ", pkg)
				return()
			}
			x <- p[1L]
		}
	}
	# try development packages
	res <- try(devtools::as.package(x), silent=TRUE)
	if( !is(res, 'try-error') )
		return(res)
	# try loaded or installed packages
	if( length(res <- find.package(package=x, quiet=TRUE)) )
		return(devtools::as.package(res))
	if( quiet )
		stop("Could not find package ", x)
	NULL
}

NotImplemented <- function(msg){
	stop("Not implemented - ", msg)
}

#' Loading Package Data
#' 
#' Loads package data using \code{\link[utils]{data}}, but allows the user to avoid
#' NOTEs for a \sQuote{non visible binding variable} to be thrown when checking a package.
#' This is possible because this function returns the loaded data.
#'  
#' @param list character vector containing the names of the data to load.
#' @inheritParams utils::data
#' @param ... other arguments eventually passed to \code{\link[utils]{data}}.
#' 
#' @return the loaded data.
#' 
#' @export
#' @examples 
#' 
#' \dontrun{ mydata <- packageData('mydata') }
#' 
packageData <- function(list, envir = .GlobalEnv, ...){
	
	# same as utils::data if no 'list' argument
	if( missing(list) ) return( data(..., envir=envir) )
	# load into environment
	data(list=list, ..., envir = envir)
	# return the loaded data
	if( length(list) == 1L ) get(list, envir=envir)
	else sapply(list, get, envir=envir, simplify=FALSE)
	
}

#' \code{ldata} loads a package data in the parent frame.
#' It is a shortcut for \code{packageData(list, ..., envir=parent.frame())}.
#' 
#' @rdname packageData
#' @export
#' @examples
#' 
#' \dontrun{ 
#' # in a package' source => won't issue a NOTE
#' myfunction function(){
#' 	mydata <- ldata('mydata') 
#' }
#' }
#' 
ldata <- function(list, ...){
	e <- parent.frame()
	packageData(list=list, ..., envir=e)
}

# Logging system
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

#' @include utils.R options.R
NULL

#' Logging Feature
#' 
#' \code{lverbose} returns/sets the current verbosity level.
#' 
#' @param val logical/numeric value that sets the verbosity level.
#' @param global logical that indicates if the verbose level of 
#' all log handlers should be set to \var{val}.
#' 
#' @return the old verbose level   
#' @export
#' @rdname logging
lverbose <- local({
      .val <- as.integer(FALSE)
      function(val, global = FALSE){
        if( missing(val) ) return( max(getOption('pkg_verbose_level', 0L), .val) )
        oval <- .val
        .val <<- max(as.integer(val), 0)
	# set value globally
	if( global ){
	  options(pkg_verbose_level = val)
	}
        invisible(oval)
      }
    })

#' @describeIn logging tells if all verbose messages are silenced. 
#' @export
lsilent <- function(){
	l <- lverbose()
	is.na(l) || l == 0L
}
#' @describeIn logging tells if verbosity is on, i.e. at level greater than 0.
#' @export
is.verbose <- function(){
	l <- lverbose()
	!is.na(l) && l >= 0L
}

#' @describeIn logging prints out a message (on sdtout) if the verbosity level is greater than a 
#' given value. 
#' 
#' @param ... parts of a character message that are concatenated and passed to 
#' the current logger's write function.
#' @param level verbosity level threshold (numeric value) above which the 
#' message should be printed out. 
#' This threshold is compared with the current verbosity level as returned by 
#' \code{lverbose}.
#' @param appendLF logical indicating if an endline character should be appended 
#' at the end of the message.
#' @param sep separation character, used when concatenating all arguments in \code{...}.
#' @param force logical that indicates if one should output messages or return a non 
#' null logger, even if the verbose mode is not high enough.
#' 
#' @export 
lmessage <- function(level, ..., appendLF=TRUE, sep='', force=FALSE){
  getLogger(force=force)$lmessage(level, ..., appendLF=appendLF, sep=sep, force=force)
}

#' @describeIn logging prints out a log message (at level 1) using the current logger, 
#' typically on stdout.
#' It is a shortcut for \code{lmessage(1L, ...)}.
#'  
#' @export
vmessage <- function(...){
  lmessage(1L, ...)
}

#' @describeIn logging directly appends some message to the current log line.
#' @export 
log_append <- function(...){
  if( !is.null(w <- getLogger(new=FALSE)$write) ) w(...)
}

# return a list of data about a given frame (i.e. caller function)
callInfo <- function(n=-1L){
  if( n<0L ) n <- n-1 
  f <- sys.function(n)
  e <- sys.frame(n)
  sysn <- if( n<0L ) sys.nframe()-n else n
  list(fd=digest(f), env=capture.output(e), nframe=sysn)
}

log_lastLF <- sVariable(list(Inf, TRUE))

.LOG_OPTIONS <- setupPackageOptions(NAME='logger', RESET=TRUE,
    # should a the next logger be indented
    autoindent = TRUE
)
logger.options <- .LOG_OPTIONS$options
logger.getOption <- .LOG_OPTIONS$getOption

getLogger <- local({
      
      # store info for top call
      .topCall <- NULL
      # list of logger objects
      .lastLogger <- list()
      
      function(..., type='STDOUT', new=TRUE, force=FALSE){
        
        # return NULL logger if not in verbose mode
        if( !force && !lverbose() ) return( new_logger('NULL') )
        
        # top caller
        call1 <- sys.call(1)
        topCall <- callInfo(1L)
        caller <- callInfo(-1L)
        # reset all static variables
        if( !identical(.topCall, topCall) ){
          .topCall <<- topCall
          .lastLogger <<- list()
        }
        
        ## build logger object
        if( !length(.lastLogger) ){ # new stored for future calls
          logger <- new_logger(type, ...)
          logger$caller(caller)
          .lastLogger[[as.character(caller$nframe)]] <<- logger
        }else{ # increment indentation
          
          autonew <- missing(new)
          if( new ){ # get new logger object
            last <- getLogger(new=FALSE)
            if( !autonew || (!is.null(last$caller()) && last$caller()$nframe < caller$nframe) ){
              # instanciate a new logger object only if the current caller is lower in the stack
              logger <- new_logger(type, ...)
              logger$caller(caller)
              # auto indent if last logger is higher in the stack
              if( !logger.getOption('autoindent') ){
                # reset autoindent option on.exit
                on.exit( logger.options(autoindent=TRUE), add=TRUE)
                logger$nindent(last$nindent(), add=FALSE)
              } else if( last$caller()$nframe < caller$nframe ){
                logger$nindent(last$nindent() + 1L, add=FALSE)
              }
              .lastLogger[[as.character(caller$nframe)]] <<- logger
            }else logger <- last
            # add initial new line if the last log message was from a higher logger
            logger$breaknext(log_lastLF()[[1L]] < caller$nframe)
          }else{
            # get logger for exact frame number 
            if( is.null(logger <- .lastLogger[[as.character(caller$nframe)]]) ){
              # get first higher logger
              i <- which(as.numeric(names(.lastLogger)) <= caller$nframe)
#					str(.lastLogger)
              if( length(i) )	logger <- .lastLogger[[tail(i, 1L)]]
              else logger <- .lastLogger[[1L]]
              
            }
          }
        }
        
        # return logger
        logger
      }
    })


new_logger <- function(type, ..., nindent=0L, indent=' '){
  
  .data <- list(
      lastLF = TRUE
      , breaknext = FALSE
      , cindent = indent
      , nindent = nindent
      , indent = rep(indent, nindent)
      , caller = NULL
  )
  
  # init logger object
  .logger <- list()
  
  .logger$data <- function(value){
    if( missing(value) ) .data
    else .data <<- value
  }
  
  # init type-specific slots
  f <- match.fun(paste('new_logger', type, sep=''))
  .logger <- f(.logger, ...)
  
  # get last used appendLF value
  .logger$lastLF <- function(){ .data$lastLF }
  
  # get initial new line flag
  .logger$breaknext <- function(val){
    if( missing(val) ) .data$breaknext
    else .data$breaknext <<- val
  }
  
  # get caller data
  .logger$caller <- function(val){
    if( missing(val) ) .data$caller
    else .data$caller <<- val
  }
  
  # get/set indentation
  .logger$indent <- function(val){
    if( missing(val) ) .data$indent
    else{
      if( is.numeric(val) ) return( .logger$nindent(val) )
      old <- .data$indent
      .data$indent <<- val
      old
    }
  }
  # get/set number of indentation
  .logger$nindent <- function(val, add=TRUE){
    if( missing(val) ) .data$nindent
    else{
      old <- .data$nindent
      .data$nindent <<- val + if( add ) .data$nindent else 0L
      if( .data$nindent >= 0L )
        .data$indent <<- paste(rep(.data$cindent, .data$nindent), collapse='')
      old
    }
  }
  
  # new message with auto-indentation and breakline
  .logger$message <- function(..., appendLF=TRUE, sep=''){
    
    if( is.null(.logger$write) ) return()
    
    msg <- if( .data$breaknext || isTRUE(logger.getOption('breaknext')) ){
      logger.options(breaknext=NULL)
      .data$breaknext <<- FALSE
      "\n"
    }
    # add indentation if on a new line
    lastLFframe <- log_lastLF()
    callerFrame <- .logger$caller()$nframe
    if( .data$lastLF || lastLFframe[[2L]] || (is.finite(lastLFframe[[1L]]) && lastLFframe[[1L]] >= callerFrame) )
      msg <- paste(msg, .data$indent, sep='')
    msg <- paste(msg, paste(..., sep='', collapse=sep), if( appendLF ) "\n", sep='')
    
    # store in global variable the frame number if it the line is left open
    log_lastLF(list(if( !appendLF ) callerFrame else Inf, appendLF))
    .data$lastLF <<- appendLF
    
    # call logger write function
    .logger$write(msg)
  }
  
  # log message with specific level
  .logger$lmessage <- function(level, ..., force=FALSE){
    if( force || lverbose() >= level ) .logger$message(...)
  }
  
  # show info about the logger
  .logger$show <- function(){
    cat("<Type: ", class(.logger)[1L], ">\n", sep='')
    .logger$info()
  }
  
  # wrap into a logger object
  structure(.logger, class=c(paste('logger', type, sep=''), 'logger'))
}

# NULL logger
new_loggerNULL <- function(.logger, ...){
  
  # write to log
  .logger$write <- NULL
  # special info
  .logger$info <- function(){
    cat("Output: NULL\n")
  }
  # return logger object
  .logger
}

# Logger that writes on the standard output
new_loggerSTDOUT <- function(.logger, ...){
  
  # append to log
  .logger$write <- function(...){
    cat(..., sep='')
  }
  
  .logger$info <- function(){
    cat("Output: stdout\n")
  }
  
  # return logger object
  .logger
  
}

# Logger that writes on the standard error
new_loggerSTDERR <- function(.logger, ...){
  
  # append to log
  .logger$write <- function(...){
    message(..., appendLF=FALSE)
  }
  
  .logger$info <- function(){
    cat("Output: stderr\n")
  }
  
  # return logger object
  .logger
  
}

#' General Log Formating
#' 
#' Generate a formatted diagnostic message.
#' This function is a shortcut for \code{message(sprintf(...))}.
#' 
#' @inheritParams base::sprintf
#' @inheritParams base::message
#' 
#' @return Returns nothing. Used for their side effects of printing messages/warnings.
#' @seealso \code{\link{sprintf}}, \code{\link{message}}
#' @export
#' @examples 
#' 
#' messagef("Hello %s number %i", 'world', 4)
#' 
messagef <- function(fmt, ..., domain = NULL, appendLF = TRUE) message(sprintf(fmt, ...), domain = domain, appendLF = appendLF)

#' @describeIn messagef throws a simple note as an immediate warning.
#' It is a shorcut for `warning(..., immediate. = TRUE, call. = FALSE)`.
#' 
#' @inheritParams base::warning
#' @export
wnote <- function(..., immediate. = TRUE){
  warning(..., immediate. = immediate., call. = FALSE)
}


#' Simple Text Iteration Counter
#' 
#' @param n number of total steps
#' @param i0 starting step
#' @param title character string to use as title
#' @param extra character vector providing extra text to add at each step
#' @param verbose logical that toggles the counter
#' 
#' @return A function that can be used to increment progress.
#' @export
#' @examples 
#' 
#' progress <- iterCount(LETTERS)
#' res <- sapply(LETTERS, function(x){
#'  Sys.sleep(.1)
#' 	progress()
#' })
#' # terminate counter
#' i_end <- progress(NULL)
#' i_end 
#' 
iterCount <- function(n = 100, i0 = 0L, title = 'Iterations', extra = NULL, verbose = TRUE){
  
  if( !verbose ) return( function(...) NULL )
  
  if( length(n) > 1L || is.character(n) ){
    extra <- extra %||% n
    n <- length(n)
  }
  
  # setup
  size_i <- nchar(as.character(n))
  if( nzchar(title) ) title <- paste0(title, ': ')
  .msg_fmt <- sprintf("%s%%%ii/%i", title, size_i, n)
  
  inc_msg <- function(i, addon = extra[i]){
    if( !length(addon) || is.na(addon) ) sprintf(.msg_fmt, i, '')
    else paste0(sprintf(.msg_fmt, i, addon))
  }
  
  .cat_msg <- function(msg){
    cat(msg)
    flush.console()
    nchar(msg)
  } 
  
  if( !is.null(extra) ){
    .msg_fmt <- paste0(.msg_fmt, " (%s)")
    .msg_size <- inc_msg(n, extra[which.max(nchar(extra))])
  }else .msg_size <- inc_msg(n)
  
  .msg_size <- nchar(.msg_size)
  .back <- paste0(rep("\b", .msg_size), collapse ='')
  .i <- i0
  .last_msg_size <- .cat_msg(inc_msg(i0))
  
  # increment function
  function(i, appendLF = i == n){
    if( missing(i) ) i <- .i
    else if( is.null(i) ){
      if( .i < n ) cat("\n")
      i_stop <- .i
      .i <<- i0
      return(invisible(i_stop))
    }
    .i <<- i+1
    msg <- paste0(substr(.back, 1L, .last_msg_size), msg0 <- inc_msg(.i))
    .cat_msg(msg)
    .last_msg_size <<- nchar(msg0)
    if( appendLF ) cat("\n")
    invisible(.i)
  }
}

# Project: pkgmaker
# 
# Author: renaud
# Created: Jul 2, 2014
###############################################################################

#' @include packages.R
NULL

#' User Queries
#' 
#' This function is an improved version of \code{userQuery} from Bioconductor \pkg{Biobase} 
#' package, which asks the user about some task that needs her intervention to proceed, 
#' e.g., ask if one should perform a computation, install a package, etc.. 
#' 
#' @inheritParams Biobase::userQuery
#' @param idefault default response in interactive mode.
#' This answer will be in upper case in the question and will be the one returned if the 
#' user simply hits return.
#' @param default default response in non-interactive mode.
#' 
#' If \code{NA}, then the user is forced to provide an answer, even in non-interactive mode 
#' (e.g., when run through \code{Rscript}).  
#' 
#' @return the character string typed/agreed by the user or directly the default answer in 
#' non-interactive mode.  
#' 
#' @export
askUser <- function (msg, allowed = c("y", "n"), idefault = "n", default = "n", case.sensitive = FALSE) 
{
    ucon <- stdin()
    if ( !interactive() ) {
        if( is_NA(default) ) ucon <- 'stdin'
        else return(default)
    }
    
    fallowed <- allowed
    # add extra info on answer options
    if( !is.null(nm <- names(allowed)) ){
        allowed[nm != ''] <- nm[nm != '']  
    }
    if( !isFALSE(idefault) )
        fallowed[fallowed == idefault] <- toupper(idefault)
    repeat {
        allowMsg <- paste("[", paste(fallowed, collapse = "/"), 
                "]: ", sep = "")
        outMsg <- paste(msg, allowMsg)
        cat("\n", outMsg, sep='')
        if (case.sensitive) 
            ans <- readLines(ucon, n = 1)
        else ans <- tolower(readLines(ucon, n = 1))
        if( !isFALSE(idefault) && !nchar(ans) ) 
            ans <- idefault
        if (ans %in% allowed) 
            break
        else cat(paste(ans, "is not a valid response, try again.\n"))
    }
    # return answer
    ans
}


#' User Data Directory
#'  
#' \code{userData} returns the path to a local directory/file where package-related user data can be stored.
#' Note that a base directory is \strong{always} created if necessary (see details).
#' 
#' The package-specific user data base directory is the sub-directory \emph{R-data/}, 
#' located in the user's home or within a diredctory defined by global option 'userData.path'.
#'  
#' If in interactive mode, and the base directory does not exist yet, 
#' the user is asked if it should be created in his home directory. 
#' Otherwise, or if the user does not allow the creation in his home, this directory is created 
#' in the current R session's temporary directory.
#' 
#' @param ... path parts passed to \code{\link{file.path}} to be appended to 
#' the main path.
#' @param create logical that indicates if the \strong{base} directory should be 
#' created if it does not exists.
#'  
#' Note that directories -- and files -- under the base directory are not automatically 
#' created. The user should therefore care of it in the caller function if necessary.
#' 
#' If \code{create=TRUE}, then the base directory is forced to be created in the user's home directory.  
#' If \code{create=FALSE}, then the base directory is never created.
#' 
#' See also section \emph{Details}.
#'   
#' @param package name of the package associated with the user data path.
#' It is used to prefix the path, within the user R data directory. 
#' 
#' @seealso \code{\link{tempdir}}
#' 
#' @return Path to the user data directory.
#' 
#' @export
userData <- function(..., create=NULL, package = topenv(parent.frame())){
        
    if( is.environment(package) ) package <- utils::packageName(package)
    
    root_dir <- getOption('userData.path', Sys.getenv('HOME'))
    p <- file.path(root_dir, 'R-data', package)
    
    # ask the user about creating the directory
    if( !file.exists(p) && (is.null(create) || isTRUE(create))  ){
        if( is.null(create) ){
            ans <- askUser(str_c("The ", package, " user data directory '", p, "' doen't exist. Do you want to create it?")
                	, idefault='y', default='n')
            if( ans == 'n' ){
                p <- file.path(tempdir(), 'R-data', package)
            }
        }
        
        if( !file.exists(p) ){
            message("Creating user data directory '", p, "'")
            dir.create(p, recursive=TRUE)
        }
    }
    file.path(p, ...)
}

#' Require a Package with User Interaction
#'
#' Like base \code{\link{require}}, \code{irequire} tries to find and load a package, 
#' but in an interactive way, i.e. offering the user to install it if not found.
#' 
#' @param package name of the package
#' @param lib path to the directory (library) where the package is to be
#' looked for and installed if agreed by the user.
#' @param ... extra arguments passed to \code{\link{install.packages}}.
#' @param load a logical that indicates if the package should be loaded, 
#' possibly after installation.
#' @param msg message to display in case the package is not found when first 
#' trying to load/find it.
#' This message is appended to the string `"Package '<packagename>' is required"`.
#' @param quiet logical that indicates if loading a package should be done quietly 
#' with \code{\link{require.quiet}} or normally with \code{\link{require}}.
#' @param prependLF logical that indicates if the message should start at a new line.
#' @param ptype type of package: from CRAN-like repositories, Bioconductor, Bioconductor software, Bioconductor annotation.
#' Bioconductor packages are installed using \code{biocLite} from the 
#' \pkg{BiocInstaller} package or fetched on line at \url{http://bioconductor.org/biocLite.R}.
#' @param autoinstall logical that indicates if missing packages should just be installed 
#' without asking with the user, which is the default in non-interactive sessions.
#' 
#' @return \code{TRUE} if the package was successfully loaded/found (installed), 
#' \code{FALSE} otherwise.  
#'  
#' @family require
#' @export
irequire <- function(package, lib=NULL, ..., load=TRUE, msg=NULL, quiet=TRUE, prependLF=FALSE
        , ptype=c('CRAN-like', 'BioC', 'BioCsoft', 'BioCann')
        , autoinstall = !interactive() ){
    
    .reqpkg <- if( quiet ) qrequire else{
                if( prependLF ) message()
                require
            }
    reqpkg <- function(...){
        .reqpkg(..., lib=lib, character.only=TRUE)
    }
    
    # vectorized version
    if( length(package) >1L ){
        return( all(sapply(package, irequire, lib = lib, ...
                                , load = load, msg = msg, quiet = quiet
                                , prependLF = prependLF, autoinstall = autoinstall)) )
    }
    # try loading it
    if( load && reqpkg(package) ) return( TRUE )
    # try finding it without loading
    else if( length(find.package(package, lib.loc=lib, quiet=TRUE)) ) return( TRUE )
    
    # package was not found: ask to install
    msg <- paste0("Package '", package, "' is required",
            if( is.null(msg) ) '.' else msg)
    
    # stop if not auto-install and not interactive
    if( !interactive() && !autoinstall ) stop(msg)
    
    # non-interactive mode: force CRAN mirror if not already set
    if( !interactive() && length(iCRAN <- grep("@CRAN@", getOption('repos'))) ){
        repos <- getOption('repos')
        repos[iCRAN] <- 'https://cran.rstudio.com'
        op <- options(repos = repos)
        on.exit(options(op), add = TRUE)
    }
    
    # detect annotation packages
    if( missing(ptype) && grepl("\\.db$", package) ) ptype <- 'BioCann'
    ptype <- match.arg(ptype)
    
    if( !autoinstall ){
        msg <- paste0(msg, "\nDo you want to install it from known repositories [", ptype, "]?\n"
                , " Package(s) will be installed in '", if(is.null(lib) ) .libPaths()[1L] else lib, "'")
        if( quiet && prependLF ) message()
        repeat{
            ans <- askUser(msg, allowed = c('y', 'n', r='(r)etry'), idefault='y', default = 'y')
            if( ans == 'n' ) return( FALSE )
            if( ans == 'y' ) break
            if( ans == 'r' && reqpkg(package) ) return(TRUE)
        }
    }
    
    ## install
    # check Bioconductor repositories
    hasRepo <- function(p){ any(grepl(p, getOption('repos'))) }
    install_type <- ptype
    if( ptype == 'CRAN-like' 
            || ( ptype == 'BioC' && hasRepo('/((bioc)|(data/annotation))/?$') )
            || ( ptype == 'BioCsoft' && hasRepo('/bioc/?$') )
            || ( ptype == 'BioCann' && hasRepo('/data/annotation/?$') )
            ){
        install_type <- 'CRAN'
    }
    
    if( install_type == 'CRAN' ){
        pkginstall <- install.packages
    }else{ # Bioconductor 
        if( testRversion("3.6", -1L) ){ # Before 3.6 use BiocInstaller::biocLite
          if( !reqpkg('BiocInstaller') ){
            sourceURL("http://bioconductor.org/biocLite.R")
          }
          pkginstall <- function(...){
            f <- ns_get('biocLite', 'BiocInstaller')
            f(..., suppressUpdates = TRUE)
          }
          
       }else { # >= 3.6 -> use BiocManager::install
          irequire("BiocManager", autoinstall = TRUE)
          pkginstall <- function(...){
            f <- ns_get('install', 'BiocManager')
            f(..., update = FALSE)
          }
          
        }
    }
    message()
    pkginstall(package, lib=lib, ...)
    #
    
    # try  reloading
    if( load ) reqpkg(package)
    else length(find.package(package, lib.loc=lib, quiet=TRUE))
}

# General utility functions
# 
# Author: Renaud Gaujoux
# Creation: 25 Apr 2012
###############################################################################

# or-NULL operator (borrowed from Hadley Wickham)
'%||%' <- function(x, y) if( !is.null(x) ) x else y

#' Get Anywhere
#' 
#' Similar to \code{\link{getAnywhere}}, but looks for the value of its argument. 
#' 
#' @param x a single character string
#' 
#' @export
cgetAnywhere <- function(x){
	do.call("getAnywhere", list(x))
}


#' Silencing Functions
#' 
#' Generates a wrapper function that silences the output, messages, and/or warnings of a given function.
#' 
#' @param f function to silence
#' @param level a single numeric (integer) that indicates the silencing level, which encodes the set of 
#' output to be silenced.
#' 
#' It is interpreted like unix permission bit system, where each bit of the binary expression of the silencing 
#' level corresponds to a given type of output: 
#' \itemize{
#' \item 0: nothing silenced;
#' \item 1: \emph{stdout};
#' \item 2: \emph{stderr} messages;
#' \item 4: \emph{stderr} warnings.
#' }
#' 
#' For example, level \code{3 = 2 + 1} means silencing \emph{stdout} and \emph{stderr}, while 
#' \code{5 = 3 + 2} means silencing \emph{stderr} messages and warnings, but not outputs to \emph{stdout}.
#' The default value is \code{7 = 4 + 2 + 1}, which silences all output.
#'  
#' Negative values are supported and mean \emph{"silence everything except the corresponding type"}, 
#' e.g., \code{level = -1} silences all except \emph{stdout} (computed as the binary complementary of 7, i.e. \code{7 - 1 = 5 = 3 + 2}).
#' See examples. 
#' @return a function 
#' @export 
#' @examples 
#' 
#' f <- function(){
#' 	cat("stdout message\n")
#'  message("stderr message")
#' 	warning("stderr warning", immediate. = TRUE)
#' }
#' 
#' # example of generated wrapper
#' g <- .silenceF(f)
#' g
#' 
#' # use of silencing level
#' for(l in 7:-7){ message("\nLevel: ", l); .silenceF(f, l)() }
#' 
#' # inline functions
#' ifun <- .silenceF(function(){ f(); invisible(1) })
#' ifun()
#' ifun <- .silenceF(function(){ f(); 1 })
#' ifun()
#' ifun <- .silenceF(function(){ f(); 1 }, 2L)
#' ifun()
#' 
.silenceF <- function(f, level = 7L){
    
    # switch inverse level specification
    if( level < 0 ) level <- 7L + level
    # early exit if not silencing
    if( !level ) return(f)
                
    silencer <- c('utils::capture.output(', 'suppressPackageStartupMessages(suppressMessages(', 'suppressWarnings(')
    wrapper <- character()
    for( i in 1:3 ){
        if( bitwAnd(level, 2^(i-1)) ) wrapper <- c(wrapper, silencer[i])
    }
    wrapper <- paste0(wrapper, collapse = "")
    npar <- length(gregexpr("(", wrapper, fixed = TRUE)[[1]])
    
    # build source code of wrapper function 
    f_str <- paste0(as.character(substitute(f)), collapse = "")
	ca <- match.call()
    use_env <- languageEl(ca$f, 1) == as.symbol('function')
    if( use_env ) f_str <- 'f'
    txt <- sprintf("function(...){ %s res <- withVisible(%s(...))%s; if( res$visible ) res$value else invisible(res$value) }", wrapper, f_str, paste0(rep(")", npar), collapse = ""))
    
    e <- parent.frame()
    if( use_env ){
        e <- new.env(parent = e)
        e$f <- f
    }
    force(eval(parse(text = txt), e))
}

#' Testing R Version
#' 
#' Compares current R version with a given target version, which may be useful  
#' for implementing version dependent code. 
#' 
#' @param x target version to compare with.
#' @param test numeric value that indicates the comparison to be carried out.
#' The comparison is based on the result from 
#' \code{utils::compareVersion(R.version, x)}:
#' \itemize{
#' \item 1: is R.version > \code{x}?
#' \item 0: is R.version = \code{x}?
#' \item -1: is R.version < \code{x}?
#' } 
#' 
#' @return a logical
#' @export
#' @examples
#' 
#' testRversion("2.14")
#' testRversion("2.15")
#' testRversion("10")
#' testRversion("10", test = -1)
#' testRversion("< 10")
#' testRversion(Rversion())
#' testRversion(paste0('=', Rversion()))
#' 
testRversion <- function(x, test=1L){
	rv <- Rversion()
    op <- '=='
    if( grepl("^[=<>]", str_trim(x)) ){
        m <- str_match(x, "^([<>=]=?)(.*)")
        if( is.na(m[, 1]) ) stop('Invalid version specification: ', x)
        op <- m[, 2]
        if( op == '=' ) op <- '=='
        x <- str_trim(m[, 3L])
        if( !missing(test) ) warning("Ignoring argument `test`: comparison operator was passed in argument `x`")
        test <- 0L
    }
	do.call(op, list(utils::compareVersion(rv, x), test))
}

#' Complete R version
#' 
#' Returns the complete R version, e.g. 2.15.0
#' 
#' @export
#' @examples
#' Rversion()
#' 
Rversion <- function(){
	paste(R.version$major, R.version$minor, sep='.')
}

#' Formatting Utilities
#' 
#' \code{str_out} formats character vectors for use in show methods or 
#' error/warning messages.
#' 
#' @param x character vector
#' @param max maximum number of values to appear in the list. If \code{x} has 
#' more elements than \code{max}, a \code{"..."} suffix is appended.
#' @param quote a logical indicating whether the values should be quoted with 
#' single quotes (defaults) or not. 
#' @param use.names a logical indicating whether names should be added to the 
#' list as \code{NAME=VAL, ...} or not (default).
#' @param sep separator character
#' @param total logical that indicates if the total number of elements should be 
#' appended to the formatted string as \code{"'a', ..., 'z' (<N> total)"}.  
#' 
#' @return a single character string
#' 
#' @examples
#' 
#' x <- letters[1:10]
#' str_out(x)
#' str_out(x, 8)
#' str_out(x, Inf)
#' str_out(x, quote=FALSE)
#' str_out(x, total = TRUE)
#' 
#' @export
str_out <- function(x, max=3L, quote=is.character(x), use.names=FALSE, sep=", ", total = FALSE){
	if( is_NA(max) ) max <- Inf
	suffix <- NULL
    nTotal <- length(x)
	if( max > 2 && length(x) > max ){
		suffix <- "..."
		x <- c(head(x, max-1), tail(x, 1))
	}
	x <- head(x, max)
	
	# add quotes if necessary
	quote <- 
			if( isTRUE(quote) ) "'"
			else if( is.character(quote) ) quote
	if( !is.null(quote) ) x <- unlist(lapply(x, function(v) paste(quote,v,quote, sep='')))
	else if( all(sapply(x, isInteger)) ) x <- unlist(lapply(x, function(v) str_c(v,'L')))
	# add names if necessary
	if( use.names && !is.null(names(x)) ){
		nm <- str_c(names(x),'=')
		x <- paste(ifelse(nm=='=','',nm), x, sep='')
	}
	# insert suffix
	if( !is.null(suffix) ){
		x <- c(head(x, length(x)-1L), suffix, tail(x, 1L))
	}
	s <- paste(paste(x, collapse=sep), sep='')
	
	if( total ) s <- paste0(s, ' (', format(nTotal, big.mark=",", scientific=F), ' total)')
	
	# return formatted string 
	s
}

#' @describeIn str_out builds formatted string from a list of complex values.
#' 
#' @param object an R object
#' @param exdent extra indentation passed to str_wrap, and used if the output 
#' should spread over more than one lines.
#' 
#' @export
str_desc <- function(object, exdent=0L){
	p <- sapply(object, function(x){
				if( is.atomic(x) && length(x) == 1L ) x
				else paste("<", class(x), ">", sep='')
			})
	str_wrap(str_out(p, NA, use.names=TRUE, quote=FALSE), exdent=exdent)
}

#' @describeIn str_out extracts and formats a function signature.
#' It typically formats the output \code{capture.output(args(object))}.
#' 
#' @export
#' @examples 
#' str_fun(install.packages)
str_fun <- function(object){
	s <- capture.output(args(object))
	paste(s[-length(s)], collapse="\n")
}


#' @describeIn str_out outputs the class(es) of an object using \code{str_out}. 
#' 
#' @param ... other arguments passed to [str_out].
#' 
#' @export
#' @examples 
#' str_class(matrix())
str_class <- function(x, max = Inf, ...){
  str_out(class(x), max = max, ...)
  
}

#' @describeIn str_out formats a package name and version
#' 
#' @param pkg package name
#' @param lib.loc path to a library of R packages
#' 
#' @export 
str_pkg <- function(pkg, lib.loc = NULL){
    sprintf("%s (version %s)", pkg, packageVersion(pkg, lib.loc = lib.loc))
}

#' @describeIn str_out computes md5sum on character vector using \code{\link[tools]{md5sum}}.
#' 
#' @importFrom tools md5sum
#' @export
str_md5sum <- function(x){
    
    tmp <- tempfile()
    on.exit( unlink(tmp) )
    cat(x, sep = "\n", file = tmp)
    md5sum(tmp)
    
}

#' @describeIn str_out computes hash of a character vector using \code{\link[digest]{digest}}.
#' 
#' @inheritParams digest::digest
#' @import digest
#' @export
str_hash <- function(x, algo = 'md5'){
    
    digest(x, algo = algo, serialize = FALSE)
    
}

#' @describeIn str_out builds a string that describes the dimension of an object, in the form
#' `n x m` for 2D-objects, `n x m x p` for 3D-objects, and so on.
#' 
#' @param dims a numeric vector of dimensions.
#' Default is to use the input object dimensions (via function `dims()`)
#' 
#' @export
str_dim <- function(x, dims = dim(x)){
  if( !is.null(dims) ) paste0(dims, collapse = ' x ')
  else length(x)
}


# From example in ?toupper
capwords <- function(s, strict = FALSE) {
    cap <- function(s) paste(toupper(substring(s,1,1)),
                {s <- substring(s,2); if(strict) tolower(s) else s},
                sep = "", collapse = " " )
    sapply(strsplit(s, split = " "), cap, USE.NAMES = !is.null(names(s)))
}

#' Finding Differences Between Strings
#' 
#' Computes which characters differ between two strings.
#' 
#' @param x a single string
#' @param y a single string
#' @return an integer vector containing the index of all mis-matched characters
#' in the first string.
#' @export
#' 
#' @examples
#' 
#' # strings to compare
#' x <- "once upon a time"
#' y <- "once upon a time there was"
#' z <- "once upon two times"
#' 
#' # diff: x - y
#' d <- str_diff(x, y)
#' d
#' str(d)
#' 
#' # other comparisons 
#' str_diff(y, x)
#' str_diff(x, x)
#' str_diff(x, z)
#' str_diff(y, z)
#' 
str_diff <- function(x, y){
	sx <- strsplit(x,'')[[1]]
	sy <- strsplit(y,'')[[1]]
	n <- min(length(sx), length(sy))
	res <- mapply('!=', head(sx,n), head(sy,n))
	wres <- which(res)
	if( length(sx) > length(sy) )
		wres <- c(wres, (n+1):length(sx))
	attr(wres, 'str') <- list(x=x,y=y)
	class(wres) <- 'str_diff'
	wres
}

#' @export
print.str_diff <- function(x, ...){
	s <- attr(x, 'str')
	n <- max(nchar(s$x), nchar(s$y))
	d <- rep('.', n)
	d[x] <- '*'
	if( (n2 <- nchar(s$y)-nchar(s$x)) )
		d[(n-abs(n2)+1):n] <- if( n2 > 0L ) '-' else '+'
	cat(str_c(s$x, collapse=''), "\n")
	cat(str_c(d, collapse=''), "\n")
	cat(str_c(s$y, collapse=''), "\n")				
}

#' Extracting Local Function Definition
#' 
#' @description
#' \code{extractLocalFun} Extracts local function from wrapper functions of the following type, typically 
#' used in S4 methods:
#' \samp{
#' function(a, b, ...)\{
#' 	.local <- function(a, b, c, d, ...)\{\}
#'	.local(a, b, ...)
#' \}
#' }
#'
#' @param f definition of the wrapper function
#' 
#' @return a function
#' @export
#' @rdname formals
extractLocalFun <- function(f){
	bf <- body(f)
	
	txt <- as.character(bf)[2]
	# in R-2.14.2 -- at least, as.character does not return the complete body
	# so some text manipulation is necessary 
	if( !grepl("\\{", txt) ){
		sf <- capture.output(print(bf))
		w <- tail(grep("^\\s*\\.local\\(", sf), 1L)
		txt <- paste(sf[-w], collapse="\n")
	}
	expr <- parse(text=txt)
	e <- new.env()
	eval(expr, e)
} 

#' Extended Formal Extraction
#'
#' Works for methods that are created (setMethod) as a wrapper function to an 
#' internal function named .local.
#'
#' @inheritParams extractLocalFun
#' @return a paired list like the one returned by \code{\link{formals}}. 
#' 
#' @export
#' @import codetools
#' @rdname formals
allFormals <- function(f){
	
	# look inside method for S4 methods
	if( is(f, 'MethodDefinition') ){
		
		# check if the method is defined as a wrapper function
		f <- f@.Data
		lf <- try(codetools::getAssignedVar(body(f)), silent=TRUE)
		if( !identical(lf, '.local') ) return( formals(f) )
		# extract arguments from local function
		lfun <- extractLocalFun(f)
		res <- formals(lfun)
		# set default values from the generic, only for arguments that have no 
		# default values in the method
		generic_args <- formals(f)
		meth_no_default <- sapply(res, is.symbol) 
		gen_no_default <- sapply(generic_args, is.symbol)
		generic_args <- generic_args[ !gen_no_default ]
		generic_args <- generic_args[ names(generic_args) %in% names(res[meth_no_default]) ]
		if( length(generic_args) ){
			res[names(generic_args)] <- generic_args
		}
		# return complete list of arguments
		res
		
	}else if( is.function(f) ) formals(f)
	
}

#' Alternative S4 Constructor
#' 
#' An alternative version of \code{\link{new}} to create objects based on a list
#' of values. 
#' 
#' @param class Class name to instanciate
#' @param ... extra arguments from which slot values are extracted by exact 
#' matching of names.
#' 
#' @export
#' @examples
#' 
#' setClass('A', contain='character', representation(x='numeric', y='character'))
#' 
#' # identical behaviour with standard calls
#' identical(new('A'), new2('A'))
#' identical(new('A', x=1), new2('A', x=1))
#' 
#' # but if passing that are names not slots 
#' identical(new('A'), new2('A', b=1))
#' identical(new('A', x=1), new2('A', x=1, b=3))
#' identical(new('A', x=1), new2('A', x=1, b=3))
#' 
#' # standard `new` would coerce first unnamed argument into parent of 'A' (i.e. 'character') 
#' new('A', list(x=1))
#' new('A', list(x=1, y='other'))
#' # `new2` rather use it to initialise the slots it can find in the list 
#' identical(new('A', x=1), new2('A', list(x=1)))
#' identical(new('A', x=1, y='other'), new2('A', list(x=1, y='other')))
#' 
#' 
new2 <- function(class, ...){
	sl <- getSlots(class)
	if( nargs() == 1L ) return( new(class) )
	
	dots <- list(...)
	if( nargs() == 2L && is.null(names(dots)) ){
		l <- dots[[1]]
		if( !is.list(l) )
			stop("Invalid call: single unnamed argument must be a list")
		dots <- l
	}
	
	if( is.null(names(dots)) || any(names(dots)=='') )
		stop("Invalid call: all slot arguments must be named")
	dots <- dots[names(dots) %in% names(sl)]
	do.call('new', c(list(class), dots))
}


#' One-off Global Variables
#' 
#' Defines a function that allow to get/assign a global variable whose value is 
#' ensured to be reset after each access.
#'   
#' @param default default value to which the global variable is reset after each 
#' access. Default is \code{NULL}.
#' 
#' @return a function with one argument (\code{value}) that provides get/set access
#' to a global variable.
#' If called with a value, it assigns this value to the global variable.
#' If called with no argument, it returns the current value of the global variable and 
#' reset it to its default value -- as defined at its creation. 
#'
#' @export
#' 
#' @examples
#' 
#' x <- oneoffVariable(0)
#' # returns default value
#' x()
#' # assign a value
#' x(3)
#' # get the value
#' x()
#' # second call returns default value again 
#' x()
#'  
oneoffVariable <- function(default=NULL){
	.var <- default
	function(value){
		if( missing(value) ){
			res <- .var
			.var <<- default
			res
		}else
			.var <<- value
	}
}


##  Exit Error Checker
##  
##  This function defines a function that checks if an error has been 
##  thrown after its definition.
##  It may be used to perform tasks on function exit depending on 
##  how the function exit (normal return or with an error).
##  
##  The function \code{errorCheck} itself is meant to be called at 
##  the beginning of functions that use \code{\link{on.exit}} to 
##  perform tasks when exiting.
##  The error checker function returned, when used in \code{on.exit} 
##  expressions, enables to distinguish between a normal exit and 
##  an exit due to an error, allowing is to perform tasks specific 
##  to each scenario.
##  
##  IMPORTANT: this function is not 100\% perfect in the sense that 
##  it will detect an error as soon as one has been thrown, even it 
##  is catched before the exit -- with \code{\link{try}} or 
##  \code{\link{tryCatch}}.
##  
##  @export
##  @examples 
##  
##  # define some function
##  f <- function(err){
##  
##   # initialise an error checker
##  	isError <- errorCheck()
##  
##   # do something on exit that depends on the error status
##  	on.exit({
##  		if(isError()) cat("with error: cleanup\n") 
##  		else cat("no error: do nothing\n") 
##  	})
##  	
##   # throw an error here
##  	if( err ) stop('There is an error')
##   
##  	1+1
##  }
##  
##  # without error
##  f(FALSE)
##  # with error
##  try( f(TRUE) )
##  
#errorCheck <- function(){
#	
#	# initialise with unique error message
#	.err <- tryCatch(stop('ERROR_CHECK:', digest(tempfile())), error=function(e) conditionMessage(e))
#	tb_digest <- function() digest(capture.output(traceback(max.lines=NULL)))
#	.traceback <- tb_digest()
#	
#	function(){
#		# error message is different
#		# tb_digest() != .traceback
#		length(grep(.err, msg, fixed=TRUE, invert=TRUE)) == 1L
#	}
#}


#' Global Static Variable
#' 
#' \code{sVariable} defines a function that acts as a global
#' static variable.
#' 
#' @param default default value for the static variable. 
#' 
#' @export
#' @examples 
#' 
#' # define variable
#' x <- sVariable(1)
#' # get value (default)
#' x()
#' # set new value: return old value
#' old <- x(3)
#' old
#' # get new value
#' x()
#' 
sVariable <- function(default=NULL){
	.val <- default
	function(value){
		if( missing(value) ) .val
		else{
			old <- .val
			.val <<- value
			old
		}
	}
}

#' Exit Error Checks
#' 
#' \code{exitCheck} provides a mechanism to distinguish the exit status
#' in \code{\link{on.exit}} expressions.
#' 
#' It generates a function that is used wihtin a function's body to 
#' "flag" normal exits and in its \code{\link{on.exit}} expression
#' to check the exit status of a function.
#' Note that it will correctly detect errors only if all normal exit 
#' are wrapped into a call to it. 
#' 
#' @export
#' 
#' @examples
#' 
#' # define some function
#' f <- function(err){
#' 
#'  # initialise an error checker
#' 	success <- exitCheck()
#' 
#'  # do something on exit that depends on the error status
#' 	on.exit({
#' 		if(success()) cat("Exit with no error: do nothing\n") 
#' 		else cat("Exit with error: cleaning up the mess ...\n") 
#' 	})
#' 	
#'  # throw an error here
#' 	if( err ) stop('There is an error')
#'  
#' 	success(1+1)
#' }
#' 
#' # without error
#' f(FALSE)
#' # with error
#' try( f(TRUE) )
#' 
exitCheck <- function(){
	
	.success <- FALSE
	function(x){
		if( nargs() == 0L ) .success
		else{
			.success <<- TRUE
			x
		}
	}
}

#' Ordering Version Numbers
#' 
#' Orders a vector of version numbers, in natural order.
#' 
#' @param x a character vector of version numbers
#' @param decreasing a logical that indicates if the ordering should be decreasing
#' 
#' @export
#' @examples
#' 
#' v <- c('1.0', '1.03', '1.2')
#' order(v)
#' orderVersion(v)
#' 
orderVersion <- function(x, ..., decreasing=FALSE){
    
    NAs <- which(is.na(x))
	tx <- gsub("[^0-9]+",".", paste('_', x, sep=''))
	stx <- strsplit(tx, ".", fixed=TRUE)
	mtx <- max(sapply(stx, length))
	tx <- sapply(stx, 
			function(v) paste(sprintf("%06i", c(as.integer(v[-1]),rep(0, mtx-length(v)+1))), collapse='.')
	)	
	res <- order(tx, ..., decreasing = decreasing)
    # put NAs at the end
    if( length(NAs) ){
        res <- c(setdiff(res, NAs), NAs)
    }
    res
}

#' @param ... extra parameters passed to \code{orderVersion} and \code{\link{order}}
#' 
#' @export
#' @rdname orderVersion
#' @examples
#' 
#' sort(v)
#' sortVersion(v)
sortVersion <- function(x, ...){
	x[orderVersion(x, ...)]
}

#' Checking for Missing Arguments
#' 
#' This function is identical to \code{\link{hasArg}}, except that 
#' it accepts the argument name as a character string.
#' This avoids to have a check NOTE about invisible binding variable.  
#' 
#' @param name the name of an argument as a character string.
#' 
#' @export
#' @examples
#' 
#' f <- function(...){ hasArg2('abc') }
#' f(a=1)
#' f(abc=1)
#' f(b=1)
#' 
hasArg2 <- function (name) 
{
	name <- as.name(name)
	## apply methods::hasArg
	aname <- as.character(substitute(name))
	fnames <- names(formals(sys.function(sys.parent())))
	if (is.na(match(aname, fnames))) {
		if (is.na(match("...", fnames))) 
			FALSE
		else {
			dotsCall <- eval(quote(substitute(list(...))), sys.parent())
			!is.na(match(aname, names(dotsCall)))
		}
	}
	else eval(substitute(!missing(name)), sys.frame(sys.parent()))
	##
}

#' Exposing Object Attributes
#' 
#' The function \code{ExposeAttribute} creates an S3 object that 
#' exposes all attributes of any R object, by making them accessible via 
#' methods \code{\link{$}} and/or \code{\link{$<-}}.
#' 
#' @param object any R object whose attributes need to be exposed
#' @param ... attributes, and optionally their respective values or
#' access permissions.
#' See argument \code{value} of \code{attr_mode} for details on the
#' way of specifying these. 
#' @param .MODE access mode:
#' \describe{
#' \item{\dQuote{r}:}{ (read-only) only method \code{$} is defined}
#' \item{\dQuote{w}:}{ (write-only) only method \code{$<-} is defined}
#' \item{\dQuote{rw}:}{ (read-write) both methods \code{$} and \code{$<-} 
#' are defined}
#' } 
#' @param .VALUE logical that indicates if the values of named arguments 
#' in \code{...} should be considered as attribute assignments, 
#' i.e. that the result object has these attributes set with the specified values.
#' In this case all these attributes will have the access permission
#' as defined by argument \code{.MODE}.
#' 
#' @export
ExposeAttribute <- function(object, ..., .MODE='rw', .VALUE=FALSE){
	
	# setup exposed arguments
	args <- list(...)
	if( length(args) ){
		# use the same mode for all attributes
		if( isString(.MODE) == 1L )
			.MODE <- rep(.MODE, length(args))
		else if( length(.MODE) != length(args) ){
			stop("Argument .MODE must provide an access mode for each argument in `...`.")
		}
		
		if( is.null(names(args)) ) # add names if necessary
			args <- setNames(args, rep('', length(args)))
		un <- names(args)==''
		if( any(!sapply(args[un], isString)) )
			stop("All unnamed argument must be the name of an attribute, i.e. a character string.")
		
		# set attributes that have values if requested
		if( .VALUE ){
			sapply(names(args)[!un], function(x){
				attr(object, x) <<- args[[x]]
			})
		}else{ # or use the values as access permission
			.MODE[!un] <- args[!un]
		}
		#
		
		# store exposed attributes with names as regular expressions
		eargs <- ifelse(un, args, names(args))
		eargs <- as.list(setNames(.MODE, eargs))
		# add ereg start-end
		names(eargs) <- paste('^', names(eargs), '$', sep='')
	}else{
		eargs <- .MODE
	}
	
	# store access rights
	attr(object, '.ExposeAttribute') <- eargs
	class(object) <- c(class(object), 'ExposeAttribute')
	object
}

.getEAmode <- function(x, name, ..., RAW..=FALSE){
	ea <- attr(x, '.ExposeAttribute')
	if( is.null(ea) ) return()
	if( is.character(ea) && !RAW.. )
		ea <- list(`^.*$`=ea)
	if( missing(name) )	return(ea)

	name <- name[name != '.ExposeAttribute']
	# determine access mode
	m <- lapply(names(ea), function(p){
		m <- grep(p, name, value=TRUE)
		setNames(rep(ea[[p]], length(m)), m)
	})
	unlist(m)
	#
}

#' @export 
.DollarNames.ExposeAttribute <- function(x, pattern=""){ 
	
	att <- grep(pattern, names(attributes(x)), value=TRUE)
	if( nchar(pattern) > 1 )
		att <- unique(c(substring(pattern, 2), att))
	# filter out based on the access permissions
	mode <- .getEAmode(x, att)
	if( !length(mode) ) return(character())
	mode <- mode[mode != '']
	# add `<-` suffix to write only attributes
	if( length(wonly <- which(mode=='w')) )
		names(mode)[wonly] <- paste(names(mode)[wonly], '<- ')
	
	names(mode)
}

#' @export
`$.ExposeAttribute` <- function(x, name){
	if( is.null(attr(x, name)) )
		stop("Object `", deparse(substitute(x)),"` has no attribute '", name, "'.")
	mode <- .getEAmode(x, name)
	if( !length(mode) ){
		stop("Could not access attribute via `$`: attribute '", name, "' is not exposed. Use `attr(x, '", name, "').")
	}
	if( !any(grepl('r', mode)) ){
		stop("Could not access exposed attribute '", name, "': permission denied [mode='", mode,"'].")
	}
	attr(x, name)
	
}

#' @export
`$<-.ExposeAttribute` <- function(x, name, value){
	mode <- .getEAmode(x, name)
	if( !length(mode) ){
		stop("Could not write attribute via `$<-`: attribute '", name, "' is not exposed. Use `attr(x, '", name, "') <- value.")
	}
	if( !any(grepl('w', mode)) ){
		stop("Could not write attribute '", name, "': permission denied [mode='", mode,"'].")
	}
	attr(x, name) <- value
	x
}

#' @export
print.ExposeAttribute <- function(x, ...){
	# remove EA stuff 
	attr_mode(x) <- NULL
	# call next print method
	print(x, ...)
}

#' \code{attr_mode} and \code{attr_mode<-} get and sets the access mode of 
#' \code{ExposeAttribute} objects.
#' 
#' @param x an \code{ExposeAttribute} object
#' @param value replacement value for mode.
#' It can be \code{NULL} to remove the ExposeAttribute wrapper, 
#' a single character string to define a permission for all atributes
#' (e.g., \code{'rw'} or \code{'r'}), or a list specifying access permission 
#' for specific attributes or classes of attributes defined by regular expressions.
#' For example, \code{list(a='r', b='w', `blabla.*`='rw')} set attribute \code{'a'} 
#' as read-only, attribute \code{'b'} as write-only, all attributes that start with 
#' \code{'blabla'} in read-write access.
#' 
#' @export
#' @rdname ExposeAttribute
attr_mode <- function(x){
	.getEAmode(x, RAW..=TRUE)
}
#' @export
#' @rdname ExposeAttribute
`attr_mode<-` <- function(x, value){
	if( is.null(value) ){
		attr(x, '.ExposeAttribute') <- NULL
		class(x) <- class(x)[!class(x) %in% "ExposeAttribute"]
	}else if( isString(value) ){
		x <- ExposeAttribute(x, .MODE=value)
	}else if( is.list(value) ){
		args <- c(list(x), names(value), list(.MODE=setNames(value, NULL), .VALUE=FALSE))
		x <- do.call('ExposeAttribute', args)
	}else{
		stop("Invalid value: a character string or a list is expected")
	}
	x
}


#' Checking R User 
#' 
#' Tests if the current R user is amongst a given set of users.
#' 
#' @param user the usernames to check for, as a character vector. 
#' 
#' @export
userIs <- function(user){
	setNames(Sys.info()['user'], NULL) %in% user
}

#' Expanding Lists
#' 
#' \code{expand_list} expands a named list with a given set of default items,
#' if these are not already in the list, partially matching their names.  
#' 
#' @param x input list
#' @param ... extra named arguments defining the default items.
#' A list of default values can also be passed as a a single unnamed argument.
#' @param .exact logical that indicates if the names in \code{x} should be 
#' partially matched against the defaults.
#' @param .names logical that only used when \code{.exact=FALSE} and indicates
#' that the names of items in \code{x} that partially match some defaults should
#' be expanded in the returned list.
#' 
#' @return a list  
#' 
#' @export 
#' @examples 
#' 
#' expand_list(list(a=1, b=2), c=3)
#' expand_list(list(a=1, b=2, c=4), c=3)
#' # with a list
#' expand_list(list(a=1, b=2), list(c=3, d=10))
#' # no partial match
#' expand_list(list(a=1, b=2, c=5), cd=3)
#' # partial match with names expanded
#' expand_list(list(a=1, b=2, c=5), cd=3, .exact=FALSE)
#' # partial match without expanding names
#' expand_list(list(a=1, b=2, c=5), cd=3, .exact=FALSE, .names=FALSE)
#' 
#' # works also inside a function to expand a call with default arguments
#' f <- function(...){
#' 	cl  <- match.call()
#' 	expand_list(cl, list(a=3, b=4), .exact=FALSE)
#' }
#' f()
#' f(c=1)
#' f(a=2)
#' f(c=1, a=2)
#' 
expand_list <- function(x, ..., .exact=TRUE, .names=!.exact){
	
	# extract defaults from ... arguments
	defaults <- list(...)
	if( length(defaults) == 1L && is.null(names(defaults)) ){
		defaults <- defaults[[1L]]
	}
	# early exit if no defaults
	if( !length(defaults) ) return(x)
	
	# match names from x in defaults
	x_ex <- x
	if( !.exact ){
		i <- pmatch(names(x), names(defaults))
		# first expand names if necessary
		if( length(w <- which(!is.na(i))) ){
			names(x_ex)[w] <- names(defaults)[i[w]]
			# apply to as well if necessary
			if( .names ) names(x)[w] <- names(defaults)[i[w]]
		}
	}
	
	# expand list
	i <- match(names(defaults), names(x_ex))
	if( length(w <- which(is.na(i))) ){
		n <- names(defaults)[w]
		lapply(n, function(m){
			if( is.null(defaults[[m]]) ) x[m] <<- list(NULL) 
			else x[[m]] <<- defaults[[m]]
		})
	}
	
	x
}

#' @describeIn expand_list expands the \code{...} arguments of the function
#' in which it is called with default values, using \code{expand_list}.
#' It can \strong{only} be called from inside a function.
#' 
#' @param .exclude optional character vector of argument names to exclude 
#' from expansion. 
#'
#' @export
#' @examples
#' # expanding dot arguments
#' 
#' f <- function(...){ 
#' 	expand_dots(list(a=2, bcd='a', xxx=20), .exclude='xxx') 
#' }
#' 
#' # add default value for all arguments 
#' f()
#' # add default value for `bcd` only
#' f(a=10)
#' # expand names
#' f(a=10, b=4)
#' 
expand_dots <- function(..., .exclude=NULL){
	
	dotsCall <- as.list(eval(quote(substitute(list(...))), sys.parent()))
	if( length(dotsCall) >= 1L ) dotsCall <- dotsCall[-1L]
	
	# extract defaults from ... arguments
	defaults <- list(...)
	if( length(defaults) == 1L && is.null(names(defaults)) ){
		defaults <- defaults[[1L]]
	}
	if( length(defaults) ){
		excl <- names(allFormals(sys.function(sys.parent())))
		if( !is.null(.exclude) ) excl <- c(excl, .exclude)
		defaults <- defaults[!names(defaults) %in% excl]
		dotsCall <- expand_list(dotsCall, defaults, .exact=FALSE)
	}
	#
	
	# return expanded dot args
	dotsCall
}

#' Check Environment Variables
#' 
#' Tells if some environment variable(s) are defined.
#' 
#' @param x environment variable name, as a character vector.
#' 
#' @export
#' @examples 
#' 
#' hasEnvar('_R_CHECK_TIMINGS_')
#' hasEnvar('ABCD')
#' 
hasEnvar <- function(x){
	is.na(Sys.getenv(x, unset = NA, names = FALSE))
}


#' Substituting Strings Against a Mapping Table
#' 
#' Match the elements of a character vectors against a mapping table, 
#' that can specify multiple exact or partial matches.
#' 
#' @param x character vector to match
#' @param maps mapping tables.
#' May be a character vector, a list of character vectors or a function.
#' @param nomatch character string to be used for non-matched elements of \code{x}.
#' If \code{NULL}, these elements are left unchanged.
#' @param partial logical that indicates if partial matches are allowed, 
#' in which case mappings are used as regular expressions. 
#' @param rev logical that indicates if the mapping should be interpreted in the 
#' reverse way. 
#' 
#' 
#' @export
charmap <- function(x, maps, nomatch = NULL, partial = FALSE, rev = FALSE){
	
	x <- as.character(x)
	res <- setNames(as.character(rep(NA, length(x))), x)
	if( !is.list(maps) ) maps <- list(maps)
	for( k in seq_along(maps) ){
		# stop as soon as all type is mapped
		if( !length(i <- which(is.na(res))) ) break;
		
		# match unmapped type
		ct <- names(res)[i]
		map <- maps[[k]]
		if( is.function(map) ){
			m <- map(ct)
			
		}else if( is.character(map) ){
			if( is.null(names(map)) ) map <- setNames(rep(names(maps)[k], length(map)), map)
			m <- .charmap(ct, map, partial = partial, rev = rev)
			
		}else if( is.list(map) ){
			map <- unlist2(map)
			m <- .charmap(ct, setNames(names(map), map), partial = partial, rev = rev)
			
		}else stop("Invalid cell type map [", class(map), ']')
		# update result map
		if( !is.null(m) ) res[i] <- m
	}
	
	if( anyNA(res) ){
		i <- is.na(res)
		if( is.null(nomatch) ) res[i] <- x[i]
		else res[i] <- nomatch
	}
	res
}


.charmap <- function(x, map, partial = FALSE, rev = FALSE){
	map <- if( !rev ) setNames(as.character(map), names(map))
			else if( !is.null(names(map)) ) setNames(names(map), as.character(map))
			else stop("Impossible to map data: the provided map has no names.")
	
	if( isFALSE(partial) ) i <- match(tolower(x), tolower(names(map)))
	else if( isTRUE(partial) ){
		i <- sapply(tolower(x), function(x){
					m <- pmatch(tolower(names(map)), x)
					i <- which(!is.na(m))[1L]
					if( !length(i) ) NA else i
				})
	}else{
		i <- rep(NA, length(x))
		sapply(seq_along(map), function(j){
					e <- names(map)[j]
					mi <- grep(e, x)
					if( length(mi) ) i[mi] <<- j 
				}) 
	}
	ok <- !is.na(i)
	i[ok] <- as.character(map[i[ok]])
	as.character(i)
}


#' Open a File Graphic Device
#'
#' Opens a graphic device depending on the file extension.
#' 
#' @param filename path to the image file to create.
#' @param width output width
#' @param height output height
#' @param ... other arguments passed to the relevant device function
#' such as \code{\link{png}} or \code{\link{pdf}}.
#' 
#' importFrom grDevices bmp jpeg pdf png svg tiff
#' @export
gfile <- function(filename, width, height, ...){ 
  # Get file type
  r = regexpr("\\.[a-zA-Z]*$", filename)
  if(r == -1) stop("Improper filename")
  ending = substr(filename, r + 1, r + attr(r, "match.length"))
  
  f = switch(ending,
      pdf = function(x, ...) pdf(x, ...),
      svg = function(x, ...) svg(x, ...),			
      png = function(x, ...) png(x, ...),
      jpeg = function(x, ...) jpeg(x, ...),
      jpg = function(x, ...) jpeg(x, ...),
      tiff = function(x, ...) tiff(x, compression = "lzw", ...),
      bmp = function(x, ...) bmp(x, ...),
      stop("File type should be: pdf, svg, png, bmp, jpg, tiff")
  )
  
  args <- c(list(filename), list(...))	
  if( !missing(width) ){
    args$width <- as.numeric(width)
    args$height <- as.numeric(height)
    if( !ending %in% c('pdf','svg') && is.null(args[['res']]) ){
      args$units <- "in"
      args$res <- 300
    }
  }
  do.call('f', args)	
}

#' Flatten a List Conserving Names 
#' 
#' `unlist2` is a replacement for [base::unlist] that does not mangle the names.
#' 
#' Use this function if you don't like the mangled names returned by the standard `unlist` function from the base package. 
#' Using `unlist` with annotation data is dangerous and it is highly recommended to use `unlist_` instead.
#' 
#' @inheritParams AnnotationDbi::unlist2
#' 
#' @author Herve Pages
#' @source Bioconductor AnnotationDbi::unlist2
#' @export
#' @examples 
#' x <- list(A=c(b=-4, 2, b=7), B=3:-1, c(a=1, a=-2), C=list(c(2:-1, d=55), e=99))
#' unlist(x)
#' unlist_(x)
#' 
#' # annotation maps (as in AnnotationDbi objects
#' egids2pbids <- list('10' = 'a', '100' = c('b', 'c'), '1000' = c('d', 'e'))
#' egids2pbids
#' 
#' unlist(egids2pbids)   # 1001, 1002, 10001 and 10002 are not real
#'                       # Entrez ids but are the result of unlist()
#'                       # mangling the names!
#' unlist_(egids2pbids)  # much cleaner! yes the names are not unique
#'                       # but at least they are correct...
#' 
unlist_ <- function (x, recursive = TRUE, use.names = TRUE, what.names = "inherited") 
{
  ans <- unlist(x, recursive, FALSE)
  if (!use.names) 
    return(ans)
  if (!is.character(what.names) || length(what.names) != 1) 
    stop("'what.names' must be a single string")
  what.names <- match.arg(what.names, c("inherited", "full"))
  names(ans) <- unlist(make.name.tree(x, recursive, what.names), 
      recursive, FALSE)
  ans
}

# taken from Bioconductor AnnotatiobnDbi::make.name.tree
make.name.tree <- function (x, recursive, what.names) 
{
  if (!is.character(what.names) || length(what.names) != 1) 
    stop("'what.names' must be a single string")
  what.names <- match.arg(what.names, c("inherited", "full"))
  .make.name.tree.rec <- function(x, parent_name, depth) {
    if (length(x) == 0) 
      return(character(0))
    x_names <- names(x)
    if (is.null(x_names)) 
      x_names <- rep.int(parent_name, length(x))
    else if (what.names == "full") 
      x_names <- paste0(parent_name, x_names)
    else x_names[x_names == ""] <- parent_name
    if (!is.list(x) || (!recursive && depth >= 1L)) 
      return(x_names)
    if (what.names == "full") 
      x_names <- paste0(x_names, ".")
    lapply(seq_len(length(x)), function(i) .make.name.tree.rec(x[[i]], 
              x_names[i], depth + 1L))
  }
  .make.name.tree.rec(x, "", 0L)
}

#' Reordering Columns
#' 
#' Reorders columns according to a prefered target order
#' 
#' Column names will be reordered so that their order match the one in `target`.
#' Any column that does not appear in `target` will be put after those that are
#' listed in `target`.
#' 
#' @param x an object with columns, such as a `matrix` or a `data.frame`, 
#' or from a class that support subsetting via `x[, i, drop = FALSE]` and has a method `colnames`.
#' @param target a character or named numeric vector that specifies the column prefered order.
#' If a numeric vector, then its names are assumed to correspond to columns, 
#' and its values determine the target order -- according to argument `decreasing`.
#' @param decreasing logical that indicates in which direction a numeric target vector should
#' be ordered.
#' 
#' @return an object of the same type and dimension 
#' 
#' @export
#' 
reorder_columns <- function(x, target, decreasing = FALSE){
  if( is.numeric(target) ){
    target <- names(target)[order(target, decreasing = decreasing)]
    
  }
  
  x[, order(match(colnames(x), target)), drop = FALSE]
  
}

#' Converting Factors to Character Vectors
#' 
#' Converts all `factor` variables to character vectors in a `data.frame`
#' or phenotypic data.
#' 
#' @param x `data.frame` or `ExpressionSet` object
#' 
#' @return an object of the same class as `x`.
#'  
#' @export 
factor2character <- function(x){
  if( is(x, 'ExpressionSet') ){
    if( !requireNamespace('Biobase') ){
      stop("Missing dependency: package 'Biobase' is required to handle ExpressionSet objects.\n"
              , "  Try installing with: source('https://bioconductor.org/biocLite.R')")
      
    }
    Biobase::pData(x) <- factor2character(Biobase::pData(x))
    return(x)
  }
  
  for(v in colnames(x)){
    if( is.factor(x[[v]]) ) x[[v]] <- as.character(x[[v]]) 
  }
  x
  
}

#' Compute Function Digest Hash
#' 
#' Computes a digest hash of the body and signature of a function.
#' Note that the hash is not affected by attributes or the 
#' function's environment.
#' 
#' The hash itself is computed using [digest::digest].
#' 
#' @param fun a function
#' @param n a single numeric that indicates the length of the hash.
#' 
#' @return a character string
#' 
#' @import digest
#' @importFrom assertthat is.number
#' @export 
digest_function <- function(fun, n = Inf){
  
  assert_that(is.number(n) & n>0, msg = "Invalid argument 'n': must be Inf or a positive number.")
  # get function body (handle primitive in a special way)
  bd <- if( !is.primitive(fun) ) body(fun) else capture.output(fun)
  attributes(bd) <- NULL
  bd <- as.character(bd)
  # include formals in the hash for non-primitive functions
  if( !is.primitive(fun) ) bd <- list(bd, formals(fun))
  hash <- digest::digest(bd)
  # limit to a given size if requested
  if( is.finite(n) ) hash <- substr(hash, 1L, n)
  hash
  
}

#' System Call Stack Utilities
#' 
#' @name sys_call_stack
NULL

#' @describeIn sys_call_stack computes digest hash for each function in the call stack.
#' @param n a single frame
#' @export
sys.function_digest <- function(n = NULL){
  assert_that(is.null(n) || (is.number(n) & n>0), msg = "Invalid argument 'n': must be NULL or a positive number.")
  n <- n %||% sys.nframe()
  sapply(1:n, function(i){
        f <- sys.function(i)
        digest_function(f)
        
      })
  
}

#' @describeIn sys_call_stack returns the index of the frame that calls a given function.
#' @param fun the function object to find in the call stack.
#' @export 
sys.function_nframe <- function(fun){
  which(sys.function_digest() == digest_function(fun))
  
}

#' @describeIn sys_call_stack returns the frame that calls a given function.
#' @export
sys.function_frame <- function(fun){
  n <- sys.function_nframe(fun)
  if( length(n) > 1L )
    warning(sprintf("Multiple call frames found for target function '%s': using last call.", digest_function(fun)))
  else if( !length(n) )
    stop(sprintf("No call frames found for target function '%s'", digest_function(fun)))
  
  sys.frame(n)
  
}

#' @describeIn sys_call_stack returns path to the script that is being sourced either
#' by [base::source] or [base::sys.source].
#' @export
sys.source_file <- function(){
  res <- try(sys.function_frame(base::source), silent = TRUE)
  if( is(res, "try-error") ){
    res <- try(sys.function_frame(base::sys.source), silent = TRUE)
    if( is(res, "try-error") ) stop("Could not find call frame for 'source' or 'sys.source'")
    res[["file"]]
    
  }else{
    if( isString(res[["ofile"]]) ) res[["ofile"]]
    else res[["filename"]]
    
  }
  
}

# Data related functions 
# 
# Author: Renaud Gaujoux
# Creation: 18 Jun 2012
###############################################################################

#' Generating Names
#' 
#' Generates names or dimnames for objects.
#' 
#' @param x object whose names are generated.
#' 
#' @export
#' @rdname addnames
addnames <- function(x, ...){
	UseMethod('addnames')
}

#' @export
#' @rdname addnames
addnames.default <- function(x, ...){
	if( is.vector(x) ) addnames.vector(x, ...)
	else 
		stop("no applicable method for 'addnames' applied to an object of class ", class(x))
}

#' @param prefix prefix string to use. A vector can be used to specify a prefix 
#' for each dimension of \code{x}. 
#' Names are build as \code{<prefix><sep><index>}.
#' @param sep separator used between the prefix and the numeric index. 
#' @param ... extra arguments to allow extension and passed to the next method.
#' 
#' @export
#' @rdname addnames
addnames.vector <- function(x, prefix='x', sep='', ...){
	names(x) <- paste(prefix, 1:length(x), sep=sep) 
	x
} 


#' @export
#' @rdname addnames
addnames.array <- function(x, prefix=letters[1:length(dim(x))], sep='', ...){
	d <- dim(x)
	# recycle prefix if necessary
	if( length(prefix) != length(d) )
		prefix <- rep(prefix, length(d))
	
	dimnames(x) <- 
			lapply(seq_along(d), function(i) paste(prefix[i], 1:d[i], sep=sep))
	x
} 

#' @export
#' @rdname addnames
addnames.matrix <- function(x, prefix=c('row', 'col'), ...){
	addnames.array(x, prefix=prefix, ...)
}


#' Flattens All List Levels Using Separated Names
#' 
#' @param x a list object, usually containing other lists -- of lists.
#' @param sep character string used to separate each component of the final element names.
#' @param use.names logical that indicates if the original names of each the sucessive
#' nested list elements should be used to build the final names of the result list.
#' @param depth maximum number of levels to unlist. 
#' Root level is `1L`. 
#' 
#' @export
#' @examples 
#' 
#' x <- list(X = list(a = 1
#'                    , b = list(b.1 = 2
#'                               , b.2 = list(b.2.1 = 4, b.2.2 = data.frame())
#'                               , b.3 = 3)
#'                    , c = matrix()))
#' unlist_with_sep(x)
#' unlist_with_sep(x, '###')
#' 
unlist_with_sep <- function(x, sep = '/', use.names = TRUE, depth = Inf){
  
  # early exit for non-list input
  if( !is.list(x) ) return(x)
  
  .local <- function(y, n = 1L){
    if( is.null(names(y)) || !use.names ) names(y) <- seq_along(y)
    if( anyDuplicated(names(y)) ){
      stop(sprintf("Invalid names at level %i: foudn duplucated names %s", n, str_out(unique(names(y)[duplicated(names(y))]), Inf)))
    }
    
    leaves <- c()
    res <- lapply(seq_along(y), function(i){
          m <- names(y)[i]
          e <- y[[i]]
          if( is.list(e) && !is.data.frame(e) && n < depth){
            e <- .local(e, n+1L)
            names(e) <- paste(m, names(e), sep = sep)
            
          }else{
            e <- y[i]
            leaves <<- c(leaves, i)
          }
          
          e
        })
    
#    # use original names for leaves
#    names(res) <- rep('', length(res))
#    names(res)[leaves] <- names(y)[leaves]
    
    # concatenate result
    do.call(c, res)
    
  }
  
  .local(x)
  
}

# Unit test for data utilities
# 
# Author: Renaud Gaujoux
###############################################################################

context('Data utilities')

test_that('unlist_with_sep', {
      
  l1 <- list(X = list(a = 1, b = list(b.1 = 2, b.2 = list(b.2.1 = 4, b.2.2 = data.frame()), b.3 = 3), c = matrix()))
  ul1 <- list(`X/a` = 1
      , `X/b/b.1` = 2, `X/b/b.2/b.2.1` = 4, `X/b/b.2/b.2.2` = data.frame()
      , `X/b/b.3` = 3
      , `X/c` = matrix())
  
  # defaults
  expect_identical( unlist_with_sep(l1), ul1, "List is correctly flatten")
  expect_identical( unlist_with_sep(matrix()), matrix(), "Result is identical to input for non list objects")
  # depth
  expect_identical( unlist_with_sep(l1, depth = 0L), l1, "Result is identical to input for depth = 0L")
  expect_identical( unlist_with_sep(l1, depth = 1L), l1, "Result is identical to input for depth = 1L")
  ul_depth <- list(`X/a` = 1, `X/b` = list(b.1 = 2, b.2 = list(b.2.1 = 4, b.2.2 = data.frame()), b.3 = 3), `X/c` = matrix())
  expect_identical( unlist_with_sep(l1, depth = 2L), ul_depth, "Result is correct for depth = 2L")
  ul_depth2 <- list(`X/a` = 1, `X/b/b.1` = 2, `X/b/b.2` = list(b.2.1 = 4, b.2.2 = data.frame()), `X/b/b.3` = 3, `X/c` = matrix())
  expect_identical( unlist_with_sep(l1, depth = 3L), ul_depth2, "Result is correct for depth = 2L")
  # sep
  ul2 <- ul1
  names(ul2) <- gsub("/", "#", names(ul2))
  expect_identical( unlist_with_sep(l1, sep = "#"), ul2, "Separator argument is honoured")
  # use.names
  ul_numbered <- ul1
  names(ul_numbered) <- c('1/1', '1/2/1', '1/2/2/1', '1/2/2/2', '1/2/3', '1/3')
  expect_identical( unlist_with_sep(l1, use.names = FALSE), ul_numbered, "Argument use.names works correctly")
  # levels with empty names are corrected
  l_empty <- l1
  names(l_empty$X$b) <- NULL
  ul_empty <- ul1
  names(ul_empty) <- c('X/a', 'X/b/1', 'X/b/2/b.2.1', 'X/b/2/b.2.2', 'X/b/3', 'X/c')
  expect_identical( unlist_with_sep(l_empty), ul_empty, "Empty intermediate names are filled with incremental numbers")
  # check error on duplicated levels
  l2 <- l1
  names(l2$X)[2] <- 'c'
  expect_error( unlist_with_sep(l2), "Invalid names at level 2.*'c'") 

})

%\VignetteIndexEntry{Overview of Utilities in the pkgmaker Package}
%\VignetteDepends{knitr,bibtex}
%\VignetteCompiler{knitr}
%\VignetteEngine{knitr::knitr}
 
\documentclass[a4paper]{article}
\usepackage[colorlinks]{hyperref} % for hyperlinks
\usepackage{a4wide}
\usepackage{xspace}
\usepackage[all]{hypcap} % for linking to the top of the figures or tables

% add preamble from pkgmaker
<<pkgmaker_preamble, message = FALSE, echo=FALSE, results='asis'>>=
library(pkgmaker)
pkgmaker::latex_preamble()
@

\title{Overview of Utilities in the pkgmaker Package\\
\small Version \Sexpr{utils::packageVersion('pkgmaker')}}
\author{Renaud Gaujoux}

\begin{document}

\maketitle

\section{Knitr hooks}

\subsection{hook\_try: showing error message in try statements}

This hooks aims at showing the error message generated by \code{try}
blocks, which are otherwise not shown in the generated document.

\subsubsection*{Hook registration}
<<hook_try>>=
library(knitr)
knit_hooks$set(try = pkgmaker::hook_try)
@

\subsubsection*{Usage}
<<without_try>>=
try( stop('this error will not appear in the document'))
@

Using chunk option \code{try = TRUE}, the error message is shown in the
document:
<<with_try, try = NA>>=
txt <- 'this error will be shown'
try( stop(txt) )
@

Note that by default, the error message is not highlighted by \code{knitr} as an
error but only as a normal message.
This is because highlighting apparently relies on an actual error signal
condition being raised, which in turn makes the package building process \code{R CMD build} fail.
However, it is still possible to hightlight the error when generating a document
outside a package, by using the chunk option \code{try = FALSE}:

<<with_try_highlight, try = TRUE>>=
txt <- 'this error will be shown'
try( stop(txt) )
@

\pagebreak 
\section{Session Info}
<<sessionInfo, echo=FALSE, results='asis'>>=
toLatex(sessionInfo())
@

\end{document}

\documentclass[10pt]{article}
%\VignetteDepends{knitr}
%\VignetteIndexEntry{pkgmaker-unitTests}
%\VignetteCompiler{knitr}
%\VignetteEngine{knitr::knitr}
\usepackage{vmargin}
\setmargrb{0.75in}{0.75in}{0.75in}{0.75in}

<<setup, include=FALSE>>=
pkg <- 'pkgmaker'
require( pkg, character.only=TRUE )
prettyVersion <- packageDescription(pkg)$Version
prettyDate <- format(Sys.Date(), '%B %e, %Y')
authors <- packageDescription(pkg)$Author
@

\usepackage[colorlinks]{hyperref}
\author{\Sexpr{authors}}
\title{\texttt{\Sexpr{pkg}}: Unit testing results\footnote{Vignette computed  on Wed May 14 13:48:07 2014}}
\date{\texttt{\Sexpr{pkg}} version \Sexpr{prettyVersion} as of \Sexpr{prettyDate}}
\begin{document}
\maketitle

\section{Details}
\begin{verbatim}

RUNIT TEST PROTOCOL -- Wed May 14 13:48:07 2014 
*********************************************** 
Number of test functions: 6 
Number of errors: 0 
Number of failures: 0 

 
1 Test Suite : 
package:pkgmaker - 6 test functions, 0 errors, 0 failures



Details 
*************************** 
Test Suite: package:pkgmaker 
Test function regexp: ^test. 
Test file regexp: ^runit.*.[rR]$ 
Involved directory: 
/tmp/Rpkglib_74c07de1ec8/pkgmaker/tests 
--------------------------- 
Test file: /tmp/Rpkglib_74c07de1ec8/pkgmaker/tests/runit.options.r 
test.option_link: (12 checks) ... OK (0.01 seconds)
test.resetOptions: (9 checks) ... OK (0 seconds)
--------------------------- 
Test file: /tmp/Rpkglib_74c07de1ec8/pkgmaker/tests/runit.utils.r 
test.errorCheck: (8 checks) ... OK (0 seconds)
test.ExposeAttribute: (20 checks) ... OK (0.01 seconds)
test.str_bs: (7 checks) ... OK (0 seconds)
test.Sys.getenv_value: (6 checks) ... OK (0 seconds)

Total execution time
***************************
   user  system elapsed 
  0.187   0.000   0.187 

\end{verbatim}

\section*{Session Information}
\begin{itemize}\raggedright
  \item R version 3.1.0 (2014-04-10), \verb|x86_64-pc-linux-gnu|
  \item Locale: \verb|LC_CTYPE=en_US.UTF-8|, \verb|LC_NUMERIC=C|, \verb|LC_TIME=en_US.UTF-8|, \verb|LC_COLLATE=en_US.UTF-8|, \verb|LC_MONETARY=en_US.UTF-8|, \verb|LC_MESSAGES=en_US.UTF-8|, \verb|LC_PAPER=en_US.UTF-8|, \verb|LC_NAME=C|, \verb|LC_ADDRESS=C|, \verb|LC_TELEPHONE=C|, \verb|LC_MEASUREMENT=en_US.UTF-8|, \verb|LC_IDENTIFICATION=C|
  \item Base packages: base, datasets, graphics, grDevices, methods,
    stats, utils
  \item Other packages: pkgmaker~0.22, registry~0.2, RUnit~0.4.26,
    stringr~0.6.2
  \item Loaded via a namespace (and not attached): codetools~0.2-8,
    digest~0.6.4, tools~3.1.0, xtable~1.7-3
\end{itemize}

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\name{simpleRegistry}
\alias{simpleRegistry}
\title{Simple Package Registry}
\usage{
simpleRegistry(name, envir = topenv(parent.frame()), verbose = FALSE)
}
\arguments{
\item{name}{name of the registry object, with which it will
be assigned in \code{envir}.}

\item{envir}{environment where to store the registry object.
Defaults to the caller's top environment.}

\item{verbose}{logical that toggle a verbose message when
the object is first created.}
}
\value{
a simple registry object that is similar to an R5 object.
}
\description{
Simple Package Registry
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/colors.R
\name{alphacol}
\alias{alphacol}
\title{Colour utilities}
\usage{
alphacol(col, alpha = FALSE)
}
\arguments{
\item{col}{vector of any of the three kinds of \R color specifications,
    i.e., either a color name (as listed by \code{\link[grDevices]{colors}()}), a
    hexadecimal string of the form \code{"#rrggbb"} or
    \code{"#rrggbbaa"} (see \code{\link[grDevices]{rgb}}), or a positive integer
    \code{i} meaning \code{\link[grDevices]{palette}()[i]}.}

\item{alpha}{logical value indicating whether the alpha channel (opacity)
    values should be returned.}
}
\value{
a character vector of color specifications.
}
\description{
\code{alphacol} adds an alpha value to a colour specification and convert to
a hexadecimal colour string.
}
\examples{

# Alphas
alphacol('red') # do nothing
alphacol('red', 10)
alphacol('#aabbcc', 5)
alphacol(4, 5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{utestPath}
\alias{utestPath}
\title{Unit Tests Result Directory}
\usage{
utestPath(...)
}
\arguments{
\item{...}{extra arguments passed to \code{\link{packagePath}}, e.g., \code{package}.}
}
\value{
The path tot the unit-test results.
}
\description{
Returns the path to the directory where the results of unit tests are stored.
This path is used by \code{\link{utest}} to save unit test results, which are
read by \code{\link{makeUnitVignette}} to update the unit test vignette when
runnning R CMD check.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{unlist_with_sep}
\alias{unlist_with_sep}
\title{Flattens All List Levels Using Separated Names}
\usage{
unlist_with_sep(x, sep = "/", use.names = TRUE, depth = Inf)
}
\arguments{
\item{x}{a list object, usually containing other lists -- of lists.}

\item{sep}{character string used to separate each component of the final element names.}

\item{use.names}{logical that indicates if the original names of each the sucessive
nested list elements should be used to build the final names of the result list.}

\item{depth}{maximum number of levels to unlist.
Root level is \code{1L}.}
}
\description{
Flattens All List Levels Using Separated Names
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


x <- list(X = list(a = 1
                   , b = list(b.1 = 2
                              , b.2 = list(b.2.1 = 4, b.2.2 = data.frame())
                              , b.3 = 3)
                   , c = matrix()))
unlist_with_sep(x)
unlist_with_sep(x, '###')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{packageTestEnv}
\alias{packageTestEnv}
\title{Returns the package internal environment where unit tests are stored.}
\usage{
packageTestEnv(pkg)
}
\arguments{
\item{pkg}{package name.
If missing the caller's package is assumed.}
}
\value{
An environment.
}
\description{
Returns the package internal environment where unit tests are stored.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/files.R
\name{source_files}
\alias{source_files}
\title{Source Multiple Files}
\usage{
source_files(x, pattern = NULL, ...)
}
\arguments{
\item{x}{character vector containing filenames}

\item{pattern}{an optional \link[base]{regular expression}.  Only file names
    which match the regular expression will be returned.}

\item{...}{extra arguments passed to \code{\link{source}}.}
}
\value{
the return value of running \link{source} on each individual file.
}
\description{
Vectorised version of \code{source}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{regfetch}
\alias{regfetch}
\alias{pkgreg_fetch}
\alias{pkgreg_remove}
\title{Finds an entry in a registry.}
\usage{
regfetch(
  regobj,
  ...,
  all = FALSE,
  error = TRUE,
  exact = FALSE,
  KEYS = NULL,
  verbose = FALSE,
  entry = FALSE,
  msg = NULL
)

pkgreg_fetch(regname, ..., msg = NULL, where = topenv(parent.frame()))

pkgreg_remove(
  regname,
  ...,
  msg = NULL,
  where = topenv(parent.frame()),
  quiet = FALSE
)
}
\arguments{
\item{regobj}{a registry object}

\item{...}{key value(s) to look up.
If multiple indexes are used, then the primary key should come first.}

\item{all}{logical to indicate if hidden keys (starting with a '.') should be
returned and output in message.}

\item{error}{a logical that indicates if an error should be thrown if the key has no match
or multiple matches}

\item{exact}{a logical that indicates if matching should be exact or partial.
Note that if exact matches exist then they are returned, independently of the
value of \code{exact}.}

\item{KEYS}{alternative way of passing the key value(s).
If not missing, then arguments in \code{...} are discarded.}

\item{verbose}{a logical that indicates if verbosity should be toggle on}

\item{entry}{a logical that indicates if the}

\item{msg}{a header to use in case of error.}

\item{regname}{Name of a sub-registry, used as its identifier.}

\item{where}{package name or namespace that owns the registry.}

\item{quiet}{a logical that indicates if the operation should be performed quietly,
without throwing errors or warnings.}
}
\value{
\code{regfetch} returns a registry entry.
}
\description{
This function provides extra control on how entries are queried
from a \code{\link[registry:regobj]{registry}} object.
}
\section{Functions}{
\itemize{
\item \code{pkgreg_fetch()}: fetches entries in a package registry, as set up by
\code{\link{setPackageRegistry}}.

It loads the requested package registry and uses \code{regfetch}
to retrieve data from it.

\item \code{pkgreg_remove()}: removes an entry from a package registry.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{parsePackageCitation}
\alias{parsePackageCitation}
\title{Formatting Package Citations in Sweave/knitr Documents}
\usage{
parsePackageCitation(x)
}
\arguments{
\item{x}{output document, as a single string.}
}
\value{
A character vecotr of citation references.
}
\description{
Formatting Package Citations in Sweave/knitr Documents
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/user.R
\name{userData}
\alias{userData}
\title{User Data Directory

\code{userData} returns the path to a local directory/file where package-related user data can be stored.
Note that a base directory is \strong{always} created if necessary (see details).}
\usage{
userData(..., create = NULL, package = topenv(parent.frame()))
}
\arguments{
\item{...}{path parts passed to \code{\link{file.path}} to be appended to
the main path.}

\item{create}{logical that indicates if the \strong{base} directory should be
created if it does not exists.

Note that directories -- and files -- under the base directory are not automatically
created. The user should therefore care of it in the caller function if necessary.

If \code{create=TRUE}, then the base directory is forced to be created in the user's home directory.
If \code{create=FALSE}, then the base directory is never created.

See also section \emph{Details}.}

\item{package}{name of the package associated with the user data path.
It is used to prefix the path, within the user R data directory.}
}
\value{
Path to the user data directory.
}
\description{
The package-specific user data base directory is the sub-directory \emph{R-data/},
located in the user's home or within a diredctory defined by global option 'userData.path'.

If in interactive mode, and the base directory does not exist yet,
the user is asked if it should be created in his home directory.
Otherwise, or if the user does not allow the creation in his home, this directory is created
in the current R session's temporary directory.
}
\seealso{
\code{\link{tempdir}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cgetAnywhere}
\alias{cgetAnywhere}
\title{Get Anywhere}
\usage{
cgetAnywhere(x)
}
\arguments{
\item{x}{a single character string}
}
\description{
Similar to \code{\link{getAnywhere}}, but looks for the value of its argument.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{unit.test}
\alias{unit.test}
\title{Embedded Unit Tests}
\usage{
unit.test(x, expr, framework = NULL, envir = parent.frame())
}
\arguments{
\item{x}{single character string used as test identifier/label}

\item{expr}{expression containing the actual test commands.
It is not evaluated, but only stored in the package namespace.}

\item{framework}{Unit test framework}

\item{envir}{the definition environment of object \code{x}.}
}
\value{
a test function with no arguments that wrapping around \code{expr}
}
\description{
The function \code{unit.test} provides a way to write unit tests embedded within
package source files.
These tests are stored and organised in the package namespace, and can be run using
the unified interface provided by the function \code{link{utest}}.
Both Runit and testthat tests are supported -- and automatically detected.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{userIs}
\alias{userIs}
\title{Checking R User}
\usage{
userIs(user)
}
\arguments{
\item{user}{the usernames to check for, as a character vector.}
}
\value{
A logical flag
}
\description{
Tests if the current R user is amongst a given set of users.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project.R
\name{find_devpackage}
\alias{find_devpackage}
\title{Find Path to Development Package Root Directory}
\usage{
find_devpackage(x, error = TRUE)
}
\arguments{
\item{x}{name of the development package to lookup.}

\item{error}{logical that indicates if an error is thrown when the project root directory
could not be found.}
}
\description{
Development packages are looked-up according to rules
defined in a file \code{.Rpackages} in the user's home directory.
}
\section{Specification of package path}{

Package paths are specified in a list with:
\itemize{
\item unnamed elements: character strings give path to directories to lookup for sub-directories that match
exactly the package's name;
\item named element containing character strings: these are paths that are looked up only for packages that
match the element name. If the element name contains any of the characters \code{*?()$^\\][}, then it is matched using
regular expression.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/namespace.R
\name{getLoadingNamespace}
\alias{getLoadingNamespace}
\alias{isLoadingNamespace}
\alias{isNamespaceLoaded2}
\alias{isDevNamespace}
\alias{addNamespaceExport}
\alias{ns_get}
\title{Namespace Development Functions}
\usage{
getLoadingNamespace(env = FALSE, info = FALSE, nodev = FALSE)

isLoadingNamespace(ns, nodev = FALSE)

isNamespaceLoaded2(ns)

isDevNamespace(ns)

addNamespaceExport(x)

ns_get(x, ns = NULL, ...)
}
\arguments{
\item{env}{logical that indicates that the namespace's environment (i.e. the
namespace itself) should be returned.}

\item{info}{logical that indicates that the complete information list should
be returned}

\item{nodev}{logical that indicates if loading devtools namespace should
be discarded.}

\item{ns}{the name of a namespace or a namespace whose loading state is tested.
If missing \code{isLoadingNamespace} test if any namespace is being loaded.}

\item{x}{character vector containing the names of R objects to export in the
loading namespace.}

\item{...}{extra arguments passed to \link{get0}.}
}
\value{
\code{getLoadingNamespace}: the name of the loading namespace if \code{env} and \code{info} are
\code{FALSE}, an environment if \code{env=TRUE}, a list with elements
\code{pkgname} and \code{libname} if \code{info=TRUE}.
}
\description{
\code{getLoadingNamespace} returns information about the loading namespace.
It is a wrapper to \code{\link{loadingNamespaceInfo}}, that does not throw
an error.
}
\section{Functions}{
\itemize{
\item \code{isLoadingNamespace}: Tests if a namespace is being loaded.

\item \code{isNamespaceLoaded2}: tests if a given namespace is loaded, without loading it,
contrary to \code{\link{isNamespace}}.
It is similar to \code{\link{isNamespaceLoaded}} -- which it uses -- but also accepts
environments.

\item \code{isDevNamespace}: tests the -- current -- namespace is a devtools namespace.

\item \code{addNamespaceExport}: Dynamically adds exported objects into the loading namespace.

\item \code{ns_get}: gets an object from a given namespace.
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{.packageMetaRegistry}
\alias{.packageMetaRegistry}
\title{Creates or Retrieves a Package Meta Registry}
\usage{
.packageMetaRegistry(package, quiet = FALSE, create = FALSE)
}
\description{
This function is used in \code{\link{setPackageRegistry}} and
\code{\link{packageRegistry}} to create or query meta registries.
}
\keyword{internal}

\name{packageCLI}
\alias{packageCLI}
\title{Package Specific Command Line Interface}
\usage{
  packageCLI(package, altfile = NULL, local = TRUE,
    ARGS = commandArgs(TRUE), ...)
}
\arguments{
  \item{package}{package name}

  \item{altfile}{alternative file that defines the main CLI
  entry point. That is a function named \code{CLI}, which
  takes the list of parsed command line arguments as its
  first argument.}

  \item{local}{logical that indicates if the main CLI
  function should be defined and evaluated in a local
  environment, or in the user's Global environment.}

  \item{ARGS}{list of parsed arguments passed to the main
  CLI function.}

  \item{...}{extra arugments passed to the package's CLI
  function.}
}
\description{
  Package Specific Command Line Interface
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{setBiocMirror}
\alias{setBiocMirror}
\alias{getBiocMirror}
\alias{getBiocRepos}
\alias{setCRANMirror}
\title{Setting Mirrors and Repositories}
\usage{
setBiocMirror(
  url = "http://www.bioconductor.org",
  version = NULL,
  unique = TRUE
)

getBiocMirror()

getBiocRepos(url = "http://www.bioconductor.org", version = NULL)

setCRANMirror(url = CRAN, unique = TRUE)
}
\arguments{
\item{url}{or Bioconductor mirror url}

\item{version}{version number}

\item{unique}{logical that indicate if duplicated urls or names should be
removed.}
}
\value{
\code{setBiocMirror} returns the old set of Bioc repositories.
}
\description{
\code{setBiocMirror} sets all Bioconductor repositories (software, data,
annotation, etc.).
so that they are directly available to \code{\link{install.packages}}.
It differs from \code{\link{chooseBioCmirror}} in that it effectively enables
the repositories.
}
\section{Functions}{
\itemize{
\item \code{getBiocMirror()}: is a shortcut for \code{getOption('BioC_mirror')}, which
returns the current Bioconductor mirror as used by \code{biocLite}.

\item \code{getBiocRepos()}: returns urls to all Bioconductor repositories on a
given mirror.

\item \code{setCRANMirror()}: sets the preferred CRAN mirror.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{oneoffVariable}
\alias{oneoffVariable}
\title{One-off Global Variables}
\usage{
oneoffVariable(default = NULL)
}
\arguments{
\item{default}{default value to which the global variable is reset after each
access. Default is \code{NULL}.}
}
\value{
a function with one argument (\code{value}) that provides get/set access
to a global variable.
If called with a value, it assigns this value to the global variable.
If called with no argument, it returns the current value of the global variable and
reset it to its default value -- as defined at its creation.
}
\description{
Defines a function that allow to get/assign a global variable whose value is
ensured to be reset after each access.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


x <- oneoffVariable(0)
# returns default value
x()
# assign a value
x(3)
# get the value
x()
# second call returns default value again 
x()
 
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/devutils.R
\name{R.exec}
\alias{R.exec}
\alias{R.CMD}
\alias{R.SHLIB}
\title{Executing R Commands}
\usage{
R.exec(..., lib.loc = NULL)

R.CMD(cmd, ...)

R.SHLIB(libname, ...)
}
\arguments{
\item{...}{extra arguments that are concatenated and appended to
the command.}

\item{lib.loc}{logical that indicates if the current library locations
should be used.
If a character vector, then it is used as the library path specification.}

\item{cmd}{command to run, e.g. \sQuote{check} or \sQuote{INSTALL}.}

\item{libname}{name of the output compiled library}
}
\value{
Returns the value of executing the R command via \link{system}.
}
\description{
Functions to execute R commands.
}
\section{Functions}{
\itemize{
\item \code{R.exec()}: executes a single R command via \code{\link{system2}}.

\item \code{R.CMD()}: executes R CMD commands.

\item \code{R.SHLIB()}: executes R CMD SHLIB commands.

}}

\name{packageDependencies}
\alias{packageDependencies}
\title{List Package Dependencies}
\usage{
  packageDependencies(x, all = TRUE, as.list = FALSE,
    available = NULL)
}
\arguments{
  \item{x}{path to package source directory or file.}

  \item{all}{logical that indicates if all dependencies
  should be returned, or only the required ones.}

  \item{as.list}{logical that indicates if the result
  should be a list with one element per type of
  dependency.}

  \item{available}{a matrix of available packages (as
  returned by \code{\link{available.packages}}), from which
  the dependencies are retrieved. This means that there
  must be a row for the package \code{x}.}
}
\description{
  List Package Dependencies
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logging.R
\name{lverbose}
\alias{lverbose}
\alias{lsilent}
\alias{is.verbose}
\alias{lmessage}
\alias{vmessage}
\alias{log_append}
\title{Logging Feature}
\usage{
lverbose(val, global = FALSE)

lsilent()

is.verbose()

lmessage(level, ..., appendLF = TRUE, sep = "", force = FALSE)

vmessage(...)

log_append(...)
}
\arguments{
\item{val}{logical/numeric value that sets the verbosity level.}

\item{global}{logical that indicates if the verbose level of
all log handlers should be set to \var{val}.}

\item{level}{verbosity level threshold (numeric value) above which the
message should be printed out.
This threshold is compared with the current verbosity level as returned by
\code{lverbose}.}

\item{...}{parts of a character message that are concatenated and passed to
the current logger's write function.}

\item{appendLF}{logical indicating if an endline character should be appended
at the end of the message.}

\item{sep}{separation character, used when concatenating all arguments in \code{...}.}

\item{force}{logical that indicates if one should output messages or return a non
null logger, even if the verbose mode is not high enough.}
}
\value{
\code{lverbose}: the old verbose level
}
\description{
\code{lverbose} returns/sets the current verbosity level.
}
\section{Functions}{
\itemize{
\item \code{lsilent}: tells if all verbose messages are silenced.

\item \code{is.verbose}: tells if verbosity is on, i.e. at level greater than 0.

\item \code{lmessage}: prints out a message (on sdtout) if the verbosity level is greater than a
given value.

\item \code{vmessage}: prints out a log message (at level 1) using the current logger,
typically on stdout.
It is a shortcut for \code{lmessage(1L, ...)}.

\item \code{log_append}: directly appends some message to the current log line.
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{gfile}
\alias{gfile}
\title{Open a File Graphic Device}
\usage{
gfile(filename, width, height, ...)
}
\arguments{
\item{filename}{path to the image file to create.}

\item{width}{output width}

\item{height}{output height}

\item{...}{other arguments passed to the relevant device function
such as \code{\link{png}} or \code{\link{pdf}}.

importFrom grDevices bmp jpeg pdf png svg tiff}
}
\value{
The value of the called device function.
}
\description{
Opens a graphic device depending on the file extension.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{make_vignette_auxfiles}
\alias{make_vignette_auxfiles}
\title{Generate RMarkdown Vignette Axiliary Files}
\usage{
make_vignette_auxfiles(
  PACKAGE,
  input = NULL,
  bibfile = "library.bib",
  Rpkg.prefix = "Rpackage_",
  ...
)
}
\arguments{
\item{PACKAGE}{package name}

\item{input}{vignette source file.
If \code{NULL} then the current file is obtained via a call to \link[knitr:current_input]{knitr::current_input}.}

\item{bibfile}{output file for R package citations.}

\item{Rpkg.prefix}{prefix to use when generating the bibtex entries of cited R packages.
If \code{Rpkg.prefix = 'Rpackage_'}, then Rmardown citations should be \verb{@Rpackage_mypkg}.}

\item{...}{other arguments passed to \link{latex_preamble}}
}
\value{
Path to the preamble file.
}
\description{
Generate RMarkdown Vignette Axiliary Files
}
\details{
To use this feature add the following in your YAML header:

\if{html}{\out{<div class="sourceCode">}}\preformatted{header-includes:
  - \\input\{"`r pkgmaker::make_vignette_auxfiles('pkgmaker')`"\}
bibliography: library.bib
}\if{html}{\out{</div>}}
}

\name{CLIArgumentParser}
\alias{CLIArgumentParser}
\alias{parseCMD}
\title{Enhanced Command Line Argument Parser}
\usage{
  CLIArgumentParser(prog = CLIfile(), description = "",
    ..., epilog = "", show.defaults = TRUE)

  parseCMD(parser, ARGS = commandArgs(TRUE), debug = FALSE,
    envir = parent.frame())
}
\arguments{
  \item{prog}{program name}

  \item{description}{program description}

  \item{...}{extra arguments passed to
  \code{\link[argparse]{ArgumentParser}}.}

  \item{epilog}{epilog messages to display at the end of
  the man pages}

  \item{show.defaults}{logical that indicates if default
  arugment values should be displayed.}

  \item{parser}{parser object as returned by
  \code{CLIArgumentParser}.}

  \item{ARGS}{command line argument to parse, as a named
  list or a character string.}

  \item{debug}{logical that indicate if debugging
  information should be printed.}

  \item{envir}{environment that contains where the
  sub-command functions are looked for.}
}
\description{
  Extends the capabilities of package \pkg{argparse}, e.g.,
  in defining sub commands.

  \code{parseCMD} parse command line arguments for
  sub-commands, and dispatch to the associated function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{render_notes}
\alias{render_notes}
\title{Renders rmarkdown Documents Using User Default Options}
\usage{
render_notes(
  input,
  output_format = NULL,
  output_options = NULL,
  ...,
  .config = NULL
)
}
\arguments{
\item{input}{The input file to be rendered. This can be an R script (.R),
an R Markdown document (.Rmd), or a plain markdown document.}

\item{output_format}{The R Markdown output format to convert to. The option
\code{"all"} will render all formats defined within the file. The option can
be the name of a format (e.g. \code{"html_document"}) and that will render
the document to that single format. One can also use a vector of format
names to render to multiple formats. Alternatively, you can pass an output
format object (e.g. \code{html_document()}). If using \code{NULL} then the
output format is the first one defined in the YAML frontmatter in the input
file (this defaults to HTML if no format is specified there).
If you pass an output format object to \code{output_format}, the options
specified in the YAML header or \code{_output.yml} will be ignored and you
must explicitly set all the options you want when you construct the object.
If you pass a string, the output format will use the output parameters in
the YAML header or \code{_output.yml}.}

\item{output_options}{List of output options that can override the options
specified in metadata (e.g. could be used to force \code{self_contained} or
\code{mathjax = "local"}). Note that this is only valid when the output
format is read from metadata (i.e. not a custom format object passed to
\code{output_format}).}

\item{...}{other arguments passed to \code{\link[rmarkdown]{render}}}

\item{.config}{location of the default options (a YAML file).
Default behaviour is to look for file \code{'.rmarkdown.yaml'} in the user's
home directory, or, if missing, for a yaml section \code{rmarkdown::render}
in the user's R profile.}
}
\value{
the path to the rendered file, like \link[rmarkdown:render]{rmarkdown::render}.
}
\description{
Renders rmarkdown Documents Using User Default Options
}
\seealso{
\code{\link{read.yaml_section}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logging.R
\name{messagef}
\alias{messagef}
\alias{wnote}
\title{General Log Formating}
\usage{
messagef(fmt, ..., domain = NULL, appendLF = TRUE)

wnote(..., immediate. = TRUE)
}
\arguments{
\item{fmt}{a character vector of format strings, each of up to 8192 bytes.}

\item{...}{values to be passed into \code{fmt}.  Only logical,
    integer, real and character vectors are supported, but some coercion
    will be done: see the \sQuote{Details} section. Up to 100.}

\item{domain}{see \code{\link[base]{gettext}}.  If \code{NA}, messages will
    not be translated, see also the note in \code{\link[base]{stop}}.}

\item{appendLF}{logical: should messages given as a character string
    have a newline appended?}

\item{immediate.}{logical, indicating if the call should be output
    immediately, even if \code{\link[base]{getOption}("warn") <= 0}.}
}
\value{
Returns nothing. Used for their side effects of printing messages/warnings.
}
\description{
Generate a formatted diagnostic message.
This function is a shortcut for \code{message(sprintf(...))}.
}
\section{Functions}{
\itemize{
\item \code{wnote()}: throws a simple note as an immediate warning.
It is a shorcut for \code{warning(..., immediate. = TRUE, call. = FALSE)}.

}}
\examples{

messagef("Hello \%s number \%i", 'world', 4)

}
\seealso{
\code{\link{sprintf}}, \code{\link{message}}
}

\name{write_PACKAGES_index}
\alias{write_PACKAGES_index}
\title{Generate CRAN-like Repository Index}
\usage{
  write_PACKAGES_index(path = ".", output = "index.html",
    pattern = NULL, title = "Packages", robots.file = TRUE)
}
\arguments{
  \item{path}{path to the repository's root directory}

  \item{output}{output filename -- relative to the
  repository root \code{path}.}

  \item{pattern}{regular expression used to filter the
  names of the packages that will appear in the index.}

  \item{title}{title of the index page}

  \item{robots.file}{logical that indicates if a file
  \code{robots.txt} that hides the repository from search
  engine robots should be created.}
}
\description{
  Generate CRAN-like Repository Index
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{exitCheck}
\alias{exitCheck}
\title{Exit Error Checks}
\usage{
exitCheck()
}
\description{
\code{exitCheck} provides a mechanism to distinguish the exit status
in \code{\link{on.exit}} expressions.
}
\details{
It generates a function that is used wihtin a function's body to
"flag" normal exits and in its \code{\link{on.exit}} expression
to check the exit status of a function.
Note that it will correctly detect errors only if all normal exit
are wrapped into a call to it.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


# define some function
f <- function(err){

 # initialise an error checker
	success <- exitCheck()

 # do something on exit that depends on the error status
	on.exit({
		if(success()) cat("Exit with no error: do nothing\\n") 
		else cat("Exit with error: cleaning up the mess ...\\n") 
	})
	
 # throw an error here
	if( err ) stop('There is an error')
 
	success(1+1)
}

# without error
f(FALSE)
# with error
try( f(TRUE) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{checkWarning}
\alias{checkWarning}
\title{Extra Check Functions for RUnit}
\usage{
checkWarning(expr, expected = TRUE, msg = NULL)
}
\arguments{
\item{expr}{an R expression}

\item{expected}{expected value as regular expression pattern.
If a logical, then it specifies if a warning is expected or not.

For backward compatibility, a \code{NULL} value is equivalent to \code{TRUE}.}

\item{msg}{informative message to add to the error in case of failure}
}
\description{
\code{checkWarning} checks if a warning is generated by an expression, and
optionally follows an expected regular expression pattern.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


# check warnings
checkWarning({ warning('ah ah'); 3})
checkWarning({ warning('ah oh ah'); 3}, 'oh')
try( checkWarning(3) )
try( checkWarning({ warning('ah ah'); 3}, 'warn you') )

}

\name{lverbose}
\alias{is.verbose}
\alias{lmessage}
\alias{lsilent}
\alias{lverbose}
\alias{vmessage}
\title{Internal verbosity option}
\usage{
  lverbose(val)

  lsilent()

  is.verbose()

  vmessage(...)

  lmessage(..., level = 1L, appendLF = TRUE)
}
\arguments{
  \item{val}{logical that sets the verbosity level.}

  \item{...}{arguments passed to \code{...}
  \code{\link{cat}}}

  \item{appendLF}{logical indicating if an endline
  character should be appended at the end of the message.
  Passed to \code{\link{cat}}.}

  \item{level}{verbosity level threshold (numeric value)
  above which the message should be printed out. This
  threshold is compared with the current verbosity level as
  returned by \code{lverbose}.}

  \item{...}{arguments passed to \code{...}
  \code{\link{lmessage}} or \code{\link{cat}}}

  \item{appendLF}{logical indicating if an endline
  character should be appended at the end of the message.
  Passed to \code{\link{cat}}.}
}
\value{
  the old verbose level
}
\description{
  Internal verbosity option

  Tells if all verbose messages should be

  Tells if verbosity is on.

  Prints out a message (on sdtout) if verbose mode is on.

  Prints out a message (on sdtout) if the verbosity level
  is greater than a given value.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/devutils.R
\name{list.data}
\alias{list.data}
\title{List Package Data Objects}
\usage{
list.data(package = NULL)
}
\arguments{
\item{package}{a single character string that specifies the name of a particular
package where to look for data objects.}
}
\value{
a \code{data.frame} object with columns:
\itemize{
\item \code{package}: name of the package that holds the data object.
\item \code{data}: name of the key to use in \link[utils:data]{utils::data} or \link{ldata} to load the data object.
\item \code{object}: name of the (sub-)object that is contained in the data object.
}
}
\description{
Lists data objects that are shipped within package(s).
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


# list all data objects
head(list.data())

# list all data objects in package 'datasets'
subset(list.data("datasets"), data \%in\% "beavers")

}
\seealso{
\link[utils:data]{utils::data}, \link{ldata}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{requireRUnit}
\alias{requireRUnit}
\title{Load RUnit Compatible Package}
\usage{
requireRUnit(...)
}
\arguments{
\item{...}{arguments not used.}
}
\value{
nothing
}
\description{
Loads the package responsible for the implementation of the RUnit framework,
choosing amongst \sQuote{RUnitX}, \sQuote{svUnit} and \sQuote{RUnit}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{setClassRegistry}
\alias{setClassRegistry}
\title{Automatic S4 Class for Registry Entries}
\usage{
setClassRegistry(registry, Class, ...)
}
\arguments{
\item{registry}{a registry object}

\item{Class}{name of the class to generate}

\item{...}{extra arguments passed to \code{\link{setClass}}.}
}
\description{
Automatic S4 Class for Registry Entries
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sys_call_stack}
\alias{sys_call_stack}
\alias{sys.function_digest}
\alias{sys.function_nframe}
\alias{sys.function_frame}
\alias{sys.source_file}
\title{System Call Stack Utilities}
\usage{
sys.function_digest(n = NULL)

sys.function_nframe(fun)

sys.function_frame(fun)

sys.source_file()
}
\arguments{
\item{n}{a single frame}

\item{fun}{the function object to find in the call stack.}
}
\value{
\itemize{
\item \code{sys.function_digest} returns a character vector of length \code{n}.
}

\itemize{
\item \code{sys.function_nframe} returns a integer vector.
}

\itemize{
\item \code{sys.function_frame} returns an environment.
}
}
\description{
System Call Stack Utilities
}
\section{Functions}{
\itemize{
\item \code{sys.function_digest()}: computes digest hash for each function in the call stack.

\item \code{sys.function_nframe()}: returns the index of the frame that calls a given function.

\item \code{sys.function_frame()}: returns the frame that calls a given function.

\item \code{sys.source_file()}: returns path to the script that is being sourced either
by \link[base:source]{base::source} or \link[base:sys.source]{base::sys.source}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R, R/packages.R
\name{pkgmaker-deprecated}
\alias{pkgmaker-deprecated}
\alias{requirePackage}
\title{Deprecated Functions in pkgmaker}
\usage{
requirePackage(pkg, ...)
}
\arguments{
\item{pkg}{package name to load.}

\item{...}{extra arguments}
}
\value{
\itemize{
\item \code{requirePackage}: returned no value, called to load a package.
}
}
\description{
These functions have been deprecated and will be defunct in the next release.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{addToLogger}
\alias{addToLogger}
\title{Enhancing RUnit Logger}
\usage{
addToLogger(name, value, logger = NULL)
}
\arguments{
\item{name}{name of the function or variable to add}

\item{value}{object to append to the logger.
If \code{value} is a function it is added to the list and is accessible via
\code{.testLogger$name}.
If \code{value} is a variable it is added to the local environment and is
therefore accessible in all logging functions.}

\item{logger}{an optional RUnit logger object.
If missing or \code{NULL}, the object \code{.testLogger} is searched in
\code{.GlobalEnv} -- and an error is thrown if it does not exist.}
}
\value{
the modified logger object. Note that the global object is also
modified if \code{logger} is \code{NULL}.
}
\description{
Adds a function or a local variable to RUnit global logger.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{require.quiet}
\alias{require.quiet}
\alias{qrequire}
\alias{qlibrary}
\alias{mrequire}
\title{Loading Packages}
\usage{
require.quiet(...)

qrequire(...)

qlibrary(...)

mrequire(msg, package, lib.loc = NULL, quietly = FALSE)
}
\arguments{
\item{...}{extra arguments passed to \code{\link{library}} or \code{\link{require}}.}

\item{msg}{error message to use, to which is appended the string
\code{' requires package <pkg>'} to build the error message.}

\item{package}{name of the package to load.}

\item{lib.loc}{a character vector describing the location of \R
    library trees to search through, or \code{NULL}.  The default value
    of \code{NULL} corresponds to all libraries currently known to
    \code{\link[base]{.libPaths}()}.
    Non-existent library trees are silently ignored.}

\item{quietly}{a logical.  If \code{TRUE}, no message confirming
    package attaching is printed, and most often, no errors/warnings are
    printed if package attaching fails.}
}
\value{
No return value, called to load packages.
}
\description{
\code{require.quiet} silently requires a package, and \code{qrequire} is an alias to \code{require.quiet}.
}
\section{Functions}{
\itemize{
\item \code{qlibrary()}: silently loads a package.

\item \code{mrequire()}: tries loading a package with base \code{\link{require}}
and stops with a -- custom -- error message if it fails to do so.

}}
\examples{

mrequire('Running this example', 'stringr')
try( mrequire('Doing impossible things', 'notapackage') )

}
\seealso{
Other require: 
\code{\link{irequire}()}
}
\concept{require}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{runVignette}
\alias{runVignette}
\title{Compile a Vignette Object}
\usage{
runVignette(x, ...)
}
\arguments{
\item{x}{an object that represents a vignette}

\item{...}{other arguments passed down to the relevant method.}
}
\value{
Returns the value returned by the vignette compiler.
}
\description{
Compile a Vignette Object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{charmap}
\alias{charmap}
\title{Substituting Strings Against a Mapping Table}
\usage{
charmap(x, maps, nomatch = NULL, partial = FALSE, rev = FALSE)
}
\arguments{
\item{x}{character vector to match}

\item{maps}{mapping tables.
May be a character vector, a list of character vectors or a function.}

\item{nomatch}{character string to be used for non-matched elements of \code{x}.
If \code{NULL}, these elements are left unchanged.}

\item{partial}{logical that indicates if partial matches are allowed,
in which case mappings are used as regular expressions.}

\item{rev}{logical that indicates if the mapping should be interpreted in the
reverse way.}
}
\description{
Match the elements of a character vectors against a mapping table,
that can specify multiple exact or partial matches.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\name{onLoad}
\alias{onLoad}
\alias{onUnload}
\title{Default Load/Unload Functions}
\usage{
onLoad(libname = NULL, pkgname, chname = packageName())

onUnload(libpath)
}
\arguments{
\item{libname}{a character string giving the library directory where
    the package defining the namespace was found.}

\item{pkgname}{a character string giving the name of the package.}

\item{chname}{a character string naming a DLL (also known as a dynamic
    shared object or library) to load.}

\item{libpath}{a character string giving the complete path to the package.}
}
\value{
Returns nothing, used only for their side-effects.
}
\description{
Default Load/Unload Functions
}
\examples{

# in a package namespace:
.onLoad <- function(libname=NULL, pkgname){

	pkgmaker::onLoad(libname, pkgname)

}

# in a package namespace:
.onUnload <- function(libpath){

	pkgmaker::onUnload(libpath)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{digest_function}
\alias{digest_function}
\title{Compute Function Digest Hash}
\usage{
digest_function(fun, n = Inf)
}
\arguments{
\item{fun}{a function}

\item{n}{a single numeric that indicates the length of the hash.}
}
\value{
a character string
}
\description{
Computes a digest hash of the body and signature of a function.
Note that the hash is not affected by attributes or the
function's environment.
}
\details{
The hash itself is computed using \link[digest:digest]{digest::digest}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{reorder_columns}
\alias{reorder_columns}
\title{Reordering Columns}
\usage{
reorder_columns(x, target, decreasing = FALSE)
}
\arguments{
\item{x}{an object with columns, such as a \code{matrix} or a \code{data.frame},
or from a class that support subsetting via \code{x[, i, drop = FALSE]} and has a method \code{colnames}.}

\item{target}{a character or named numeric vector that specifies the column prefered order.
If a numeric vector, then its names are assumed to correspond to columns,
and its values determine the target order -- according to argument \code{decreasing}.}

\item{decreasing}{logical that indicates in which direction a numeric target vector should
be ordered.}
}
\value{
an object of the same type and dimension
}
\description{
Reorders columns according to a prefered target order
}
\details{
Column names will be reordered so that their order match the one in \code{target}.
Any column that does not appear in \code{target} will be put after those that are
listed in \code{target}.
}

\name{makeUnitVignette}
\alias{makeUnitVignette}
\title{Make Vignette for Unit Tests}
\usage{
  makeUnitVignette(pkg,
    file = paste(pkg, "-unitTests.pdf", sep = ""), ...,
    check = FALSE)
}
\arguments{
  \item{pkg}{Package name}

  \item{file}{Output file (.Rnw, .tex, or .pdf)}

  \item{...}{extra arguments passed to
  \code{\link{utest}}.}

  \item{check}{logical that indactes the cal was made from
  R CMD check, in which case the vignette is updated only
  if results of unit tests can be found in the unit test
  output directory, where they would have been generated by
  \code{\link{utest}}.}
}
\value{
  Result of running unit test suite
}
\description{
  Builds a vignette for unit tests in a package using the
  \code{\link{utest}} and a template vignette file.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibtex.R
\name{write.pkgbib}
\alias{write.pkgbib}
\title{Generate a Bibtex File from Package Citations}
\usage{
write.pkgbib(
  entry = NULL,
  file = "Rpackages.bib",
  prefix = "",
  append = FALSE,
  verbose = TRUE
)
}
\arguments{
\item{entry}{a \code{\link{bibentry}} object or a character vector of package
names. If \code{NULL}, then the list of all installed packages is used.}

\item{file}{output Bibtex file. It can be specified as a filename (as a single
character string), NULL for \code{stdout}, or a \code{link{connection}} object.
If \code{file} is a character string, an extension '.bib' is appended if not
already present.}

\item{prefix}{character string to prepend to the generated packages' Bibtex key.}

\item{append}{a logical that indicates that the Bibtex entries should be added
to the file. If \code{FALSE} (default), the file is overwritten.}

\item{verbose}{a logical to toggle verbosity. If \code{file=NULL}, verbosity
is forced off.}
}
\value{
the list of Bibtex objects -- invisibly.
}
\description{
Generates a Bibtex file from a list of packages or all the installed packages.
It is useful for adding relevant citations in Sweave documents.
}
\details{
Multiple citations are handled by adding a numeric suffix to the Bibtex key
(other than the first/main citation) as \code{"<pkgname>\%i"} (e.g. pkg, pkg2, pkg3).
}
\note{
The Old version of this function {write.bib} has now been integrated
by Romain Francois in the bibtex package.
}
\examples{

write.pkgbib(c('rbibutils', 'utils', 'tools'), file='references')
bibs <- rbibutils::readBib('references.bib', "UTF-8")
write.pkgbib(bibs, 'references2.bib')
bibs2 <- rbibutils::readBib('references.bib', "UTF-8")
identical(bibs, bibs2)
\dontshow{ stopifnot(identical(bibs, bibs2)) }

# write to stdout()
write.pkgbib(c('rbibutils', 'utils', 'tools'), file=NULL)

# clean up 
unlink(c('references.bib', 'references2.bib'))

}
\references{
\emph{Creating bibtex file of all installed packages?}
Achim Zeileis. R-help mailing list.
\url{https://stat.ethz.ch/pipermail/r-help/2009-December/415181.html}
}
\seealso{
\code{link{connection}}, \code{link{bibentry}}
}
\author{
Renaud Gaujoux, based on the function \code{Rpackages.bib}
from Achim Zeileis (see \emph{References}).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{Rversion}
\alias{Rversion}
\title{Complete R version}
\usage{
Rversion()
}
\value{
A character string.
}
\description{
Returns the complete R version, e.g. 2.15.0
}
\examples{
Rversion()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project.R
\name{is_package_path}
\alias{is_package_path}
\title{Test for Package Root Directory}
\usage{
is_package_path(x, error = FALSE)
}
\arguments{
\item{x}{path to the directory to test}

\item{error}{logical that indicates if an error should be raised
if the directory is not a package directory.}
}
\description{
Tells if a directory is a package directory, i.e. that it contains
a \code{DESCRIPTION} file.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\docType{methods}
\name{utest}
\alias{utest}
\alias{utest,function-method}
\alias{utest,character-method}
\alias{utest,RUnitTestSuite-method}
\title{Running Unit Tests}
\usage{
utest(x, ...)

\S4method{utest}{`function`}(x, run = TRUE)

\S4method{utest}{character}(x, filter = "^runit.+\\\\.[rR]$",
  fun = "^test\\\\.", ..., testdir = "tests", framework = c("RUnit",
  "testthat"), quiet = Sys.getenv("RCMDCHECK") != "FALSE",
  lib.loc = NULL)

\S4method{utest}{RUnitTestSuite}(x, ..., quiet = FALSE, outdir = NULL)
}
\arguments{
\item{x}{object to which a unit test is attached}

\item{...}{extra arguments to allow extensions and are passed to
the unit framework running funcitons.}

\item{run}{a logical that indicates if the unit test should be run}

\item{filter}{pattern to match files that contain the definition of
the unit tests functions to run.}

\item{fun}{patter to match the test functions to run.}

\item{testdir}{directory where to look for the test files}

\item{framework}{unit test framework}

\item{quiet}{a logical that indicates if the tests should be run silently}

\item{lib.loc}{path to a library where installed packages are searched for.
Used is of the form \code{x='package:*'}.}

\item{outdir}{output directory}
}
\description{
Run unit tests in a variety of settings.
This is still \strong{very} experimental.
}
\section{Methods (by class)}{
\itemize{
\item \code{function}: Run the unit test assoicated to a function.

\item \code{character}: Run a package test suite

\item \code{RUnitTestSuite}: Runs a RUnit test suite
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/devutils.R
\name{compile_src}
\alias{compile_src}
\title{Compile Source Files from a Development Package}
\usage{
compile_src(pkg = NULL, load = TRUE)
}
\arguments{
\item{pkg}{the name of the package to compile}

\item{load}{a logical indicating whether the compiled library should be loaded
after the compilation (default) or not.}
}
\value{
None
}
\description{
Compile Source Files from a Development Package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/devutils.R
\name{packageData}
\alias{packageData}
\alias{ldata}
\title{Loading Package Data}
\usage{
packageData(list, envir = .GlobalEnv, ..., options = NULL,
  stringsAsFactors = getOption("stringsAsFactors", TRUE))

ldata(list, ..., package = NULL, error = TRUE, simplify = TRUE)
}
\arguments{
\item{list}{character vector containing the names of the data to load.}

\item{envir}{the \link{environment} where the data should be loaded.}

\item{...}{other arguments eventually passed to \code{\link[utils]{data}}.}

\item{options}{list of R options to set before calling \code{\link[utils]{data}}.
This may be useful the data is shipped as an R script.}

\item{stringsAsFactors}{logical that indicates if character columns of tabular data should be
converted into factors.}

\item{package}{
    a character vector giving the package(s) to look
    in for data sets, or \code{NULL}.

    By default, all packages in the search path are used, then
    the \file{data} subdirectory (if present) of the current working
    directory.
  }

\item{error}{a logical that indicates whether an error should be thrown if
the requested data cannot be found.}

\item{simplify}{logical that indicates if queries of one object only (i.e. argument \code{list}
is of length one) should return the data object itself.}
}
\value{
the loaded data.
}
\description{
Loads package data using \code{\link[utils]{data}}, but allows the user to avoid
NOTEs for a \sQuote{non visible binding variable} to be thrown when checking a package.
This is possible because this function returns the loaded data.
}
\section{Functions}{
\itemize{
\item \code{ldata}: loads a package data in the parent frame.
It is a shortcut for \code{packageData(list, ..., envir=parent.frame())}.
}}

\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


\dontrun{ mydata <- packageData('mydata') }


\dontrun{ 
# in a package' source => won't issue a NOTE
myfunction function(){
	mydata <- ldata('mydata') 
}
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{writeUnitVignette}
\alias{writeUnitVignette}
\title{Writes Unit Tests Vignette}
\usage{
writeUnitVignette(pkg, file, results = NULL, check = FALSE)
}
\arguments{
\item{pkg}{Package name}

\item{file}{Output Sweave (.Rnw) file}

\item{results}{result file or output character vector}

\item{check}{logical that indactes the cal was made from R CMD check,
in which case the vignette is updated only if results of unit tests can
be found in the unit test output directory, where they would have been
generated by \code{\link{utest}}.}
}
\value{
Path to the vignette file.
}
\description{
Writes a vignette that contains the results from running unit test suites.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{packageRegistry}
\alias{packageRegistry}
\alias{packageRegistries}
\alias{hasPackageRegistry}
\alias{setPackageRegistry}
\alias{setPackageRegistryEntry}
\title{Package Registry}
\usage{
packageRegistry(
  regname = NULL,
  quiet = FALSE,
  entry = FALSE,
  update = !entry,
  package = topenv(parent.frame())
)

packageRegistries(regname = NULL, package = NULL, primary = FALSE)

hasPackageRegistry(regname = NULL, package)

setPackageRegistry(
  regname,
  regobj,
  description = "",
  entrydesc = NA,
  ...,
  package = topenv(parent.frame()),
  overwrite = FALSE
)

setPackageRegistryEntry(
  regname,
  key,
  ...,
  overwrite = FALSE,
  verbose = FALSE,
  where = topenv(parent.frame()),
  msg = NULL
)
}
\arguments{
\item{regname}{Name of a sub-registry, used as its identifier.}

\item{quiet}{a logical that indicates that one should return the (meta-)registry if it exists,
or \code{NULL} otherwise, without throwing any error.}

\item{entry}{logical that indicates if the corresponding meta registry entry should
be directly returned, without any other processing.}

\item{update}{logical that indicates if the package registry should be updated, by adding/removing
entries from other loaded/unloaded packages.}

\item{package}{package where to store or look for the registry.}

\item{primary}{logical that indicates if only primary registries
should be listed.}

\item{regobj}{a \code{\link[registry:regobj]{registry}} object or a single character
string that indicates the class of the objects that are stored in the
sub-registry.
See details for the list of the sub-registry's fields in this latter case.}

\item{description}{short description line about the registry.
It is recommended to provide such description as it makes clearer the purpose of the
registry.
This description is shown when the registry object is printed/formated/listed.}

\item{entrydesc}{human readable description that is used in log messages
when registering/removing entries.}

\item{...}{named values used to set extra information about the new registry, that
are stored in the corresponding fields of the meta-registry.
Currently not used, as no extra field other than \code{'description'} is defined.}

\item{overwrite}{a logical that indicate if an existing registry with the same
should be overwritten if it exists.}

\item{key}{entry identifier.}

\item{verbose}{a logical that indicates if verbosity should be toggle on.}

\item{where}{package name or namespace that owns the registry.}

\item{msg}{addon message to print at the end of the output log line,
when \code{verbose=TRUE}.}
}
\value{
a \code{\link[registry:regobj]{registry}} object or \code{NULL} (see argument
\code{quiet}).
}
\description{
\code{packageRegistry} provides ways to create query package specific
registries.
}
\details{
Package registries are organised in a meta-registry (a registry of registries) within a package's namespace.
Each registry can be used to store sets of built-in or user-defined objects
in an organised way, e.g. algorithms or datasets.

A package meta-registry is a \code{\link[registry:regobj]{registry}} object,
whose entries are \code{\link[registry:regobj]{registry}} objects themselves.
A sub-registry entry is defined by the following fields:
\describe{
\item{key}{The sub-registry's accession key/identifier (a character string).}
\item{regobj}{The sub-registry itself (a \code{registry} object)}
\item{description}{Human readable description of the purpose of the registry (a character string)}
\item{description}{Short human readable description of the type of entries (a character string)}
\item{package}{owner package, which is forced to be the package in which the meta registry
is defined.}
\item{parent}{The name of the package that holds the parent registry, which we
call the primary package.
This field is non empty for cross-package registries, i.e. registries that
derive from primary package's own registry.
Their entries are defined when (lazy-)loading the dependent package's namespace.}
}

Note that this function cannot be called from the global environment, but from
a package namespace, e.g., when a package is lazy-loaded on installation or loaded
via the function \code{\link[devtools]{load_all}} from the \pkg{devtools} package.
}
\section{Functions}{
\itemize{
\item \code{packageRegistries()}: lists registries from loaded packages.

\item \code{hasPackageRegistry()}: tells if a given package has a meta-registry or
a given registry.

\item \code{setPackageRegistry()}: creates a package-specific registry within a package.

Each package sub-registry has its own set of fields.
Sub-registries defined by passing a character string in argument \code{regobj} of
\code{setPackageRegistry} have the following fields: \code{'key'} and \code{'object'}

\item \code{setPackageRegistryEntry()}: adds an entry in a package registry.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibtex.R
\name{citecmd}
\alias{citecmd}
\title{Citing Package References}
\usage{
citecmd(key, ..., REFERENCES = NULL)
}
\arguments{
\item{key}{character vector of BibTex keys}

\item{...}{extra arguments passed to \code{format.bibentry}.}

\item{REFERENCES}{package or bibentry specification}
}
\value{
a character string containing the text formated BibTex entries
}
\description{
Create a citation command from package specific BibTex entries, suitable to
be used in Rd files or Latex documents.
The entries are looked in a file named REFERNCES.bib in the package's root
directory (i.e. inst/ in development mode).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{expand_list}
\alias{expand_list}
\alias{expand_dots}
\title{Expanding Lists}
\usage{
expand_list(x, ..., .exact = TRUE, .names = !.exact)

expand_dots(..., .exclude = NULL)
}
\arguments{
\item{x}{input list}

\item{...}{extra named arguments defining the default items.
A list of default values can also be passed as a a single unnamed argument.}

\item{.exact}{logical that indicates if the names in \code{x} should be
partially matched against the defaults.}

\item{.names}{logical that only used when \code{.exact=FALSE} and indicates
that the names of items in \code{x} that partially match some defaults should
be expanded in the returned list.}

\item{.exclude}{optional character vector of argument names to exclude
from expansion.}
}
\value{
a list
}
\description{
\code{expand_list} expands a named list with a given set of default items,
if these are not already in the list, partially matching their names.
}
\section{Functions}{
\itemize{
\item \code{expand_dots()}: expands the \code{...} arguments of the function
in which it is called with default values, using \code{expand_list}.
It can \strong{only} be called from inside a function.

}}
\examples{

expand_list(list(a=1, b=2), c=3)
expand_list(list(a=1, b=2, c=4), c=3)
# with a list
expand_list(list(a=1, b=2), list(c=3, d=10))
# no partial match
expand_list(list(a=1, b=2, c=5), cd=3)
# partial match with names expanded
expand_list(list(a=1, b=2, c=5), cd=3, .exact=FALSE)
# partial match without expanding names
expand_list(list(a=1, b=2, c=5), cd=3, .exact=FALSE, .names=FALSE)

# works also inside a function to expand a call with default arguments
f <- function(...){
	cl  <- match.call()
	expand_list(cl, list(a=3, b=4), .exact=FALSE)
}
f()
f(c=1)
f(a=2)
f(c=1, a=2)

# expanding dot arguments

f <- function(...){ 
	expand_dots(list(a=2, bcd='a', xxx=20), .exclude='xxx') 
}

# add default value for all arguments 
f()
# add default value for `bcd` only
f(a=10)
# expand names
f(a=10, b=4)

}

\name{require.quiet}
\alias{require.quiet}
\title{Silent Require}
\usage{
  require.quiet(package, character.only = FALSE, ...)
}
\arguments{
  \item{...}{extra arguments passed to
  \code{\link{require}}.}

  \item{package}{the name of a package, given as a
  \link{name} or literal character string, or a character
  string, depending on whether \code{character.only} is
  \code{FALSE} (default) or \code{TRUE}).}

  \item{character.only}{a logical indicating whether
  \code{package} or \code{help} can be assumed to be
  character strings.}
}
\description{
  Silently require a package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{read.yaml_section}
\alias{read.yaml_section}
\title{Reads YAML Options Embbeded into a File}
\usage{
read.yaml_section(section, file = "~/.Rprofile", text = NULL)
}
\arguments{
\item{section}{section name to lookup in the file.
In the file this defined by paired tags \code{"#<section_name>", "#</section_name>"},
or a single tag \code{"#<section_name@file_path>"} that redirect to a YAML file.}

\item{file}{path to the file to parse. Default is to parse the user's \emph{.Rprofile}.}

\item{text}{text to parse.
If provided, then argument \code{file} is not used.}
}
\value{
Returns a list representation of the YAML header
}
\description{
Reads YAML Options Embbeded into a File
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{str_diff}
\alias{str_diff}
\title{Finding Differences Between Strings}
\usage{
str_diff(x, y)
}
\arguments{
\item{x}{a single string}

\item{y}{a single string}
}
\value{
an integer vector containing the index of all mis-matched characters
in the first string.
}
\description{
Computes which characters differ between two strings.
}
\examples{

# strings to compare
x <- "once upon a time"
y <- "once upon a time there was"
z <- "once upon two times"

# diff: x - y
d <- str_diff(x, y)
d
str(d)

# other comparisons 
str_diff(y, x)
str_diff(x, x)
str_diff(x, z)
str_diff(y, z)

}

\name{new2}
\alias{new2}
\title{Alternative S4 Constructor}
\usage{
  new2(class, ...)
}
\arguments{
  \item{class}{Class name to instanciate}

  \item{...}{extra arguments from which slot values are
  extracted by exact matching of names.}
}
\description{
  An alternative version of \code{\link{new}} to create
  objects based on a list of values.
}
\examples{
\dontshow{# roxygen generated flag
options(R_CHECK_RUNNING_EXAMPLES_=TRUE)
}

setClass('A', contain='character', representation(x='numeric', y='character'))

# identical behaviour with standard calls
identical(new('A'), new2('A'))
identical(new('A', x=1), new2('A', x=1))

# but if passing that are names not slots
identical(new('A'), new2('A', b=1))
identical(new('A', x=1), new2('A', x=1, b=3))
identical(new('A', x=1), new2('A', x=1, b=3))

# standard `new` would coerce first unnamed argument into parent of 'A' (i.e. 'character')
new('A', list(x=1))
new('A', list(x=1, y='other'))
# `new2` rather use it to initialise the slots it can find in the list
identical(new('A', x=1), new2('A', list(x=1)))
identical(new('A', x=1, y='other'), new2('A', list(x=1, y='other')))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/devutils.R
\name{load_all_file}
\alias{load_all_file}
\title{Generate a Loading Script for Development Packages}
\usage{
load_all_file(path = path.package(package), package, dest = NULL)
}
\arguments{
\item{path}{a character string that contains the path to the development package.}

\item{package}{the name of the package for which the loading script must be generated.
It must be a package that has already been loaded with \link[devtools:load_all]{devtools::load_all} in the current
session, so that its path can be retrieved.}

\item{dest}{the path to script file to create (as a character string).
If not provided, then the script is written in a temporary .R file with prefix
\code{"load_all_<pkgname>_"}.}
}
\value{
a character string that contains the path to the script file.
}
\description{
Writes a script file that contains code that loads a given development package.
}
\details{
This is useful when we want to load a development package in \code{batchtools} registries:

\if{html}{\out{<div class="sourceCode">}}\preformatted{library(devtools)
library(batchtools)

load_all("path/to/pkgA")
makeRegistry(..., source = load_all_file("pkgA"))
}\if{html}{\out{</div>}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/user.R
\name{irequire}
\alias{irequire}
\title{Require a Package with User Interaction}
\usage{
irequire(package, lib = NULL, ..., load = TRUE, msg = NULL,
  quiet = TRUE, prependLF = FALSE, ptype = c("CRAN-like", "BioC",
  "BioCsoft", "BioCann"), autoinstall = !interactive())
}
\arguments{
\item{package}{name of the package}

\item{lib}{path to the directory (library) where the package is to be
looked for and installed if agreed by the user.}

\item{...}{extra arguments passed to \code{\link{install.packages}}.}

\item{load}{a logical that indicates if the package should be loaded,
possibly after installation.}

\item{msg}{message to display in case the package is not found when first
trying to load/find it.
This message is appended to the string \dQuote{Package '<packagename>' is required}.}

\item{quiet}{logical that indicates if loading a package should be done quietly
with \code{\link{require.quiet}} or normally with \code{\link{require}}.}

\item{prependLF}{logical that indicates if the message should start at a new line.}

\item{ptype}{type of package: from CRAN-like repositories, Bioconductor, Bioconductor software, Bioconductor annotation.
Bioconductor packages are installed using \code{biocLite} from the
\pkg{BiocInstaller} package or fetched on line at \url{http://bioconductor.org/biocLite.R}.}

\item{autoinstall}{logical that indicates if missing packages should just be installed
without asking with the user, which is the default in non-interactive sessions.}
}
\value{
\code{TRUE} if the package was successfully loaded/found (installed),
\code{FALSE} otherwise.
}
\description{
Like base \code{\link{require}}, \code{irequire} tries to find and load a package,
but in an interactive way, i.e. offering the user to install it if not found.
}
\seealso{
Other require: \code{\link{require.quiet}}
}
\concept{require}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{unlist_}
\alias{unlist_}
\title{Flatten a List Conserving Names}
\source{
Bioconductor AnnotationDbi::unlist2
}
\usage{
unlist_(x, recursive = TRUE, use.names = TRUE, what.names = "inherited")
}
\arguments{
\item{x, recursive, use.names}{See \code{?unlist}.}

\item{what.names}{\code{"inherited"} or \code{"full"}.}
}
\value{
A vector.
}
\description{
\code{unlist_} is a replacement for \link[base:unlist]{base::unlist} that does not mangle the names.
}
\details{
Use this function if you don't like the mangled names returned by the standard \code{unlist} function from the base package.
Using \code{unlist} with annotation data is dangerous and it is highly recommended to use \code{unlist_} instead.
}
\examples{
x <- list(A=c(b=-4, 2, b=7), B=3:-1, c(a=1, a=-2), C=list(c(2:-1, d=55), e=99))
unlist(x)
unlist_(x)

# annotation maps (as in AnnotationDbi objects
egids2pbids <- list('10' = 'a', '100' = c('b', 'c'), '1000' = c('d', 'e'))
egids2pbids

unlist(egids2pbids)   # 1001, 1002, 10001 and 10002 are not real
                      # Entrez ids but are the result of unlist()
                      # mangling the names!
unlist_(egids2pbids)  # much cleaner! yes the names are not unique
                      # but at least they are correct...

}
\author{
Herve Pages
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.silenceF}
\alias{.silenceF}
\title{Silencing Functions}
\usage{
.silenceF(f, level = 7L)
}
\arguments{
\item{f}{function to silence}

\item{level}{a single numeric (integer) that indicates the silencing level, which encodes the set of
output to be silenced.

It is interpreted like unix permission bit system, where each bit of the binary expression of the silencing
level corresponds to a given type of output:
\itemize{
\item 0: nothing silenced;
\item 1: \emph{stdout};
\item 2: \emph{stderr} messages;
\item 4: \emph{stderr} warnings.
}

For example, level \code{3 = 2 + 1} means silencing \emph{stdout} and \emph{stderr}, while
\code{5 = 3 + 2} means silencing \emph{stderr} messages and warnings, but not outputs to \emph{stdout}.
The default value is \code{7 = 4 + 2 + 1}, which silences all output.

Negative values are supported and mean \emph{"silence everything except the corresponding type"},
e.g., \code{level = -1} silences all except \emph{stdout} (computed as the binary complementary of 7, i.e. \code{7 - 1 = 5 = 3 + 2}).
See examples.}
}
\value{
a function
}
\description{
Generates a wrapper function that silences the output, messages, and/or warnings of a given function.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


f <- function(){
	cat("stdout message\\n")
 message("stderr message")
	warning("stderr warning", immediate. = TRUE)
}

# example of generated wrapper
g <- .silenceF(f)
g

# use of silencing level
for(l in 7:-7){ message("\\nLevel: ", l); .silenceF(f, l)() }

# inline functions
ifun <- .silenceF(function(){ f(); invisible(1) })
ifun()
ifun <- .silenceF(function(){ f(); 1 })
ifun()
ifun <- .silenceF(function(){ f(); 1 }, 2L)
ifun()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{ExposeAttribute}
\alias{ExposeAttribute}
\alias{attr_mode}
\alias{attr_mode<-}
\title{Exposing Object Attributes}
\usage{
ExposeAttribute(object, ..., .MODE = "rw", .VALUE = FALSE)

attr_mode(x)

attr_mode(x) <- value
}
\arguments{
\item{object}{any R object whose attributes need to be exposed}

\item{...}{attributes, and optionally their respective values or
access permissions.
See argument \code{value} of \code{attr_mode} for details on the
way of specifying these.}

\item{.MODE}{access mode:
\describe{
\item{\dQuote{r}:}{ (read-only) only method \code{$} is defined}
\item{\dQuote{w}:}{ (write-only) only method \code{$<-} is defined}
\item{\dQuote{rw}:}{ (read-write) both methods \code{$} and \code{$<-}
are defined}
}}

\item{.VALUE}{logical that indicates if the values of named arguments
in \code{...} should be considered as attribute assignments,
i.e. that the result object has these attributes set with the specified values.
In this case all these attributes will have the access permission
as defined by argument \code{.MODE}.}

\item{x}{an \code{ExposeAttribute} object}

\item{value}{replacement value for mode.
It can be \code{NULL} to remove the ExposeAttribute wrapper,
a single character string to define a permission for all atributes
(e.g., \code{'rw'} or \code{'r'}), or a list specifying access permission
for specific attributes or classes of attributes defined by regular expressions.
For example, \code{list(a='r', b='w', `blabla.*`='rw')} set attribute \code{'a'}
as read-only, attribute \code{'b'} as write-only, all attributes that start with
\code{'blabla'} in read-write access.}
}
\value{
\code{ExposeAttribute} returns an S3 object of class \code{ExposeAttribute}.
}
\description{
The function \code{ExposeAttribute} creates an S3 object that
exposes all attributes of any R object, by making them accessible via
methods \code{\link{$}} and/or \code{\link{$<-}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{winbuild}
\alias{winbuild}
\title{Build a Windows Binary Package}
\usage{
winbuild(path, outdir = ".", verbose = TRUE)
}
\arguments{
\item{path}{path to a source or already installed package}

\item{outdir}{output directory}

\item{verbose}{logical or numeric that indicates the verbosity level}
}
\value{
Invisibly returns the full path to the generated zip file.
}
\description{
Build a Windows Binary Package
}
\examples{
\dontrun{

# from source directory
winbuild('path/to/package/source/dir/')
# from tar ball
winbuild('PKG_1.0.tar.gz')

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{latex_preamble}
\alias{latex_preamble}
\alias{latex_bibliography}
\title{LaTeX Utilities for Vignettes}
\usage{
latex_preamble(PACKAGE, R = TRUE, CRAN = TRUE, Bioconductor = TRUE,
  GEO = TRUE, ArrayExpress = TRUE, biblatex = FALSE, only = FALSE,
  file = "")

latex_bibliography(PACKAGE, file = "")
}
\arguments{
\item{PACKAGE}{package name}

\item{R}{logical that indicate if general R commands should be added
(e.g. package names, inline R code format commands)}

\item{CRAN}{logical that indicate if general CRAN commands should be added
(e.g. CRAN package citations)}

\item{Bioconductor}{logical that indicate if general Bioconductor commands
should be added (e.g. Bioc package citations)}

\item{GEO}{logical that indicate if general GEOmnibus commands should be added
(e.g. urls to GEO datasets)}

\item{ArrayExpress}{logical that indicate if general ArrayExpress commands
should be added (e.g. urls to ArrayExpress datasets)}

\item{biblatex}{logical that indicates if a \code{\\bibliography} command
should be added to include references from the package's REFERENCES.bib file.}

\item{only}{a logical that indicates if the only the commands whose
dedicated argument is not missing should be considered.}

\item{file}{connection where to print. If \code{NULL} the result is returned
silently.}
}
\description{
\code{latex_preamble} outputs/returns command definition LaTeX commands to
be put in the preamble of vignettes.
}
\details{
Argument \code{PACKAGE} is not required for \code{latex_preamble}, but must
be correctly specified to ensure \code{biblatex=TRUE} generates the correct
bibliography command.
}
\section{Functions}{
\itemize{
\item \code{latex_bibliography}: \code{latex_bibliography} prints or return a LaTeX command that includes a
package bibliography file if it exists.
}}

\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


latex_preamble()
latex_preamble(R=TRUE, only=TRUE)
latex_preamble(R=FALSE, CRAN=FALSE, GEO=FALSE)
latex_preamble(GEO=TRUE, only=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{knit_ex}
\alias{knit_ex}
\alias{hook_try}
\alias{hook_backspace}
\alias{hook_toggle}
\title{Knitr Extensions}
\usage{
knit_ex(x, ..., quiet = TRUE, open = FALSE)

hook_try(before, options, envir)

hook_backspace()

hook_toggle()
}
\arguments{
\item{x}{text to knit as a character vector}

\item{...}{arguments passed to \code{\link[knitr]{knit2html}} or \code{\link[knitr]{knit}}}

\item{quiet}{logical that indicates if knitting should be quiet (no progress bars etc..).}

\item{open}{logical, only used when \code{x} is in .Rmd format, that indicates
if the generated document result should be open in a browse, instead of
being printed on screen.
Not that a browser will not open in non-interactive sessions, and the result will
be returned invisibly.}

\item{before}{logical that indicates when the hook is being called:
before or after the chunk is processed.}

\item{options}{list of current knitr chunk options}

\item{envir}{environment where the chunk is evaluated}
}
\value{
\code{knit_ex} returns the generated code, although invisibly when \code{open=TRUE}.
}
\description{
\code{knit_ex} is a utility function for running small knitr examples,
e.g., to illustrate functionalities or issues.

\code{hook_backspace} is a chunk hook that enables the use of backspace
characters in the output (e.g., as used in progress bars), and still
obtain a final output as in the console.
}
\section{Functions}{
\itemize{
\item \code{hook_try}: is a knitr hook to enable showing error
messages thrown by \code{\link{try}}.
The function is not meant to be called directly, but only registered
using \link[knitr:knit_hooks]{knitr::knit_hooks} (see details on this dedicated man page).

This simply defines a function \code{try} in \code{envir} that prints
the error message if any, and is called instead of base \code{\link{try}}.

\item \code{hook_toggle}: is a chunk hook that adds clickable elements to toggle \emph{indvidual}
code chunks in HTML documents generated from .Rmd files.
}}

\examples{

library(knitr)
knit_ex("1 + 1")


library(knitr)

# standard error message is caught
knit_ex("stop('ah ah')")

# with try the error is output on stderr but not caughted by knitr
knit_ex("try( stop('ah ah') )")

# no message caught
knit_ex("
^^^{r, include = FALSE}
knit_hooks$set(try = pkgmaker::hook_try)
^^^

^^^{r, try=TRUE}
try( stop('ah ah') )
^^^")


# Correctly formatting backspaces in chunk outputs
tmp <- tempfile(fileext = '.Rmd')
cat(file = tmp, "
^^^{r, include = FALSE}
library(knitr)
knit_hooks$set(backspace = pkgmaker::hook_backspace())
^^^
Default knitr does not handle backspace and adds a special character:
^^^{r}
cat('abc\bd')
^^^

Using the hook backspace solves the issue:
^^^{r, backspace=TRUE}
cat('abc\bd')
^^^
")

# knit
out <- knitr::knit2html(tmp, fragment.only = TRUE)
# look at output
\dontrun{
  browseURL(out)
  edit( file = out)
}
# cleanup
out_files <- list.files(dirname(out), full.names = TRUE,
                         pattern = paste0("^", tools::file_path_sans_ext(out))) 
unlink(c(tmp, out_files))



knit_ex("

Declare chunk hook:
^^^{r, setup}
library(knitr)
knit_hooks$set(toggle = hook_toggle())
^^^

The R code of this chunk can be toggled on/off, and starts visible:
^^^{r, toggle=TRUE}
print(1:10)
^^^
The R code of this chunk can be toggled on/off, and starts hidden:
^^^{r, toggle=FALSE}
print(1:10)
^^^

This is a plain chunk that cannot be toggled on/off:
^^^{r}
print(1:10)
^^^

Now all chunks can be toggled and start visible:
^^^{r, toggle_all}
opts_chunk$set(toggle = TRUE)
^^^

^^^{r}
sample(5)
^^^

To diable the toggle link, one can pass anything except TRUE/FALSE:
^^^{r, toggle = NA}
sample(5)
^^^

", open = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibtex.R, R/package.R
\name{write.bib}
\alias{write.bib}
\alias{pkgmaker-defunct}
\title{Defunct Functions in pkgmaker}
\usage{
write.bib(...)
}
\arguments{
\item{...}{extra arguments}
}
\value{
\itemize{
\item \code{write.bib}: returns no value, called to write a bib file.
}
}
\description{
These functions have been defunct or superseded by other
functions.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.R
\name{is_something}
\alias{is_something}
\alias{is_NA}
\alias{isFALSE}
\alias{isNumber}
\alias{isReal}
\alias{isInteger}
\alias{isString}
\alias{is.dir}
\alias{is.file}
\alias{hasNames}
\title{Testing Object Type}
\usage{
is_NA(x)

isFALSE(x)

isNumber(x)

isReal(x)

isInteger(x)

isString(x, y, ignore.case = FALSE)

is.dir(x)

is.file(x)

hasNames(x, all = FALSE)
}
\arguments{
\item{x}{an R object}

\item{y}{character string to compare with.}

\item{ignore.case}{logical that indicates if the comparison
should be case sensistive.}

\item{all}{logical that indicates if the object needs all names non empty}
}
\value{
\code{TRUE} or \code{FALSE}
}
\description{
Testing Object Type

\code{is_NA} tests if a variable is exactly NA (logical, character, numeric or integer)

\code{isFALSE} Tests if a variable is exactly FALSE.

\code{isNumber} tests if a variable is a single number

\code{isReal} tests if a variable is a single real number

\code{isInteger} tests if an object is a single integer

\code{isString} tests if an object is a character string.

\code{is.dir} tests if a filename is a directory.

\code{is.file} tests if a filename is a file.

\code{hasNames} tests if an object has names.
}
\seealso{
\code{\link{isTRUE}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package-extra.R
\name{setPackageExtraHandler}
\alias{setPackageExtraHandler}
\alias{packageExtraHandler}
\alias{setPackageExtra}
\alias{packageExtra}
\alias{packageExtraRunner}
\alias{install.extras}
\alias{install.extrapackages}
\title{Install/Run Extra Things After Standard Package Installation}
\usage{
setPackageExtraHandler(handler, fun, ...)

packageExtraHandler(handler = NULL, ...)

setPackageExtra(handler, extra, ...)

packageExtra(handler = NULL, extra = NULL, package = NULL, .wrap = FALSE)

packageExtraRunner(handler)

install.extras(
  package,
  extra = NULL,
  handler = NULL,
  ...,
  .verbose = getOption("verbose")
)

install.extrapackages(
  package,
  extra = NULL,
  handler = NULL,
  ...,
  .verbose = getOption("verbose")
)
}
\arguments{
\item{handler}{name of a handler, e.g, \code{'install'}.
It must be unique across all handlers registered by any other packages.}

\item{fun}{handler function that will be called with the arguments registered
with \code{packageExtra(name, ...)}}

\item{...}{extra arguments passed to internal function calls.
In \code{packageExtraHandler}, these are passed to \code{\link{pkgreg_fetch}}.

In \code{setPackageExtra}, these define default arguments for the handler function.
These are overwritten by arguments in the call to runner function if any.}

\item{extra}{name of the extra action.}

\item{package}{package name where to store/look for the internal registries.
End users should not need to use this argument.}

\item{.wrap}{logical that indicates if a function that runs the extra action should
be returned or only the default arguments}

\item{.verbose}{logical that indicates if verbose messages about the extra actions being
run should be displayed.}
}
\value{
the runner function associated with the newly registered handler,
as built by \code{packageExtraRunner}.
}
\description{
These functions define a framework to register actions for which default sets of arguments
can be defined when (lazy-)loading a package, and run later on, e.g., after the package
is installed using dedicated commands.

\code{setPackageExtraHandler} defines main action handler functions, for which
actions are defined as a set of arguments and registered using \code{setPackageExtra}.
}
\section{Functions}{
\itemize{
\item \code{packageExtraHandler()}: retrieves a given handler from the registry.

\item \code{setPackageExtra()}: registers extra actions for a given handler.

For example, calling \code{setPackageExtra('install', pkgs='non_CRAN_pkg', repos='http://non-standard-repo')}
in a source file of package 'myPkg' registers the call
\code{install.packages('non_CRAN_pkg', repos='http://non-standard-repo', ...)}
in a registry internal to the package.
All calls to \code{setPackageExtra('install', ...)} can then be run by the user, as
a post installation step via \code{install.extrapackages('myPkg', ..)}.

\item \code{packageExtra()}: retrieve a given extra action, either as its registry entry,
or as a function that would perform the given action.

\item \code{packageExtraRunner()}: defines a function to run all or some of the actions registered
for a given handler in a given package.
For example, the function \code{install.extrapackages} is the runner defined for the extra handler \code{'install'}
via \code{packageExtraRunner('install')}.

\item \code{install.extras()}: runs all extra actions registered for a given package.

\item \code{install.extrapackages()}: install sets of packages that can enhance a
package, but may not be available from CRAN.

It is defined as the extra handler for
the extra action handler \code{'install.packages'}.
All arguments in \code{...} are passed to \code{\link{install.packages}}.
By default, packages that are already installed are not re-installed.
An extra argument \code{force} allows to force their installation.
The packages are loaded if their installation is successful.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{testRversion}
\alias{testRversion}
\title{Testing R Version}
\usage{
testRversion(x, test = 1L)
}
\arguments{
\item{x}{target version to compare with.}

\item{test}{numeric value that indicates the comparison to be carried out.
The comparison is based on the result from
\code{utils::compareVersion(R.version, x)}:
\itemize{
\item 1: is R.version > \code{x}?
\item 0: is R.version = \code{x}?
\item -1: is R.version < \code{x}?
}}
}
\value{
a logical
}
\description{
Compares current R version with a given target version, which may be useful
for implementing version dependent code.
}
\examples{

testRversion("2.14")
testRversion("2.15")
testRversion("10")
testRversion("10", test = -1)
testRversion("< 10")
testRversion(Rversion())
testRversion(paste0('=', Rversion()))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{inSweave}
\alias{inSweave}
\title{Identifying Sweave Run}
\usage{
inSweave()
}
\value{
\code{TRUE} or \code{FALSE}
}
\description{
Tells if the current code is being executed within a Sweave document.
}
\examples{

# Not in a Sweave document 
inSweave()

# Within a Sweave document

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{orderVersion}
\alias{orderVersion}
\alias{sortVersion}
\title{Ordering Version Numbers}
\usage{
orderVersion(x, ..., decreasing = FALSE)

sortVersion(x, ...)
}
\arguments{
\item{x}{a character vector of version numbers}

\item{...}{extra parameters passed to \code{orderVersion} and \code{\link{order}}}

\item{decreasing}{a logical that indicates if the ordering should be decreasing}
}
\description{
Orders a vector of version numbers, in natural order.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


v <- c('1.0', '1.03', '1.2')
order(v)
orderVersion(v)


sort(v)
sortVersion(v)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{hasArg2}
\alias{hasArg2}
\title{Checking for Missing Arguments}
\usage{
hasArg2(name)
}
\arguments{
\item{name}{the name of an argument as a character string.}
}
\value{
A logical flag.
}
\description{
This function is identical to \code{\link{hasArg}}, except that
it accepts the argument name as a character string.
This avoids to have a check NOTE about invisible binding variable.
}
\examples{

f <- function(...){ hasArg2('abc') }
f(a=1)
f(abc=1)
f(b=1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{checkPlot}
\alias{checkPlot}
\title{Plot in Unit Tests}
\usage{
checkPlot(expr, msg = NULL, width = 1000, height = NULL)
}
\arguments{
\item{expr}{expression that generate th eplot}

\item{msg}{plot msg explaining the plot . It will be used as the caption}

\item{width}{plot width}

\item{height}{plot height (not used)}
}
\description{
Saves a plot in a PNG file that will be included in unit test HTML reports.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/user.R
\name{askUser}
\alias{askUser}
\title{User Queries}
\usage{
askUser(msg, allowed = c("y", "n"), idefault = "n", default = "n",
  case.sensitive = FALSE)
}
\arguments{
\item{msg}{The output message}

\item{allowed}{Allowed input from the user}

\item{idefault}{default response in interactive mode.
This answer will be in upper case in the question and will be the one returned if the
user simply hits return.}

\item{default}{default response in non-interactive mode.

If \code{NA}, then the user is forced to provide an answer, even in non-interactive mode
(e.g., when run through \code{Rscript}).}

\item{case.sensitive}{Is the response case sensitive? Defaults to
    \code{FALSE}}
}
\value{
the character string typed/agreed by the user or directly the default answer in
non-interactive mode.
}
\description{
This function is an improved version of \code{userQuery} from Bioconductor \pkg{Biobase}
package, which asks the user about some task that needs her intervention to proceed,
e.g., ask if one should perform a computation, install a package, etc..
}

\name{requirePackage}
\alias{requirePackage}
\title{Require a Package}
\usage{
  requirePackage(pkg, ...)
}
\arguments{
  \item{pkg}{package name as a character string}

  \item{...}{extra arguments concatenated to for the header
  of the error message}
}
\description{
  Require a package with a custom error message
}

\name{install.dependencies}
\alias{install.dependencies}
\title{Installing All Package Dependencies}
\usage{
  install.dependencies(pkg = NULL, all = FALSE, ...,
    dryrun = FALSE)
}
\arguments{
  \item{pkg}{package path or source file}

  \item{all}{logical that indicates if 'Suggests' packages
  should be installed.}

  \item{...}{extra arguments passed to
  \code{\link{install.packages}}.}

  \item{dryrun}{logical that indicates if the packages
  should be effectively installed or only shown.}
}
\description{
  Install all dependencies from a package source directory
  or package source file.
}
\examples{
\dontshow{# roxygen generated flag
options(R_CHECK_RUNNING_EXAMPLES_=TRUE)
}

try( install.dependencies('Matrix', dryrun=TRUE) )
\dontrun{
install.dependencies("mypackage_1.0.tar.gz", dryrun=TRUE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rd.R
\name{RdSection2latex}
\alias{RdSection2latex}
\title{Format Rd Sections into LatTeX}
\usage{
RdSection2latex(topic, package, i = 1L, notitle = TRUE)
}
\arguments{
\item{topic}{Rd topic}

\item{package}{package in which to search the topic}

\item{i}{index of the section to format}

\item{notitle}{logical that indicates if the section's title should be removed}
}
\value{
Nothing, just prints the LaTeX code on console.
}
\description{
This function extract sections from Rd files and convert them into
LaTeX code.
This can be useful to include Rd text into vignettes, hence keeping them
up to date.
}
\section{Example section}{
 This is a nice section, with a bullet list:
\itemize{
\item tata
\item toto
}
}

\examples{
RdSection2latex('RdSection2latex', package = 'pkgmaker')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/project.R
\name{load_project}
\alias{load_project}
\alias{library_project}
\title{Load Development Package}
\usage{
load_project(
  pkg,
  reset = FALSE,
  ...,
  utests = TRUE,
  verbose = FALSE,
  addlib = TRUE,
  character.only = FALSE,
  try.library = FALSE
)

library_project(...)
}
\arguments{
\item{pkg}{name of the package/project to load.}

\item{reset}{logical that indicates if the package should be reloaded (passed to \code{\link[devtools]{load_all}}.}

\item{...}{other arguments passed to \code{\link[devtools]{load_all}}.}

\item{utests}{logical that indicates if an environment containing the unit test functions should be created.
If \code{TRUE} this environment is accessible at \code{pkgname::UnitTests$test.filename.r$function.name}.}

\item{verbose}{logical that indicates if log messages should be printed.}

\item{addlib}{logical that indicates if the \code{lib/} sub-directory, if it exists, should be prepended
to the library path.
This enables to control the version of the loaded dependencies.}

\item{character.only}{logical that indicates if argument \var{pkg} should be evaluated or taken litteral.}

\item{try.library}{logicatl that indicates if projects that could not be found should be looked up in
the installed packages.}
}
\value{
Invisibly the \code{package} object of the loaded package.
}
\description{
Load Development Package
}
\section{Functions}{
\itemize{
\item \code{library_project()}: shortcut for \code{load_project(..., try.library = TRUE)}, to load project
code from installed library if not found as a development project.
All its arguments are passed to \code{load_project}.

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/registry.R
\name{fix_registry}
\alias{fix_registry}
\title{Fix Registry Access Functions}
\usage{
fix_registry(regobj)
}
\arguments{
\item{regobj}{registry object}
}
\description{
Fixes some of the member functions for registry objects, as defined in the
\pkg{registry} package.
The main fixed issue is due to the fact that key filtering does not return
the correct entry, when an entry key is a prefix of another entry key,
even when passed the exact key.
}
\concept{registry-internals}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibtex.R
\name{bibtex}
\alias{bibtex}
\alias{packageReferenceFile}
\alias{package_bibliography}
\title{Bibtex Utilities}
\usage{
packageReferenceFile(PACKAGE = NULL, check = FALSE)

package_bibliography(PACKAGE = NULL, action = c("path", "copy", "load"))
}
\arguments{
\item{PACKAGE}{package name. If \code{NULL}, then the name of the calling package is used.}

\item{check}{logical that indicates if the result should be an empty string if the
bibliography file (or package) does not exist.}

\item{action}{single character string that specifies the action to be performed:
\itemize{
\item 'path': return the path to the bibliography file. It returns an empty character
string if the file does not exist.
\item 'copy': copy the bibliography file to the current directory, overwriting any existing
\code{REFERENCES.bib} file.
\item 'load': load the bibliography file and return a list of \link[utils:bibentry]{utils::bibentry}
objects. It returns \code{NULL} if the file does not exist.
}}
}
\value{
\itemize{
\item \code{packageReferenceFile}: returns the path to the REFERENCES file as a character string.
}

\itemize{
\item \code{package_bibliography}: returns the bibiliography as a bibtex list object,
as returned by \link[rbibutils:readBib]{rbibutils::readBib}.
}
}
\description{
Utility functions to work with BiBTeX files.
}
\section{Functions}{
\itemize{
\item \code{packageReferenceFile()}: returns the path to a package REFERENCES.bib file.

\item \code{package_bibliography()}: returns the bibliography associated with a package.
This can

}}
\examples{

packageReferenceFile('pkgmaker')
packageReferenceFile('pkgmaker', check = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphics.R
\name{graphics-utils}
\alias{graphics-utils}
\alias{mfrow}
\title{Utility Functions for Graphics}
\usage{
mfrow(n)
}
\arguments{
\item{n}{number of plots to be arranged.}
}
\description{
Utility Functions for Graphics

\code{mfrow} returns a 2-long numeric vector suitable to use in \code{\link{par}(mfrow=x)},
that will arrange \code{n} panels in a single plot.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}

 
mfrow(1)
mfrow(2)
mfrow(3)
mfrow(4)
mfrow(10)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\docType{data}
\name{CRAN}
\alias{CRAN}
\title{Main CRAN Mirror URL}
\format{An object of class \code{character} of length 1.}
\usage{
CRAN
}
\description{
\code{CRAN} simply contains the url of CRAN main mirror
(\url{https://cran.r-project.org}), and aims at simplifying its use, e.g., in
calls to \code{\link{install.packages}}.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}

\dontrun{
install.packages('pkgmaker', repos=CRAN)
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/with.r
\name{using_something}
\alias{using_something}
\alias{using_envvar}
\alias{using_env}
\alias{using_locale}
\alias{using_collate}
\alias{using_dir}
\alias{using_libpaths}
\alias{using_lib}
\alias{using_options}
\alias{using_par}
\alias{using_path}
\title{Execute code in temporarily altered environment.}
\usage{
using_envvar(new, code, action = "replace")

using_env(new, code)

using_locale(new, code)

using_collate(new, code)

using_dir(new, code)

using_libpaths(new, code)

using_lib(new, code)

using_options(new, code)

using_par(new, code)

using_path(new, code, add = TRUE, prepend = FALSE)
}
\arguments{
\item{new}{values for setting}

\item{code}{code to execute in that environment}

\item{action}{(for \code{using_envvar} only): should new values
\code{"replace"}, \code{"suffix"}, \code{"prefix"} existing environmental
variables with the same name.}

\item{add}{Combine with existing values? Currently for
\code{\link{using_path}} only. If \code{FALSE} all existing
paths are overwritten, which you don't usually want.}

\item{prepend}{logical that indicates if the new paths should
be added in front of the current ones.}
}
\value{
Nothing, used for side effect.
}
\description{
These functions were extracted from the \pkg{devtools} package
to make them available without a dependency to \pkg{devtools}.
}
\details{
\itemize{
\item \code{using_dir}: working directory
\item \code{using_collate}: collation order
\item \code{using_envvar}: environmental variables
\item \code{using_libpaths}: library paths, replacing current libpaths
\item \code{using_lib}: library paths, prepending to current libpaths
\item \code{using_locale}: any locale setting
\item \code{using_options}: options
\item \code{using_path}: PATH environment variable
\item \code{using_par}: graphics parameters
}
}
\section{Deprecation}{

\code{using_env} will be deprecated in devtools 1.2 and removed in
devtools 1.3
}

\examples{
getwd()
using_dir(tempdir(), getwd())
getwd()

Sys.getenv("HADLEY")
using_envvar(c("HADLEY" = 2), Sys.getenv("HADLEY"))
Sys.getenv("HADLEY")

using_envvar(c("A" = 1),
  using_envvar(c("A" = 2), action = "suffix", Sys.getenv("A"))
)
}
\author{
Hadley Wickham
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unitTests.R
\name{utestFramework}
\alias{utestFramework}
\title{Inferring Unit Test Framework}
\usage{
utestFramework(x, eval = FALSE)
}
\arguments{
\item{x}{an filename, a function or the body of a function}

\item{eval}{a logical that indicates if the value of \code{x} should be used.}
}
\value{
the name of the framework as a character string or NULL if
it could not be detected.
}
\description{
Inferring Unit Test Framework
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/devutils.R
\name{packageEnv}
\alias{packageEnv}
\alias{topns_name}
\alias{topns}
\alias{packageName}
\alias{str_ns}
\alias{packagePath}
\alias{isPackageInstalled}
\alias{as_package}
\title{Package Development Utilities}
\usage{
packageEnv(pkg, skip = FALSE, verbose = FALSE)

topns_name(n = 1L, strict = TRUE, unique = TRUE)

topns(strict = TRUE)

packageName(envir = packageEnv(), .Global = FALSE, rm.prefix = TRUE)

str_ns(envir = packageEnv())

packagePath(..., package = NULL, lib.loc = NULL, check = TRUE)

isPackageInstalled(..., lib.loc = NULL)

as_package(x, ..., quiet = FALSE, extract = FALSE)
}
\arguments{
\item{pkg}{package name. If missing the environment of the runtime caller
package is returned.}

\item{skip}{a logical that indicates if the calling namespace should be
skipped.}

\item{verbose}{logical that toggles verbosity}

\item{n}{number of namespaces to return}

\item{strict}{a logicical that indicates if the global environment should
be considered as a valid namespace.}

\item{unique}{logical that indicates if the result should be reduced
to contain only one occurence of each namespace.}

\item{envir}{environment where to start looking for a package name.
The default is to use the \strong{runtime} calling package environment.}

\item{.Global}{a logical that indicates if calls from the global
environment should throw an error (\code{FALSE}: default) or the string
\code{'R_GlobalEnv'}.}

\item{rm.prefix}{logical that indicates if an eventual prefix 'package:'
should be removed from the returned string.}

\item{...}{arguments passed to \code{\link{file.path}}.}

\item{package}{optional name of an installed package}

\item{lib.loc}{path to a library of R packages where to search the package}

\item{check}{logical that indicates if an error should be thrown if the path to the
package root directory cannot be found.
If this is the case and \code{check = FALSE}, then the function returns \code{NULL}.}

\item{x}{package specified by its installation/development path or its name
as \code{'package:*'}.}

\item{quiet}{a logical that indicate if an error should be thrown if a
package is not found. It is also passed to \code{\link{find.package}}.}

\item{extract}{logical that indicates if DESCRIPTION of package
source files should be extracted.
In this case there will be no valid path.}
}
\value{
\code{packageEnv}: \code{packageEnv} returns an environment

\code{packageName}: a character string

\code{packagePath}: a character string
}
\description{
\code{packageEnv} is a slight modification from \code{\link{topenv}}, which
returns the top environment, which in the case of development
packages is the environment into which the source files are loaded by
\code{\link[devtools]{load_all}}.
}
\section{Functions}{
\itemize{
\item \code{topns_name}: returns the name of the runtime sequence of top namespace(s),
i.e. the name of the top calling package(s), from top to bottom.

The top namespace is is not necessarily the namespace where \code{topns_name}
is effectively called.
This is useful for packages that define functions that need to access the
calling namespace, even from calls nested into calls to another function from
the same package -- in which case \code{topenv} would not give the desired
environment.

\item \code{topns}: returns the runtime top namespace, i.e. the namespace of
the top calling package, possibly skipping the namespace where \code{topns}
is effectively called.
This is useful for packages that define functions that need to access the
calling namespace, even from calls nested into calls to another function from
the same package -- in which case \code{topenv} would not give the desired
environment.

\item \code{packageName}: returns the current package's name.
It was made internal from version 0.16, since the package \pkg{utils}
exported its own \code{\link[utils]{packageName}} function in R-3.0.0.

\item \code{str_ns}: formats a package environment/namespace for log/info messages.

\item \code{packagePath}: returns the current package's root directory, which is
its installation/loading directory in the case of an installed package, or
its source directory served by devtools.

\item \code{isPackageInstalled}: checks if a package is installed.

\item \code{as_package}: an enhanced version of \code{\link[devtools]{as.package}},
that is not exported not to mask the original function.
It could eventually be incorporated into \code{devtools} itself.
Extra arguments in \code{...} are passed to \code{\link{find.package}}.
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{factor2character}
\alias{factor2character}
\title{Converting Factors to Character Vectors}
\usage{
factor2character(x)
}
\arguments{
\item{x}{\code{data.frame} or \code{ExpressionSet} object}
}
\value{
an object of the same class as \code{x}.
}
\description{
Converts all \code{factor} variables to character vectors in a \code{data.frame}
or phenotypic data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{isCRANcheck}
\alias{isCRANcheck}
\alias{isCRAN_timing}
\alias{isCHECK}
\title{Package Check Utils}
\usage{
isCRANcheck(...)

isCRAN_timing()

isCHECK()
}
\arguments{
\item{...}{each argument specifies a set of tests to do using an AND operator.
The final result tests if any of the test set is true.
Possible values are:
\describe{
\item{\code{'timing'}}{Check if the environment variable \code{_R_CHECK_TIMINGS_} is set,
as with the flag \code{'--timing'} was set.}
\item{\code{'cran'}}{Check if the environment variable \code{_R_CHECK_CRAN_INCOMING_} is set,
as with the flag \code{'--as-cran'} was set.}
}}
}
\value{
A logical flag.
}
\description{
\code{isCRANcheck} \strong{tries} to identify if one is running CRAN-like checks.
}
\details{
Currently \code{isCRANcheck} returns \code{TRUE} if the check is run with
either environment variable \code{_R_CHECK_TIMINGS_} (as set by flag \code{'--timings'})
or \code{_R_CHECK_CRAN_INCOMINGS_} (as set by flag \code{'--as-cran'}).

\strong{Warning:} the checks performed on CRAN check machines are on purpose not always
run with such flags, so that users cannot effectively "trick" the checks.
As a result, there is no guarantee this function effectively identifies such checks.
If really needed for honest reasons, CRAN recommends users rely on custom dedicated environment
variables to enable specific tests or examples.
}
\section{Functions}{
\itemize{
\item \code{isCRAN_timing()}: tells if one is running CRAN check with flag \code{'--timing'}.

\item \code{isCHECK()}: tries harder to test if running under \code{R CMD check}.
It will definitely identifies check runs for:
\itemize{
\item unit tests that use the unified unit test framework defined by \pkg{pkgmaker} (see \code{\link{utest}});
\item examples that are run with option \code{R_CHECK_RUNNING_EXAMPLES_ = TRUE},
which is automatically set for man pages generated with a fork of \pkg{roxygen2} (see \emph{References}).
}

Currently, \code{isCHECK} checks both CRAN expected flags, the value of environment variable
\code{_R_CHECK_RUNNING_UTESTS_}, and the value of option \code{R_CHECK_RUNNING_EXAMPLES_}.
It will return \code{TRUE} if any of these environment variables is set to
anything not equivalent to \code{FALSE}, or if the option is \code{TRUE}.
For example, the function \code{\link{utest}} sets it to the name of the package
being checked (\code{_R_CHECK_RUNNING_UTESTS_=<pkgname>}),
but unit tests run as part of unit tests vignettes are run with
\code{_R_CHECK_RUNNING_UTESTS_=FALSE}, so that all tests are run and reported when
generating them.

}}
\examples{

isCHECK()

}
\references{
Adapted from the function \code{CRAN}
in the \pkg{fda} package.

\url{https://github.com/renozao/roxygen}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/files.R
\name{list.libs}
\alias{list.libs}
\alias{libname}
\title{Library Files Utilities}
\usage{
list.libs(dir, ..., all.platforms = FALSE)

libname(x)
}
\arguments{
\item{dir}{directory}

\item{...}{extra arguments passed to \code{\link{list.files}}.}

\item{all.platforms}{a logical that indicates whether to list library files for
the current platform only (default) or all platforms (Unix, Windows, Mac).}

\item{x}{a filename}
}
\value{
\code{list.libs}: a character vector
}
\description{
Lists binary library files in a directory
}
\section{Functions}{
\itemize{
\item \code{libname}: extracts library names from a path, removing the
directory part of the path, as well as the platform
specific library extension.
}}

\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


libname('mylib.so')
libname('/some/path/somewhere/mylib.dll') 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\name{postponeAction}
\alias{postponeAction}
\alias{runPostponedAction}
\title{Postponing Actions}
\usage{
postponeAction(expr, key = digest(tempfile()), group = NULL,
  envir = topns(strict = FALSE), verbose = getOption("verbose"))

runPostponedAction(group = NULL, verbose = getOption("verbose"))
}
\arguments{
\item{expr}{expression that define the action to postpone.
Currently only functions are supported.}

\item{key}{identifier for this specific action.
It should be unique across the postponed actions from the same group.}

\item{group}{optional parent action group.
This enables to define meaningful sets of actions that can be run all at once.}

\item{envir}{environment in which the action should be executed.
Currently not used.}

\item{verbose}{logical that toggles verbose messages.}
}
\description{
This function implement a mechanism to postpone actions, which can be executed
at a later stage.
This is useful when developing packages, where actions that need to be run in the
\code{link{.onLoad}} function but can be defined close to their context.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}

opt <- options(verbose=2)

# define actions
postponeAction(function(){print(10)}, "print")
postponeAction(function(){print(1:10)}, "more")
postponeAction()
# execute actions
runPostponedAction()
runPostponedAction()

# restore options
options(opt)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{quickinstall}
\alias{quickinstall}
\title{Quick Installation of a Source Package}
\usage{
quickinstall(path, destdir = NULL, vignettes = FALSE, force = TRUE,
  ..., lib.loc = if (!is.null(destdir)) TRUE)
}
\arguments{
\item{path}{path to the package source directory}

\item{destdir}{installation directory.
If \code{NULL}, the package is installed in the default installation library.
If \code{NA}, the package is installed in a temporary directory, whose path is returned
as a value.}

\item{vignettes}{logical that indicates if the vignettes should be
rebuilt and installed.}

\item{force}{logical that indicates if the package should be installed even if a previous
installation exists in the installation library.}

\item{...}{extra arguments passed to \code{\link{R.CMD}}}

\item{lib.loc}{library specification.
If \code{TRUE} then the installation directory \code{destdir} is added to the default
library paths.
This can be usefull if dependencies are installed in this directory.
If \code{NULL}, then the default library path is left unchanged.}
}
\value{
The path of the library where the package was installed.
}
\description{
Builds and install a minimal version of a package from its
source directory.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{rnw}
\alias{rnw}
\alias{isManualVignette}
\alias{as.rnw}
\alias{rnwCompiler}
\alias{rnwWrapper}
\alias{rnwDriver}
\alias{rnwIncludes}
\alias{rnwChildren}
\alias{vignetteMakefile}
\alias{compactVignettes}
\title{Utilities for Vignettes}
\usage{
rnw(x, file = NULL, ..., raw = FALSE)

isManualVignette()

as.rnw(x, ..., load = TRUE)

rnwCompiler(x, verbose = TRUE)

rnwWrapper(x, verbose = TRUE)

rnwDriver(x)

rnwIncludes(x)

rnwChildren(x)

vignetteMakefile(
  package = NULL,
  skip = NULL,
  print = TRUE,
  template = NULL,
  temp = FALSE,
  checkMode = isCHECK() || vignetteCheckMode(),
  user = NULL,
  tests = TRUE
)

compactVignettes(paths, ...)
}
\arguments{
\item{x}{vignette source file specification as a path or a \code{rnw} object.}

\item{file}{output file}

\item{...}{extra arguments passed to \code{as.rnw} that can be used to force
certain building parameters.}

\item{raw}{a logical that indicates if the raw result for the compilation
should be returned, instead of the result file path.}

\item{load}{logical to indicate if all the object's properties should loaded,
which is done by parsing the file and look up for specific tags.}

\item{verbose}{logical that toggles verbosity}

\item{package}{package name.
If \code{NULL}, a DESRIPTION file is looked for one directory up: this
meant to work when building a vignette directly from a package's
\code{'vignettes'} sub-directory.}

\item{skip}{Vignette files to skip (basename).}

\item{print}{logical that specifies if the path should be printed or
only returned.}

\item{template}{template Makefile to use.
The default is to use the file \dQuote{vignette.mk} shipped with the package
\pkg{pkgmaker} and can be found in its install root directory.}

\item{temp}{logical that indicates if the generated makefile should using a
temporary filename (\code{TRUE}), or simply named \dQuote{vignette.mk}}

\item{checkMode}{logical that indicates if the vignettes should be generated as in a
CRAN check (\code{TRUE}) or in development mode, in which case \code{pdflatex}, \code{bibtex},
and, optionally, \code{qpdf} are required.}

\item{user}{character vector containing usernames that enforce \code{checkMode=TRUE},
if the function is called from within their session.}

\item{tests}{logical that enables the compilation of a vignette that gathers all unit
test results.
Note that this means that all unit tests are run before generating the vignette.
However, unit tests are not (re)-run at this stage when the vignettes are built
when checking the package with \code{R CMD check}.}

\item{paths}{A character vector of paths to PDF files, or a length-one
    character vector naming a directory, when all \file{.pdf} files in
    that directory will be used.}
}
\value{
\code{rnw} returns the result of compiling the vignette with \link{runVignette}.
}
\description{
\code{rnw} provides a unified interface to run vignettes that detects
the type of vignette (Sweave or knitr), and which Sweave driver
to use (either automatically or from an embedded command \code{\\VignetteDriver}
command).
}
\section{Functions}{
\itemize{
\item \code{isManualVignette()}: tells if a vignette is being run through the function \code{runVignette}
of \pkg{pkgmker}, allowing disabling behaviours not allowed in package vignettes that are
checked vi \code{R CMD check}.

\item \code{as.rnw()}: creates a S3 \code{rnw} object that contains information
about a vignette, e.g., source filename, driver, fixed included files, etc..

\item \code{rnwCompiler()}: tries to detect the vignette compiler to use on a vignette
source file, e.g., \code{\link{Sweave}} or \link[knitr:knitr-package]{knitr}.

\item \code{rnwWrapper()}: tries to detect the type of vignette and if it is meant
to be wrapped into another main file.

\item \code{rnwDriver()}: tries to detect Sweave driver to use on a vignette source
file, e.g., \code{SweaveCache}, \code{highlight}, etc..

\item \code{rnwIncludes()}: detects fixed includes, e.g., image or pdf files, that are
required to build the final document.

\item \code{rnwChildren()}: detects included vignette documents and return them as a
list of vignette objects.

\item \code{vignetteMakefile()}: returns the path to a generic makefile used to make
vignettes.

\item \code{compactVignettes()}: compacts vignette PDFs using either \code{gs_quality='none'} or \code{'ebook'},
depending on which compacts best (as per CRAN check criteria).

}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{sVariable}
\alias{sVariable}
\title{Global Static Variable}
\usage{
sVariable(default = NULL)
}
\arguments{
\item{default}{default value for the static variable.}
}
\value{
A function that can be used to set/get the static variable.
}
\description{
\code{sVariable} defines a function that acts as a global
static variable.
}
\examples{

# define variable
x <- sVariable(1)
# get value (default)
x()
# set new value: return old value
old <- x(3)
old
# get new value
x()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{makeFakeVignette}
\alias{makeFakeVignette}
\title{Generate a Fake Vignette}
\usage{
makeFakeVignette(src, out, PACKAGE = NULL)
}
\arguments{
\item{src}{original Sweave file}

\item{out}{output file}

\item{PACKAGE}{package name where to look the source vignette}
}
\description{
Generate a Fake Vignette
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{Sys.getenv_value}
\alias{Sys.getenv_value}
\title{System Environment Variables}
\usage{
Sys.getenv_value(name, raw = FALSE)
}
\arguments{
\item{name}{variable name as a character string.}

\item{raw}{logical that indicates if one should return the raw value or
the convertion of any false value to \code{FALSE}.}
}
\value{
the value of the environment variable as a character string or
\code{NA} is the variable is not defined \strong{at all}.
}
\description{
System Environment Variables
}
\examples{

# undefined returns FALSE
Sys.getenv_value('TOTO')
# raw undefined returns NA
Sys.getenv_value('TOTO', raw = TRUE)

Sys.setenv(TOTO='bla')
Sys.getenv_value('TOTO')

# anything false-like returns FALSE
Sys.setenv(TOTO='false'); Sys.getenv_value('TOTO')
Sys.setenv(TOTO='0'); Sys.getenv_value('TOTO')

# cleanup
Sys.unsetenv('TOTO')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\name{addnames}
\alias{addnames}
\alias{addnames.default}
\alias{addnames.vector}
\alias{addnames.array}
\alias{addnames.matrix}
\title{Generating Names}
\usage{
addnames(x, ...)

\method{addnames}{default}(x, ...)

\method{addnames}{vector}(x, prefix = "x", sep = "", ...)

\method{addnames}{array}(x, prefix = letters[1:length(dim(x))],
  sep = "", ...)

\method{addnames}{matrix}(x, prefix = c("row", "col"), ...)
}
\arguments{
\item{x}{object whose names are generated.}

\item{...}{extra arguments to allow extension and passed to the next method.}

\item{prefix}{prefix string to use. A vector can be used to specify a prefix
for each dimension of \code{x}.
Names are build as \code{<prefix><sep><index>}.}

\item{sep}{separator used between the prefix and the numeric index.}
}
\description{
Generates names or dimnames for objects.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{add_lib}
\alias{add_lib}
\title{Adding Package Libraries}
\usage{
add_lib(..., append = FALSE)
}
\arguments{
\item{...}{paths to add to .libPath}

\item{append}{logical that indicates that the paths should be appended
rather than prepended.}
}
\description{
Prepend/append paths to the library path list, using \code{\link{.libPaths}}.
}
\details{
This function is meant to be more convenient than \code{.libPaths}, which requires
more writing if one wants to:
\itemize{
\item sequentially add libraries;
\item append and not prepend new path(s);
\item keep the standard user library in the search path.
}
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}

ol <- .libPaths()
# called sequentially, .libPaths only add the last library
show( .libPaths('.') )
show( .libPaths(tempdir()) )
# restore
.libPaths(ol)

# .libPaths does not keep the standard user library
show( .libPaths() ) 
show( .libPaths('.') )
# restore
.libPaths(ol)

# with add_lib
show( add_lib('.') )
show( add_lib(tempdir()) )
show( add_lib('..', append=TRUE) )

# restore 
.libPaths(ol)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logging.R
\name{iterCount}
\alias{iterCount}
\title{Simple Text Iteration Counter}
\usage{
iterCount(n = 100, i0 = 0L, title = "Iterations", extra = NULL,
  verbose = TRUE)
}
\arguments{
\item{n}{number of total steps}

\item{i0}{starting step}

\item{title}{character string to use as title}

\item{extra}{character vector providing extra text to add at each step}

\item{verbose}{logical that toggles the counter}
}
\description{
Simple Text Iteration Counter
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}


progress <- iterCount(LETTERS)
res <- sapply(LETTERS, function(x){
 Sys.sleep(.1)
	progress()
})
# terminate counter
i_end <- progress(NULL)
i_end 

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bibtex.R
\name{packageReference}
\alias{packageReference}
\title{Package References}
\usage{
packageReference(key, short = FALSE, PACKAGE = NULL)
}
\arguments{
\item{key}{character vector of BibTex keys}

\item{short}{logical that indicates if the reference should be shorten as
First Author et al. if it has more than one author.}

\item{PACKAGE}{package in which the BiBTeX entry is defined.}
}
\value{
a character string containing the text formated BibTex entries
}
\description{
Create a citation string from package specific BibTex entries, suitable to
be used in Rd files.
The entries are looked in a file named REFERNCES.bib in the package's root
directory (i.e. inst/ in development mode).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/files.R
\name{file_extension}
\alias{file_extension}
\title{Extract File Extension}
\usage{
file_extension(x, ext = NULL)
}
\arguments{
\item{x}{path as a character vector.}

\item{ext}{extension to append instead of the original extension.}
}
\description{
Extract File Extension
}
\examples{

file_extension('alpha.txt')
file_extension(paste('aa.tt', 1:5, sep=''))
# change extension
file_extension(paste('aa.tt', 1:5, sep=''), 'pdf')
file_extension(paste('aatt', 1:5, sep=''), 'pdf')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{setupPackageOptions}
\alias{setupPackageOptions}
\title{Package Specific Options}
\usage{
setupPackageOptions(..., NAME = NULL, ENVIR = topenv(parent.frame()),
  RESET = isLoadingNamespace())
}
\arguments{
\item{...}{a single named list or named arguments that provide the default
options and their values.}

\item{NAME}{name of the set of options.
This is used as a prefix for the name of the associated global
option: \code{package:<name>}.}

\item{ENVIR}{environment where the option wrapper functions will be defined.
No function is defined if \code{ENVIR=NULL}}

\item{RESET}{a logical that indicates whether the option set should overwrite
one that already exists if necessary.
The default is \code{FALSE} (i.e. no reset), except when loading a namespace,
either from an installed package or a development package -- with devtools.
If \code{FALSE}, an error is thrown if trying to setup options with the same name.}
}
\description{
The following functions to access/set the options from the set are assigned
in \code{envir}:
\describe{
\item{<subset>Options}{}
\item{<subset>GetOption}{}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{extractLocalFun}
\alias{extractLocalFun}
\alias{allFormals}
\title{Extracting Local Function Definition}
\usage{
extractLocalFun(f)

allFormals(f)
}
\arguments{
\item{f}{definition of the wrapper function}
}
\value{
a function

a paired list like the one returned by \code{\link{formals}}.
}
\description{
\code{extractLocalFun} Extracts local function from wrapper functions of the following type, typically
used in S4 methods:
\samp{
function(a, b, ...)\{
	.local <- function(a, b, c, d, ...)\{\}
.local(a, b, ...)
\}
}

Works for methods that are created (setMethod) as a wrapper function to an
internal function named .local.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{option_symlink}
\alias{option_symlink}
\alias{is_option_symlink}
\alias{option_symlink_target}
\alias{as.package_options}
\alias{[[.package_options}
\alias{packageOptions}
\alias{listPackageOptions}
\title{\code{option_symlink} creates a symbolic link to option \code{x}.}
\usage{
option_symlink(x)

is_option_symlink(x, opts)

option_symlink_target(x, opts)

as.package_options(..., defaults = NULL)

\method{[[}{package_options}(x, ...)

packageOptions(..., PACKAGE = packageName())

listPackageOptions()
}
\arguments{
\item{x}{a character string, a list or an object of class
\code{package_options}.}

\item{opts}{a list of options}

\item{...}{arguments passed to \code{getOption} (only first one is used).}

\item{defaults}{\code{NULL} or a list of default options
with their values.}

\item{PACKAGE}{a package name}
}
\value{
\code{listPackageOptions}: a character vector (possibly empty).
}
\description{
\code{option_symlink} creates a symbolic link to option \code{x}.

\code{is_option_symlink} tests if \code{x} is a symbolic link option.

\code{option_symlink_target} returns the end target option of a symbolic link
option \code{x}.

\code{as.package_options} creates an object such as the
ones used to stores package specific options.

The method \code{[[} is equivalent to \code{options()} or \code{getOption(...)}:
e.g. \code{obj[[]]} returns the list of options defined in \code{obj}, and
\code{obj[['abc']]} returns the value of option \code{'abc'}.

\code{packageOptions} provides access to package specific options from a
given package that were defined with \code{setupPackageOptions}, and behaves as the base function \code{\link[base]{options}}.

\code{listPackageOptions} returns the names of all option
currently defined with \code{setupPackageOptions}.
}
\examples{
\dontshow{
options(R_CHECK_RUNNING_EXAMPLES_=TRUE) ## roxygen generated flag
}

listPackageOptions()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{hasEnvar}
\alias{hasEnvar}
\title{Check Environment Variables}
\usage{
hasEnvar(x)
}
\arguments{
\item{x}{environment variable name, as a character vector.}
}
\value{
A logical flag.
}
\description{
Tells if some environment variable(s) are defined.
}
\examples{

hasEnvar('_R_CHECK_TIMINGS_')
hasEnvar('ABCD')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R, R/knitr.R
\name{str_out}
\alias{str_out}
\alias{str_desc}
\alias{str_fun}
\alias{str_class}
\alias{str_pkg}
\alias{str_md5sum}
\alias{str_hash}
\alias{str_dim}
\alias{str_bs}
\title{Formatting Utilities}
\usage{
str_out(
  x,
  max = 3L,
  quote = is.character(x),
  use.names = FALSE,
  sep = ", ",
  total = FALSE
)

str_desc(object, exdent = 0L)

str_fun(object)

str_class(x, max = Inf, ...)

str_pkg(pkg, lib.loc = NULL)

str_md5sum(x)

str_hash(x, algo = "md5")

str_dim(x, dims = dim(x))

str_bs(x)
}
\arguments{
\item{x}{character vector}

\item{max}{maximum number of values to appear in the list. If \code{x} has
more elements than \code{max}, a \code{"..."} suffix is appended.}

\item{quote}{a logical indicating whether the values should be quoted with
single quotes (defaults) or not.}

\item{use.names}{a logical indicating whether names should be added to the
list as \code{NAME=VAL, ...} or not (default).}

\item{sep}{separator character}

\item{total}{logical that indicates if the total number of elements should be
appended to the formatted string as \code{"'a', ..., 'z' (<N> total)"}.}

\item{object}{an R object}

\item{exdent}{extra indentation passed to str_wrap, and used if the output
should spread over more than one lines.}

\item{...}{other arguments passed to \link{str_out}.}

\item{pkg}{package name}

\item{lib.loc}{path to a library of R packages}

\item{algo}{The algorithms to be used; currently available choices are
    \code{md5}, which is also the default, \code{sha1}, \code{crc32},
    \code{sha256}, \code{sha512}, \code{xxhash32}, \code{xxhash64},
    \code{murmur32}, \code{spookyhash} and \code{blake3}.}

\item{dims}{a numeric vector of dimensions.
Default is to use the input object dimensions (via function \code{dims()})}
}
\value{
a single character string

\itemize{
\item \code{str_bs} returns a character string.
}
}
\description{
\code{str_out} formats character vectors for use in show methods or
error/warning messages.
}
\section{Functions}{
\itemize{
\item \code{str_desc()}: builds formatted string from a list of complex values.

\item \code{str_fun()}: extracts and formats a function signature.
It typically formats the output \code{capture.output(args(object))}.

\item \code{str_class()}: outputs the class(es) of an object using \code{str_out}.

\item \code{str_pkg()}: formats a package name and version

\item \code{str_md5sum()}: computes md5sum on character vector using \code{\link[tools]{md5sum}}.

\item \code{str_hash()}: computes hash of a character vector using \code{\link[digest]{digest}}.

\item \code{str_dim()}: builds a string that describes the dimension of an object, in the form
\verb{n x m} for 2D-objects, \verb{n x m x p} for 3D-objects, and so on.

\item \code{str_bs()}: substitutes backspace characters (\verb{\\\\b}) to produce
a character string as it would be displayed in the console.

}}
\examples{

x <- letters[1:10]
str_out(x)
str_out(x, 8)
str_out(x, Inf)
str_out(x, quote=FALSE)
str_out(x, total = TRUE)

str_fun(install.packages)
str_class(matrix())

# Backspace substitution
str_bs("abc")
str_bs("abc\b")
str_bs("abc\b\b")
str_bs("abc\bd")
str_bs("abc\b\bde\b")

# more complex example
x <- "\bab\nc\bd\n\babc\b\bd"
cat(x, "\n")
y <- str_bs(x)
y
cat(y, "\n")

}
\author{
Renaud Gaujoux

\code{str_bs} was adapted from a proposal from Yihui Xie.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vignette.R
\name{cite_pkg}
\alias{cite_pkg}
\title{\code{bibcite} provides an inline package citation functionnality.
Technically it adds a given Bibtex key to a cache that is used at the end of the
document processing to generate a .bib file with all citation keys.}
\usage{
cite_pkg(key, cache = NA, ...)
}
\arguments{
\item{key}{citation Bibtex key(s) as a character vector}

\item{cache}{specifies what to do with the previsouly chached keys.
If \code{TRUE}, then \code{key} is added to the cache.
If \code{NULL}, then all previously cached keys are deleted, before .
If a character string, then it specifies the path to a Bibtex file that is loaded
to initialise the cache.}

\item{...}{extra arguments passed to \link[rbibutils:readBib]{rbibutils::readBib}.}
}
\description{
\code{bibcite} provides an inline package citation functionnality.
Technically it adds a given Bibtex key to a cache that is used at the end of the
document processing to generate a .bib file with all citation keys.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{mkoptions}
\alias{mkoptions}
\alias{.options}
\title{Quick Option-like Feature}
\usage{
mkoptions(...)

.options(..., .DATA)
}
\arguments{
\item{...}{list of keys or key-value pairs.
For \code{mkoptions} these define inital/default key-value pairs.}

\item{.DATA}{a list or an environment with an element \code{.options}.}
}
\value{
\code{mkoptions} returns a function.
}
\description{
\code{mkoptions} is a function that returns a function that
behaves like \code{\link[base]{options}}, with an attached
internal/local list of key-value pairs.
}
\section{Functions}{
\itemize{
\item \code{.options()}: is a low-level function that mimics the behaviour
of the base function \code{\link[base]{options}}, given a set
of key-value pairs.
It is the workhorse function used in \code{mkoptions} and package-specific
option sets (see \code{\link{setupPackageOptions}})

}}
\examples{
f <- mkoptions(a=3, b=list(1,2,3))
str(f())
f('a')
f('b')
str(old <- f(a = 10))
str(f())
f(old)
str(f())

}
\seealso{
\code{\link{setupPackageOptions}}
}
